@@ -18,7 +18,7 @@ gboolean screem_page_open_with_filename( ScreemSite *site,
 					 ScreemWindow *window,
 					 const gchar *filename );
 gint screem_page_save_as( ScreemPage *page, ScreemWindow *window );
-gint screem_page_save_confirmation( ScreemPage *page );
+gint screem_page_save_confirmation( ScreemPage *page, gboolean closing );
 void screem_page_create_blank_page( ScreemSite *site );
 gboolean screem_page_close_page( ScreemPage *page, ScreemWindow *window,
 				gboolean confirm );
