@@ -6,6 +6,9 @@
 #include "screem-dtd-db.h"
 #include "screem-dtd.h"
 
+#include "screem-page.h"
+#include "screem-site.h"
+
 const gchar* screem_markup_char_to_ent( guint c );
 
 gboolean screem_markup_is_entity( const gchar *text, gint pos,
@@ -43,5 +46,7 @@ void screem_markup_update_from_template( ScreemSite *site,
 gchar *screem_markup_basic_html( ScreemDTDDB *db, const gchar *doctype,
 		gboolean frameset, gboolean xhtml );
 
+gchar *screem_markup_get_charset( const gchar *text, 
+		gint *retpos, gint *retlen );
 
 #endif
