@@ -2,8 +2,7 @@
 #define SCREEM_SESSION_H
 
 #include <glib-object.h>
-
-#include "screem-application.h"
+#include <gtk/gtkwidget.h>
 
 #define SCREEM_TYPE_SESSION            (screem_session_get_type ())
 #define SCREEM_SESSION(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), SCREEM_TYPE_SESSION, ScreemSession))
@@ -27,10 +26,21 @@ typedef struct {
 } ScreemSessionClass;
 
 GType screem_session_get_type( void );
-ScreemSession* screem_session_new( ScreemApplication *app );
+ScreemSession* screem_session_new( GObject *app );
 
 void screem_session_load( ScreemSession *session,
 			  const gchar *filename );
 gboolean screem_session_save( ScreemSession *session, const gchar *filename );
 
+void screem_session_set_dialog( ScreemSession *session, 
+		const gchar *name, gint x, gint y, gint w, gint h );
+gboolean screem_session_get_dialog( ScreemSession *session, 
+		const gchar *name, gint *x, gint *y, gint *w, gint *h );
+
+/* util functions for screem_session_get/set_dialog() */
+void screem_session_restore_dialog( ScreemSession *session,
+		GtkWidget *widget );
+void screem_session_store_dialog( ScreemSession *Session,
+		GtkWidget *widget );
+
 #endif
