@@ -47,4 +47,7 @@ void screem_view_print( ScreemView *view, gboolean preview );
 void screem_view_undo( ScreemView *view );
 void screem_view_redo( ScreemView *view );
 
+void screem_view_show_message( ScreemView *view, const gchar *msg );
+void screem_view_show_error( ScreemView *view, const gchar *msg );
+
 #endif
