@@ -29,12 +29,7 @@
 #include <libgnomeui/gnome-uidefs.h>
 #include <libgnome/gnome-i18n.h>
 
-#include <gtk/gtkdialog.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtkcombo.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtklabel.h>
+#include <gtk/gtk.h>
 
 #include "screem-plugin.h"
 #include "screem-window.h"
@@ -42,8 +37,6 @@
 
 #include "fileops.h"
 
-#include "libegg/menu/egg-menu.h"
-
 /* per wizard struct */
 typedef struct {
 	ScreemWindow *window;
@@ -63,7 +56,7 @@ static GList *wizards = NULL;
 
 /* we don't want to try and load old plugins
    so this is a version symbol for screem to spot */
-int screem_plugin_version = 3;
+int screem_plugin_version = 4;
 
 #define NAME "Form Wizard"
 #define AUTHOR "David A Knight (david@screem.org)"
@@ -77,7 +70,7 @@ G_MODULE_EXPORT void add_ui( GtkWidget *window, GtkWidget *editor,
 			     GtkWidget *preview, GtkWidget *link_view );
 G_MODULE_EXPORT void remove_ui( GtkWidget *window, GtkWidget *editor,
 				GtkWidget *preview, GtkWidget *link_view );
-void form_wizard_display( EggAction *action, gpointer user_data );
+void form_wizard_display( GtkAction *action, gpointer user_data );
 void form_wizard_clicked( GtkWidget *widget, gint button, gpointer data );
 
 static GtkWidget* create_dialog( FormWizard *form_wizard );
@@ -115,27 +108,23 @@ G_MODULE_EXPORT void add_ui( GtkWidget *window, GtkWidget *editor,
 {
 	FormWizard *form_wizard;
 	const gchar *ui = "\
-<Root>\
-<menu>\
-<submenu name=\"Insert\" verb=\"Insert\">\
-<submenu name=\"Wizards\" verb=\"Wizards\">\
-<menuitem name=\"FormWizard\" verb=\"FormWizard\"/>\
-</submenu>\
-</submenu>\
+<ui>\
+<menubar>\
+<menu action=\"Insert\">\
+<menu action=\"Wizards\">\
+<menuitem action=\"FormWizard\"/>\
 </menu>\
-<dockitem name=\"Wizards Toolbar\">\
-<toolitem name=\"FormWizard\" verb=\"FormWizard\" />\
-</dockitem>\
-</Root>";
-	static EggActionGroupEntry entries[] = {
-		{ "FormWizard", "label",
-	  	GTK_STOCK_JUSTIFY_FILL, NULL, "tip",
-	  	G_CALLBACK( form_wizard_display ), NULL },
-	};
-	EggActionGroupEntry *entry;
+</menu>\
+</menubar>\
+<toolbar name=\"Wizards Toolbar\">\
+<toolitem action=\"FormWizard\" />\
+</toolbar>\
+</ui>";
+	GtkAction *action;
 
 	gchar *label;
 	gchar *tip;
+	GError *error;
 
 	form_wizard = g_new0( FormWizard, 1 );
 	form_wizard->window = SCREEM_WINDOW( window );
@@ -143,19 +132,22 @@ G_MODULE_EXPORT void add_ui( GtkWidget *window, GtkWidget *editor,
 
 	label = g_strdup( _( "Form Wizard" ) );
 	tip = g_strdup( _( "Create a Form" ) );
-	
-	entry = entries;
-	entry->label = label;
-	entry->tooltip = tip;
-	entry->user_data = form_wizard;
-	
-	egg_action_group_add_actions( EGG_ACTION_GROUP( form_wizard->window->action_group ),
-					entry, 1 );
-	egg_menu_merge_add_ui_from_string( EGG_MENU_MERGE( form_wizard->window->merge ),
-						ui, strlen( ui ), NULL );
+	action = gtk_action_new( "FormWizard", label, tip,
+			GTK_STOCK_JUSTIFY_FILL );
+	g_signal_connect( G_OBJECT( action ), "activate",
+			G_CALLBACK( form_wizard_display ), form_wizard );
+	gtk_action_group_add_action( GTK_ACTION_GROUP( form_wizard->window->action_group ), action );
 	g_free( label );
 	g_free( tip );
 
+	error = NULL;
+	if( ! gtk_ui_manager_add_ui_from_string( GTK_UI_MANAGER( form_wizard->window->merge ),
+						ui, strlen( ui ), &error ) ) {
+		g_message( "%s ui error = %s", "form wizard",
+				error->message );
+		g_error_free( error );
+	}
+	
 	/* NOTE: make sure that the window being obtained isn't visible */
 	form_wizard->dialog = create_dialog( form_wizard );
 
@@ -186,7 +178,7 @@ G_MODULE_EXPORT void remove_ui( GtkWidget *window, GtkWidget *editor,
 
 /* End of required section */
 
-void form_wizard_display( EggAction *action, gpointer user_data )
+void form_wizard_display( GtkAction *action, gpointer user_data )
 {
 	FormWizard *wizard;
 	ScreemPage *page;
@@ -205,12 +197,11 @@ void form_wizard_display( EggAction *action, gpointer user_data )
 
 void form_wizard_clicked( GtkWidget *widget, gint button, gpointer data )
 {
+	GtkWidget *entry;
 	const gchar *method;
 	const gchar *action;
 	const gchar *enctype;
 
-	GtkCombo *combo;
-
 	gchar *open;
 	gchar *close;
 
@@ -220,12 +211,12 @@ void form_wizard_clicked( GtkWidget *widget, gint button, gpointer data )
 						 "form_wizard" );
 
 	if( button == GTK_RESPONSE_OK || button == GTK_RESPONSE_APPLY ) {
-		combo = GTK_COMBO( wizard->method );
-		method = gtk_entry_get_text( GTK_ENTRY( combo->entry ) );
-		combo = GTK_COMBO( wizard->action );
-		action = gtk_entry_get_text( GTK_ENTRY( combo->entry ) );
-		combo = GTK_COMBO( wizard->enctype );
-		enctype = gtk_entry_get_text( GTK_ENTRY( combo->entry ) );
+		entry = GTK_BIN( wizard->method )->child;
+		method = gtk_entry_get_text( GTK_ENTRY( entry ) );
+		entry = GTK_BIN( wizard->action )->child;
+		action = gtk_entry_get_text( GTK_ENTRY( entry ) );
+		entry = GTK_BIN( wizard->enctype )->child;
+		enctype = gtk_entry_get_text( GTK_ENTRY( entry ) );
 		if( ! strcmp( "", enctype ) )
 			enctype = NULL;
 
@@ -281,17 +272,14 @@ static GtkWidget* create_dialog( FormWizard *form_wizard )
 			"GET",
 			NULL
 		};
-		GList *method_list = NULL;
 		gint i;
 
-		form_wizard->method = gtk_combo_new();
+		form_wizard->method = gtk_combo_box_entry_new_text();
 		
 		for( i = 0; methods[ i ]; ++ i ) {
-			method_list = g_list_append( method_list,
-						     (gchar*)methods[ i ] );
+			gtk_combo_box_append_text( GTK_COMBO_BOX( form_wizard->method ),
+					methods[ i ] );
 		}
-		gtk_combo_set_popdown_strings(GTK_COMBO(form_wizard->method),
-					      method_list);
 		gtk_table_attach( GTK_TABLE( table ), form_wizard->method,
 				  1, 2, 0, 1, 
 				  GTK_EXPAND | GTK_FILL,
@@ -303,19 +291,14 @@ static GtkWidget* create_dialog( FormWizard *form_wizard )
 		const gchar *actions[] = {
 			NULL
 		};
-		GList *action_list = NULL;
 		gint i;
 
-		form_wizard->action = gtk_combo_new();
+		form_wizard->action = gtk_combo_box_entry_new_text();
 		
 		for( i = 0; actions[ i ]; ++ i ) {
-			action_list = g_list_append( action_list,
-						     (gchar*)actions[ i ] );
+			gtk_combo_box_append_text( GTK_COMBO_BOX( form_wizard->action ),
+					actions[ i ] );
 		}
-		/*
-		  we don't have any actions to set! 
-		  gtk_combo_set_popdown_strings(GTK_COMBO(form_wizard->action),
-		  action_list);*/
 		gtk_table_attach( GTK_TABLE( table ), form_wizard->action,
 				  1, 2, 1, 2, 				 
 				  GTK_EXPAND | GTK_FILL,
@@ -337,17 +320,14 @@ static GtkWidget* create_dialog( FormWizard *form_wizard )
 			"text/xml",
 			NULL
 		};
-		GList *enctype_list = NULL;
 		gint i;
 
-		form_wizard->enctype = gtk_combo_new();
+		form_wizard->enctype = gtk_combo_box_entry_new_text();
 		
 		for( i = 0; types[ i ]; ++ i ) {
-			enctype_list = g_list_append( enctype_list,
-						      (gchar*)types[ i ] );
+			gtk_combo_box_append_text( GTK_COMBO_BOX( form_wizard->enctype ),
+					types[ i ] );
 		}
-		gtk_combo_set_popdown_strings(GTK_COMBO(form_wizard->enctype),
-					      enctype_list);
 		gtk_table_attach( GTK_TABLE( table ), form_wizard->enctype,
 				  1, 2, 2, 3, 
 				  GTK_EXPAND | GTK_FILL,
