@@ -32,13 +32,7 @@
 
 #include <glib/gfileutils.h>
 
-#include <gtk/gtktoolbar.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtkspinbutton.h>
-#include <gtk/gtktogglebutton.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkdialog.h>
-#include <gtk/gtkoptionmenu.h>
+#include <gtk/gtk.h>
 
 #include <glade/glade.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
@@ -52,8 +46,6 @@
 
 #include "fileops.h"
 
-#include "libegg/menu/egg-menu.h"
-
 static const gchar *align[] = {
 	"bottom",
 	"middle",
@@ -83,7 +75,7 @@ static GList *wizards = NULL;
 
 /* we don't want to try and load old plugins
    so this is a version symbol for screem to spot */
-int screem_plugin_version = 3;
+int screem_plugin_version = 4;
 
 #define NAME "Image Wizard"
 #define AUTHOR "David A Knight (david@screem.org)"
@@ -99,7 +91,7 @@ G_MODULE_EXPORT void remove_ui( GtkWidget *window, GtkWidget *editor,
 				GtkWidget *preview, GtkWidget *link_view );
 G_MODULE_EXPORT void popup( ScreemWindow *window,
 			    guint start, guint len, GSList *attrs );
-void image_wizard_display( EggAction *action, gpointer user_data );
+void image_wizard_display( GtkAction *action, gpointer user_data );
 void image_wizard_clicked( GtkWidget *widget, gint button, gpointer data );
 gboolean image_wizard_preview( GtkWidget *widget );
 
@@ -137,27 +129,23 @@ G_MODULE_EXPORT void add_ui( GtkWidget *window, GtkWidget *editor,
 {
 	ImageWizard *image_wizard;
 	const gchar *ui = "\
-<Root>\
-<menu>\
-<submenu name=\"Insert\" verb=\"Insert\">\
-<submenu name=\"Wizards\" verb=\"Wizards\">\
-<menuitem name=\"ImageWizard\" verb=\"ImageWizard\"/>\
-</submenu>\
-</submenu>\
+<ui>\
+<menubar>\
+<menu action=\"Insert\">\
+<menu action=\"Wizards\">\
+<menuitem action=\"ImageWizard\"/>\
 </menu>\
-<dockitem name=\"Wizards Toolbar\">\
-<toolitem name=\"ImageWizard\" verb=\"ImageWizard\" />\
-</dockitem>\
-</Root>";
-	static EggActionGroupEntry entries[] = {
-		{ "ImageWizard", "label",
-	  	"Screem_Image", NULL, "tip",
-	  	G_CALLBACK( image_wizard_display ), NULL },
-	};
-	EggActionGroupEntry *entry;
+</menu>\
+</menubar>\
+<toolbar name=\"Wizards Toolbar\">\
+<toolitem action=\"ImageWizard\" />\
+</toolbar>\
+</ui>";
+	GtkAction *action;
 	
 	gchar *label;
 	gchar *tip;
+	GError *error;
 
 	image_wizard = g_new0( ImageWizard, 1 );
 	image_wizard->window = SCREEM_WINDOW( window );
@@ -165,20 +153,22 @@ G_MODULE_EXPORT void add_ui( GtkWidget *window, GtkWidget *editor,
 
 	label = g_strdup( _( "Image Wizard" ) );
 	tip = g_strdup( _( "Insert an Image" ) );
-
-	entry = entries;
-	entry->label = label;
-	entry->tooltip = tip;
-	entry->user_data = image_wizard;
-	
-	egg_action_group_add_actions( EGG_ACTION_GROUP( image_wizard->window->action_group ),
-					entry, 1 );
-	egg_menu_merge_add_ui_from_string( EGG_MENU_MERGE( image_wizard->window->merge ),
-						ui, strlen( ui ), NULL );
-		
+	action = gtk_action_new( "ImageWizard", label, tip,
+			"Screem_Image" );
+	g_signal_connect( G_OBJECT( action ), "activate",
+			G_CALLBACK( image_wizard_display ), image_wizard );
+	gtk_action_group_add_action( GTK_ACTION_GROUP( image_wizard->window->action_group ), action );
 	g_free( label );
 	g_free( tip );
 
+	error = NULL;
+	if( ! gtk_ui_manager_add_ui_from_string( GTK_UI_MANAGER( image_wizard->window->merge ),
+						ui, strlen( ui ), &error ) ) {
+		g_message( "%s ui error = %s", "image wizard",
+				error->message );
+		g_error_free( error );
+	}
+
 	/* NOTE: make sure that the window being obtained isn't visible */
 	image_wizard->xml = glade_xml_new( GLADE_PATH"/imageWizard.glade",
 					   "imagewizard_dialog", NULL );
@@ -395,18 +385,27 @@ G_MODULE_EXPORT void popup( ScreemWindow *window,
 /* End of required section */
 
 
-void image_wizard_display( EggAction *action, gpointer user_data )
+void image_wizard_display( GtkAction *action, gpointer user_data )
 {
 	ImageWizard *wizard;
 	ScreemPage *page;
-
+	ScreemApplication *app;
+	ScreemSession *session;
+		
 	wizard = (ImageWizard*)user_data;
+	app = SCREEM_APPLICATION( wizard->window->application );
+	session = screem_application_get_session( app );
 
 	page = screem_window_get_document( wizard->window );
 
 	if( ! page )
 		return;
-        
+       
+	if( ! GTK_WIDGET_VISIBLE( wizard->dialog ) ) {
+		screem_session_restore_dialog( session,
+				wizard->dialog );
+	}
+	
 	gtk_widget_show_all( wizard->dialog );
 	gdk_window_raise( wizard->dialog->window );
 }
@@ -484,6 +483,8 @@ void image_wizard_clicked( GtkWidget *widget, gint button, gpointer data )
 	GladeXML *xml;
 	GList *list;
 	ScreemPage *page;
+	ScreemApplication *app;
+	ScreemSession *session;
 
 	xml = glade_get_widget_tree( widget );
 	for( wizard = NULL, list = wizards; list; list = list->next ) {
@@ -495,6 +496,8 @@ void image_wizard_clicked( GtkWidget *widget, gint button, gpointer data )
 	g_assert( wizard );
 
 	page = screem_window_get_document( wizard->window );
+	app = wizard->window->application;
+	session = screem_application_get_session( app );
 	
 	if( ! page ) {
 		return;
@@ -649,7 +652,8 @@ void image_wizard_clicked( GtkWidget *widget, gint button, gpointer data )
 		if( copyNeeded ) {
 			uri = g_strdup_printf( "%s/%s", dir, 
 					       g_basename( filename ) );
-			copy_file( filename, uri );
+			copy_file( filename, uri, 
+				screem_application_file_op, app );
 		} else {
 			uri = g_strdup( filename );
 		}
@@ -684,7 +688,9 @@ void image_wizard_clicked( GtkWidget *widget, gint button, gpointer data )
 								 GDK_INTERP_BILINEAR );
 			}
 			if( scale ) {
-				gdk_pixbuf_save( scale, rel, ext, NULL, NULL );
+				if( gdk_pixbuf_save( scale, rel, ext, NULL, NULL ) ) {
+					screem_application_file_op( GNOME_VFS_MONITOR_EVENT_CREATED, rel, app );
+				}
 				g_object_unref( scale );
 				g_object_unref( pixbuf );
 			}
@@ -712,6 +718,8 @@ void image_wizard_clicked( GtkWidget *widget, gint button, gpointer data )
 		g_string_free( tag, TRUE );
 	}
 
+	screem_session_store_dialog( session, widget );
+	
 	if( button != GTK_RESPONSE_APPLY ) {
 		gtk_widget_hide( GTK_WIDGET( widget ) );
 	}
