@@ -30,21 +30,12 @@
 
 #include <glade/glade.h>
 
-#include <gtk/gtkentry.h>
-#include <gtk/gtkdialog.h>
-#include <gtk/gtkoptionmenu.h>
-#include <gtk/gtkmenu.h>
-#include <gtk/gtkmenuitem.h>
-#include <gtk/gtktogglebutton.h>
-#include <gtk/gtkspinbutton.h>
-
+#include <gtk/gtk.h>
 
 #include "screem-plugin.h"
 #include "screem-window.h"
 #include "screem-editor.h"
 
-#include "libegg/menu/egg-menu.h"
-
 static const gchar *protocols[] = {
 	"http://", "ftp://",
 	"mailto:", "irc://",
@@ -95,7 +86,7 @@ static GList *wizards = NULL;
 
 /* we don't want to try and load old plugins
    so this is a version symbol for screem to spot */
-int screem_plugin_version = 3;
+int screem_plugin_version = 4;
 
 #define NAME "Link Wizard"
 #define AUTHOR "David A Knight (david@screem.org)"
@@ -112,11 +103,13 @@ G_MODULE_EXPORT void remove_ui( GtkWidget *window, GtkWidget *editor,
 G_MODULE_EXPORT void popup( ScreemWindow *window,
 			    guint start, guint len, GSList *attrs );
 
-void link_wizard_display( EggAction *action, gpointer user_data );
+void link_wizard_display( GtkAction *action, gpointer user_data );
 void link_wizard_clicked( GtkWidget *widget, gint button, gpointer data );
 void link_wizard_edit_popup( GtkWidget *widget );
 
 static void create_dialog( LinkWizard *link_wizard );
+static void popup_response( GtkWidget *dialog, gint button, 
+		gpointer data );
 
 
 G_MODULE_EXPORT const gchar* g_module_check_init( GModule *module )
@@ -151,27 +144,23 @@ G_MODULE_EXPORT void add_ui( GtkWidget *window, GtkWidget *editor,
 {
 	LinkWizard *link_wizard;
 	const gchar *ui = "\
-<Root>\
-<menu>\
-<submenu name=\"Insert\" verb=\"Insert\">\
-<submenu name=\"Wizards\" verb=\"Wizards\">\
-<menuitem name=\"LinkWizard\" verb=\"LinkWizard\"/>\
-</submenu>\
-</submenu>\
+<ui>\
+<menubar>\
+<menu action=\"Insert\">\
+<menu action=\"Wizards\">\
+<menuitem action=\"LinkWizard\"/>\
 </menu>\
-<dockitem name=\"Wizards Toolbar\">\
-<toolitem name=\"LinkWizard\" verb=\"LinkWizard\" />\
-</dockitem>\
-</Root>";
-	static EggActionGroupEntry entries[] = {
-		{ "LinkWizard", "label",
-	  	"Screem_Link", NULL, "tip",
-	  	G_CALLBACK( link_wizard_display ), NULL },
-	};
-	EggActionGroupEntry *entry;
-
+</menu>\
+</menubar>\
+<toolbar name=\"Wizards Toolbar\">\
+<toolitem action=\"LinkWizard\" />\
+</toolbar>\
+</ui>";
+	GtkAction *action;
+	
 	gchar *label;
 	gchar *tip;
+	GError *error;
 
 	link_wizard = g_new0( LinkWizard, 1 );
 	link_wizard->window = SCREEM_WINDOW( window );
@@ -179,20 +168,22 @@ G_MODULE_EXPORT void add_ui( GtkWidget *window, GtkWidget *editor,
 
 	label = g_strdup( _( "Link Wizard" ) );
 	tip = g_strdup( _( "Insert a Link" ) );
-
-	entry = entries;
-	entry->label = label;
-	entry->tooltip = tip;
-	entry->user_data = link_wizard;
-	
-	egg_action_group_add_actions( EGG_ACTION_GROUP( link_wizard->window->action_group ),
-					entry, 1 );
-	egg_menu_merge_add_ui_from_string( EGG_MENU_MERGE( link_wizard->window->merge ),
-						ui, strlen( ui ), NULL );
-	
+	action = gtk_action_new( "LinkWizard", label, tip,
+			"Screem_Link" );
+	g_signal_connect( G_OBJECT( action ), "activate",
+			G_CALLBACK( link_wizard_display ), link_wizard );
+	gtk_action_group_add_action( GTK_ACTION_GROUP( link_wizard->window->action_group ), action );
 	g_free( label );
 	g_free( tip );
 
+	error = NULL;
+	if( ! gtk_ui_manager_add_ui_from_string( GTK_UI_MANAGER( link_wizard->window->merge ),
+						ui, strlen( ui ), &error ) ) {
+		g_message( "%s ui error = %s", "link wizard",
+				error->message );
+		g_error_free( error );
+	}
+	
 	create_dialog( link_wizard );
 
 	wizards = g_list_append( wizards, link_wizard );
@@ -286,6 +277,7 @@ G_MODULE_EXPORT void popup( ScreemWindow *window,
 		} else if( ! g_strcasecmp( "target", name ) ) {
 			widget = glade_xml_get_widget( link_wizard->xml,
 						       "target" );
+			widget = GTK_BIN( widget )->child;
 			gtk_entry_set_text( GTK_ENTRY( widget ), value );
 		} else if( ! g_strcasecmp( "onclick", name ) ) {
 			onclick = FALSE;
@@ -312,19 +304,27 @@ G_MODULE_EXPORT void popup( ScreemWindow *window,
 /* End of required section */
 
 /* now the stuff that actually does the work */
-void link_wizard_display( EggAction *action, gpointer user_data )
+void link_wizard_display( GtkAction *action, gpointer user_data )
 {
 	LinkWizard *link_wizard;
 	ScreemPage *page;
 	GtkWidget *widget;
+	ScreemApplication *app;
+	ScreemSession *session;
 	
 	link_wizard = (LinkWizard*)user_data;
+	app = SCREEM_APPLICATION( link_wizard->window->application );
+	session = screem_application_get_session( app );
 
 	page = screem_window_get_document( link_wizard->window );
 
 	if( page ) {
 		widget = glade_xml_get_widget( link_wizard->pxml, "onclick" );
 		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( widget ), FALSE );
+		if( ! GTK_WIDGET_VISIBLE( link_wizard->dialog ) ) {
+			screem_session_restore_dialog( session,
+					link_wizard->dialog );
+		}
 		gtk_widget_show_all( link_wizard->dialog );
 		gdk_window_raise( link_wizard->dialog->window );
 
@@ -346,10 +346,16 @@ void link_wizard_clicked( GtkWidget *dialog, gint button, gpointer data )
 		NULL
 	};
 	gint i;
+	ScreemApplication *app;
+	ScreemSession *session;
+
 
 	link_wizard = (LinkWizard*)g_object_get_data( G_OBJECT( dialog ),
 						      "link_wizard" );
 
+	app = SCREEM_APPLICATION( link_wizard->window->application );
+	session = screem_application_get_session( app );
+	
 	if( button == GTK_RESPONSE_APPLY || button == GTK_RESPONSE_OK ) { 
 		GtkWidget *widget;
 		const gchar *text;
@@ -376,6 +382,7 @@ void link_wizard_clicked( GtkWidget *dialog, gint button, gpointer data )
 
 		widget = glade_xml_get_widget( link_wizard->xml,
 					       "target" );
+		widget = GTK_BIN( widget )->child;
 		target = gtk_entry_get_text( GTK_ENTRY( widget ) );
 		if( ! GTK_WIDGET_SENSITIVE( widget ) ) {
 			target = "";
@@ -532,6 +539,9 @@ void link_wizard_clicked( GtkWidget *dialog, gint button, gpointer data )
 		
 		g_string_free( tag, TRUE );
 	}
+
+	screem_session_store_dialog( session, dialog );
+	
 	if( button != GTK_RESPONSE_APPLY ) {
 		gtk_widget_hide( dialog );
         }
@@ -541,13 +551,22 @@ void link_wizard_edit_popup( GtkWidget *widget )
 {
 	LinkWizard *wiz;
 	GladeXML *xml;
-
+	ScreemApplication *app;
+	ScreemSession *session;
+		
 	xml = glade_get_widget_tree( widget );
 	widget = glade_xml_get_widget( xml, "linkwizard" );
 
 	wiz = (LinkWizard*)g_object_get_data( G_OBJECT( widget ),
 					      "link_wizard" );
 
+	app = SCREEM_APPLICATION( wiz->window->application );
+	session = screem_application_get_session( app );
+
+	if( ! GTK_WIDGET_VISIBLE( wiz->pdialog ) ) {
+		screem_session_restore_dialog( session,
+				wiz->pdialog );
+	}
 	gtk_widget_show_all( wiz->pdialog );
 	gdk_window_raise( wiz->pdialog->window );
 	
@@ -642,8 +661,27 @@ static void create_dialog( LinkWizard *link_wizard )
 	
 	link_wizard->pdialog = glade_xml_get_widget( link_wizard->pxml, 
 						    "popup" );
+	g_signal_connect( G_OBJECT( link_wizard->pdialog ),
+			"response",
+			G_CALLBACK( popup_response ),
+			link_wizard );
 	g_object_set_data( G_OBJECT( link_wizard->pdialog ),
 			   "link_wizard", link_wizard );
 	glade_xml_signal_autoconnect( link_wizard->pxml );
 
 }
+
+static void popup_response( GtkWidget *dialog, gint button, gpointer data )
+{
+	LinkWizard *wiz;
+	ScreemApplication *app;
+	ScreemSession *session;
+		
+	wiz = (LinkWizard*)data;
+
+	app = SCREEM_APPLICATION( wiz->window->application );
+	session = screem_application_get_session( app );
+
+	screem_session_store_dialog( session, dialog );
+}
+
