@@ -8,10 +8,7 @@
 #include <libgnomeui/gnome-file-entry.h>
 #include <libgnome/gnome-i18n.h>
 
-#include <gtk/gtktoolbar.h>
-#include <gtk/gtktogglebutton.h>
-#include <gtk/gtkspinbutton.h>
-#include <gtk/gtkdialog.h>
+#include <gtk/gtk.h>
 
 #include <glade/glade.h>
 
@@ -19,8 +16,6 @@
 #include "screem-window.h"
 #include "screem-editor.h"
 
-#include "libegg/menu/egg-menu.h"
-
 /* per wizard struct */
 typedef struct {
 	GladeXML *xml;
@@ -37,7 +32,7 @@ static GList *wizards = NULL;
 
 /* we don't want to try and load old plugins
    so this is a version symbol for screem to spot */
-int screem_plugin_version = 3;
+int screem_plugin_version = 4;
 
 #define NAME "Table Wizard"
 #define AUTHOR "David A Knight (david@screem.org)"
@@ -53,7 +48,7 @@ G_MODULE_EXPORT void remove_ui( GtkWidget *window, GtkWidget *editor,
 				GtkWidget *preview, GtkWidget *link_view );
 
 void table_wizard_clicked( GtkWidget *w, gint button );
-void table_wizard_display( EggAction *action, gpointer user_data );
+void table_wizard_display( GtkAction *action, gpointer user_data );
 G_MODULE_EXPORT const gchar* g_module_check_init( GModule *module )
 {
 	return NULL;
@@ -86,27 +81,23 @@ G_MODULE_EXPORT void add_ui( GtkWidget *window, GtkWidget *editor,
 {
 	TableWizard *table_wizard;
 	const gchar *ui = "\
-<Root>\
-<menu>\
-<submenu name=\"Insert\" verb=\"Insert\">\
-<submenu name=\"Wizards\" verb=\"Wizards\">\
-<menuitem name=\"TableWizard\" verb=\"TableWizard\"/>\
-</submenu>\
-</submenu>\
+<ui>\
+<menubar>\
+<menu action=\"Insert\">\
+<menu action=\"Wizards\">\
+<menuitem action=\"TableWizard\"/>\
 </menu>\
-<dockitem name=\"Wizards Toolbar\">\
-<toolitem name=\"TableWizard\" verb=\"TableWizard\" />\
-</dockitem>\
-</Root>";
-	static EggActionGroupEntry entries[] = {
-		{ "TableWizard", "label",
-	  	"Screem_Table", NULL, "tip",
-	  	G_CALLBACK( table_wizard_display ), NULL },
-	};
-	EggActionGroupEntry *entry;
+</menu>\
+</menubar>\
+<toolbar name=\"Wizards Toolbar\">\
+<toolitem action=\"TableWizard\" />\
+</toolbar>\
+</ui>";
+	GtkAction *action;
 	
 	gchar *label;
 	gchar *tip;
+	GError *error;
 
 	table_wizard = g_new0( TableWizard, 1 );
 	table_wizard->xml = glade_xml_new( GLADE_PATH"/tablewizard.glade", 
@@ -116,17 +107,21 @@ G_MODULE_EXPORT void add_ui( GtkWidget *window, GtkWidget *editor,
 
 	label = g_strdup( _( "Table Wizard" ) );
 	tip = g_strdup( _( "Insert a Table" ) );
-	entry = entries;
-	entry->label = label;
-	entry->tooltip = tip;
-	entry->user_data = table_wizard;
-	
-	egg_action_group_add_actions( EGG_ACTION_GROUP( table_wizard->window->action_group ),
-					entry, 1 );
-	egg_menu_merge_add_ui_from_string( EGG_MENU_MERGE( table_wizard->window->merge ),
-						ui, strlen( ui ), NULL );
+	action = gtk_action_new( "TableWizard", label, tip,
+			"Screem_Table" );
+	g_signal_connect( G_OBJECT( action ), "activate",
+			G_CALLBACK( table_wizard_display ), table_wizard );
+	gtk_action_group_add_action( GTK_ACTION_GROUP( table_wizard->window->action_group ), action );
 	g_free( label );
 	g_free( tip );
+
+	error = NULL;
+	if( ! gtk_ui_manager_add_ui_from_string( GTK_UI_MANAGER( table_wizard->window->merge ),
+						ui, strlen( ui ), &error ) ) {
+		g_message( "%s ui error = %s", "table wizard",
+				error->message );
+		g_error_free( error );
+	}
 	
 	table_wizard->dialog = glade_xml_get_widget( table_wizard->xml, 
 						    "tablewizard" );
@@ -183,6 +178,9 @@ void table_wizard_clicked( GtkWidget *w, gint button )
 	GladeXML *xml;
 	TableWizard *wizard;
 	GList *list;
+	ScreemApplication *app;
+	ScreemSession *session;
+	
 	xml = glade_get_widget_tree( w );
 
 	wizard = NULL;
@@ -194,6 +192,9 @@ void table_wizard_clicked( GtkWidget *w, gint button )
 	}
 	g_assert( wizard );
 
+	app = SCREEM_APPLICATION( wizard->window->application );
+	session = screem_application_get_session( app );
+
     	if( button == GTK_RESPONSE_APPLY || button == GTK_RESPONSE_OK ) {
 		widget = glade_xml_get_widget( 	wizard->xml, "caption" );
 		caption = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) );
@@ -296,22 +297,33 @@ void table_wizard_clicked( GtkWidget *w, gint button )
 		g_string_free( text, TRUE );
 	}
 
-	if( button == GTK_RESPONSE_OK || button == GTK_RESPONSE_CLOSE )
+	screem_session_store_dialog( session, wizard->dialog );
+	
+	if( button == GTK_RESPONSE_OK || button == GTK_RESPONSE_CLOSE ){
 		gtk_widget_hide( wizard->dialog );
+	}
 }
 
-void table_wizard_display( EggAction *action, gpointer user_data )
+void table_wizard_display( GtkAction *action, gpointer user_data )
 {
 	TableWizard *wizard;
 	ScreemPage *page;
-
+	ScreemApplication *app;
+	ScreemSession *session;
+		
 	wizard = (TableWizard*)user_data;
+	app = SCREEM_APPLICATION( wizard->window->application );
+	session = screem_application_get_session( app );
 
         page = screem_window_get_document( wizard->window );
         
 	if( ! page )
 		return;
 
+	if( ! GTK_WIDGET_VISIBLE( wizard->dialog ) ) {
+		screem_session_restore_dialog( session, wizard->dialog );
+	}
+
 	gtk_widget_show_all( wizard->dialog );
 	gdk_window_raise( wizard->dialog->window );
 }
