@@ -1,6 +1,6 @@
 /* 
    sitecopy, manage remote web sites.
-   Copyright (C) 1998-2000, Joe Orton <joe@orton.demon.co.uk>.
+   Copyright (C) 1998-2002, Joe Orton <joe@manyfish.co.uk>.
                                                                      
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,6 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-   $Id: common.c,v 1.4 2000/10/15 16:49:13 davek Exp $
 */
 
 #include <config.h>
@@ -33,29 +32,35 @@
 #include <stdlib.h>
 #endif /* HAVE_STDLIB_H */
 
+#ifdef HAVE_LANGINFO_H
+#include <langinfo.h>
+#endif
+
 #include <ctype.h>
 
-#include <string_utils.h>
+#include <ne_string.h>
+#include <ne_utils.h>
 
 #include "i18n.h"
 #include "common.h"
 
-#ifdef DEBUGGING
+#ifdef NE_DEBUGGING
 
 static const struct {
     const char *name;
     int val;
 } debug_map[] = {
-    { "socket", DEBUG_SOCKET },
+    { "socket", NE_DBG_SOCKET },
     { "files", DEBUG_FILES },
     { "rcfile", DEBUG_RCFILE },
     { "ftp", DEBUG_FTP },
-    { "xml", DEBUG_XML },
-    { "xmlparse", DEBUG_XMLPARSE },
-    { "http", DEBUG_HTTP },
-    { "httpauth", DEBUG_HTTPAUTH },
-    { "httpbody", DEBUG_HTTPBODY },
-    { "cleartext", DEBUG_HTTPPLAIN },
+    { "xml", NE_DBG_XML },
+    { "xmlparse", NE_DBG_XMLPARSE },
+    { "http", NE_DBG_HTTP },
+    { "httpauth", NE_DBG_HTTPAUTH },
+    { "httpbody", NE_DBG_HTTPBODY },
+    { "cleartext", NE_DBG_HTTPPLAIN },
+    { "ssl", NE_DBG_SSL },
     { NULL, 0 }
 };
 
@@ -100,4 +105,3 @@ char *strerror (errnum)
   return _("Unknown system error");
 }
 # endif /* HAVE_STRERROR */
-
