@@ -1,6 +1,6 @@
 /* 
    sitecopy, for managing remote web sites.
-   Copyright (C) 1998-2000, Joe Orton <joe@orton.demon.co.uk>
+   Copyright (C) 1998-2002, Joe Orton <joe@manyfish.co.uk>
                                                                      
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
 
 #include <sys/types.h>
 
-#include <nsocket.h> /* for sock_block_reader */
+#include <ne_request.h>
 
 #include "sites.h"
 #include "common.h"
@@ -80,13 +80,13 @@ struct proto_driver {
      */
     int (*file_upload_cond)(void *session,
 	const char *local, const char *remote,
-	int ascii, time_t time);
+	int ascii, time_t t);
     /* Retrieve the remote file modification time and file size */
     int (*file_get_modtime)(void *sess, const char *remote, time_t *modtime);
     int (*file_download)(void *sess, const char *local, const char *remote,
 			 int ascii);
     int (*file_read)(void *sess, const char *remote, 
-		     sock_block_reader reader, void *userdata);
+		     ne_block_reader reader, void *userdata);
 
     int (*file_delete)(void *sess, const char *filename);
     int (*file_chmod)(void *sess, const char *filename, mode_t mode);
@@ -95,11 +95,11 @@ struct proto_driver {
     int (*dir_remove)(void *sess, const char *dirname);
 
     /* Creates a link with given target */
-    int (*link_create)(void *session, const char *link, const char *target);
+    int (*link_create)(void *session, const char *fn, const char *target);
     /* Changes a link to point to a different target */
-    int (*link_change)(void *session, const char *link, const char *target);
+    int (*link_change)(void *session, const char *fn, const char *target);
     /* Deletes a link */
-    int (*link_delete)(void *session, const char *link);
+    int (*link_delete)(void *session, const char *fn);
     
     /* Fetches the list of files.
      * Returns:
@@ -122,4 +122,7 @@ struct proto_driver {
     const char *protocol_name; /* The user-visible name for this protocol */
 };
 
+/* Callback of type 'sock_progress'. */
+void site_sock_progress_cb(void *userdata, off_t progress, off_t total);
+
 #endif /* PROTO_H */
