@@ -292,7 +292,7 @@ enum site_protocol_modes {
     siteproto_ftp,
     siteproto_dav,
     siteproto_rsh,
-    siteproto_local,
+    siteproto_vfs,
     siteproto_unknown
 };
 
