@@ -33,8 +33,7 @@
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 
-#include <gtk/gtkdialog.h>
-#include <gtk/gtkmessagedialog.h>
+#include <gtk/gtk.h>
 
 #include <errno.h>
 #include <gmodule.h>
@@ -66,8 +65,6 @@
 
 #include "screem-site.h"
 
-#include "libegg/menu/egg-menu.h"
-
 /* per wizard struct */
 typedef struct {
 	ScreemWindow *window;
@@ -93,7 +90,7 @@ typedef struct {
 	float upload_total;
 	float uploaded_bytes;
 
-	EggAction *action;
+	GtkAction *action;
 } UploadWizard;
 
 /* keep track of how many link wizards we have */
@@ -110,7 +107,7 @@ static GMutex *wizard_lock;
 
 /* we don't want to try and load old plugins
    so this is a version symbol for screem to spot */
-int screem_plugin_version = 3;
+int screem_plugin_version = 4;
 
 #define NAME "Upload Wizard (sitecopy)"
 #define AUTHOR "Lee Mallabone, David A Knight (david@screem.org)"
@@ -133,9 +130,8 @@ static void destroy_fnlist( struct fnlist *list );
 
 /* the drivers */
 extern const struct proto_driver ftp_driver;
-extern const struct proto_driver dav_driver;
 extern const struct proto_driver rsh_driver;
-extern const struct proto_driver local_driver;
+extern const struct proto_driver vfs_driver;
 
 
 G_MODULE_EXPORT const gchar* g_module_check_init( GModule *module );
@@ -152,7 +148,7 @@ static struct site* screem_site_to_sitecopy_site( ScreemSite *site,
 
 static void upload_wizard_error( const gchar *message );
 static void upload_wizard_message( const gchar *message );
-static void upload_site( EggAction *action, gpointer user_data );
+static void upload_site( GtkAction *action, gpointer user_data );
 static gboolean should_init( void );
 static gboolean verify_site( struct site *site );
 
@@ -197,22 +193,18 @@ G_MODULE_EXPORT void add_ui( GtkWidget *window, GtkWidget *editor,
 {
 	UploadWizard *upload_wizard;
 	const gchar *ui = "\
-<Root>\
-<menu>\
-<submenu name=\"Tools\" verb=\"Tools\">\
-<menuitem name=\"UploadWizard\" verb=\"UploadWizard\"/>\
-</submenu>\
+<ui>\
+<menubar>\
+<menu action=\"Tools\">\
+<menuitem action=\"UploadWizard\"/>\
 </menu>\
-</Root>";
-	static EggActionGroupEntry entries[] = {
-		{ "UploadWizard", "label",
-	  	GTK_STOCK_JUMP_TO, NULL, "tip",
-	  	G_CALLBACK( upload_site ), NULL },
-	};
-	EggActionGroupEntry *entry;
+</menubar>\
+</ui>";
+	GtkAction *action;
 
 	gchar *label;
 	gchar *tip;
+	GError *error;
 
 	GtkSizeGroup *group;
 	GtkWidget *widget;
@@ -222,19 +214,22 @@ G_MODULE_EXPORT void add_ui( GtkWidget *window, GtkWidget *editor,
 	
 	label = g_strdup( _( "Upload" ) );
 	tip = g_strdup( _( "Upload the Current Site or Page" ) );
-	entry = entries;
-	entry->label = label;
-	entry->tooltip = tip;
-	entry->user_data = upload_wizard;
-	
-	egg_action_group_add_actions( EGG_ACTION_GROUP( upload_wizard->window->action_group ),
-					entry, 1 );
-	egg_menu_merge_add_ui_from_string( EGG_MENU_MERGE( upload_wizard->window->merge ),
-						ui, strlen( ui ), NULL );
+	action = gtk_action_new( "UploadWizard", label, tip,
+			GTK_STOCK_JUMP_TO );
+	g_signal_connect( G_OBJECT( action ), "activate",
+			G_CALLBACK( upload_site ), upload_wizard );
+	gtk_action_group_add_action( GTK_ACTION_GROUP( upload_wizard->window->action_group ), action );
 	g_free( label );
 	g_free( tip );
 
-	upload_wizard->action = egg_action_group_get_action( EGG_ACTION_GROUP( upload_wizard->window->action_group ), "UploadWizard" );
+	error = NULL;
+	if( ! gtk_ui_manager_add_ui_from_string( GTK_UI_MANAGER( upload_wizard->window->merge ), ui, strlen( ui ), &error ) ) {
+		g_message( "%s ui error = %s", "upload wizard",
+				error->message );
+		g_error_free( error );
+	}
+
+	upload_wizard->action = gtk_action_group_get_action( GTK_ACTION_GROUP( upload_wizard->window->action_group ), "UploadWizard" );
 	
 	upload_wizard->xml = glade_xml_new( GLADE_PATH, "update_dialog",
 					    NULL );
@@ -296,6 +291,7 @@ static struct site* screem_site_to_sitecopy_site( ScreemSite *ssite,
 	struct site_host *host;
 	const gchar *string;
 	GList *list;
+	GList *tmp;
 	struct fnlist *exclude;
 	struct fnlist *ignores;
 	struct fnlist *ascii;
@@ -303,7 +299,11 @@ static struct site* screem_site_to_sitecopy_site( ScreemSite *ssite,
 	const gchar* home;
 
 	gchar *temp;
+	GnomeVFSURI *uri;
+	GnomeVFSURI *full;
 
+	UploadMethods method;
+	
 	sitecopy_site = g_new0( struct site, 1 );
 
 	host = &sitecopy_site->server;
@@ -324,12 +324,12 @@ static struct site* screem_site_to_sitecopy_site( ScreemSite *ssite,
 		host->password = NULL;
 
 	if( host->hostname ) {
-		if( ( temp = strchr( host->hostname, ':' ) ) ) {
+		if( ( temp = strrchr( host->hostname, ':' ) ) ) {
 			host->port = atoi( temp + 1 );
 			temp = g_strndup( host->hostname, 
 					 temp - host->hostname );
+			g_free( host->hostname );
 			host->hostname = temp;
-			/* FIXME: memory leak */
 		}
 	}
 
@@ -338,17 +338,13 @@ static struct site* screem_site_to_sitecopy_site( ScreemSite *ssite,
 	sitecopy_site->url = g_strdup( string );
 
 	/* get transfer options */
-	switch( screem_site_get_remote_method( ssite ) ) {
+	method = screem_site_get_remote_method( ssite );
+	switch( method ) {
 	case FTP:
 		sitecopy_site->protocol = siteproto_ftp;
 		sitecopy_site->proto_string = "ftp";
 		sitecopy_site->driver = &ftp_driver;
 		break;
-	case WEBDAV:
-		sitecopy_site->protocol = siteproto_dav;
-		sitecopy_site->proto_string = "dav";
-		sitecopy_site->driver = &dav_driver;
-		break;
 	case RSH:
 		sitecopy_site->protocol = siteproto_rsh;
 		sitecopy_site->proto_string = "rsh";
@@ -356,18 +352,20 @@ static struct site* screem_site_to_sitecopy_site( ScreemSite *ssite,
 		sitecopy_site->rsh_cmd = "rsh";
 		sitecopy_site->rcp_cmd = "rcp";
 		break;
-	case SSH:
-		sitecopy_site->protocol = siteproto_rsh;
-                sitecopy_site->proto_string = "ssh";
-                sitecopy_site->driver = &rsh_driver;
-		sitecopy_site->rsh_cmd = "ssh";
-		sitecopy_site->rcp_cmd = "scp";
-                break;
-	case LOCAL:
 	default:
-		sitecopy_site->protocol = siteproto_local;
-		sitecopy_site->proto_string = "local";
-		sitecopy_site->driver = &local_driver;
+		sitecopy_site->protocol = siteproto_vfs;
+		sitecopy_site->proto_string = "vfs";
+		sitecopy_site->driver = &vfs_driver;
+		host->port = 0;
+		g_free( host->hostname );
+		string = screem_site_get_remote_url( ssite );
+		if( method == UPLOAD_OTHER ) {
+			host->hostname = g_strdup( string );
+		} else {
+			host->hostname = 
+				g_strconcat( upload_schemes[ method ],
+						string, NULL );
+		}
 		break;
 	}
 
@@ -400,16 +398,46 @@ static struct site* screem_site_to_sitecopy_site( ScreemSite *ssite,
 	sitecopy_site->remote_root_user = g_strdup( string );
 
 	sitecopy_site->remote_isrel = ( string[ 0 ] == '~' );
+	if( sitecopy_site->protocol == siteproto_vfs ) {
+		sitecopy_site->remote_isrel = FALSE;
+	}
+
 	if( string[ 0 ] != '~' && string[ 0 ] != G_DIR_SEPARATOR ) {
 		/* must start with ~ or /  */
-		if( show_alert )
+		if( show_alert ) {
 			upload_wizard_error( _("Remote path must being with ~ or /") );
+		}
 		g_free( sitecopy_site );
 		return NULL;
 	}
 
-	if( screem_site_get_remote_method( ssite ) == LOCAL ) {
-		host->hostname = g_strdup( string );
+	if( method != FTP && method != RSH ) {
+		/* vfs method */
+		g_free( sitecopy_site->remote_root_user );
+		uri = gnome_vfs_uri_new( host->hostname );
+		if( ! uri ) {
+			if( show_alert ) {
+				upload_wizard_error( _( "Invalid URI for remote host" ) );
+			}
+			g_free( sitecopy_site );
+			return NULL;
+		}
+		if( host->port ) {
+			gnome_vfs_uri_set_host_port( uri, host->port );
+		}
+		full = gnome_vfs_uri_append_path( uri, string );
+		gnome_vfs_uri_unref( uri );
+		if( ! full ) {
+			if( show_alert ) {
+				upload_wizard_error( _( "Invalid URI for remote host" ) );
+			}
+		}
+		if( host->username ) {
+			gnome_vfs_uri_set_user_name( full, 
+					host->username );
+		}
+		sitecopy_site->remote_root_user = gnome_vfs_uri_to_string( full, 0 );
+		gnome_vfs_uri_unref( full );
 	}
 
 
@@ -422,11 +450,9 @@ static struct site* screem_site_to_sitecopy_site( ScreemSite *ssite,
 	{
 		GnomeVFSURI *uri;
 		gboolean islocal;
-		gchar *temp;
 
 		uri = gnome_vfs_uri_new( string );
 		islocal = gnome_vfs_uri_is_local( uri );
-
 		gnome_vfs_uri_unref( uri );
 		
 		if( ( ! islocal ) && strncmp( "file:", string,
@@ -436,23 +462,18 @@ static struct site* screem_site_to_sitecopy_site( ScreemSite *ssite,
 			g_free( sitecopy_site );
 			return NULL;
 		}
-
-		temp = gnome_vfs_get_local_path_from_uri( string );
-		if( ! temp ) {
-			string = screem_site_get_pathname( ssite );
-		} else {
-			/* memleak */
-			string = temp;
-		}
 	}
 
-	sitecopy_site->local_root_user = g_strdup( string );
+	temp = screem_get_local_site_path( ssite );
+	string = g_strconcat( temp, G_DIR_SEPARATOR_S, NULL );
+	g_free( temp );
+	sitecopy_site->local_root_user = string;
 	sitecopy_site->local_isrel = ( string[ 0 ] == '~' );
 	if( string[ 0 ] != '~' && string[ 0 ] != G_DIR_SEPARATOR ) {
 		/* should never occur, screem should not allow this
 		   case to happen */
 		if( show_alert )
-			upload_wizard_error( _("Local path must being with ~ or /") );
+			upload_wizard_error( _("Local path must begin with ~ or /") );
 		g_free( sitecopy_site );
 		return NULL;
 	}
@@ -473,20 +494,22 @@ static struct site* screem_site_to_sitecopy_site( ScreemSite *ssite,
 	exclude->prev->next = exclude;
 	exclude = exclude->prev;
 	exclude->haspath = FALSE;
-	exclude->pattern = ".project.screem";
-	for(list = screem_site_get_excludes( ssite ); list; list = list->next){
+	exclude->pattern = g_strdup( ".project.screem" );
+	for( tmp = list = screem_site_get_excludes( ssite ); 
+			list; list = list->next ){
 		struct fnlist *newexclude;
+		gchar *pattern;
 
 		newexclude = g_new0( struct fnlist, 1 );
 
 		if( ! strncmp( "glob:", (gchar*)list->data, 
 				strlen( "glob:" ) ) ) {
 			newexclude->haspath = FALSE;
-			newexclude->pattern = list->data + strlen( "glob:" );
+			pattern = ((gchar*)list->data) + strlen( "glob:" );
 		} else if( ! strncmp( "regex:", (gchar*)list->data, 
 				strlen( "regex:" ) ) ) {
 			newexclude->haspath = FALSE;
-			newexclude->pattern = list->data + strlen( "regex: " );
+			pattern = ((gchar*)list->data) + strlen( "regex: " );
 
 			/* don't support this yet */
 			g_free( newexclude );
@@ -494,31 +517,37 @@ static struct site* screem_site_to_sitecopy_site( ScreemSite *ssite,
 			g_warning( "regex: patterns not supported yet\n" );
 		} else {
 			newexclude->haspath = TRUE;
-			newexclude->pattern = ((gchar*)list->data) + strlen( string );
+			pattern = ((gchar*)list->data) + strlen( string );
+			pattern ++;
 		}
 
 		if( newexclude ) {
+			newexclude->pattern = g_strdup( pattern );
 			exclude->prev = newexclude;
 			exclude->prev->next = exclude;
 			exclude = exclude->prev;
 		}
+		g_free( list->data );
 	}
+	g_list_free( tmp );
 	sitecopy_site->excludes = exclude;
 	
 	/* setup ignores */
-	for( ignores = NULL, list = screem_site_get_ignores( ssite ); list;
+	for( ignores = NULL, 
+		tmp = list = screem_site_get_ignores( ssite ); list;
 	     list = list->next ) {
 		struct fnlist *newignore;
+		gchar *pattern;
 
 		newignore = g_new0( struct fnlist, 1 );
 		if( ! strncmp( "glob:", (gchar*)list->data, 
 				strlen( "glob:" ) ) ) {
 			newignore->haspath = FALSE;
-			newignore->pattern = list->data + strlen( "glob:" );
+			pattern = ((gchar*)list->data) + strlen( "glob:" );
 		} else if( ! strncmp( "regex:", (gchar*)list->data, 
 				strlen( "regex:" ) ) ) {
 			newignore->haspath = FALSE;
-			newignore->pattern = list->data + strlen( "regex: " );
+			pattern = ((gchar*)list->data) + strlen( "regex: " );
 
 			/* don't support this yet */
 			g_free( newignore );
@@ -526,10 +555,12 @@ static struct site* screem_site_to_sitecopy_site( ScreemSite *ssite,
 			g_warning( "regex: patterns not supported yet\n" );
 		} else {
 			newignore->haspath = TRUE;
-			newignore->pattern = ((gchar*)list->data) + strlen( string );
+			pattern = ((gchar*)list->data) + strlen( string );
+			pattern ++;
 		}
 
 		if( newignore ) {
+			newignore->pattern = g_strdup( pattern );
 			if( ! ignores ) {
 				ignores = newignore;
 			} else {
@@ -538,23 +569,27 @@ static struct site* screem_site_to_sitecopy_site( ScreemSite *ssite,
 				ignores = ignores->prev;
 			}
 		}
+		g_free( list->data );
 	}
+	g_list_free( tmp );
 	sitecopy_site->ignores = ignores;
 
 	/* setup asciis */
-	for( ascii = NULL, list = screem_site_get_asciis( ssite ); list;
+	for( ascii = NULL, 
+		tmp = list = screem_site_get_asciis( ssite ); list;
 	     list = list->next ) {
 		struct fnlist *newascii;
+		gchar *pattern;
 
 		newascii = g_new0( struct fnlist, 1 );
 		if( ! strncmp( "glob:", (gchar*)list->data, 
 				strlen( "glob:" ) ) ) {
 			newascii->haspath = FALSE;
-			newascii->pattern = list->data + strlen( "glob:" );
+			pattern = ((gchar*)list->data) + strlen( "glob:" );
 		} else if( ! strncmp( "regex:", (gchar*)list->data, 
 				strlen( "regex:" ) ) ) {
 			newascii->haspath = FALSE;
-			newascii->pattern = list->data + strlen( "regex: " );
+			pattern = ((gchar*)list->data) + strlen( "regex: " );
 
 			/* don't support this yet */
 			g_free( newascii );
@@ -562,10 +597,12 @@ static struct site* screem_site_to_sitecopy_site( ScreemSite *ssite,
 			g_warning( "regex: patterns not supported yet\n" );
 		} else {
 			newascii->haspath = TRUE;
-			newascii->pattern = ((gchar*)list->data) + strlen( string );
+			pattern = ((gchar*)list->data) + strlen( string );
+			pattern ++;
 		}
 
 		if( newascii ) {
+			newascii->pattern = g_strdup( pattern );
 			if( ! ascii ) {
 				ascii = newascii;
 			} else {
@@ -574,7 +611,9 @@ static struct site* screem_site_to_sitecopy_site( ScreemSite *ssite,
 				ascii = ascii->prev;
 			}
 		}
+		g_free( list->data );
 	}
+	g_list_free( tmp );
 	sitecopy_site->asciis = ascii;
 
 	sitecopy_site->use_this = TRUE;
@@ -642,15 +681,19 @@ static void upload_page(ScreemPage *page)
 	gtk_widget_destroy( dialog );
 }
 
-static void upload_site( EggAction *action, gpointer user_data )
+static void upload_site( GtkAction *action, gpointer user_data )
 {
 	UploadWizard *wizard;
 	struct stat s;
 	ScreemSite *site;
 	ScreemPage *page;
 	GtkWidget *widget;
+	ScreemApplication *app;
+	ScreemSession *session;
 	
 	wizard = (UploadWizard*)user_data;
+	app = SCREEM_APPLICATION( wizard->window->application );
+	session = screem_application_get_session( app );
 
 	site = screem_window_get_current( wizard->window );
 
@@ -760,6 +803,9 @@ static void upload_site( EggAction *action, gpointer user_data )
 		       update_thread, wizard);
 
        	/* all setup, display upload dialog */
+	if( ! GTK_WIDGET_VISIBLE( wizard->dialog ) ) {
+		screem_session_restore_dialog( session, wizard->dialog );
+	}
 	gtk_widget_show_all( wizard->dialog );
 	gdk_window_raise( wizard->dialog->window );	
 
@@ -885,42 +931,23 @@ static void *update_thread( void *arg )
 		gtk_widget_set_sensitive(keep_going_button, FALSE);
 		gdk_threads_leave();
 
+		wizard->site->keep_going = tmp;
 		ret = my_abortable_transfer_wrapper(wizard->site, 
 						    site_op_update);
 
 		gdk_threads_enter();
 
-		message = NULL;
-		
-		switch( ret ) {
-		case SITE_OK:
-			/* be sure to add some 'free' commands in here */
+		message = wizard->site->last_error;
+
+		if( ret == SITE_OK ) {
 			site_write_stored_state( wizard->site );
-			/*rescan_selected(1);*/
-			break;
-		case SITE_AUTH:
-			message = _("There was a problem authenticating with the remote server.");
-			break;
-		case SITE_LOOKUP:
-			message = _("Could not resolve the remote site's hostname.");
-			break;
-		case SITE_CONNECT:
-			message = _("Could not establish a connection to the remote site.");
-			break;
-		case SITE_FAILED:
-			message = _("Update Failed (Authentication problems)" );
-			break;
-		case SITE_ABORTED:
-			message = _("Update was aborted." );
-			break;
-		case SITE_ERRORS:
-			/* replace this with a custom dialog containing the errors, a la Xsitecopy */
-			message = _("There were errors.");
-			break;
-		}
-		if( message ) {
+			if( message ) {
+				upload_wizard_message( message );
+			}
+		} else if( message ) {
 			upload_wizard_error( message );
 		}
+
 		gtk_widget_hide (stop_button);
 		gtk_widget_show (close_button); 
 		gtk_widget_set_sensitive(close_button, TRUE);
@@ -943,10 +970,10 @@ static void upload_wizard_error( const gchar *message )
 {
 	gchar *temp;
 
-	temp = g_strdup_printf( _( "Upload Wizard: %s" ), message );
+	temp = g_strdup_printf( _( "Upload Wizard: %s\n" ), message );
 	
 	screem_window_show_error( SCREEM_WINDOW( currentWiz->window ),
-				    temp );
+				    temp, TRUE );
 
 	g_free( temp );	
 }
@@ -955,10 +982,10 @@ static void upload_wizard_message( const gchar *message )
 {
 	gchar *temp;
 
-	temp = g_strdup_printf( _( "Upload Wizard: %s" ), message );
+	temp = g_strdup_printf( _( "Upload Wizard: %s\n" ), message );
 	
 	screem_window_show_message( SCREEM_WINDOW( currentWiz->window ),
-				    temp );
+				    temp, TRUE );
 
 	g_free( temp );
 }
@@ -1002,11 +1029,15 @@ void close_button_clicked(GtkButton *button, gpointer user_data )
 	GladeXML *xml;
 	GtkWidget *update_dialog;
 	UploadWizard *wizard;
+	ScreemApplication *app;
+	ScreemSession *session;
 
 	xml = glade_get_widget_tree( GTK_WIDGET( button ) );
 	update_dialog = glade_xml_get_widget( xml, "update_dialog" );
 	wizard = (UploadWizard*)g_object_get_data( G_OBJECT(update_dialog),
 						     "wizard" );
+	app = SCREEM_APPLICATION( wizard->window->application );
+	session = screem_application_get_session( app );
 
 	wizard->closing = TRUE;
 	sem_post( wizard->update_semaphore );
@@ -1020,6 +1051,8 @@ void close_button_clicked(GtkButton *button, gpointer user_data )
 	/* clean up */
 	g_free( wizard->site );
 
+	screem_session_store_dialog( session, wizard->dialog );
+	
 	gtk_widget_hide( wizard->dialog );
 }
 
@@ -1122,15 +1155,15 @@ void do_abort( UploadWizard *wizard ) {
 
 int fe_gtk_question(char *question, GnomeReplyCallback yes_action)
 {
-    //GtkWidget *question_window;
+/*    GtkWidget *question_window;
 
     gdk_threads_enter();
-    //    question_window = gnome_app_question(GNOME_APP(app),
-    //                                   (const gchar *) question,
-    //                                       yes_action, NULL);
-//gtk_widget_show(question_window);
+        question_window = gnome_app_question(GNOME_APP(app),
+                                       (const gchar *) question,
+                                           yes_action, NULL);
+gtk_widget_show(question_window);
     gdk_threads_leave();
-g_print( "QUESTION: %s\n", question );
+g_print( "QUESTION: %s\n", question );*/
     return 1;
 }
 
@@ -1192,7 +1225,7 @@ void fe_warning(const char *description, const char *subject,
 	temp = g_strdup_printf( _( "Upload Wizard: %s" ), description );
 	
 	screem_window_show_message( SCREEM_WINDOW( currentWiz->window ),
-				    temp );
+				    temp, FALSE );
 
 	gdk_threads_leave();
 	
@@ -1301,7 +1334,7 @@ void fe_updated( const struct site_file *file, const int success,
 	if( ! success ) {
 		temp = g_strdup_printf( "%s: %s", file_name( file ), error );
 		screem_window_show_message( SCREEM_WINDOW( currentWiz->window ),
-					    temp );
+					    temp, FALSE );
 		g_free( temp );
 	}
 
