@@ -27,9 +27,3 @@ enum site_op
 {
     site_op_update, site_op_fetch, site_op_resync
 };
-
-void create_update_dialog (void);
-void create_error_log_window (void);
-void uploadSite(void);
-void *update_thread(void *bler);
-
