@@ -35,7 +35,8 @@
 #include <gtk/gtklabel.h>
 #include <gtk/gtkprogressbar.h>
 
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
+#include <libgnomeui/gnome-authentication-manager.h>
 
 #include <gconf/gconf-client.h>
 #include <glade/glade.h>
@@ -52,9 +53,26 @@
 #include <libgnomevfs/gnome-vfs-file-info.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 
+#include <libgnomevfs/gnome-vfs-module-callback.h>
+#include <libgnomevfs/gnome-vfs-standard-callbacks.h>
+
 #include "fileops.h"
 
-gboolean copy_file( const gchar *source, const gchar *dest )
+#include "support.h"
+
+#include "screem-errors.h"
+
+/* not nice including these, we need access to the ScreemSession
+ * so we can keep track of the file selector window size */
+#include "screem-application.h"
+#include "screem-window.h"
+
+/* yuk, needed for our hack on GnomeFileEntry as we can't
+ * provide the ScreemWindow in glade dialogs to get access to the app */
+extern ScreemApplication *app;
+
+gboolean copy_file( const gchar *source, const gchar *dest,
+		ScreemFileOp cb, gpointer data )
 {
 	GnomeVFSURI *suri;
 	GnomeVFSURI *duri;
@@ -91,10 +109,12 @@ gboolean copy_file( const gchar *source, const gchar *dest )
 			/* get permissions for source */
 			options = GNOME_VFS_FILE_INFO_FOLLOW_LINKS;
 			info = gnome_vfs_file_info_new();
-			gnome_vfs_get_file_info_uri( suri, info, options );
-			result = gnome_vfs_create_uri( &dhandle, duri,
+			result = gnome_vfs_get_file_info_uri( suri, info, options );
+			if( result == GNOME_VFS_OK ) {
+				result = gnome_vfs_create_uri( &dhandle, duri,
 						       mode, TRUE, 
 						       info->permissions );
+			}
 			gnome_vfs_file_info_unref( info );
 		}
 		while( result == GNOME_VFS_OK ) {
@@ -110,8 +130,7 @@ gboolean copy_file( const gchar *source, const gchar *dest )
 				result = gnome_vfs_write( dhandle, buffer, rsize,
 							  &rsize );
 				wsize += rsize;
-			} else
-				break;
+			} 
 		}
 		if( result == GNOME_VFS_ERROR_EOF ) {
 			ret = TRUE;
@@ -120,6 +139,10 @@ gboolean copy_file( const gchar *source, const gchar *dest )
 		if( wrote ) {
 			gnome_vfs_truncate_handle( dhandle, wsize );
 			gnome_vfs_close( dhandle );
+			if( cb ) {
+				cb( GNOME_VFS_MONITOR_EVENT_CREATED,
+						dest, data );
+			}
 		}
 		gnome_vfs_close( shandle );
 	}
@@ -130,7 +153,8 @@ gboolean copy_file( const gchar *source, const gchar *dest )
 	return ret;
 }
 
-gboolean copy_dir( const gchar *source, const gchar *dest, gboolean move )
+gboolean copy_dir( const gchar *source, const gchar *dest, gboolean move,
+		ScreemFileOp cb, gpointer data )
 {
 	gchar *orig;
 	gchar *new_path;
@@ -159,11 +183,14 @@ gboolean copy_dir( const gchar *source, const gchar *dest, gboolean move )
 	result = gnome_vfs_get_file_info( dest, vfsinfo, options );
 	
 	if( result != GNOME_VFS_OK ) {
-		if( ! mkdir_recursive( dest, perms ) ) {
+		if( ! mkdir_recursive( dest, perms, cb, data ) ) {
 			gnome_vfs_file_info_unref( vfsinfo );
 			return FALSE;
 		}
 		made = TRUE;
+		if( cb ) {
+			cb( GNOME_VFS_MONITOR_EVENT_CREATED, dest, data );
+		}
 	}
 
 	if( ! made && vfsinfo->type != GNOME_VFS_FILE_TYPE_DIRECTORY ) {
@@ -192,12 +219,12 @@ gboolean copy_dir( const gchar *source, const gchar *dest, gboolean move )
 						NULL );
 			if( vfsinfo->type != GNOME_VFS_FILE_TYPE_DIRECTORY ) {
 				if( move ) {
-					move_file( orig, new_path );
+					move_file( orig, new_path, cb, data );
 				} else {
-					copy_file( orig, new_path );
+					copy_file( orig, new_path, cb, data );
 				}
 			} else {
-				copy_dir( orig, new_path, move );
+				copy_dir( orig, new_path, move, cb, data );
 			}
 			g_free( orig );
 			g_free( new_path );
@@ -209,22 +236,30 @@ gboolean copy_dir( const gchar *source, const gchar *dest, gboolean move )
 	}
 
 	if( move ) {
-		rmdir( source );
+		delete_dir( source, cb, data );
 	}
 
 	return TRUE;
 }
 
-gboolean move_file( const gchar *source, const gchar *dest )
+gboolean move_file( const gchar *source, const gchar *dest,
+		ScreemFileOp cb, gpointer data )
 {
-	if( ! overwrite( dest ) ) {
-		return FALSE;
+	gboolean ret;
+	
+	if( ( ! overwrite( dest ) ) ||
+		( gnome_vfs_move( source, dest, TRUE ) != GNOME_VFS_OK ) ) {
+		ret = FALSE;
+	} else {
+		ret = TRUE;
+		cb( GNOME_VFS_MONITOR_EVENT_CREATED, dest, data );
+		cb( GNOME_VFS_MONITOR_EVENT_DELETED, source, data );
 	}
 
-	return ( gnome_vfs_move( source, dest, TRUE ) == GNOME_VFS_OK );
+	return ret;
 }
 
-gboolean delete_file( const gchar *file )
+gboolean delete_file( const gchar *file, ScreemFileOp cb, gpointer data )
 {
 	GnomeVFSURI *uri;
 	gboolean ret;
@@ -235,21 +270,26 @@ gboolean delete_file( const gchar *file )
 
 	gnome_vfs_uri_unref( uri );
 
+	if( ret && cb ) {
+		cb( GNOME_VFS_MONITOR_EVENT_DELETED, file, data );
+	}
+	
 	return ret;
 }
 
-gboolean delete_dir( const gchar *path )
+gboolean delete_dir( const gchar *path, ScreemFileOp cb, gpointer data )
 {
 	GnomeVFSDirectoryHandle *handle;
 	GnomeVFSFileInfoOptions options;
 	GnomeVFSResult result;
 	GnomeVFSFileInfo *info;
-
+	gboolean ret;
+	
 	options = GNOME_VFS_DIRECTORY_VISIT_LOOPCHECK;
 
 	result = gnome_vfs_directory_open( &handle, path, options );
 	if( result != GNOME_VFS_OK) {
-		return delete_file( path );
+		return delete_file( path, cb, data );
 	}
        
 	info = gnome_vfs_file_info_new();
@@ -261,7 +301,7 @@ gboolean delete_dir( const gchar *path )
 			gchar *new_path = g_strdup_printf( "%s%c%s", path,
 							   G_DIR_SEPARATOR, 
 							   info->name );
-			delete_dir( new_path );
+			delete_dir( new_path, cb, data );
 			g_free( new_path );
 		}
 	}
@@ -270,20 +310,31 @@ gboolean delete_dir( const gchar *path )
 
 	gnome_vfs_directory_close( handle );
 
-	return ( gnome_vfs_remove_directory( path ) == GNOME_VFS_OK );
+	ret = ( gnome_vfs_remove_directory( path ) == GNOME_VFS_OK );
+	if( ret ) {
+		if( cb ) {
+			cb( GNOME_VFS_MONITOR_EVENT_DELETED, path, data );
+		}
+	}
+
+	return ret;
 }
 
 /* recusivley makes directories (if needed), starting at current path */
-gboolean mkdir_recursive( const gchar *path, GnomeVFSFilePermissions perms )
+gboolean mkdir_recursive( const gchar *path, GnomeVFSFilePermissions perms,
+		ScreemFileOp cb, gpointer data )
 {
 	GnomeVFSURI *uri;
 	gboolean ret = FALSE;
+	gboolean exists;
 
 	uri = gnome_vfs_uri_new( path );
 	if( uri ) {
 		ret = TRUE;
+		
+		exists = gnome_vfs_uri_exists( uri );
 
-		if( gnome_vfs_uri_has_parent( uri ) ) {
+		if( ( ! exists ) && gnome_vfs_uri_has_parent( uri ) ) {
 			GnomeVFSURI *parent;
 			gchar *parentname;
 
@@ -293,11 +344,11 @@ gboolean mkdir_recursive( const gchar *path, GnomeVFSFilePermissions perms )
 							      GNOME_VFS_URI_HIDE_NONE );
 			gnome_vfs_uri_unref( parent );
 
-			ret = mkdir_recursive( parentname, perms );
+			ret = mkdir_recursive( parentname, perms, cb, data );
 
 			g_free( parentname );
 		}
-		if( ret ) {
+		if( ret && ! exists ) {
 			GnomeVFSResult result;
 
 			result = gnome_vfs_make_directory_for_uri( uri, 
@@ -309,9 +360,11 @@ gboolean mkdir_recursive( const gchar *path, GnomeVFSFilePermissions perms )
 				
 				info = gnome_vfs_file_info_new();
 				options = GNOME_VFS_FILE_INFO_DEFAULT;
-				gnome_vfs_get_file_info_uri( uri, info,
-							     options );
-				if(info->type ==GNOME_VFS_FILE_TYPE_DIRECTORY){
+				if( gnome_vfs_get_file_info_uri( uri, 
+							info,
+							options ) ==
+							GNOME_VFS_OK &&
+				    info->type ==GNOME_VFS_FILE_TYPE_DIRECTORY) {
 					result = GNOME_VFS_OK;
 				}
 				gnome_vfs_file_info_unref( info );
@@ -322,6 +375,12 @@ gboolean mkdir_recursive( const gchar *path, GnomeVFSFilePermissions perms )
 		gnome_vfs_uri_unref( uri );
 	}
 
+	if( ret ) {
+		if( cb ) {
+			cb( GNOME_VFS_MONITOR_EVENT_CREATED, path, data );
+		}
+	}
+	
 	return ret;
 }
 
@@ -477,28 +536,46 @@ gboolean uri_exists( const gchar *filename, GnomeVFSResult *result )
 {
 	GnomeVFSFileInfo *info;
 	GnomeVFSURI *uri;
-	GnomeVFSResult res;
 	gboolean ret;
+	
+	uri = gnome_vfs_uri_new( filename );
 
 	if( ! result ) {
-		result = &res;
+		ret = gnome_vfs_uri_exists( uri );
+	} else {
+		info = gnome_vfs_file_info_new();
+		*result = gnome_vfs_get_file_info_uri( uri, info,
+				       GNOME_VFS_FILE_INFO_FOLLOW_LINKS );
+
+		/* does dest already exist? */
+		ret = ( *result == GNOME_VFS_OK );
+
+		gnome_vfs_file_info_unref( info );
 	}
-
-	uri = gnome_vfs_uri_new( filename );
-	
-	info = gnome_vfs_file_info_new();
-	*result = gnome_vfs_get_file_info_uri( uri, info,
-					       GNOME_VFS_FILE_INFO_FOLLOW_LINKS );
-
-	/* does dest already exist? */
-	ret = ( *result == GNOME_VFS_OK );
-
-	gnome_vfs_file_info_unref( info );
 	gnome_vfs_uri_unref( uri );
 
 	return ret;
 }
 
+gboolean uri_accessible( const gchar *filename, 
+		GnomeVFSFilePermissions perms )
+{
+	gboolean ret;
+	GnomeVFSFileInfo *info;
+	GnomeVFSResult res;
+	
+	ret = FALSE;
+	info = gnome_vfs_file_info_new();
+	res = gnome_vfs_get_file_info( filename, info,
+				GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS |
+			       GNOME_VFS_FILE_INFO_FOLLOW_LINKS );
+	if( res == GNOME_VFS_OK ) {
+		ret = info->permissions & perms;
+	}
+	gnome_vfs_file_info_unref( info );
+
+	return ret;
+}
 
 gboolean overwrite( const gchar *filename )
 {
@@ -512,7 +589,6 @@ gboolean overwrite( const gchar *filename )
 	if( ! ret ) {
 		ret = TRUE;
 	} else if( result == GNOME_VFS_OK ) {
-		GConfClient *client;
 		gchar *glade_path;
 		GladeXML *xml;
 		GString *str;
@@ -522,13 +598,9 @@ gboolean overwrite( const gchar *filename )
 		gchar *tmp;
 		guint pos;
 
-		client = gconf_client_get_default();
-		glade_path = gconf_client_get_string( client,
-						      "/apps/screem/general/glade_path",
-						      NULL );
+		glade_path = screem_get_glade_path();
 		xml = glade_xml_new( glade_path, "overwrite", NULL );
 		g_free( glade_path );
-		g_object_unref( client );
 	
 		label = glade_xml_get_widget( xml, "overwrite_label" );
 		ltxt = gtk_label_get_text( GTK_LABEL( label ) );
@@ -599,7 +671,8 @@ void load_file_internal_cancel( GtkWidget *widget )
 static GString *load_file_internal( GnomeVFSHandle *handle, 
 				   const gchar *path,
 				   GnomeVFSFileSize size,
-				   gboolean interactive )
+				   gboolean interactive,
+				   GnomeVFSResult *res )
 {
 	GString *ret;
 	GnomeVFSResult result;
@@ -607,7 +680,6 @@ static GString *load_file_internal( GnomeVFSHandle *handle,
 	GnomeVFSFileSize total_read;
 	
 	GladeXML *xml;
-	GConfClient *client;
 	gchar *glade_path;
 	const gchar *ltxt;
 	const gchar *temp;
@@ -621,15 +693,16 @@ static GString *load_file_internal( GnomeVFSHandle *handle,
 	gdouble fraction;
 	gboolean pulse;
 	
+	/* shut compiler up, they won't be used uninitialised */
+	xml = NULL;
+	widget = NULL;
+	
 	ret = g_string_new( NULL );
 
 	pulse = ( size == 0 );
 	
 	if( interactive ) {
-		client = gconf_client_get_default();
-		glade_path = gconf_client_get_string( client,
-						      "/apps/screem/general/glade_path",
-						      NULL );
+		glade_path = screem_get_glade_path();
 		xml = glade_xml_new( glade_path, "loading", NULL );
 		g_free( glade_path );
 
@@ -664,8 +737,6 @@ static GString *load_file_internal( GnomeVFSHandle *handle,
 		gtk_window_set_type_hint( GTK_WINDOW( widget ), 
 					  GDK_WINDOW_TYPE_HINT_DIALOG );
 
-		g_object_unref( client );
-
 		gtk_widget_show( widget );
 		glade_xml_signal_autoconnect( xml );
 
@@ -677,9 +748,18 @@ static GString *load_file_internal( GnomeVFSHandle *handle,
 	do {
 		result = gnome_vfs_read( handle, buffer,
 				READ_CHUNK_SIZE, &bytes_read );
-
+		
 		if( result == GNOME_VFS_OK || 
 		    result == GNOME_VFS_ERROR_EOF ) {
+			/* gzip method never gives EOF, assume
+			 * 0 bytes is EOF,
+			 *
+			 * see http://bugzilla.gnome.org/show_bug.cgi?id=41173
+			 *
+			 * */
+			if( bytes_read == 0 ) {
+				result = GNOME_VFS_ERROR_EOF;
+			}
 			if( interactive ) {
 				total_read += bytes_read;
 
@@ -690,10 +770,9 @@ static GString *load_file_internal( GnomeVFSHandle *handle,
 					gtk_progress_bar_pulse( GTK_PROGRESS_BAR( widget ) );
 				}
 				gdk_threads_leave();
-				while( g_main_context_pending( NULL ) ) {
-					g_main_iteration( FALSE );
-				}
-gdk_threads_enter();
+				while( g_main_iteration( FALSE ) ) {}
+				
+				gdk_threads_enter();
 
 				if( g_object_get_data( G_OBJECT( widget ),
 							"cancel" ) ) {
@@ -708,6 +787,8 @@ gdk_threads_enter();
 	if( result != GNOME_VFS_ERROR_EOF ) {
 		g_string_free( ret, TRUE );
 		ret = NULL;
+	} else {
+		result = GNOME_VFS_OK;
 	}
 	
 	if( interactive ) {
@@ -716,64 +797,123 @@ gdk_threads_enter();
 		g_object_unref( G_OBJECT( xml ) );
 	}
 	
+	if( res ) {
+		*res = result;
+	}
+	
 	return ret;
 }
 
-GString *load_file( const gchar *path )
+GString *load_file( const gchar *path, gboolean *compress,
+		gchar **charset, GError **error )
 {
 	GnomeVFSResult result;
 	GnomeVFSFileInfo *info;
 	GnomeVFSHandle *handle;
 	gboolean interactive;
+
+	gchar *gzpath;
+	GnomeVFSFileSize size;
+	gboolean local;
+	gboolean compressed;
+	gchar *temp;
 	
 #define FILE_SIZE_FOR_DIALOG ( 8 * 1024000 )
 	
 	GString *ret;
 
+	if( error ) {
+		*error = NULL;
+	}
+	if( ! compress ) {
+		compress = &compressed;
+	}
+	
 	ret = NULL;
 	
 	info = gnome_vfs_file_info_new();
 	
+	gzpath = g_strconcat( path, "#ugzip:", NULL );
+	
 	handle = NULL;
-	result = gnome_vfs_open( &handle, path, GNOME_VFS_OPEN_READ );
+	result = gnome_vfs_open( &handle, gzpath, GNOME_VFS_OPEN_READ );
+	*compress = TRUE;
+	if( result != GNOME_VFS_OK ) {
+		result = gnome_vfs_open( &handle, path, GNOME_VFS_OPEN_READ );
+		*compress = FALSE;
+	}
 	if( result == GNOME_VFS_OK ) {
 		result = gnome_vfs_get_file_info_from_handle( handle, 
 				info,
 				GNOME_VFS_FILE_INFO_DEFAULT );
-	}
+		size = 0;
+		local = FALSE;
+		if( result == GNOME_VFS_OK ) {
+			local = GNOME_VFS_FILE_INFO_LOCAL( info );
+			if( info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_SIZE )  {
+				size = info->size;
+			}
+		}
+		gnome_vfs_file_info_unref( info );
 	
-	if( result == GNOME_VFS_OK ) {
-		interactive = ( ( info->size > FILE_SIZE_FOR_DIALOG ) ||
-			        ( ! GNOME_VFS_FILE_INFO_LOCAL( info ) ) );
-		ret = load_file_internal( handle, path, info->size,
-					interactive );
+		interactive = ( ( size > FILE_SIZE_FOR_DIALOG ) ||
+				! local );
+		ret = load_file_internal( handle, path, size,
+					interactive, &result );
 	}
 
 	if( handle ) {
 		gnome_vfs_close( handle );
 	}
 	
-	gnome_vfs_file_info_unref( info );
+	/* if we have a result convert it to UTF-8 if possible */
+	if( ret ) {
+		temp = screem_support_charset_convert( ret->str, 
+				ret->len, charset );
+		g_string_assign( ret, temp );
+	} else if( error ) {
+		*error = g_error_new_literal( g_quark_from_string( SCREEM_LOAD_FILE_ERR ),
+				(gint)result,
+				gnome_vfs_result_to_string( result ) );
+	}
+	
+	g_free( gzpath );
 	
 	return ret;
 }
 
 gboolean save_file( const gchar *path, const gchar *data,
-		    GnomeVFSFilePermissions perms )
+		    GnomeVFSFilePermissions perms,
+		    gboolean compress,
+		    GError **error )
 {
 	GnomeVFSURI *uri;
 	GnomeVFSResult result;
 	GnomeVFSHandle *handle;
 	GnomeVFSFileSize wsize;
 	gint len;
+	gchar *gzpath;
 	
 	gboolean ret;
 
 	if( ! data ) {
 		data = "";
 	}
+
+	if( error ) {
+		*error = NULL;
+	}
 	
-	uri = gnome_vfs_uri_new( path );
+	if( ! compress ) {
+		uri = gnome_vfs_uri_new( path );
+	} else {
+		gzpath = g_strconcat( path, "#gzip:", NULL );
+		uri = gnome_vfs_uri_new( gzpath );
+		g_free( gzpath );
+	}
+
+	/* FIXME: make a backup of the original first
+	 * to avoid any data loss if the save fails? */
 	
 	if( uri ) {
 		len = strlen( data );
@@ -784,7 +924,10 @@ gboolean save_file( const gchar *path, const gchar *data,
 					       FALSE, perms );
 	}
 	
-	if( ! uri || result != GNOME_VFS_OK ) {
+	if( ! uri ) {
+		ret = FALSE;
+		result = GNOME_VFS_ERROR_INVALID_URI;
+	} else if( result != GNOME_VFS_OK ) {
 		ret = FALSE;
 	} else {
 		while( len > 0 && result == GNOME_VFS_OK ) {
@@ -804,6 +947,12 @@ gboolean save_file( const gchar *path, const gchar *data,
 		ret = TRUE;
 	}
 
+	if( error && ! ret ) {
+		*error = g_error_new_literal( g_quark_from_string( SCREEM_SAVE_FILE_ERR ),
+				(gint)result,
+				gnome_vfs_result_to_string( result ) );
+	}
+	
 	return ret;
 }
 
@@ -879,7 +1028,7 @@ gchar *screem_get_dot_dir( void )
 
 	ret = g_build_filename( home, ".screem", NULL );
 
-	mkdir_recursive( ret, GNOME_VFS_PERM_USER_ALL );
+	mkdir_recursive( ret, GNOME_VFS_PERM_USER_ALL, NULL, NULL );
 
 	return ret;
 }
@@ -902,6 +1051,7 @@ GSList *screem_vfs_scandir( const gchar *dir,
 	GnomeVFSFileInfo *info;
 	GnomeVFSResult result;
 	gboolean ignore;
+	gchar *utf;
 	
 	ret = NULL;
 
@@ -926,18 +1076,27 @@ GSList *screem_vfs_scandir( const gchar *dir,
 				       info->name[ 2 ] == '\0' )
 				   )
 				 );
+	
+			if( ! g_utf8_validate( info->name, -1, NULL ) ) {
+				utf = g_filename_to_utf8( info->name, -1,
+						NULL, NULL, NULL );
+			} else {
+				utf = g_strdup( info->name );
+			}
 			
 			if( ! filter( info ) ) {
 				full = g_build_path( G_DIR_SEPARATOR_S,
-						     dir, info->name,
+						     dir, utf,
 						     NULL );
 				ret = g_slist_prepend( ret, full );
 			} else if( recurse && ( ! ignore ) &&
 				info->mime_type &&
-				! strcmp( "x-directory/normal",
-					info->mime_type ) ) {
+				( ! strcmp( "x-directory/normal",
+					info->mime_type ) ||
+				  ! strcmp( "inode/directory",
+					  info->mime_type ) ) ) {
 				full = g_build_path( G_DIR_SEPARATOR_S,
-						     dir, info->name,
+						     dir, utf,
 						     NULL );
 				rec = screem_vfs_scandir( full,
 						filter, compare,
@@ -945,6 +1104,7 @@ GSList *screem_vfs_scandir( const gchar *dir,
 				g_free( full );
 				ret = g_slist_concat( rec, ret );
 			}
+			g_free( utf );
 		}
 		gnome_vfs_file_info_clear( info );
 	}
@@ -961,3 +1121,300 @@ GSList *screem_vfs_scandir( const gchar *dir,
 	
 	return ret;
 }
+
+
+/* vfs callbacks */
+static void screem_vfs_full_auth_cb( gconstpointer in,
+				gsize in_size,
+				gpointer out,
+				gsize out_size,
+				gpointer data )
+{
+	const GnomeVFSModuleCallbackFullAuthenticationIn *full_in;
+	GnomeVFSModuleCallbackFullAuthenticationOut *full_out;
+
+	full_in = (GnomeVFSModuleCallbackFullAuthenticationIn*) in;
+	full_out = (GnomeVFSModuleCallbackFullAuthenticationOut*) out;
+	
+}
+
+static void screem_vfs_fill_auth_cb( gconstpointer in,
+				gsize in_size,
+				gpointer out,
+				gsize out_size,
+				gpointer data )
+{
+	const GnomeVFSModuleCallbackFillAuthenticationIn *fill_in;
+	GnomeVFSModuleCallbackFillAuthenticationOut *fill_out;
+
+	fill_in = (GnomeVFSModuleCallbackFillAuthenticationIn*) in;
+	fill_out = (GnomeVFSModuleCallbackFillAuthenticationOut*) out;
+
+}
+
+static void screem_vfs_save_auth_cb( gconstpointer in,
+				gsize in_size,
+				gpointer out,
+				gsize out_size,
+				gpointer data )
+{
+	const GnomeVFSModuleCallbackSaveAuthenticationIn *save_in;
+	GnomeVFSModuleCallbackSaveAuthenticationOut *save_out;
+
+	save_in = (GnomeVFSModuleCallbackSaveAuthenticationIn*) in;
+	save_out = (GnomeVFSModuleCallbackSaveAuthenticationOut*) out;
+
+}
+#if 0
+static void screem_vfs_auth_cb( gconstpointer in,
+				gsize in_size,
+				gpointer out,
+				gsize out_size,
+				gpointer data )
+{
+
+}
+
+static void screem_vfs_proxy_auth_cb( gconstpointer in,
+				gsize in_size,
+				gpointer out,
+				gsize out_size,
+				gpointer data )
+{
+
+}
+#endif
+void screem_init_vfs_callbacks( void )
+{
+	static const struct {
+		const gchar *type;
+		GnomeVFSModuleCallback cb;
+	} callbacks[] = {
+		{
+			GNOME_VFS_MODULE_CALLBACK_FULL_AUTHENTICATION,
+			screem_vfs_full_auth_cb
+		},
+		{
+			GNOME_VFS_MODULE_CALLBACK_FILL_AUTHENTICATION,
+			screem_vfs_fill_auth_cb
+		},
+		{
+			GNOME_VFS_MODULE_CALLBACK_SAVE_AUTHENTICATION,
+			screem_vfs_save_auth_cb
+		},
+/*		{
+			GNOME_VFS_MODULE_CALLBACK_AUTHENTICATION,
+			screem_vfs_auth_cb
+		},
+		{
+			GNOME_VFS_MODULE_CALLBACK_HTTP_PROXY_AUTHENTICATION,
+			screem_vfs_proxy_auth_cb
+		}*/
+	};
+	static const guint n_cbs = G_N_ELEMENTS( callbacks );
+	guint i;
+	
+	
+	for( i = 0; FALSE && i < n_cbs; ++ i ) {
+		gnome_vfs_module_callback_set_default( callbacks[ i ].type,
+				callbacks[ i ].cb,
+				NULL, /* data */
+				NULL );
+	}
+
+	gnome_authentication_manager_init();
+}
+
+gchar *screem_fileselect( const gchar *title, GtkWindow *parent,
+			GtkFileChooserAction action,
+			const gchar *dir,
+			const GSList *filters )
+{
+	ScreemSession *session;
+	
+	GtkWidget *dialog;
+	gchar *ret;
+	const gchar *button;
+	gchar *temp;
+	GtkFileFilter *all;
+
+	gint x;
+	gint y;
+	gint w;
+	gint h;
+
+	const gchar *role = "screem_file_selector";
+	
+	session = screem_application_get_session( app );
+	ret = NULL;
+
+	button = GTK_STOCK_OPEN;
+	if( action == GTK_FILE_CHOOSER_ACTION_SAVE ||
+	    action == GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER ) {
+		button = GTK_STOCK_SAVE;
+	}
+		
+	dialog = gtk_file_chooser_dialog_new( title, parent,
+				action, 
+				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+				button, GTK_RESPONSE_ACCEPT,
+				NULL );
+	gtk_window_set_role( GTK_WINDOW( dialog ), role );
+	gtk_file_chooser_set_local_only( GTK_FILE_CHOOSER( dialog ), 
+					FALSE );
+
+	while( filters ) {
+		gtk_file_chooser_add_filter( GTK_FILE_CHOOSER( dialog ),
+						filters->data );
+		filters = filters->next;
+	}
+	all = screem_get_file_filter( _( "All Files" ) );
+	gtk_file_chooser_add_filter( GTK_FILE_CHOOSER( dialog ), all );
+	if( dir ) {
+		gtk_file_chooser_set_current_folder_uri( GTK_FILE_CHOOSER( dialog ), dir );
+	}
+	
+	screem_session_restore_dialog( session, dialog );
+
+	if( gtk_dialog_run( GTK_DIALOG( dialog ) ) == GTK_RESPONSE_ACCEPT ) {
+		ret = gtk_file_chooser_get_uri( GTK_FILE_CHOOSER( dialog ) );
+		if( ret ) {
+			if( ! g_utf8_validate( ret, -1, NULL ) ) {
+				temp = g_filename_to_utf8( ret, -1,
+						NULL, NULL, NULL );
+				g_free( ret );
+				ret = temp;
+			}
+		}
+	}
+
+	/* don't use screem_session_store_dialog(), we don't
+	 * want to store the width/height for save dialogs */
+	if( session ) {
+		if( action == GTK_FILE_CHOOSER_ACTION_SAVE ||
+		    action == GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER ) {
+			w = h = -1;
+		} else {
+			gtk_window_get_size( GTK_WINDOW( dialog ),
+				&w, &h );
+		}
+		gtk_window_get_position( GTK_WINDOW( dialog ),
+				&x, &y );
+		screem_session_set_dialog( session, 
+				gtk_window_get_role( GTK_WINDOW( dialog ) ),
+				x, y, w, h );
+	}
+	
+	gtk_widget_destroy( dialog );
+	
+	return ret;
+}
+
+GSList *screem_fileselect_multi( const gchar *title, GtkWindow *parent,
+			GtkFileChooserAction action,
+			const gchar *dir,
+			const GSList *filters )
+{
+	ScreemSession *session;
+
+	GtkWidget *dialog;
+	GSList *ret;
+	gchar *temp;
+	GSList *tmp;
+	const gchar *button;
+	GtkFileFilter *all;
+
+	gint x;
+	gint y;
+	gint w;
+	gint h;
+	
+	const gchar *role = "screem_file_selector";
+	
+	session = screem_application_get_session( app );
+	ret = NULL;
+
+	button = GTK_STOCK_OPEN;
+	if( action == GTK_FILE_CHOOSER_ACTION_SAVE ||
+	    action == GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER ) {
+		button = GTK_STOCK_SAVE;
+	}
+		
+	dialog = gtk_file_chooser_dialog_new( title, parent,
+				action, 
+				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+				button, GTK_RESPONSE_ACCEPT,
+				NULL );
+	gtk_window_set_role( GTK_WINDOW( dialog ), role );
+	gtk_file_chooser_set_local_only( GTK_FILE_CHOOSER( dialog ), 
+					FALSE );
+	gtk_file_chooser_set_select_multiple( GTK_FILE_CHOOSER( dialog),
+						TRUE );
+	while( filters ) {
+		gtk_file_chooser_add_filter( GTK_FILE_CHOOSER( dialog ),
+						filters->data );
+		filters = filters->next;
+	}
+	all = screem_get_file_filter( _( "All Files" ) );
+	gtk_file_chooser_add_filter( GTK_FILE_CHOOSER( dialog ), all );
+	if( dir ) {
+		gtk_file_chooser_set_current_folder_uri( GTK_FILE_CHOOSER( dialog ), dir );
+	}
+	
+	screem_session_restore_dialog( session, dialog );
+
+	if( gtk_dialog_run( GTK_DIALOG( dialog ) ) == GTK_RESPONSE_ACCEPT ) {
+		ret = gtk_file_chooser_get_uris( GTK_FILE_CHOOSER( dialog ) );
+		for( tmp = ret; tmp; tmp = tmp->next ) {
+			temp = tmp->data;
+			if( ! g_utf8_validate( temp, -1, NULL ) ) {
+				tmp->data = g_filename_to_utf8( temp, 
+						-1, NULL, NULL, NULL );
+				g_free( temp );
+			}
+		}	
+	}
+
+	/* don't use screem_session_store_dialog(), we don't
+	 * want to store the width/height for save dialogs */
+	if( session ) {
+		if( action == GTK_FILE_CHOOSER_ACTION_SAVE ||
+		    action == GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER ) {
+			w = h = -1;
+		} else {
+			gtk_window_get_size( GTK_WINDOW( dialog ),
+				&w, &h );
+		}
+		gtk_window_get_position( GTK_WINDOW( dialog ),
+				&x, &y );
+		screem_session_set_dialog( session, 
+				gtk_window_get_role( GTK_WINDOW( dialog ) ),
+				x, y, w, h );
+	}
+	
+	gtk_widget_destroy( dialog );
+	
+	return ret;
+}
+
+gint screem_compare_file_age( const gchar *a, const gchar *b )
+{
+	GnomeVFSFileInfo *info;
+	time_t atime;
+	time_t btime;
+	GnomeVFSFileInfoOptions options;
+	
+	info = gnome_vfs_file_info_new();
+	atime = btime = 0;
+	options = GNOME_VFS_FILE_INFO_FOLLOW_LINKS;
+	if( gnome_vfs_get_file_info( a, info, options ) == GNOME_VFS_OK ) {
+		atime = info->mtime;
+	}
+	if( gnome_vfs_get_file_info( b, info, options ) == GNOME_VFS_OK ) {
+		btime = info->mtime;
+	}
+	gnome_vfs_file_info_unref( info );
+
+	return ( atime - btime );
+}
+
