@@ -28,15 +28,19 @@
 
 #include <gmodule.h>
 
-#include <gtk/gtkcombo.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkmessagedialog.h>
-#include <gtk/gtkiconfactory.h>
-#include <gtk/gtkstock.h>
+#ifdef HAVE_DBUS
+
+#define DBUS_API_SUBJECT_TO_CHANGE
+#include <dbus/dbus.h>
+#include <dbus/dbus-glib-lowlevel.h>
+
+#endif
+
+#include <gtk/gtk.h>
+
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
@@ -60,6 +64,7 @@
 #include "pageUI.h"
 
 #include "screem-application.h"
+#include "screem-file-browser.h"
 #include "screem-hint.h"
 #include "screem-window.h"
 
@@ -79,21 +84,23 @@
 #include "support.h"
 #include "fileops.h"
 
-#include "rb-sidebar.h"
-#include "rb-sidebar-button.h"
-
-#include "libegg/menu/egg-menu.h"
-
-
 #include "screem-session.h"
 #include "screem-dtd-db.h"
 
+#include "screem-encodings.h"
+
 struct ScreemApplicationPrivate {
 	GList *window_list;
 	GList *loaded_sites;
 	
 	ScreemSession *session;
 	ScreemDTDDB *dtddb;
+
+	GtkTreeModel *encodings;
+#ifdef HAVE_DBUS
+	DBusConnection *conn;
+#endif
+	GSList *start_files;
 };
 
 static void screem_application_init( ScreemApplication *application );
@@ -112,15 +119,21 @@ static gboolean screem_application_load( ScreemApplication *application );
 
 static void screem_application_load_helpers( ScreemApplication *application );
 static void screem_application_helper_failed( ScreemHelper *helper,
-					      ScreemApplication *application );
+		const gchar *msg, ScreemApplication *application );
 static void screem_application_helper_error( ScreemHelper *helper,
 					     gint status,
 					     ScreemApplication *application );
 
 static void screem_application_load_macros( ScreemApplication *application );
 
+#ifdef HAVE_DBUS
+static DBusHandlerResult screem_application_dbus_filter( DBusConnection *conn,
+		DBusMessage *message,
+		gpointer data );
+#endif
+
 #define SPLASH_TEXT( w, t ) gtk_label_set_text( GTK_LABEL( w ),t ); \
-gdk_threads_leave(); while( g_main_pending() ) g_main_iteration(TRUE); \
+gdk_threads_leave(); while( g_main_context_iteration(NULL,FALSE) ){} \
 gdk_threads_enter();
 
 static void screem_application_offline_notify( GConfClient *client,
@@ -227,7 +240,10 @@ screem_application_destroyed_window( GObject *object, ScreemApplication *app )
 
 	for( list = app->plugins; list; list = list->next ) {
 		plugin = (ScreemPlugin*)list->data;
-		plugin->remove_ui( GTK_WIDGET( object ), NULL, NULL, NULL );
+		if( plugin->remove_ui ) {
+			plugin->remove_ui( GTK_WIDGET( object ), 
+					NULL, NULL, NULL );
+		}
 	}
 
 	/* if all windows are closed then exit */
@@ -262,10 +278,12 @@ ScreemWindow *screem_application_create_window( ScreemApplication *app )
 	for( list = app->plugins; list; list = list->next ) {
 		ScreemPlugin *plugin = (ScreemPlugin*)list->data;
 
-		plugin->add_ui( GTK_WIDGET( window ), 
-				window->details->editor,
-				window->details->preview, 
-				window->details->link_view );
+		if( plugin->add_ui ) {
+			plugin->add_ui( GTK_WIDGET( window ), 
+					window->details->editor,
+					window->details->preview, 
+					window->details->link_view );
+		}
 	}
 	for( list = app->helpers; list; list = list->next ) {
 		ScreemHelper *helper = (ScreemHelper*)list->data;
@@ -288,6 +306,7 @@ ScreemApplication *screem_application_new()
 }
 
 static GtkWidget *screem_application_splash( ScreemApplication *application,
+					     GtkWindow **w,
 					     gboolean close)
 {
        	GtkWidget *label;
@@ -320,6 +339,10 @@ static GtkWidget *screem_application_splash( ScreemApplication *application,
 		
 		gtk_widget_show_all( window );
 		
+		if( w ) {
+			*w = GTK_WINDOW( window );
+		}
+		
 		return label;
 	} else {
 		gtk_widget_destroy( window );
@@ -331,6 +354,7 @@ void screem_application_startup( ScreemApplication *application,
 				 const gchar *session,
 				 const gchar **start_files )
 {
+	ScreemApplicationPrivate *priv;
 	ScreemWindow *window;
 	ScreemSite *site;
 	ScreemPage *page;
@@ -341,29 +365,86 @@ void screem_application_startup( ScreemApplication *application,
 	gboolean show_hints;
 	GList *windows;
 
-gdk_threads_enter();
+	GtkWindow *splash;
 	
-	application->start_files = start_files;
+#ifdef HAVE_DBUS
+	DBusError error;
+	DBusConnection *conn;
+	DBusMessage *message;
+	DBusMessageIter it;
+#endif
+	priv = application->priv;
+	
+#ifdef HAVE_DBUS
+	dbus_g_thread_init();
+	
+	dbus_error_init( &error );
+	priv->conn = conn = dbus_bus_get( DBUS_BUS_SESSION, &error );
+	if( conn ) {
+		dbus_connection_setup_with_g_main( conn, NULL );
+
+		if( dbus_bus_service_exists( conn, "org.screem", 
+					&error ) ) {
+			message = dbus_message_new_method_call(
+					"org.screem",
+					"/org/screem/app",
+					"org.screem.app",
+					"Open" );
+			if( start_files ) {
+				while( *start_files ) {
+					dbus_message_append_iter_init( message,
+							&it );
+					dbus_message_iter_append_string( &it,
+							*start_files );
+					start_files ++;
+				}
+
+				dbus_connection_send( conn, message, 0 );
+				dbus_message_unref( message );
+			}
+			while( g_main_context_iteration( NULL, FALSE ) ) {}
+			gtk_main_quit();
+			return;
+
+		} else {
+			if( dbus_bus_acquire_service( conn, "org.screem",
+				0, &error ) != -1 ) {
+				/* acquired, listen in for 
+				 * org.screem stuff */
+				dbus_bus_add_match( conn,
+						"type='method_call',interface='org.screem.app'", NULL );
+				dbus_connection_add_filter( conn,
+					screem_application_dbus_filter,
+					application, NULL );
+			} else {
+				g_warning( "Can't acquire service:  %s: %s'", error.name, error.message);
+			}
+		}
+	}
+#endif
+
+	if( start_files ) {
+		while( *start_files ) {
+			priv->start_files = g_slist_prepend( priv->start_files,
+					g_strdup( *start_files ) );
+			start_files ++;
+		}
+	}
+	
+	gdk_threads_enter();
 
 	application->client = gconf_client_get_default();
 	
 	gconf_client_add_dir( application->client, "/apps/screem",
 			      GCONF_CLIENT_PRELOAD_RECURSIVE, NULL );
-	
-	glade_path = gconf_client_get_string(application->client,
-					     "/apps/screem/general/glade_path",
-					     NULL );
-	if( ! glade_path ) {
-		glade_path = g_strconcat( GLADE_PATH, "/screem.glade",
-					  NULL );
-		gconf_client_set_string( application->client,
-					 "/apps/screem/general/glade_path",
-					 glade_path, NULL );
-	}
+
+	glade_path = screem_get_glade_path();
 	g_free( glade_path );
 
 	/* create splash screen */
-	splash_text = screem_application_splash( application, FALSE );
+	splash_text = screem_application_splash( application, &splash, FALSE );
+	screem_set_cursor( GTK_WIDGET( splash ), GDK_WATCH );
+	
 	SPLASH_TEXT( splash_text, _( "Initialising..." ) );
 
 #ifdef HAVE_PYTHON
@@ -386,6 +467,8 @@ gdk_threads_enter();
 					 application, NULL, NULL );
 	
 	application->hint = GTK_WIDGET( screem_hint_new() );
+	g_object_ref( G_OBJECT( application->hint ) );
+	gtk_object_sink( GTK_OBJECT( application->hint ) );
 
 	/* add icons */
 	screem_application_add_icons( application );
@@ -430,6 +513,7 @@ gdk_threads_enter();
 		screem_page_set_data( page, NULL );
 
 		screem_site_add_page( site, page );
+		g_object_unref( page );
 
 		/* create initial window */
 		SPLASH_TEXT( splash_text, _( "Creating Interface..." ) );
@@ -441,7 +525,8 @@ gdk_threads_enter();
 
 		gtk_widget_show( GTK_WIDGET( window ) );
 	}
-	screem_application_splash( application, TRUE );
+	
+	screem_application_splash( application, NULL, TRUE );
 
 
 	/* display hints if necessary */
@@ -476,58 +561,80 @@ void screem_application_close( ScreemApplication *app )
 
 static gboolean screem_application_load( ScreemApplication *application )
 {
-	const gchar **start_files = application->start_files;
+	ScreemApplicationPrivate *priv;
+	GSList *start_files;
+	gchar *sfile;
 	gchar *mime_type;
 	gchar *file;
 	ScreemWindow *window;
-	gint i;
 
-       	window = application->priv->window_list->data;
+	priv = application->priv;
+	
+	if( ! priv->window_list ) {
+		/* no window created yet, this can occur when
+		 * the Open dbus method has been called while
+		 * screem is still starting up, don't do anything,
+		 * as we add the idle handler again at the end of
+		 * screem_application_startup() */
+		return FALSE;
+	}
+       	window = priv->window_list->data;
 
+	start_files = priv->start_files;
+	
 	/* process command line files / sites */
-	for( i = 0; start_files && start_files[ i ]; i ++ ) {
+	for( start_files = priv->start_files; start_files;
+			start_files = start_files->next ) {
 		GnomeVFSURI *uri;
 		gboolean isdir;
 
-		uri = gnome_vfs_uri_new( start_files[ i ] );
+		sfile = (gchar*)start_files->data;
+		uri = gnome_vfs_uri_new( sfile );
 		
 		/* is the filename local? */
 		if( gnome_vfs_uri_is_local( uri ) ) {
 			/* yes */
 			/* HACK */
-			if( *start_files[ i ] == G_DIR_SEPARATOR ) {
+			if( *sfile == G_DIR_SEPARATOR ) {
 				file = g_strconcat( "file://", 
-						    start_files[ i ], NULL );
+						    sfile, NULL );
 			} else {
-				file = relative_to_full( start_files[ i ],
-							 NULL );
+				file = relative_to_full( sfile, NULL );
 			}
 		} else {
-			file = g_strdup( start_files[ i ] );
+			file = g_strdup( sfile );
 		}
-		isdir = screem_uri_is_dir( file );
 		gnome_vfs_uri_unref( uri );
 
+		isdir = FALSE;
+		if( uri_exists( file, NULL ) ) {
+			isdir = screem_uri_is_dir( file );
+		}
+
 		/* is it a project file or an html file? */
-		mime_type = screem_get_mime_type( file );
+		mime_type = screem_get_mime_type( file, FALSE );
 		if( isdir || 
 		    ( mime_type &&
 		      ! strcmp( mime_type, "application/x-screem" ) ) ) {
-gdk_threads_enter();
+			gdk_threads_enter();
 			screem_site_open_with_filename( window, application,
 							file );
 	      		gdk_threads_leave();
 		} else {
 			ScreemSite *site;
-gdk_threads_enter();
+			gdk_threads_enter();
 			site =screem_application_get_default_site(application);
 			screem_page_open_with_filename( site, window, file );
 	      		gdk_threads_leave();
 		}
 		g_free( mime_type );
 		g_free( file );
-	}
 
+		g_free( sfile );
+	}
+	g_slist_free( priv->start_files );
+	priv->start_files = NULL;
+	
 	return FALSE;
 }
 
@@ -542,6 +649,9 @@ void screem_application_add_site( ScreemApplication *app, ScreemSite *site )
 {
 	GList *list;
 	ScreemWindow *window;
+	ScreemWindowDetails *details;
+	const gchar *pathname;
+	EggRecentItem *item;
 	
 	/* add to list of loaded sites */
 	app->priv->loaded_sites = g_list_append( app->priv->loaded_sites, site );
@@ -552,8 +662,15 @@ void screem_application_add_site( ScreemApplication *app, ScreemSite *site )
 		   the changes via gconf */
 		if( app->priv->window_list ) {
 			window = app->priv->window_list->data;
-			gedit_recent_add( window->details->recent_sites,
-					  screem_site_get_pathname( site ) );
+			details = window->details;
+			pathname = screem_site_get_pathname( site );
+			item = egg_recent_item_new_from_uri( pathname );
+			egg_recent_item_add_group( item,
+					RECENT_SITE_GROUP );
+			egg_recent_item_set_private( item, TRUE );
+			egg_recent_model_add_full( details->recent_site_model,
+					item );
+			egg_recent_item_unref( item );
 		}
 	}
 
@@ -567,7 +684,7 @@ ScreemSite *screem_application_get_site( ScreemApplication *app,
 					 const gchar *uri )
 {
 	ScreemSite *site;
-	GList *list;
+	const GList *list;
 
 	/* loaded_sites->next as the first one is guarenteed
 	   to be the fake site */
@@ -590,7 +707,7 @@ ScreemSite *screem_application_get_site_by_name( ScreemApplication *app,
 						 const gchar *name )
 {
 	ScreemSite *site;
-	GList *list;
+	const GList *list;
 
 	/* loaded_sites->next as the first one is guarenteed
 	   to be the fake site */
@@ -617,66 +734,16 @@ ScreemSite *screem_application_get_default_site( ScreemApplication *app )
 
 void screem_application_close_site( ScreemApplication *app, ScreemSite *site )
 {
-	gint item;
-	const gchar *name;
-	gchar *verb;
 	GList *list;
 
 	list = screem_application_get_loaded_sites( app );
-	item = g_list_index( list, site );
-
 	app->priv->loaded_sites = g_list_remove( list, site );
 
-	/* remove verb for switch site menus and remove button from site
-	   sidebars */
-	name = screem_site_get_name( site );
-
-	verb = g_strconcat( "SwitchSite_", name, NULL );
-	
 	for( list = screem_application_get_window_list( app ); list;
 	     list = list->next ) {
-		ScreemWindow *window;
-		GSList *sites;
-		RBSidebarButton *button;
-		EggAction *action;
-		guint mergeid;
-		gpointer sd;
-		     
-		window = SCREEM_WINDOW( list->data );
-
-		button = NULL;
-		for( sites = window->details->sitebargroup; sites;
-		     sites = sites->next ) {
-			button = RB_SIDEBAR_BUTTON( sites->data );
-			if( ! strcmp( name, button->unique_id ) ) {
-				break;
-			}
-			button = NULL;
-		}
-
-		if( button ) {
-			g_assert( site ==
-				  g_object_get_data( G_OBJECT( button ),
-						     "site" ) );
-			window->details->sitebargroup = 
-				g_slist_remove( window->details->sitebargroup,
-						button );
-			rb_sidebar_remove(RB_SIDEBAR(window->details->sitebar),
-					  button);
-		}
-
-		action = egg_action_group_get_action( EGG_ACTION_GROUP( window->action_group ),
-						      verb );
-		mergeid = GPOINTER_TO_UINT( g_object_get_data( G_OBJECT( action ),
-								"mergeid" ) );
-		sd = g_object_get_data( G_OBJECT( action ), "sitedata" );
-		egg_action_group_remove_action( EGG_ACTION_GROUP( window->action_group ),
-						action );
-		egg_menu_merge_remove_ui( EGG_MENU_MERGE( window->merge ),
-					  mergeid );
-		g_free( sd );
+		screem_window_remove_site( SCREEM_WINDOW( list->data ),
+				site );
 	}
-	g_free( verb );
 }
 
 void screem_application_close_all_sites( ScreemApplication *app )
@@ -808,9 +875,9 @@ GtkSourceLanguagesManager* screem_application_load_syntax_tables()
 static void screem_application_add_icons( ScreemApplication *application )
 {
 	static struct ScreemStockPixmap {
-		guchar const *stock_id;
-		guchar const *filename;
-		guchar const *ext;
+		gchar const *stock_id;
+		gchar const *filename;
+		gchar const *ext;
 	} const icons[] = {
 		{ "Screem_Button", "button", ".xpm" },
 		{ "Screem_Caption", "caption", ".xpm" },
@@ -818,6 +885,7 @@ static void screem_application_add_icons( ScreemApplication *application )
 		{ "Screem_Entry", "entry", ".xpm" },
 		{ "Screem_FileEntry", "gnome-fileentry", ".xpm" },
 		{ "Screem_Image", "stock_insert_image", ".png" },
+		{ "Screem_Object", "stock_insert_object", ".png" },
 		{ "Screem_Entity", "insert-symbol", ".png" },
 		{ "Screem_Link", "add-link", ".png" },
 		{ "Screem_Optionmenu", "optionmenu", ".xpm" },
@@ -832,9 +900,9 @@ static void screem_application_add_icons( ScreemApplication *application )
 		{ "Screem_Th", "th", ".xpm" },
 		{ "Screem_Tr", "add_row", ".png" },
 
-		{ "Screem_Browser1", "browser", ".xpm" },
-		{ "Screem_Browser2", "browser", ".xpm" },
-		{ "Screem_Browser3", "browser", ".xpm" },
+		{ "Screem_Browser1", "", "" },
+		{ "Screem_Browser2", "", "" },
+		{ "Screem_Browser3", "", "" },
 
 		{ "Screem_Site", "screem_site", ".png" },
 		{ "Screem_Preview", "site_preview", ".png" },
@@ -863,6 +931,7 @@ static void screem_application_add_icons( ScreemApplication *application )
 		{ "Screem_Entry", "text entry", 0, 0, 0 },
 		{ "Screem_FileEntry", "file entry", 0, 0, 0 },
 		{ "Screem_Image", "<img>", 0, 0, 0 },
+		{ "Screem_Object", "<object>", 0, 0, 0 },
 		{ "Screem_Entity", "Entity", 0, 0, 0 },
 		{ "Screem_Link", "<a>", 0, 0, 0 },
 		{ "Screem_Optionmenu", "<option>", 0, 0, 0 },
@@ -932,8 +1001,6 @@ static void screem_application_add_icons( ScreemApplication *application )
 			if( pixbuf ) {
 				src = gtk_icon_source_new();
 				gtk_icon_source_set_pixbuf( src, pixbuf );
-				
-
 				if( ! wildcarded ) {
 					wildcarded = TRUE;
 				} else {
@@ -1136,152 +1203,142 @@ ScreemDTDDB *screem_application_get_dtd_db( ScreemApplication *application )
 	return application->priv->dtddb;
 }
 
-
-static void screem_application_load_helpers( ScreemApplication *application )
+void screem_application_set_cursor( ScreemApplication *app,
+		GdkCursorType type )
 {
-	GSList *helper_list;
-	GSList *list;
-	gchar *base_path;
+	GList *windows;
 
-	helper_list = gconf_client_get_list( application->client,
-					     "/apps/screem/helpers",
-					     GCONF_VALUE_STRING, NULL );
-	base_path = screem_helper_get_gconf_path( NULL );
-	for( list = helper_list; list; list = list->next ) {
-		ScreemHelper *helper;
-		HelperInputMode imode;
-		HelperOutputMode omode;
-		
-		gchar *name = NULL;
-		gchar *pathname = NULL;
-		gchar *iconpath = NULL;
-
-		GSList *data_list;
-		GSList *dlist;
-	    
-		name = g_strconcat( base_path, list->data, NULL );
-		
-		data_list = gconf_client_get_list( application->client,
-						   name,
-						   GCONF_VALUE_STRING,
-						   NULL );
-		g_free( name );
-		dlist = data_list;
-
-
-		imode = SCREEM_HELPER_STDIN_SELECTION;
-		omode = SCREEM_HELPER_STDOUT_SELECTION;
-		
-		if( data_list ) {
-			name = data_list->data;
-			data_list = data_list->next;
-		}
-		if( data_list ) {
-			pathname = data_list->data;
-			data_list = data_list->next;
-		}
-		if( data_list ) {
-			iconpath = data_list->data;
-			data_list = data_list->next;
-		}
-		if( data_list ) {
-			imode = atoi( data_list->data );
-			data_list = data_list->next;
-		}
-		if( data_list ) {
-			omode = atoi( data_list->data );
-			data_list = data_list->next;
-		}
-		if( name && pathname ) {
-			helper = screem_helper_new( name, pathname,
-						    iconpath, imode, omode );
-			screem_application_add_helper( application,
-						       helper );
-		}
-		g_slist_foreach( dlist, (GFunc)g_free, NULL );
-		g_slist_free( dlist );
-	}
-	g_free( base_path );
-	if( helper_list ) {
-		g_slist_foreach( helper_list, (GFunc)g_free, NULL );
-		g_slist_free( helper_list );
+	for( windows = screem_application_get_window_list( app );
+			windows; windows = windows->next ) {
+		screem_set_cursor( GTK_WIDGET( windows->data ),
+				type );
 	}
 }
 
-static void screem_application_helper_failed( ScreemHelper *helper,
-					      ScreemApplication *application )
+void screem_application_file_op( GnomeVFSMonitorEventType type,
+				const gchar *uri, gpointer data )
 {
-	GtkWidget *widget;
-	const gchar *name;
+	ScreemApplication *app;
+	ScreemFileBrowser *browser;
+	gchar *dirname;
+	GList *list;
+	
+	dirname = g_path_get_dirname( uri );
+	app = SCREEM_APPLICATION( data );
+	
+	for( list = screem_application_get_loaded_sites( app );
+			list; list = list->next ) {
+		g_object_get( G_OBJECT( list->data ), "browser", &browser,
+				NULL );
+		screem_file_browser_file_mod( browser, dirname, uri, type );
+		g_object_unref( browser );
+	}
+	g_free( dirname );
+}
+
+ScreemSession *screem_application_get_session( ScreemApplication *app )
+{
+	g_return_val_if_fail( SCREEM_IS_APPLICATION( app ), NULL );
+
+	return app->priv->session;
+}
+
+GtkTreeModel *screem_application_get_encodings( ScreemApplication *app )
+{
+	g_return_val_if_fail( SCREEM_IS_APPLICATION( app ), NULL );
+
+	return app->priv->encodings;
+}
+
+static void screem_application_helper_failed( ScreemHelper *helper,
+		const gchar *msg, ScreemApplication *application )
+{
+	gchar *name;
+	gchar *primary;
 
 	g_object_get( G_OBJECT( helper ), "name", &name, NULL );
+		
+	primary = g_strdup_printf( _( "There was an error running the %s helper" ), name );
 
-	widget = gtk_message_dialog_new( NULL, 
-					 GTK_DIALOG_MODAL,
-					 GTK_MESSAGE_ERROR,
-					 GTK_BUTTONS_CLOSE,
-					 _( "Helper %s encountered a fatal error"),
-					 name );
-	gtk_window_set_wmclass( GTK_WINDOW( widget ),
-				"Screem", "helper_failed" );
-	gtk_dialog_run( GTK_DIALOG( widget ) );
-	gtk_widget_destroy( widget );
+	screem_hig_alert( GTK_STOCK_DIALOG_ERROR,
+			primary, msg, NULL );
+	g_free( primary );
+	g_free( name );
 }
 
 static void screem_application_helper_error( ScreemHelper *helper,
 					     gint status,
 					     ScreemApplication *application )
 {
+	gchar *name;
+	gchar *primary;
+	const gchar *secondary;
 
-	GtkWidget *widget;
-	const gchar *name;
+	if( status != 0 ) {
+		g_object_get( G_OBJECT( helper ), "name", &name, NULL );
+	
+		primary = g_strdup_printf( "The %s helper returned errors", 
+				name );
+		secondary = _( "The error window will contain details of these" );
+		screem_hig_alert( GTK_STOCK_DIALOG_WARNING,
+				primary, secondary, NULL );
+		g_free( primary );
+		g_free( name );
+	}
+}
 
-	g_object_get( G_OBJECT( helper ), "name", &name, NULL );
+static gint helper_filter( const GnomeVFSFileInfo *info )
+{
+	return strcmp( "application/x-desktop", info->mime_type );
+}
 
-	widget = gtk_message_dialog_new( NULL, 
-					 GTK_DIALOG_MODAL,
-					 GTK_MESSAGE_ERROR,
-					 GTK_BUTTONS_CLOSE,
-					 _( "Helper %s returned errors"),
-					 name );
-	gtk_window_set_wmclass( GTK_WINDOW( widget ),
-				"Screem", "helper_error" );
-	gtk_dialog_run( GTK_DIALOG( widget ) );
-	gtk_widget_destroy( widget );
+static void screem_application_load_helpers( ScreemApplication *app )
+{
+	gchar *path;
+	gchar *tmp;
+	GSList *helpers;
+	GSList *list;
+	ScreemHelper *helper;
+	
+	tmp = screem_get_dot_dir();
+	path = g_build_path( G_DIR_SEPARATOR_S, tmp,
+			"helpers", NULL );
+	g_free( tmp );
+
+	if( mkdir_recursive( path, 
+			GNOME_VFS_PERM_USER_ALL, 
+			screem_application_file_op, app ) ) {
+		helpers = screem_vfs_scandir( path,
+				helper_filter,
+				(GCompareFunc)strcmp, FALSE );
+		for( list = helpers; list; list = list->next ) {
+			helper = screem_helper_new_from_file( list->data );
+			if( helper ) {
+				screem_application_add_helper( app,
+						helper );
+			}
+			g_free( list->data );
+		}
+		g_slist_free( helpers );
+	}
+
+	g_free( path );
 }
 
 void screem_application_add_helper( ScreemApplication *application,
 				    ScreemHelper *helper )
 {
-	gchar *name;
-	gchar *path;
-	gchar *icon;
-	HelperInputMode imode;
-	HelperOutputMode omode;
-	
-	gchar *base_path;
-	GSList *list;
-	GSList *helpers;
-	GSList *dlist;
-	GSList *data_list;
 	GList *windows;
-
-	GConfClient *client;
+	ScreemWindow *window;
 
 	/* add to all open windows */
 	for( windows = application->priv->window_list; windows;
 	     windows = windows->next ) {
-		ScreemWindow *window;
-		
 		window = SCREEM_WINDOW( windows->data );
 		screem_helper_add_to_window( helper, window );
 	}
-
-	client = application->client;
-
-	base_path = screem_helper_get_gconf_path( helper );
-
-	application->helpers = g_list_append( application->helpers, helper );
+	application->helpers = g_list_append( application->helpers, 
+			helper );
 
 	/* hookup callbacks */
 	g_signal_connect( G_OBJECT( helper ), "helper_failed",
@@ -1290,141 +1347,102 @@ void screem_application_add_helper( ScreemApplication *application,
 	g_signal_connect( G_OBJECT( helper ), "helper_error",
 			  G_CALLBACK( screem_application_helper_error ),
 			  application );
-
-	g_object_get( G_OBJECT( helper ), 
-		      "name", &name, "pathname", &path,
-		      "iconpath", &icon, "input", &imode, "output", &omode, NULL );
-
-	/* store via gconf */
-	client = application->client;
-	helpers = gconf_client_get_list( client, "/apps/screem/helpers",
-					 GCONF_VALUE_STRING, NULL );
-
-	/* we don't want to add one we already have so check for it,
-	   based on name */
-	for( list = helpers; list; list = list->next ) {
-		const gchar *hname = (const gchar *)list->data;
-		if( hname && ! strcmp( name, hname ) ) {
-			/* found it */
-			break;
-		}
-	}
-	/* add to the list / update the list */
-	if( list ) {
-		gconf_client_unset( client, base_path, NULL );
-	} else {
-		helpers = g_slist_append( helpers, g_strdup( name ) );
-	}
-
-
-	dlist = g_slist_prepend( NULL, g_strdup( name ) );
-	dlist = g_slist_prepend( dlist, g_strdup( path ) );
-	dlist = g_slist_prepend( dlist, g_strdup( icon ) );
-	dlist = g_slist_prepend( dlist, g_strdup_printf( "%i", imode ) );
-	dlist = g_slist_prepend( dlist, g_strdup_printf( "%i", omode ) );
-	
-	data_list = g_slist_reverse( dlist );
-
-	/* store helper */
-	gconf_client_set_list( client, base_path, GCONF_VALUE_STRING,
-			       data_list, NULL );
-
-	/* store in list of helper names */
-	gconf_client_set_list( client, "/apps/screem/helpers", 
-			       GCONF_VALUE_STRING, helpers, NULL );
-
-	gconf_client_suggest_sync( client, NULL );
-
-	/* clean up */
-	if( helpers ) {
-		g_slist_foreach( helpers, (GFunc)g_free, NULL );
-		g_slist_free( helpers );
-	}
-	if( data_list ) {
-		g_slist_foreach( data_list, (GFunc)g_free, NULL );
-		g_slist_free( data_list );
-	}
-
-	g_free( icon );
-	g_free( path );
-	g_free( name );
-	
-	g_free( base_path );
 }
 
-void screem_application_remove_helper( ScreemApplication *application,
+void screem_application_remove_helper( ScreemApplication *app,
 				       ScreemHelper *helper )
 {
 	GList *hlist;
-	const gchar *name;
+	gchar *name;
+	gchar *dname;
+	ScreemWindow *window;
+	GList *windows;
 
+	gchar *path;
+	gchar *tmp;
+	
 	g_object_get( G_OBJECT( helper ), "name", &name, NULL );
 
-	/* first try and find a matching helper in application->helpers */
-	for( hlist = application->helpers; hlist; hlist = hlist->next ) {
-		const gchar *dname;
+	/* first try and find a matching helper in 
+	   app->helpers */
+	for( hlist = app->helpers; hlist; hlist = hlist->next ) {
 
-		g_object_get( G_OBJECT( hlist->data ), "name", &dname, NULL );
+		g_object_get( G_OBJECT( hlist->data ), 
+				"name", &dname, NULL );
 
 		if( ! strcmp( name, dname ) ) {
+			g_free( dname );
 			break;
 		}
+		g_free( dname );
 	}
+	tmp = screem_get_dot_dir();
+	path = g_strconcat( tmp, G_DIR_SEPARATOR_S,
+			"helpers", G_DIR_SEPARATOR_S,
+			name, ".desktop", NULL );
+	g_free( tmp );
+	g_free( name );
+	
 	if( hlist ) {
-		GSList *helpers;
-		GSList *list;
-		GList *windows;
-
 		/* found it */
 		helper = SCREEM_HELPER( hlist->data );
-		application->helpers = g_list_remove( application->helpers,
-						      hlist->data );
-
+		app->helpers = g_list_remove( app->helpers,
+				hlist->data );
 
 		/* remove from all open windows */
-		for( windows = application->priv->window_list; windows;
+		for( windows = app->priv->window_list; windows;
 		     windows = windows->next ) {
-			ScreemWindow *window;
-
 			window = SCREEM_WINDOW( windows->data );
-			screem_helper_remove_from_window( helper, window );
+			screem_helper_remove_from_window( helper, 
+					window );
 		}
 
-		/* now remove from gconf */
-		helpers = gconf_client_get_list( application->client, 
-						 "/apps/screem/helpers",
-						 GCONF_VALUE_STRING, NULL );
+		delete_file( path, screem_application_file_op, app );
+	}
+	g_free( path );
+}
 
-		for( list = helpers; list; list = list->next ) {
-			const gchar *hname = (const gchar *)list->data;
-			if( hname && ! strcmp( name, hname ) ) {
-				/* found it */
-				break;
-			}
+void screem_application_exec_helpers( ScreemApplication *app,
+		ScreemPage *page )
+{
+	const gchar *type;
+	ScreemWindow *window;
+	GList *windows;
+	GList *docs;
+	GList *helpers;
+	ScreemHelper *helper;
+	gboolean exec;
+	gchar *etype;
+
+	type = screem_page_get_mime_type( page );
+
+	/* we need a window to output into, use the first one
+	 * we find the page open in, or NULL otherwise, bit of
+	 * a messy thing to do though */
+	window = NULL;
+	for( windows = app->priv->window_list; windows;
+			windows = windows->next ) {
+		window = SCREEM_WINDOW( windows->data );
+
+		docs = screem_window_get_documents( window );
+		if( g_list_find( docs, page ) ) {
+			break;
 		}
-		if( list ) {
-			gchar *base_path;
+		window = NULL;
+	}
 	
-			helpers = g_slist_remove( helpers, list->data );
-			g_free( list->data );
-
-			gconf_client_set_list( application->client,
-					       "/apps/screem/helpers",
-					       GCONF_VALUE_STRING,
-					       helpers, NULL );
-
-			/* now remove from helpers_data */
-			base_path = screem_helper_get_gconf_path( helper );
-			
-			gconf_client_unset( application->client,
-					    base_path, NULL );
-
-			gconf_client_suggest_sync( application->client, NULL );
-
-			g_free( base_path );
+	for( helpers = app->helpers; helpers; helpers = helpers->next ) {
+		helper = SCREEM_HELPER( helpers->data );
+		g_object_get( G_OBJECT( helper ),
+				"exec_on_save", &exec,
+				"exec_type", &etype,
+				NULL );
+		if( exec &&
+			g_pattern_match_simple( etype, type ) ) {
+			screem_helper_execute( helper, page,
+					window );
 		}
-		/* clear up helpers list */
-		g_slist_free( helpers );
+		g_free( etype );
 	}
 }
 
@@ -1483,6 +1501,43 @@ static gboolean screem_application_free_features( gpointer key,
 	return FALSE;
 }
 
+#ifdef HAVE_DBUS
+static DBusHandlerResult screem_application_dbus_filter( DBusConnection *conn,
+		DBusMessage *message,
+		gpointer data )
+{
+	ScreemApplication *app;
+	ScreemApplicationPrivate *priv;
+	DBusMessageIter it;
+	gchar *uri;
+	DBusHandlerResult res;
+	gboolean add;
+	
+	app = SCREEM_APPLICATION( data );
+	priv = app->priv;
+	res = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+	
+	if( dbus_message_is_method_call( message, "org.screem.app",
+				"Open" ) ) {
+		dbus_message_iter_init( message, &it );
+		
+		add = ( ! priv->start_files ); 
+		while( ( uri = dbus_message_iter_get_string( &it ) ) ) {
+			priv->start_files = g_slist_prepend( priv->start_files, uri );	
+			if( ! dbus_message_iter_next( &it ) ) {
+				break;
+			}
+		}
+		if( add ) {
+			g_idle_add( (GSourceFunc)screem_application_load,
+					app );
+		}
+		res = DBUS_HANDLER_RESULT_HANDLED;
+	}
+	return res;
+}
+#endif
+
 /* G Object stuff */
 
 #define PARENT_TYPE G_TYPE_OBJECT
@@ -1501,15 +1556,38 @@ static void screem_application_class_init( ScreemApplicationClass *klass )
 
 static void screem_application_init( ScreemApplication *application )
 {
-	/* we properly init later by calling screem_application_startup */
+	ScreemApplicationPrivate *priv;
+	gint i;
+	GtkTreeIter it;
+	const gchar *trans;
+	gchar *tmp;
 
-	application->priv = g_new0( ScreemApplicationPrivate, 1 );
-	application->priv->session = screem_session_new( application );
+	/* we properly init later by calling screem_application_startup */
+	priv = application->priv = g_new0( ScreemApplicationPrivate, 1 );
+	priv->session = screem_session_new( G_OBJECT( application ) );
+
+	priv->encodings = GTK_TREE_MODEL( gtk_list_store_new( 2, 
+				G_TYPE_STRING, G_TYPE_STRING ) );
+
+	for( i = 0; i < screem_n_encodings; ++ i ) {
+		gtk_list_store_append( GTK_LIST_STORE( priv->encodings ), &it );
+		trans = gettext( screem_encodings[ i ].name );
+		tmp = g_strconcat( trans, " (",
+				screem_encodings[ i ].charset, ")", 
+				NULL );
+		gtk_list_store_set( GTK_LIST_STORE( priv->encodings ),
+				&it,
+				0, tmp,
+				1, screem_encodings[ i ].charset,
+				-1 );
+		g_free( tmp );
+	}
 }
 
 static void screem_application_finalize( GObject *object )
 {
 	ScreemApplication *application;
+	ScreemApplicationPrivate *priv;
 	GtkSourceLanguagesManager *lm;
 	GHashTable *features;
 	GList *list;
@@ -1518,18 +1596,38 @@ static void screem_application_finalize( GObject *object )
 	ScreemHelper *helper;
 	
 	application = SCREEM_APPLICATION( object );
-
+	priv = application->priv;
+#ifdef HAVE_DBUS
+	if( priv->conn ) {
+		dbus_connection_disconnect( priv->conn );
+	}
+#endif
+	if( priv->start_files ) {
+		g_slist_foreach( priv->start_files, (GFunc)g_free, NULL );
+		g_slist_free( priv->start_files );
+		priv->start_files = NULL;
+	}
+	
 	screem_application_close_all_windows( application );
 
 	screem_application_close_all_sites( application );
 
-	gconf_client_notify_remove( application->client, application->offline_notify );
+	if( application->offline_notify ) {
+		gconf_client_notify_remove( application->client, 
+				application->offline_notify );
+	}
 
-	g_object_unref( application->priv->dtddb );
+	if( priv->dtddb ) {
+		g_object_unref( priv->dtddb );
+	}
 
-	g_object_unref( application->client );
+	if( application->client ) {
+		g_object_unref( application->client );
+	}
 
-	g_object_unref( application->priv->session );
+	if( priv->session ) {
+		g_object_unref( priv->session );
+	}
 
 	lm = screem_application_load_syntax_tables();
 	features = g_object_get_data( G_OBJECT( lm ), "features" );
@@ -1541,13 +1639,19 @@ static void screem_application_finalize( GObject *object )
 		g_object_unref( G_OBJECT( lm ) );
 	}
 
+	if( priv->encodings ) {
+		g_object_unref( priv->encodings );
+	}
+	
 	for( list = application->plugins; list; list = list->next ) {
 		plugin = (ScreemPlugin*)list->data;
 		
 		g_module_close( plugin->module );
 		g_free( plugin );
 	}
-	g_list_free( application->plugins );
+	if( application->plugins ) {
+		g_list_free( application->plugins );
+	}
 
 	for( list = application->macros; list; list = list->next ) {
 		macro = (ScreemEditorMacro*)list->data;
@@ -1556,22 +1660,35 @@ static void screem_application_finalize( GObject *object )
 		g_free( macro->text );
 		g_free( macro );
 	}
-	g_list_free( application->macros );
+	if( application->macros ) {
+		g_list_free( application->macros );
+	}
 	
 	for( list = application->helpers; list; list = list->next ) {
 		helper = (ScreemHelper*)list->data;	
 	
 		g_object_unref( helper );
 	}
-	g_list_free( application->helpers );
-	
-	g_free( application->priv );
+	if( application->helpers ) {
+		g_list_free( application->helpers );
+	}
+
+	if( application->hint ) {
+		g_object_unref( application->hint );
+		gtk_widget_destroy( application->hint );
+	}
+
+	g_free( priv );
+
+	screem_get_file_filter( NULL );
 
 #ifdef HAVE_PYTHON
 	Py_Finalize();
 #endif
 	
 	G_OBJECT_CLASS( parent_class )->finalize( object );
+
+	screem_file_browser_cleanup();
 }
 
 GType screem_application_get_type()
