@@ -54,8 +54,6 @@ static void screem_dtd_load_catalog( ScreemDTDDB *db,
 				     const GString *pathname );
 static void screem_dtd_load_catalogs( ScreemDTDDB *db );
 static gint screem_dtd_db_sort_compare( const gchar *a, const gchar *b);
-static void screem_dtd_db_fill_list( ScreemDTDDB *db,
-				GtkListStore *store );
 static void dtds_insert( gpointer key, gpointer value, GObject *widget);
 static void screem_dtd_db_offline_notify( GConfClient *client,
 					guint cnxn_id,
@@ -71,9 +69,6 @@ static gchar *screem_dtd_db_override_get_system_id( ScreemDTDDB *db,
 static ScreemDTD *screem_dtd_db_new_dtd( ScreemDTDDB *db,
 					 const gchar *publicid,
 					 const gchar *systemid );
-static gchar *screem_dtd_db_resolve_entity( void *db,
-					  const gchar *publicid,
-					  const gchar *systemid );
 static gboolean screem_dtd_db_load_internal( ScreemDTDDB *db,
 					const gchar *publicid,
 					const gchar *systemid );
@@ -133,8 +128,11 @@ static void screem_dtd_db_instance_init( ScreemDTDDB *dtd_db )
 	priv->overridepath = g_build_filename( tmp, 
 				"dtd_override.xml", NULL );
 	g_free( tmp );
-	
-	priv->overrides = xmlParseFile( priv->overridepath );
+
+	priv->overrides = NULL;
+	if( uri_exists( priv->overridepath, NULL ) ) {
+		priv->overrides = xmlParseFile( priv->overridepath );
+	}
 	if( ! priv->overrides ) {
 		/* no overrides file, create a new doc */
 		priv->overrides = xmlNewDoc( XML_DEFAULT_VERSION );
@@ -148,8 +146,9 @@ static void screem_dtd_db_instance_init( ScreemDTDDB *dtd_db )
 
 	screem_dtd_load_catalogs( dtd_db );
 
-	priv->store = gtk_list_store_new( 2, 
+	priv->store = gtk_list_store_new( 3, 
 				G_TYPE_STRING, 
+				G_TYPE_STRING,
 				G_TYPE_STRING );
 
 	screem_dtd_db_fill_list( dtd_db, priv->store );
@@ -300,7 +299,7 @@ static void screem_dtd_load_catalog( ScreemDTDDB *db,
 			tpath = gnome_vfs_escape_host_and_path_string( path );
 			if( ! g_path_is_absolute( path ) ) {
 				
-				tmp = g_dirname( pathname->str );
+				tmp = g_path_get_dirname( pathname->str );
 				temp = g_strconcat( "file://", tmp,
 						G_DIR_SEPARATOR_S,
 						tpath, NULL );
@@ -351,7 +350,8 @@ static void screem_dtd_load_catalogs( ScreemDTDDB *db )
 	g_free( tmp );
 
 	/* make dtds dir if needed */
-	mkdir_recursive( paths[ 4 ], GNOME_VFS_PERM_USER_ALL );
+	mkdir_recursive( paths[ 4 ], GNOME_VFS_PERM_USER_ALL,
+			NULL, NULL );
 
 	for( i = 0; paths[ i ]; ++ i ) {
 		if( ! ( dir = opendir( paths[ i ] ) ) ) {
@@ -414,34 +414,6 @@ static gint screem_dtd_db_sort_compare( const gchar *a, const gchar *b )
 	return ret;
 }
 
-static void screem_dtd_db_fill_list( ScreemDTDDB *db,
-				GtkListStore *store )
-{
-	GList *list;
-	GList *tmp;
-	GtkTreeIter it;
-	gchar *sysid;
-
-	gtk_list_store_clear( store );
-	g_object_set_data( G_OBJECT( store ), "list", NULL );
-	g_hash_table_foreach( db->priv->dtd_types, 
-			      (GHFunc)dtds_insert, store );
-	list = g_object_get_data( G_OBJECT( store ), "list" );
-	list = g_list_sort( list, (GCompareFunc)screem_dtd_db_sort_compare );
-	for( tmp = list; tmp; tmp = tmp->next ) {
-		sysid = screem_dtd_db_get_system_id( db,
-				tmp->data, NULL );
-		gtk_list_store_append( store, &it );
-		gtk_list_store_set( store, &it,
-				0, tmp->data,
-				1, sysid,
-				-1 );
-
-		g_free( sysid );
-	}
-	g_list_free( list );
-}
-
 static void dtds_insert( gpointer key, gpointer value, GObject *widget )
 {
 	if( ! strstr( (gchar*)key, "ENTITIES" ) ) {
@@ -581,13 +553,13 @@ static ScreemDTD *screem_dtd_db_new_dtd( ScreemDTDDB *db,
 	return dtd;
 }
 
-static gchar *screem_dtd_db_resolve_entity( void *db,
-					  const gchar *publicid,
-					  const gchar *systemid )
+gchar *screem_dtd_db_resolve_entity( void *db,
+				  const gchar *publicid,
+				  const gchar *systemid )
 {
 	ScreemDTDDB *dtddb;
 	ScreemDTDDBPrivate *priv;
-	gchar *localuri;
+	const gchar *localuri;
 	GString *file;
 	gchar *ret;
 	GnomeVFSURI *uri;
@@ -630,7 +602,7 @@ static gchar *screem_dtd_db_resolve_entity( void *db,
 	ret = NULL;
 
 	if( localuri ) {
-		file = load_file( localuri );
+		file = load_file( localuri, NULL, NULL, NULL );
 	}
 	if( file ) {
 		ret = file->str;
@@ -672,7 +644,7 @@ static gboolean screem_dtd_db_load_internal( ScreemDTDDB *db,
 	if( uri ) {
 		local = gnome_vfs_uri_is_local( uri );
 		if( ( ! priv->offline ) || local ) {
-			file = load_file( systemid );
+			file = load_file( systemid, NULL, NULL, NULL );
 		}
 		gnome_vfs_uri_unref( uri );
 	}
@@ -690,20 +662,24 @@ static gboolean screem_dtd_db_load_internal( ScreemDTDDB *db,
 		
 		if( save_file( filename, file->str,
 				GNOME_VFS_PERM_USER_READ |
-				GNOME_VFS_PERM_USER_WRITE ) ) {
+				GNOME_VFS_PERM_USER_WRITE,
+				FALSE,
+				NULL ) ) {
 			catalog = g_build_filename( path, "catalog",
 						    NULL );
 			tmp = g_strdup_printf( "PUBLIC \"%s\" \"%s\"\n",
 						publicid, sysid );
 
-			catfile = load_file( catalog );
+			catfile = load_file( catalog, NULL, NULL, NULL );
 			if( ! catfile ) {
 				catfile = g_string_new( NULL );
 			}
 			g_string_append( catfile, tmp );
 			save_file( catalog, catfile->str,
 				   GNOME_VFS_PERM_USER_READ |
-				   GNOME_VFS_PERM_USER_WRITE );
+				   GNOME_VFS_PERM_USER_WRITE, 
+				   FALSE,
+				   NULL );
 			g_string_free( catfile, TRUE );
 
 			g_free( tmp );
@@ -766,16 +742,30 @@ ScreemDTD *screem_dtd_db_get_default_dtd( ScreemDTDDB *db )
 	return dtd;
 }
 
-void screem_dtd_db_fill_combo( ScreemDTDDB *db, GtkWidget *combo )
+void screem_dtd_db_fill_list( ScreemDTDDB *db, GtkListStore *store )
 {
 	GList *list;
+	GList *tmp;
+	GtkTreeIter it;
+	gchar *sysid;
 
-	g_object_set_data( G_OBJECT( combo ), "list", NULL );
+	gtk_list_store_clear( store );
+	g_object_set_data( G_OBJECT( store ), "list", NULL );
 	g_hash_table_foreach( db->priv->dtd_types, 
-			      (GHFunc)dtds_insert, combo );
-	list = g_object_get_data( G_OBJECT( combo ), "list" );
+			      (GHFunc)dtds_insert, store );
+	list = g_object_get_data( G_OBJECT( store ), "list" );
 	list = g_list_sort( list, (GCompareFunc)screem_dtd_db_sort_compare );
-	gtk_combo_set_popdown_strings( GTK_COMBO( combo ), list );
+	for( tmp = list; tmp; tmp = tmp->next ) {
+		sysid = screem_dtd_db_get_system_id( db,
+				tmp->data, NULL );
+		gtk_list_store_append( store, &it );
+		gtk_list_store_set( store, &it,
+				0, tmp->data,
+				1, sysid,
+				-1 );
+
+		g_free( sysid );
+	}
 	g_list_free( list );
 }
 
@@ -953,9 +943,10 @@ void screem_dtd_db_parse_doctype( ScreemDTDDB *db,
 		}
 	}
 	doctype = g_utf8_skip_space( doctype );
-	if( ! strncmp( "PUBLIC ", doctype, strlen( "PUBLIC " ) ) ) {
+	if( ( ! strncmp( "PUBLIC", doctype, strlen( "PUBLIC" ) ) ) ||
+	    ( ! strncmp( "public", doctype, strlen( "public" ) ) ) ) {
 		/* public id found */
-		doctype += strlen( "PUBLIC " );
+		doctype += strlen( "PUBLIC" );
 
 		doctype = g_utf8_skip_space( doctype );
 	
@@ -1002,10 +993,11 @@ void screem_dtd_db_parse_doctype( ScreemDTDDB *db,
 			doctype = g_utf8_skip_space( doctype );
 			term = g_utf8_get_char( doctype );
 		}	
-	} else if( ! strncmp( "SYSTEM ", doctype, strlen( "SYSTEM " ) ) ) {
+	} else if( ( ! strncmp( "SYSTEM", doctype, strlen( "SYSTEM" ) ) ||
+		   ( ! strncmp( "system", doctype, strlen( "system" ) ) ) ) ) {
 		/* system id only */
 	
-		doctype += strlen( "SYSTEM " );
+		doctype += strlen( "SYSTEM" );
 
 		doctype = g_utf8_skip_space( doctype );
 	
