@@ -36,6 +36,8 @@
 
 #include "support.h"
 
+#include "fileops.h"
+
 static void screem_dtd_class_init( ScreemDTDClass *klass );
 static void screem_dtd_instance_init( ScreemDTD *dtd );
 static void screem_dtd_finalize( GObject *object );
@@ -67,6 +69,9 @@ struct ScreemDTDPrivate {
 	GCompareFunc compare;
 	GHashFunc hfunc;
 	GEqualFunc efunc;
+
+	gchar *rooturi;
+	const gchar *toproot;
 };
 
 struct ScreemDTDEntity {
@@ -121,6 +126,11 @@ struct ScreemDTDAllowed {
 	guint ref;
 };
 
+typedef struct {
+	ScreemDTD *dtd;
+	GSList *list;
+} ScreemDTDAllowedCopy;
+
 typedef enum {
 	SCREEM_DTD_NO_TYPE = 0,
 	SCREEM_DTD_IMPLIED,
@@ -186,8 +196,7 @@ static gboolean screem_dtd_free_attribute_table( gpointer key,
 static void screem_dtd_free_attribute( ScreemDTDAttribute *attr );
 
 static gint screem_dtd_sort_element_list( gconstpointer a,
-					gconstpointer b,
-					gpointer data );
+					gconstpointer b );
 static gint screem_dtd_sort_attr_list( gconstpointer a,
 					gconstpointer b,
 					gpointer data );
@@ -259,6 +268,9 @@ static void screem_dtd_finalize( GObject *object )
 	/* free attributes */
 	g_hash_table_destroy( priv->attlists );
 
+	g_free( priv->root );
+	g_free( priv->rooturi );
+	
 	g_free( priv );	
 
 	GNOME_CALL_PARENT( G_OBJECT_CLASS, finalize, (object) );
@@ -566,6 +578,7 @@ static const gchar *screem_dtd_read_entity( const gchar *data,
 		     g_unichar_type( c )  == G_UNICODE_COMBINING_MARK )  {
 			g_string_append_unichar( str, c );
 		} else {
+			data = g_utf8_prev_char( data );
 			break;
 		}
 		data = g_utf8_next_char( data );
@@ -590,6 +603,7 @@ static const gchar *screem_dtd_parse_entity( ScreemDTD *dtd,
 	GString *name;
 	GString *value;
 	gunichar c;
+	gunichar eterm;
 	gunichar term;
 	gchar *desc;
 	GUnicodeType type;
@@ -637,22 +651,23 @@ static const gchar *screem_dtd_parse_entity( ScreemDTD *dtd,
 	
 	/* data should now point at the entity value */
 	value = g_string_new( NULL );
-	term = g_utf8_get_char( data );
-	if( term != '"' && term != '\'' ) {
-		term = ' ';
+	eterm = g_utf8_get_char( data );
+	if( eterm != '"' && eterm != '\'' ) {
+		eterm = ' ';
 	} else {
 		data = g_utf8_next_char( data );
 	}
 	while( *data != '\0' ) {
 		c = g_utf8_get_char( data );
-		if( c == term || ( term == ' ' && g_unichar_isspace( c ) ) ) {
+		if( c == eterm || ( eterm == ' ' && g_unichar_isspace( c ) ) ) {
 			break;
 		}
 		g_string_append_unichar( value, c );
 		data = g_utf8_next_char( data );
 	}
 
-	if( value->str && ! strcmp( "PUBLIC", value->str ) ) {
+	if( eterm == ' ' &&
+	    value->str && ! strcmp( "PUBLIC", value->str ) ) {
 		/* read public id */
 		g_string_append_c( value, ' ' );
 		data = g_utf8_skip_space( data );
@@ -676,9 +691,24 @@ static const gchar *screem_dtd_parse_entity( ScreemDTD *dtd,
 			data = g_utf8_next_char( data );
 			data = g_utf8_skip_space( data );
 		}
+		c = g_utf8_get_char( data );
+		if( c == '-' && *( data + 1 ) == '-' ) {
+			/* local copy of html 4.0 trans, at least
+			 * under debian comments out the system
+			 * id for the entities, so we should
+			 * handle comments here */
+			data = screem_dtd_bypass_comment( data, 
+							 NULL );	
+			c = g_utf8_get_char( data );
+		}
+		/* prevent attempt to read non existant SYSTEM uri */
+		if( c == '>' || c == '\0' ) {
+			eterm = '\0';
+		}
 	}
-	if( value->str && ( ! strcmp( "PUBLIC", value->str ) ||
-			! strcmp( "SYSTEM", value->str ) ) ) {
+	if( eterm == ' ' && value->str && 
+	    ( ! strncmp( "PUBLIC", value->str, strlen( "PUBLIC" ) ) ||
+		! strcmp( "SYSTEM", value->str ) ) ) {
 		/* read system id */
 		g_string_append_c( value, ' ' );
 		data = g_utf8_skip_space( data );
@@ -714,7 +744,7 @@ static const gchar *screem_dtd_parse_entity( ScreemDTD *dtd,
 	}
 
 	entity = NULL;
-	if( name->str && value->str ) {
+	if( name->str && value->str ) {		
 		if( ! g_hash_table_lookup( priv->entities,
 					   name->str ) ) {
 			entity = g_new0( ScreemDTDEntity, 1 );
@@ -756,10 +786,19 @@ static const gchar *screem_dtd_parse_element( ScreemDTD *dtd,
 	ScreemDTDGroup *tmp;
 	ScreemDTDElement *element;
 
+	ScreemDTDAllowedCopy copy;
 	GSList *allowedlist;
 	
+	const gchar *ndata;
+	const gchar *backtrack;
+	gchar *name;
+	gchar *val;
+	
 	g_return_val_if_fail( SCREEM_IS_DTD( dtd ), data );
 	
+	copy.dtd = dtd;
+	copy.list = NULL;
+	
 	priv = dtd->priv;
 	
 	/* check this is an element */
@@ -787,8 +826,25 @@ static const gchar *screem_dtd_parse_element( ScreemDTD *dtd,
 
 	data = g_utf8_skip_space( data );
 
-	/* check for open/close options */
+	/* check for open/close options,
+	 *
+	 * some DTDs, TEI 2 for example use an entity for this,
+	 * grrrrrr, how annoying.
+	 *
+	 */
 	offset = 0;
+	ndata = backtrack = val = NULL;
+	if( *data == '%' ) {
+		backtrack = data;
+		ndata = screem_dtd_read_entity( data, &name );
+		ndata = g_utf8_next_char( ndata );
+		val = screem_dtd_lookup_entity( dtd, name, FALSE );
+		g_free( name );
+		if( val ) {
+			data = val;
+		}
+	}
+	
 	if( ! strncmp( "O O", data, strlen( "O O" ) ) ) {
 		/* open / close optional */
 		ostate = cstate = SCREEM_DTD_SHOULD;
@@ -806,11 +862,26 @@ static const gchar *screem_dtd_parse_element( ScreemDTD *dtd,
 		/* must close */
 		ostate = cstate = SCREEM_DTD_MUST;
 		offset += strlen( "- -" );
+	} else if( ! strncmp( "EMPTY", data, strlen( "EMPTY" ) ) ) {
+		ostate = SCREEM_DTD_MUST;
+		cstate = SCREEM_DTD_MUST_NOT;
 	} else {
 		/* default to must close */
 		ostate = cstate = SCREEM_DTD_MUST;
+
+		if( ndata ) {
+			/* entity didn't expand to open/close
+			 * options, backtrack */
+			data = backtrack;	
+			ndata = NULL;
+		}
+	}
+	if( ndata ) {
+		data = ndata;
+		g_free( val );
+	} else {
+		data += offset;
 	}
-	data += offset;
 
 	data = g_utf8_skip_space( data );
 
@@ -917,11 +988,10 @@ static const gchar *screem_dtd_parse_element( ScreemDTD *dtd,
 		screem_dtd_group_free( include, NULL );
 	}
 	/* create elements */
-	allowedlist = NULL;
 	g_hash_table_foreach( table,
 			(GHFunc)screem_dtd_copy_allowed_from_table,
-			&allowedlist );
-	allowedlist = g_slist_sort( allowedlist, priv->compare );
+			&copy );
+	allowedlist = copy.list;
 	
 	for( tmp = name_group; tmp; tmp = tmp->siblings ) {
 		if( tmp->name ) {
@@ -952,7 +1022,7 @@ static const gchar *screem_dtd_parse_element( ScreemDTD *dtd,
 							efunc ); 
 			g_hash_table_insert( priv->elements, 
 					element->name, element );
-			priv->elementlist = g_slist_prepend( priv->elementlist, element );
+			priv->elementlist = g_slist_insert_sorted( priv->elementlist, element, screem_dtd_sort_element_list );
 		}
 	}
 	
@@ -1025,15 +1095,37 @@ static const gchar *screem_dtd_parse_attlist_attrs( ScreemDTD *dtd,
 			/* lookup entity name */
 			val = screem_dtd_lookup_entity( dtd, ename, 
 							FALSE );
-			if( val ) {
+			if( val && *val != '\0' ) {
+				if( str->len > 0 ) {
+					/* we already have part of
+					 * a name */
+					tmp = val;
+					val = g_strconcat( str->str,
+							tmp, NULL );
+					g_free( tmp );
+					g_string_assign( str, "" );
+				}
+					
 				/* parse val */
 				screem_dtd_parse_attlist_attrs( dtd,
 								val,
 								attrs );
-				g_free( val );
 				alist = *attrs;
-				attr = alist->data;
+				if( alist ) {
+					attr = alist->data;
+				}
+				/* it may have a name in attr now,
+				 * however it may just be a prefix
+				 * to the fullname, so we will
+				 * treat it as such */
+				if( attr && attr->name ) {
+					g_string_assign( str,
+							attr->name );
+					g_free( attr->name );
+					attr->name = NULL;
+				}
 			}
+			g_free( val );
 			g_free( ename );
 		} else if( ! attr->name ) {
 			/* we are parsing the attribute name */
@@ -1060,6 +1152,14 @@ static const gchar *screem_dtd_parse_attlist_attrs( ScreemDTD *dtd,
 				  ! strncmp( "%URI.datatype;", data, 
 					     strlen( "%URI.datatype;" ) ) );
 
+			/* some doctypes may not declare a URI type,
+			 * look at the attribute name and guess
+			 * based on that */
+			if( ! attr->isuri ) {
+				attr->isuri = ( ! strcmp( "url",
+							attr->name ) );
+			}
+			
 			data = screem_dtd_name_parse( dtd, data, 
 							&group );
 			flat = NULL;
@@ -1067,6 +1167,7 @@ static const gchar *screem_dtd_parse_attlist_attrs( ScreemDTD *dtd,
 				flat = screem_dtd_group_flatten( group,
 								NULL );
 				screem_dtd_group_free( group, NULL );
+				group = NULL;
 			}
 			for( group = flat; group; group = group->siblings ) {
 				if( group->name ) {
@@ -1075,7 +1176,7 @@ static const gchar *screem_dtd_parse_attlist_attrs( ScreemDTD *dtd,
 			}
 			if( flat ) {
 				screem_dtd_group_free( flat, NULL );
-				attr->values = g_slist_reverse( attr->values );
+				group = NULL;
 			} else {
 				/* failed to get a single value,
 				   default to CDATA so we have something */
@@ -1112,6 +1213,11 @@ static const gchar *screem_dtd_parse_attlist_attrs( ScreemDTD *dtd,
 				if( group ) {
 					screem_dtd_group_free( group, 
 							NULL );
+					group = NULL;
+				}
+				/* FIXME: HACK */
+				if( *data == '\'' || *data == '"' ) {
+					data = g_utf8_next_char( data );
 				}
 			}
 			data = g_utf8_skip_space( data );
@@ -1127,6 +1233,7 @@ static const gchar *screem_dtd_parse_attlist_attrs( ScreemDTD *dtd,
 			}
 			if( group ) {
 				screem_dtd_group_free( group, NULL );
+				group = NULL;
 			}
 		} else if( ! g_unichar_isspace( c ) ) {
 			/* we must be starting a new attribute,
@@ -1143,11 +1250,19 @@ static const gchar *screem_dtd_parse_attlist_attrs( ScreemDTD *dtd,
 	}
 
 	if( ! attr->name ) {
-		*attrs = alist->next;
-		alist->next = NULL;
-		g_slist_free( alist );
-		g_free( attr );
+		/* have a string, so use that as the attr->name */
+		if( str->len ) {
+			attr->name = g_strdup( str->str );
+		} else {
+			if( alist ) {
+				*attrs = alist->next;
+				alist->next = NULL;
+				g_slist_free( alist );
+			}
+			g_free( attr );
+		}
 	}
+	g_string_free( str, TRUE );
 
 	return data;
 }
@@ -1163,12 +1278,13 @@ static const gchar *screem_dtd_parse_attlist( ScreemDTD *dtd,
 	ScreemDTDAttribute *attribute;
 	GSList *list;
 	GSList *tlist;
+	GSList *copy;
 	
 	g_return_val_if_fail( SCREEM_IS_DTD( dtd ), data );
 	
 	priv = dtd->priv;
 	
-	/* check this is an element */
+	/* check this is an attlist */
 	
 	if( strncmp( "ATTLIST ", data, strlen( "ATTLIST " ) ) ) {
 		return data;
@@ -1183,8 +1299,11 @@ static const gchar *screem_dtd_parse_attlist( ScreemDTD *dtd,
 
 	/* flatten out the tree of names so it is simple
 	   to use */
-	name_group = screem_dtd_group_flatten( group, NULL );
-	screem_dtd_group_free( group, NULL );
+	name_group = NULL;
+	if( group ) {
+		name_group = screem_dtd_group_flatten( group, NULL );
+		screem_dtd_group_free( group, NULL );
+	}
 
 	data = g_utf8_skip_space( data );
 	list = NULL;
@@ -1192,30 +1311,47 @@ static const gchar *screem_dtd_parse_attlist( ScreemDTD *dtd,
 	if( list ) {
 		list = g_slist_sort_with_data( list, 
 				screem_dtd_sort_attr_list, dtd );
-	}
-
-	/* copy attributes to all elements */ 
-	for( tmp = name_group; tmp; tmp = tmp->siblings ) {
-		if( tmp->name ) {
-			element = g_hash_table_lookup( priv->elements,
-							tmp->name );
-
-			if( element ) {
-				for( tlist = list; tlist; 
-					tlist = tlist->next ) {
-					attribute = (ScreemDTDAttribute*)tlist->data;
-					if( ! g_hash_table_lookup( element->attributes, attribute->name ) ) {
-						g_hash_table_insert( element->attributes, attribute->name, attribute );
-						attribute->ref ++;
+	
+		/* copy attributes to all elements */ 
+		for( tmp = name_group; tmp; tmp = tmp->siblings ) {
+			if( tmp->name ) {
+				element = g_hash_table_lookup( priv->elements,
+								tmp->name );
+				/* a crash was occuring / valgrind
+				 * was giving errors when we
+				 * were using
+				 * copy = g_slist_copy( list );
+				 * theoretically that should be no
+				 * different to what is here now,
+				 * prepending each item, and reversing
+				 * should be identical, however the
+				 * crash goes, and valgrind stops
+				 * warning like this */
+				if( element ) {
+					copy = NULL;
+					for( tlist = list; tlist; 
+						tlist = tlist->next ) {
+						attribute = (ScreemDTDAttribute*)tlist->data;
+						if( ! g_hash_table_lookup( element->attributes, attribute->name ) ) {
+							g_hash_table_insert( element->attributes, attribute->name, attribute );
+							attribute->ref ++;
+							copy = g_slist_prepend( copy, attribute );
+						}
 					}
+					/* XMLResume has multiple ATTLISTs for
+					 * elements, we need to allow that */
+					copy = g_slist_reverse( copy );
+					element->attrlist = g_slist_concat( element->attrlist, copy );
 				}
-				element->attrlist = g_slist_copy( list );
 			}
 		}
+		g_slist_foreach( list, 
+				(GFunc)screem_dtd_free_attribute, NULL );
+		g_slist_free( list );
+	}
+	if( name_group ) {
+		screem_dtd_group_free( name_group, NULL );
 	}
-	g_slist_foreach( list, (GFunc)screem_dtd_free_attribute, NULL );
-	g_slist_free( list );
-	screem_dtd_group_free( name_group, NULL );
 
 	return data;
 }
@@ -1232,11 +1368,14 @@ static const gchar *screem_dtd_name_parse( ScreemDTD *dtd,
 	gboolean passed;
 	gchar *name;
 	gchar *val;
-	
+	ScreemDTDGroup *new_group;
+	gunichar qchar;
+
 	*group = NULL;
-	
+
 	string = g_string_new( NULL );
 	
+	qchar = '\0';
 	/* first pass, build up complete string */
 	for( passed = FALSE, quote = FALSE, depth = 0, pass1 = pattern;
 		*pass1 != '\0' && ! passed;
@@ -1271,7 +1410,15 @@ static const gchar *screem_dtd_name_parse( ScreemDTD *dtd,
 				g_string_append_unichar( string, c );
 				break;
 			case '"':
-				quote = !quote;
+			case '\'':
+				if( qchar == '\0' ||
+				    qchar == c ) {
+					quote = !quote;
+					qchar = c;
+					if( ! quote ) {
+						qchar = '\0';
+					}
+				}
 				g_string_append_unichar( string,
 							c );
 				break;
@@ -1289,17 +1436,18 @@ static const gchar *screem_dtd_name_parse( ScreemDTD *dtd,
 				break;
 		}
 	}
-	pattern = g_utf8_prev_char( pass1 );
-	
 	if( string->len > 0 ) {
 		/* second pass, parse string */
 
-		(*group) = g_new0( ScreemDTDGroup, 1 );
-		(*group)->type = SCREEM_DTD_OR;
+		new_group = g_new0( ScreemDTDGroup, 1 );
+		new_group->type = SCREEM_DTD_OR;
 		
-		screem_dtd_name_parse2( dtd, string->str, *group );
+		screem_dtd_name_parse2( dtd, string->str, new_group );
+		*group = new_group;
 	}
 	g_string_free( string, TRUE );
+	
+	pattern = g_utf8_prev_char( pass1 );
 
 	return pattern;
 }
@@ -1313,7 +1461,7 @@ static const gchar *screem_dtd_name_parse2( ScreemDTD *dtd,
 	gunichar c;
 	gunichar p;
 	ScreemDTDGroup *sub;
-
+	
 	string = g_string_new( NULL );
 
 	c = '\0';
@@ -1364,8 +1512,10 @@ static const gchar *screem_dtd_name_parse2( ScreemDTD *dtd,
 			}
 			
 			pass2 = g_utf8_next_char( pass2 );
-			pass2 = screem_dtd_name_parse2( dtd, 
+			if( *pass2 != '\0' ) {
+				pass2 = screem_dtd_name_parse2( dtd, 
 							pass2, sub );
+			}
 			/* backup if needed */
 			if( *pass2 == '\0' ) {
 				pass2 = g_utf8_prev_char( pass2 );
@@ -1480,6 +1630,8 @@ static ScreemDTDGroup *screem_dtd_group_flatten( ScreemDTDGroup *group, ScreemDT
 
 static void screem_dtd_group_free( ScreemDTDGroup *group, ScreemDTDGroup *parent )
 {
+	g_return_if_fail( group != NULL );
+
 	g_free( group->name );
 	if( group->children ) {
 		screem_dtd_group_free( group->children, group );
@@ -1494,18 +1646,15 @@ static gboolean screem_dtd_copy_allowed_from_table( gpointer key,
 					gpointer value,
 					gpointer user_data  )
 {
-	ScreemDTDElement *element;
+	ScreemDTDAllowedCopy *copy;
 	ScreemDTDAllowed *allowed;
-	GSList **list;
-	
-	element = (ScreemDTDElement*)user_data;
 
+	copy = (ScreemDTDAllowedCopy*)user_data;
 	allowed = (ScreemDTDAllowed*)value;
-	list = (GSList **)user_data;
-	*list = g_slist_prepend( *list, allowed->name );
+	copy->list = g_slist_insert_sorted( copy->list, 
+			allowed->name, copy->dtd->priv->compare );
 
 	return FALSE;
-
 }
 
 static gboolean screem_dtd_copy_allowed_table( gpointer key, 
@@ -1619,8 +1768,7 @@ static void screem_dtd_free_attribute( ScreemDTDAttribute *attr )
 }
 
 static gint screem_dtd_sort_element_list( gconstpointer a,
-					gconstpointer b,
-					gpointer data )
+					gconstpointer b )
 {
 	ScreemDTD *dtd;
 	const ScreemDTDElement *aelem;
@@ -1629,7 +1777,7 @@ static gint screem_dtd_sort_element_list( gconstpointer a,
 	aelem = (const ScreemDTDElement*)a;
 	belem = (const ScreemDTDElement*)b;
 
-	dtd = SCREEM_DTD( data );
+	dtd = SCREEM_DTD( aelem->dtd );
 
 	return dtd->priv->compare( aelem->name, belem->name );
 }
@@ -1675,7 +1823,7 @@ static void screem_dtd_build_optional( gpointer data,
 		child = screem_dtd_valid_element( element->dtd,
 				allowed->data );
 		if( child && child->ostate == SCREEM_DTD_SHOULD ) {
-			opt = g_slist_prepend( opt, child );	
+			opt = g_slist_prepend( opt, (gpointer)child );	
 		}
 	}
 	element->optional = opt;
@@ -1763,6 +1911,8 @@ void screem_dtd_parse( ScreemDTD *dtd, ScreemDTDParse *parse,
 	gboolean feature;
 	const gchar *tmp;
 
+	gchar *rooturi;
+	
 	g_return_if_fail( SCREEM_IS_DTD( dtd ) );
 	g_return_if_fail( parse != NULL );
 	g_return_if_fail( data != NULL );
@@ -1770,6 +1920,14 @@ void screem_dtd_parse( ScreemDTD *dtd, ScreemDTDParse *parse,
 	priv = dtd->priv;
 	db = SCREEM_DTD_DB( parse->userdata );
 
+	if( ! priv->rooturi ) {
+		rooturi = (gchar*)screem_dtd_get_system_id( dtd );
+		if( rooturi ) {
+			priv->rooturi = g_path_get_dirname( rooturi );
+			priv->toproot = priv->rooturi;
+		}
+	}
+	
 	while( *data != '\0' ) {
 
 		while( *data != '\0' && *data != '<' && *data != '%' ) {
@@ -1790,17 +1948,38 @@ void screem_dtd_parse( ScreemDTD *dtd, ScreemDTDParse *parse,
 			
 			if( parse->resolve_entity &&
 			    ( publicid || systemid ) ) {
-
+				/* set rooturi */
+				tmp = priv->rooturi;
+			
+				rooturi = relative_to_full( systemid, 
+						tmp );
+				priv->rooturi = g_path_get_dirname( rooturi );
 				data2 = parse->resolve_entity( db,
 							publicid,
-							systemid );
+							rooturi );
+				/* failed to resolve entity,
+				 * try with different root */
+				if( ! data2 ) {
+					g_free( rooturi );
+
+					rooturi = relative_to_full( systemid, 
+						priv->toproot );
+					data2 = parse->resolve_entity( db,
+							publicid,
+							rooturi );
+				}
 				g_free( publicid );
 				g_free( systemid );
+				g_free( rooturi );
 				if( data2 ) {
 					screem_dtd_parse( dtd, parse,
 							  data2 );
 					g_free( data2 );
+
 				}
+				/* restore rooturi */
+				g_free( priv->rooturi );
+				priv->rooturi = (gchar*)tmp;
 			}
 
 			g_free( name );
@@ -1851,11 +2030,16 @@ void screem_dtd_parse( ScreemDTD *dtd, ScreemDTDParse *parse,
 			data = g_utf8_next_char( data );
 		}
 	}
-	priv->elementlist = g_slist_sort_with_data( priv->elementlist,
-				screem_dtd_sort_element_list, dtd );
-	g_slist_foreach( priv->elementlist,
-			screem_dtd_build_optional, dtd );
-	screem_dtd_find_root( dtd );
+
+	/* this will occur at the top level screem_dtd_parse() for
+	 * this dtd */
+	if( priv->rooturi == priv->toproot ) {
+		g_slist_foreach( priv->elementlist,
+				screem_dtd_build_optional, dtd );
+		screem_dtd_find_root( dtd );
+		g_free( priv->rooturi );
+		priv->rooturi = NULL;
+	}
 }
 
 gchar *screem_dtd_dump( const ScreemDTD *dtd )
@@ -1899,7 +2083,7 @@ gchar *screem_dtd_lookup_entity( const ScreemDTD *dtd,
 	
 	g_return_val_if_fail( SCREEM_IS_DTD( dtd ), NULL );
 	g_return_val_if_fail( name != NULL, NULL );
-
+	
 	entity = g_hash_table_lookup( dtd->priv->entities, name );
 
 	if( ! entity ) {
@@ -1908,7 +2092,7 @@ gchar *screem_dtd_lookup_entity( const ScreemDTD *dtd,
 	}
 
 	val = entity->value;
-
+	
 	if( ! expand ) {
 		ret = g_strdup( val );
 	} else {
@@ -1918,7 +2102,7 @@ gchar *screem_dtd_lookup_entity( const ScreemDTD *dtd,
 			if( c != '%' || entity->general ) {
 				exp = g_string_append_unichar( exp, c );
 			} else { 
-				g_string_append_c( exp, ' ' );
+				tmp = NULL;
 				val = screem_dtd_read_entity( val, &tmp );		
 				if( tmp ) {
 					ret = screem_dtd_lookup_entity( dtd, tmp, TRUE );
@@ -1926,9 +2110,10 @@ gchar *screem_dtd_lookup_entity( const ScreemDTD *dtd,
 					g_free( ret );
 					g_free( tmp );
 				}
-				g_string_append_c( exp, ' ' );
 			}
-			val = g_utf8_next_char( val );
+			if( val != '\0' ) {
+				val = g_utf8_next_char( val );
+			}
 		}
 		ret = exp->str;
 		g_string_free( exp, FALSE );
@@ -1937,8 +2122,6 @@ gchar *screem_dtd_lookup_entity( const ScreemDTD *dtd,
 	return ret;
 }
 
-
-
 const ScreemDTDElement *screem_dtd_valid_element( const ScreemDTD *dtd,
 					    const gchar *element )
 {
@@ -2154,7 +2337,7 @@ const gchar *screem_dtd_attribute_get_name( const ScreemDTDAttribute *attr )
 const gchar *screem_dtd_attribute_get_description( const ScreemDTDAttribute *attr )
 {
 	g_return_val_if_fail( attr != NULL, NULL );
-
+	
 	return attr->description;
 }
 
