@@ -68,10 +68,7 @@ enum {
 
 static guint screem_file_browser_signals[ LAST_SIGNAL ] = { 0 };
 
-
-#ifdef HAVE_ICON_THEME
-static GnomeIconTheme *gnome_theme = NULL;
-#endif
+static GtkIconTheme *gtk_theme = NULL;
 
 /* static icon cache */
 static GHashTable *icon_cache = NULL;
@@ -79,6 +76,9 @@ static GHashTable *icon_cache = NULL;
 static GMutex *cache_mutex = NULL;
 
 
+/* cache mime type strings */
+static GHashTable *mime_cache = NULL;
+
 typedef enum {
 	BROWSER_NODE_DIRECTORY,
 	BROWSER_NODE_FILE
@@ -127,9 +127,6 @@ static void directory_monitor_cb( GnomeVFSMonitorHandle *handle,
 static gboolean find_iter( GtkTreeModel *model, const gchar *uri,
 			   GtkTreeIter *parent, GtkTreeIter *it );
 
-static gchar* 
-screem_file_browser_build_icon_name( const gchar *iconname );
-
 static void screem_file_browser_set_icons( ScreemFileBrowser *browser,
 					     GtkTreeIter *it );
 
@@ -248,7 +245,7 @@ void screem_file_browser_scan_directory( ScreemFileBrowser *browser,
 				/* replace */
 				gchar *temp;
 
-				temp = g_dirname( node->pathname );
+				temp = g_path_get_dirname( node->pathname );
 				g_free( node->pathname );
 				node->pathname = temp;
 				g_free( node->basename );
@@ -335,7 +332,7 @@ const gchar* screem_file_browser_scan_iter( ScreemFileBrowser *browser,
 				/* replace */
 				gchar *temp;
 
-				temp = g_dirname( node->pathname );
+				temp = g_path_get_dirname( node->pathname );
 				current = node->current = node->pathname;
 				node->pathname = temp;
 				g_free( node->basename );
@@ -452,13 +449,18 @@ GdkPixbuf *screem_file_browser_get_icon( ScreemFileBrowser *browser,
 {
 	gchar *mime_type;
 	GdkPixbuf *ret;
-	gchar *file = NULL;
+	const gchar *file;
 	GnomeVFSFileInfo *info;
 	gchar *tmp;
 
+	GtkIconInfo *iinfo;
+	
 	g_return_val_if_fail( SCREEM_IS_FILE_BROWSER( browser ), NULL );
-	g_return_val_if_fail( uri != NULL, NULL );
 
+	if( ! uri ) {
+		uri = "";
+	}
+	
 	mime_type = NULL;
 	if( mtype && *mtype ) {
 		mime_type = g_strdup( *mtype );
@@ -467,44 +469,58 @@ GdkPixbuf *screem_file_browser_get_icon( ScreemFileBrowser *browser,
 	/* Hack to avoid file access when we are accessing
 	 * directories */
 	if( uri[ strlen( uri ) -1 ] == '/' ) {
+		if( mime_type ) {
+			g_free( mime_type );
+		}
 		mime_type = g_strdup( "x-directory/normal" );
 	} else if( mime_type == NULL ){
 		info = gnome_vfs_file_info_new();
 
-		gnome_vfs_get_file_info( uri, info, 
+		if( gnome_vfs_get_file_info( uri, info, 
 				 GNOME_VFS_FILE_INFO_GET_MIME_TYPE |
 				 GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE |
-				 GNOME_VFS_FILE_INFO_FOLLOW_LINKS );
-
-		mime_type = g_strdup( info->mime_type );
+				 GNOME_VFS_FILE_INFO_FOLLOW_LINKS ) ==
+				GNOME_VFS_OK ) {
+			mime_type = g_strdup( info->mime_type );
+		} else {
+			/* FIXME: application/octet-stream ? */
+			mime_type = g_strdup( "" );
+		}
 		gnome_vfs_file_info_unref( info );
 	}
 	if( mtype && ( *mtype == NULL ) ) {
 		*mtype = g_strdup( mime_type );
 	}
 
-	file = NULL;
-#ifdef HAVE_ICON_THEME
 	/* figure out icon filename */
-	tmp = gnome_icon_lookup( gnome_theme, 
+	tmp = gnome_icon_lookup( gtk_theme, 
 				  NULL,
 				  uri,
 				  NULL,
 				  NULL,
 				  mime_type,
 				  0, NULL );
+	file = NULL;
+	iinfo = NULL;
 	if( tmp ) {
-		file = screem_file_browser_build_icon_name( tmp );
+		iinfo = gtk_icon_theme_lookup_icon( gtk_theme, tmp,
+						     width, 0 );
+		file = gtk_icon_info_get_filename( iinfo );
 		g_free( tmp );
+		if( ! file ||
+		    ! g_file_test( file, G_FILE_TEST_EXISTS ) ) {
+			gtk_icon_info_free( iinfo );
+			info = NULL;
+		}
+	
 	}
-#endif
 	g_free( mime_type );
 	g_mutex_lock( cache_mutex );
 	
 	ret = NULL;
-	if( file ) {
+	if( iinfo ) {
 		GdkPixbuf *close = NULL;
-		
+	
 		if( ! ignore_cache ) {
 			ret = g_hash_table_lookup( icon_cache, file );
 		}
@@ -536,13 +552,12 @@ GdkPixbuf *screem_file_browser_get_icon( ScreemFileBrowser *browser,
 			g_object_ref( ret );
 			if( ! ignore_cache ) {
 				g_hash_table_insert( icon_cache, 
-							g_strdup( file ),
-					     		ret );
+						g_strdup( file ), ret );
 			}
 			g_object_unref( close );
 		}
 
-		g_free( file );
+		gtk_icon_info_free( iinfo );
 	}
 
 	g_mutex_unlock( cache_mutex );
@@ -550,6 +565,27 @@ GdkPixbuf *screem_file_browser_get_icon( ScreemFileBrowser *browser,
 	return ret;
 }
 
+void screem_file_browser_file_mod( ScreemFileBrowser *browser,
+				const gchar *directory,
+				const gchar *file,
+				GnomeVFSMonitorEventType type )
+{
+	directory_monitor_cb( NULL, directory, file, type, browser );
+}
+
+void screem_file_browser_cleanup( void )
+{
+	if( cache_mutex ) {
+		g_mutex_free( cache_mutex );
+	}
+	if( icon_cache ) {
+		g_hash_table_destroy( icon_cache );
+	}
+	if( mime_cache ) {
+		g_hash_table_destroy( mime_cache );
+	}
+}
+
 /* static stuff */
 static BrowserNode *browser_node_new( ScreemFileBrowser *browser, 
 		gint depth, BrowserNodeType type, const gchar *uri,
@@ -557,46 +593,60 @@ static BrowserNode *browser_node_new( ScreemFileBrowser *browser,
 {
 	BrowserNode *node;
 	const gchar *lookup_uri;
+	gchar *temp;
 
 	node = g_new0( BrowserNode, 1 );
 	node->type = type;
 
 	node->browser = browser;
 
+	g_assert( uri != NULL );
+	
 	if( mime_type ) {
 		node->mime_type = g_strdup( mime_type );
 	}
 	
-	if( uri ) {
-		gchar *temp;
 		
-		node->pathname = g_strdup( uri );
+	node->pathname = g_strdup( uri );
 
-		temp = g_path_get_basename( node->pathname );
-		if( temp ) {
-			node->basename = gnome_vfs_unescape_string_for_display( temp );
-			g_free( temp );
-		}
-		
-		if( ! node->basename || *node->basename == '\0' ) {
-			g_free( node->basename );
-			node->basename = g_strdup( G_DIR_SEPARATOR_S );
-		}
-
-		lookup_uri = uri;
-		if( type == BROWSER_NODE_DIRECTORY ) {
-			lookup_uri = "/";
-		}
-		
-		node->close = screem_file_browser_get_icon( browser, 
-							lookup_uri,
-							ICON_X_SIZE,
-							ICON_Y_SIZE,
-							FALSE,
-							&node->mime_type );
+	temp = g_path_get_basename( node->pathname );
+	if( temp ) {
+		node->basename = gnome_vfs_unescape_string_for_display( temp );
+		g_free( temp );
+	}
+	
+	if( ! node->basename || *node->basename == '\0' ) {
+		g_free( node->basename );
+		node->basename = g_strdup( G_DIR_SEPARATOR_S );
 	}
 
-       	if( uri && type == BROWSER_NODE_DIRECTORY &&
+	lookup_uri = uri;
+	if( type == BROWSER_NODE_DIRECTORY ) {
+		lookup_uri = "/";
+	}
+		
+	node->close = screem_file_browser_get_icon( browser, 
+						lookup_uri,
+						ICON_X_SIZE,
+						ICON_Y_SIZE,
+						FALSE,
+						&node->mime_type );
+	
+	g_mutex_lock( cache_mutex );
+
+	mime_type = g_hash_table_lookup( mime_cache, node->mime_type );
+	if( ! mime_type ) {
+		g_hash_table_insert( mime_cache, node->mime_type,
+				node->mime_type );
+		mime_type = node->mime_type;
+	} else if( mime_type != node->mime_type ) {
+		g_free( node->mime_type );
+		node->mime_type = mime_type;
+	}
+	
+	g_mutex_unlock( cache_mutex );
+
+       	if( type == BROWSER_NODE_DIRECTORY &&
 	    ( ( ! ( browser->details->mode & FILE_BROWSE_FLAT ) ) ||
 	      depth == 0 ) ) {
 		/* don't monitor /dev, /proc, or /sys */
@@ -629,7 +679,6 @@ static void browser_node_free( BrowserNode *node )
 	if( node->close ) {
 		g_object_unref( node->close );
 	}
-	g_free( node->mime_type );
 	g_free( node->basename );
 
 	if( node->monitor ) {
@@ -677,7 +726,7 @@ static void scan_directory( ScreemFileBrowser *browser,
 	root = parent;
 	parent_uri = NULL;
 	if( ! parent ) {
-		if( gtk_tree_model_get_iter_root( model, &clear ) ) {
+		if( gtk_tree_model_get_iter_first( model, &clear ) ) {
 			root = &clear;
 		}
 	} else {
@@ -735,6 +784,7 @@ static void scan_directory( ScreemFileBrowser *browser,
 				    FILE_BROWSER_NODE_COL, node,
 				    FILE_BROWSER_ICON_COL, node->close,
 				    FILE_BROWSER_URI_COL, node->pathname,
+				    FILE_BROWSER_MIME_COL, node->mime_type,
 				    -1 );
 
 		g_signal_emit( G_OBJECT( browser ),
@@ -748,11 +798,9 @@ static void scan_directory( ScreemFileBrowser *browser,
 		if( ! parent_uri || strcmp( uri, parent_uri ) ) {
 			gtk_tree_store_append( GTK_TREE_STORE( model ), &it, 
 					       parent );
-			
 			node = browser_node_new( browser, depth,
 						 BROWSER_NODE_DIRECTORY,
 						 uri, mime_type );
-	
 			gtk_tree_store_set( GTK_TREE_STORE( model ), &it,
 					    FILE_BROWSER_NAME_COL,
 					    node->basename,
@@ -761,8 +809,9 @@ static void scan_directory( ScreemFileBrowser *browser,
 					    node->close,
 					    FILE_BROWSER_URI_COL,
 					    node->pathname,
+				    	    FILE_BROWSER_MIME_COL, 
+					    node->mime_type,
 					    -1 );
-			
 			g_signal_emit( G_OBJECT( browser ),
 				       screem_file_browser_signals[ ADDED ],
 				       0,
@@ -795,9 +844,12 @@ static void scan_directory( ScreemFileBrowser *browser,
 				    *info->name != '.' ) {
 					gchar *utfuri;
 					gchar *nexturi;
-					
-					utfuri = screem_support_charset_convert( info->name );
-			
+
+					if( ! g_utf8_validate( info->name, -1, NULL ) ) {
+						utfuri = g_filename_to_utf8( info->name, -1, NULL, NULL, NULL );
+					} else {
+						utfuri = g_strdup( info->name );
+					}
 					/* yuk */
 					if( ! utfuri ) {
 						continue;
@@ -808,7 +860,6 @@ static void scan_directory( ScreemFileBrowser *browser,
 							utfuri,
 							NULL );
 					g_free( utfuri );
-					
 					scan_directory( browser, &it,
 							nexturi, 
 							info->mime_type,
@@ -839,7 +890,6 @@ static void directory_monitor_cb( GnomeVFSMonitorHandle *handle,
 	GtkTreeIter it;
 	GtkTreeIter iter;
 	GtkTreeIter *parent;
-	GtkTreePath *path;
 	GtkTreeModel *model;
 	GnomeVFSFileInfo *info;
 	BrowserNodeType btype;
@@ -847,6 +897,7 @@ static void directory_monitor_cb( GnomeVFSMonitorHandle *handle,
 	gboolean hidden;
 	int mode;
 	BrowserNode *node;
+	GnomeVFSResult res;
 	
 	if( ! data ) {
 		return;
@@ -888,18 +939,36 @@ static void directory_monitor_cb( GnomeVFSMonitorHandle *handle,
 		break;
 	case GNOME_VFS_MONITOR_EVENT_CREATED:
 		model = GTK_TREE_MODEL( browser->details->model );
+		
+		/* check that we don't already have a node for info_uri */
+		if( find_iter( model, info_uri, NULL, &iter ) ) {
+			/* we do, don't do anything */
+			break;
+		}
+	
 		parent = NULL;
 		if( ( ! ( mode & FILE_BROWSE_FLAT ) ) &&
 			find_iter( model, monitor_uri, NULL, &pit ) ) {
 			/* got the parent */
 			parent = &pit;
 		}
+		
+		if( ! parent ) {
+			break;	
+		}
 			
 		/* is info_uri a directory? */
 		info = gnome_vfs_file_info_new();
-		gnome_vfs_get_file_info( info_uri, info, 
+		res = gnome_vfs_get_file_info( info_uri, info, 
 				GNOME_VFS_FILE_INFO_GET_MIME_TYPE | 
 				GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE );
+		/* file may have disappeared between notification
+		 * signal being sent, and us attempting to get file
+		 * info */
+		if( res != GNOME_VFS_OK ) {
+			break;
+		}
+		
 		if( info->type == GNOME_VFS_FILE_TYPE_DIRECTORY ) {
 			btype = BROWSER_NODE_DIRECTORY;
 		} else {
@@ -937,6 +1006,8 @@ static void directory_monitor_cb( GnomeVFSMonitorHandle *handle,
 				 	   node->close,
 				 	   FILE_BROWSER_URI_COL,
 				 	   node->pathname,
+				    	   FILE_BROWSER_MIME_COL, 
+					   node->mime_type,
 					   -1 );
 
 			g_signal_emit( G_OBJECT( browser ),
@@ -1010,27 +1081,6 @@ static gboolean find_iter( GtkTreeModel *model, const gchar *uri,
 	return found;
 }
 
-static gchar* 
-screem_file_browser_build_icon_name( const gchar *iconname )
-{
-	gchar *name;
-
-
-#ifdef HAVE_ICON_THEME
-
-      	if( iconname[ 0 ] != '/' || ! g_file_test( iconname,
-						   G_FILE_TEST_EXISTS ) ) {
-	      	name = gnome_icon_theme_lookup_icon( gnome_theme, iconname,
-						     24, NULL, NULL );
-	} else {
-		name = g_strdup( iconname );
-	}
-#else
-	name = g_strdup( iconname );
-#endif
-	return name;
-}
-
 static void screem_file_browser_set_icons( ScreemFileBrowser *browser,
 					     GtkTreeIter *it )
 {
@@ -1066,8 +1116,8 @@ static void screem_file_browser_set_icons( ScreemFileBrowser *browser,
 			mimetype = node->mime_type;
 		}
 		
-		if( uri ) {
-			if( node && node->close ) {
+		if( uri && node ) {
+			if( node->close ) {
 				g_object_unref( node->close );
 			} 
 			pixbuf = screem_file_browser_get_icon( browser,
@@ -1077,11 +1127,11 @@ static void screem_file_browser_set_icons( ScreemFileBrowser *browser,
 								FALSE,
 								NULL );
 			
-			if( node ) {
-				node->close = pixbuf;
-			}
-
+			node->close = pixbuf;
+			
 			gtk_tree_store_set( GTK_TREE_STORE( model ), it,
+				    	    FILE_BROWSER_MIME_COL, 
+					    mimetype,
 					    FILE_BROWSER_ICON_COL, 
 					    pixbuf, -1 );
 		}		
@@ -1202,8 +1252,8 @@ static gboolean screem_file_browser_clear( GtkTreeModel *model,
         BrowserNode *node;
 	
         gtk_tree_model_get( model, it, 
-				FILE_BROWSER_NODE_COL, &node, -1 );
-        
+			FILE_BROWSER_NODE_COL, &node, -1 );
+       
 	if( node ) {
 		browser_node_free( node );
 	}
@@ -1279,9 +1329,21 @@ screem_file_browser_init( ScreemFileBrowser *browser )
 	if( ! cache_mutex ) {
 		cache_mutex = g_mutex_new();
 	}
+	g_mutex_lock( cache_mutex );
 	if( ! icon_cache ) {
-		icon_cache = g_hash_table_new( g_str_hash, g_str_equal );
+		icon_cache = g_hash_table_new_full( g_str_hash, 
+				g_str_equal, 
+				(GDestroyNotify)g_free, 
+				(GDestroyNotify)g_object_unref );
 	}
+	if( ! mime_cache ) {
+		mime_cache = g_hash_table_new_full( g_str_hash, 
+				g_str_equal, 
+				(GDestroyNotify)g_free, 
+				NULL );
+	}
+
+	g_mutex_unlock( cache_mutex );
 
 	browser->details = g_new0( ScreemFileBrowserDetails, 1 );
 
@@ -1291,7 +1353,8 @@ screem_file_browser_init( ScreemFileBrowser *browser )
 						      G_TYPE_POINTER,
 						      G_TYPE_STRING,
 						      G_TYPE_POINTER,
-						      G_TYPE_STRING );
+						      G_TYPE_STRING,
+						      G_TYPE_POINTER);
 
 	browser->details->sorted = 
 		GTK_TREE_MODEL( gtk_tree_model_sort_new_with_model( GTK_TREE_MODEL( browser->details->model ) ) );
@@ -1301,16 +1364,13 @@ screem_file_browser_init( ScreemFileBrowser *browser )
 					   browser );
 
 
-#ifdef HAVE_ICON_THEME
-	if( ! gnome_theme ) {
-		gnome_theme = gnome_icon_theme_new();
-		gnome_icon_theme_set_allow_svg( gnome_theme, TRUE );
+	if( ! gtk_theme ) {
+		gtk_theme = gtk_icon_theme_get_default();
 	}
-	g_signal_connect( G_OBJECT( gnome_theme ),
+	g_signal_connect( G_OBJECT( gtk_theme ),
 			  "changed",
 			  G_CALLBACK( screem_file_browser_theme_change ),
 			  browser );
-#endif
 }
 
 static void
@@ -1348,13 +1408,11 @@ screem_file_browser_finalize( GObject *object )
 
 	browser = SCREEM_FILE_BROWSER( object );
 
-#ifdef HAVE_ICON_THEME
-	g_signal_handlers_disconnect_matched( G_OBJECT( gnome_theme ),
+	g_signal_handlers_disconnect_matched( G_OBJECT( gtk_theme ),
 					      G_SIGNAL_MATCH_DATA,
 					      0, 0, NULL,
 					      NULL,
 					      browser );
-#endif
 
 	g_object_unref( browser->details->sorted );
 
