@@ -33,7 +33,7 @@
 
 #include <gmodule.h>
 
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 
 #include <libgnomeui/gnome-popup-menu.h>
 
@@ -81,8 +81,6 @@
 #include "screem-link-view-image.h"
 #include "screem-link-view-html.h"
 
-#include "libegg/menu/egg-menu.h"
-
 #include "support.h"
 
 static const gdouble half_mpi = M_PI / 2;
@@ -400,7 +398,7 @@ screem_link_view_add_key( ScreemLinkView *link_view,
 
 	gchar *font;
 
-	gint icons[] = {
+	static const gint icons[] = {
 		GOODNODE,
 		EXTERNALNODE,
 		BADNODE,
@@ -409,13 +407,13 @@ screem_link_view_add_key( ScreemLinkView *link_view,
 		CHECKNODE,
 		-1
 	};
-	gchar *names[] = {
-		_( "Local" ),
-		_( "External" ),
-		_( "Broken" ),
-		_( "Ignored" ),
-		_( "Changed" ),
-		_( "Checking" ),
+	static const gchar *names[] = {
+		N_( "Local" ),
+		N_( "External" ),
+		N_( "Broken" ),
+		N_( "Ignored" ),
+		N_( "Changed" ),
+		N_( "Checking" ),
 		NULL
 	};
 	gint i;
@@ -443,11 +441,14 @@ screem_link_view_add_key( ScreemLinkView *link_view,
 					      "x", w + 15.0,
 					      "y", y2 + 4.0,
 					      "anchor", GTK_ANCHOR_WEST,
-					      "text", names[ i ],
-					      "font", font,
+					      "text", 
+					      gettext( names[ i ] ),
 					       "fill_color", "black",
 					      NULL );
-		
+		if( font ) {
+			g_object_set( G_OBJECT( text ), "font", font,
+					NULL );
+		}
 		w += ((GnomeCanvasText*)text)->max_width;
 		w += 20.0;
 		w += 10.0;
@@ -511,7 +512,7 @@ static GNode *build_link_tree( ScreemLinkView *view,
 	} else if( FALSE && online ) {
 		/* can lead to network access, therefore we don't want
 		   to do this if we are not online */
-		mime_type = screem_get_mime_type( pathname );
+		mime_type = screem_get_mime_type( pathname, FALSE );
 	}
 	
 	if( mime_type &&
@@ -571,10 +572,14 @@ static GNode *build_link_tree( ScreemLinkView *view,
 					      g_strdup( pathname ),
 					      GINT_TO_POINTER( level + 1 ) );
 			
-			screem_page_load( page );
-			text = screem_page_get_data( page );
-			links = screem_markup_get_links( dtd, text );
-			g_free( text );
+			if( screem_page_load( page, NULL ) ) {
+				text = screem_page_get_data( page );
+				links = screem_markup_get_links( dtd, text );
+				g_free( text );
+			} else {
+				links = NULL;
+				/* FIXME: report error? */
+			}
 		}
 		for( list = links; list; list = list->next ) {
 			GnomeVFSURI *uri;
@@ -589,6 +594,11 @@ static GNode *build_link_tree( ScreemLinkView *view,
 			if( uri && path && *path == '/' ) {
 				/* absolute link, strip fragment */
 				full = gnome_vfs_uri_to_string( uri, GNOME_VFS_URI_HIDE_FRAGMENT_IDENTIFIER );
+
+				/* this may already be a full path,
+				 * or may need the site pathname
+				 * prepending */
+				
 			} else if( uri ) {
 				/* must be relative or include scheme */
 				full = relative_to_full( path, 
@@ -811,33 +821,33 @@ static gint background_event( GtkWidget *canvas, GdkEventButton *event,
 	}
 
        	if( ( ! html ) && event->button == 3 ) {
-		EggAction *action;
+		GtkAction *action;
 		
 		g_signal_stop_emission_by_name( G_OBJECT( canvas ),
 						"button_press_event" );
-		menu = egg_menu_merge_get_widget( EGG_MENU_MERGE( window->merge ),
-							"/popups/linkviewmenu" );
+		menu = gtk_ui_manager_get_widget( GTK_UI_MANAGER( window->merge ),
+							"/linkviewmenu" );
 			
-		action = egg_action_group_get_action( EGG_ACTION_GROUP( window->action_group ),
+		action = gtk_action_group_get_action( GTK_ACTION_GROUP( window->action_group ),
 							"ExcludeFlag" );
 		g_object_set( G_OBJECT( action ), "visible", FALSE, NULL );
-		action = egg_action_group_get_action( EGG_ACTION_GROUP( window->action_group ),
+		action = gtk_action_group_get_action( GTK_ACTION_GROUP( window->action_group ),
 							"IgnoreFlag" );
 		g_object_set( G_OBJECT( action ), "visible", FALSE, NULL );
-		action = egg_action_group_get_action( EGG_ACTION_GROUP( window->action_group ),
+		action = gtk_action_group_get_action( GTK_ACTION_GROUP( window->action_group ),
 							"ASCIIFlag" );
 		g_object_set( G_OBJECT( action ), "visible", FALSE, NULL );
 		
-		gnome_popup_menu_do_popup_modal( menu, 0, 0, event,
+		screem_popup_menu_do_popup_modal( menu, 0, 0, event,
 						 0, 0 );
 			
-		action = egg_action_group_get_action( EGG_ACTION_GROUP( window->action_group ),
+		action = gtk_action_group_get_action( GTK_ACTION_GROUP( window->action_group ),
 							"ExcludeFlag" );
 		g_object_set( G_OBJECT( action ), "visible", TRUE, NULL );
-		action = egg_action_group_get_action( EGG_ACTION_GROUP( window->action_group ),
+		action = gtk_action_group_get_action( GTK_ACTION_GROUP( window->action_group ),
 							"IgnoreFlag" );
 		g_object_set( G_OBJECT( action ), "visible", TRUE, NULL );
-		action = egg_action_group_get_action( EGG_ACTION_GROUP( window->action_group ),
+		action = gtk_action_group_get_action( GTK_ACTION_GROUP( window->action_group ),
 							"ASCIIFlag" );
 		g_object_set( G_OBJECT( action ), "visible", TRUE, NULL );
 	} else {
@@ -1331,16 +1341,18 @@ static void screem_link_view_finalize( GObject *link_view )
 	G_OBJECT_CLASS( parent_class )->finalize( G_OBJECT( link_view ) );
 }
 
-static void screem_link_view_popup_zoom_callback( EggAction *action,
+static void screem_link_view_popup_zoom_callback( GtkAction *action,
 						  gpointer user_data )
 {
 	ScreemLinkView *view;
 	gint zoom;
-	gchar *verb;
-	
-	verb = action->name;
-	verb += strlen( "LinkViewZoom" );
+	const gchar *verb;
+	gchar *name;
+
+	g_object_get( G_OBJECT( action ), "name", &name, NULL );
+	verb = name + strlen( "LinkViewZoom" );
 	zoom = atoi( verb );
+	g_free( name );
 
 	view = SCREEM_LINK_VIEW( user_data );
 
@@ -1348,7 +1360,7 @@ static void screem_link_view_popup_zoom_callback( EggAction *action,
 				  (gdouble)zoom );
 }
 
-static void screem_link_view_refresh_callback( EggAction *action,
+static void screem_link_view_refresh_callback( GtkAction *action,
 					       gpointer user_data )
 {
 	screem_view_display( SCREEM_VIEW( user_data ) );
@@ -1360,36 +1372,35 @@ static void screem_link_view_window_set( ScreemView *view )
 	GtkWidget *image;*/
 	ScreemWindow *window;
 
-	static EggActionGroupEntry entries[] = {
-		{ "LinkViewRefresh", N_( "Refresh" ),
-	  	GTK_STOCK_REFRESH, NULL, NULL,
-	  	G_CALLBACK( screem_link_view_refresh_callback ), NULL },
+	static GtkActionEntry entries[] = {
+		{ "LinkViewRefresh", GTK_STOCK_REFRESH, N_( "Refresh" ),
+	  	"", NULL,
+	  	G_CALLBACK( screem_link_view_refresh_callback ) },
 		
-		{ "LinkViewZoom", N_( "Zoom" ),
-	  	GTK_STOCK_ZOOM_FIT, NULL, NULL,
-	  	G_CALLBACK( NULL ), NULL },
-		{ "LinkViewZoom25", "25%",
-	  	GTK_STOCK_ZOOM_OUT, NULL, NULL,
-	  	G_CALLBACK( screem_link_view_popup_zoom_callback ), NULL },
-		{ "LinkViewZoom50", "50%",
-	  	GTK_STOCK_ZOOM_OUT, NULL, NULL,
-	  	G_CALLBACK( screem_link_view_popup_zoom_callback  ), NULL },
-		{ "LinkViewZoom100", "100%",
-	  	GTK_STOCK_ZOOM_100, NULL, NULL,
-	  	G_CALLBACK( screem_link_view_popup_zoom_callback  ), NULL },
-		{ "LinkViewZoom200", "200%",
-	  	GTK_STOCK_ZOOM_IN, NULL, NULL,
-	  	G_CALLBACK( screem_link_view_popup_zoom_callback  ), NULL },
-		{ "LinkViewZoom300", "300%",
-	  	GTK_STOCK_ZOOM_IN, NULL, NULL,
-	  	G_CALLBACK( screem_link_view_popup_zoom_callback  ), NULL },	
-		{ "LinkViewZoom400", "400%",
-	  	GTK_STOCK_ZOOM_IN, NULL, NULL,
-	  	G_CALLBACK( screem_link_view_popup_zoom_callback  ), NULL },
+		{ "LinkViewZoom", GTK_STOCK_ZOOM_FIT, N_( "Zoom" ),
+	  	"", NULL,
+	  	G_CALLBACK( NULL ) },
+		{ "LinkViewZoom25", GTK_STOCK_ZOOM_OUT, "25%",
+	  	"", NULL,
+	  	G_CALLBACK( screem_link_view_popup_zoom_callback ) },
+		{ "LinkViewZoom50", GTK_STOCK_ZOOM_OUT, "50%",
+	  	"", NULL,
+	  	G_CALLBACK( screem_link_view_popup_zoom_callback ) },
+		{ "LinkViewZoom100", GTK_STOCK_ZOOM_100, "100%",
+	  	"", NULL,
+	  	G_CALLBACK( screem_link_view_popup_zoom_callback ) },
+		{ "LinkViewZoom200", GTK_STOCK_ZOOM_IN, "200%",
+	  	"", NULL,
+	  	G_CALLBACK( screem_link_view_popup_zoom_callback  ) },
+		{ "LinkViewZoom300", GTK_STOCK_ZOOM_IN, "300%",
+	  	"", NULL,
+	  	G_CALLBACK( screem_link_view_popup_zoom_callback  ) },	
+		{ "LinkViewZoom400", GTK_STOCK_ZOOM_IN, "400%",
+	  	"", NULL,
+	  	G_CALLBACK( screem_link_view_popup_zoom_callback  ) }
 	};
 	static 
 	guint n_entries=G_N_ELEMENTS(entries);
-	gint i;
 	
 	g_object_get( G_OBJECT( view ), "window", &window, NULL );
 	
@@ -1399,12 +1410,10 @@ static void screem_link_view_window_set( ScreemView *view )
 					 GTK_ICON_SIZE_SMALL_TOOLBAR );
 	gtk_widget_show( image );*/
 	
-	for( i = 0; i < n_entries; ++ i ) {
-		entries[ i ].user_data = view;
-	}
-	egg_action_group_add_actions( EGG_ACTION_GROUP( window->action_group ),
+	gtk_action_group_add_actions( GTK_ACTION_GROUP( window->action_group ),
 					entries,
-					n_entries );
+					n_entries,
+					view );
 }
 
 static void screem_link_view_set_prop( GObject *object, guint property_id,
