@@ -21,6 +21,8 @@
  *  about box under the help menu for a contact address
  */
 
+#include <gconf/gconf.h>
+
 #include <glib/gstring.h>
 #include <glib/gstrfuncs.h>
 #include <glib/gmem.h>
@@ -585,8 +587,8 @@ gboolean screem_markup_is_tag( const gchar *text, gint pos,
 		i ++;
 
 		/* special case for comments */
-		if( c == '-' && *( tmp + 1 ) == '-' ) {
-			temp = strstr( tmp + 2, "-->" );
+		if( c == '!' && tmp[ 1 ] == '-' && tmp[ 2 ] == '-' ) {
+			temp = strstr( tmp + 3, "-->" );
 			if( temp ) {
 				i += g_utf8_pointer_to_offset( stext,
 							temp ) -
@@ -595,6 +597,22 @@ gboolean screem_markup_is_tag( const gchar *text, gint pos,
 				i += 4;
 			}
 			ret = ( temp != NULL );
+		} else if( c == '!' &&
+			tmp[ 1 ] == '[' &&
+			tmp[ 2 ] == 'C' &&
+			tmp[ 3 ] == 'D' &&
+			tmp[ 4 ] == 'A' &&
+			tmp[ 5 ] == 'T' &&
+			tmp[ 6 ] == 'A' ) {
+			/* special case for <![CDATA[ */
+			temp = strstr( tmp + 7, "]]>" );
+			if( temp ) {
+				i += g_utf8_pointer_to_offset( stext,
+							temp ) -
+					g_utf8_pointer_to_offset( stext,
+							tmp );
+				i += 4;
+			}
 		} else if( ( c == '%' || c == '?' ) ) {
 			/* special case for asp, php etc */
 			const gchar *ptemp;
@@ -643,11 +661,21 @@ gchar *screem_markup_next_tag( const gchar *text, gint pos,
 	gunichar c;
 	gchar *ret;
 
+	gint s;
+	gint e;
+	
 	if( pos < 0 ) {
 		g_warning( "ERROR\n" );
 		return NULL;
 	}
 	
+	if( ! start ) {
+		start = &s;
+	}
+	if( ! end ) {
+		end = &e;
+	}
+	
 	ret = NULL;
 	tag = g_utf8_offset_to_pointer( text, pos );
 	tag = g_utf8_strchr( tag, -1, '<' );
@@ -664,13 +692,13 @@ gchar *screem_markup_next_tag( const gchar *text, gint pos,
 					c = g_utf8_get_char( tend );
 				}
 				*name = g_strndup( temp, 
-						g_utf8_pointer_to_offset( temp, tend ) );
+						tend - temp );
 			}
 			if( tend ) {
 				tend = strstr( tend, "?>" );
 			}
 			if( tend ) {
-				tend += strlen( "?>" ) - 1;
+				tend += strlen( "?>" );
 			}
 			break;
 		case '%':
@@ -680,7 +708,7 @@ gchar *screem_markup_next_tag( const gchar *text, gint pos,
 			}
 			tend = strstr( tend, "%>" );
 			if( tend ) {
-				tend += strlen( "%>" ) - 1;
+				tend += strlen( "%>" );
 			}
 			break;
 		case '!':
@@ -691,9 +719,24 @@ gchar *screem_markup_next_tag( const gchar *text, gint pos,
 				}
 				tend = strstr( tend, "-->" );
 				if( tend ) {
-					tend += strlen( "-->" ) - 1;
+					tend += strlen( "-->" );
 				} 
-			break;
+				break;
+			} else if( tend[ 1 ] == '[' &&
+				   tend[ 2 ] == 'C' &&
+				   tend[ 3 ] == 'D' &&
+				   tend[ 4 ] == 'A' &&
+				   tend[ 5 ] == 'T' &&
+				   tend[ 6 ] == 'A' ) {
+				/* special case for CDATA sections */
+				if( name ) {
+					*name = g_strdup( "[CDATA]" );
+				}
+				tend = strstr( tend, "]]>" );
+				if( tend ) {
+					tend += strlen( "]]>" );
+				}
+				break;
 			}
 		/* deliberate drop through */
 		default:
@@ -706,9 +749,12 @@ gchar *screem_markup_next_tag( const gchar *text, gint pos,
 					! g_unichar_isspace( c ) ); 
 
 				*name = g_strndup( temp, 
-						g_utf8_pointer_to_offset( temp, tend ) );
+						tend - temp );
 			}
 			tend = g_utf8_strchr( tend, -1, '>' );
+			if( tend ) {
+				tend = g_utf8_next_char( tend );
+			}
 			break;
 		}
 		/* if the tag isn't closed we haven't found a valid tag */
@@ -718,15 +764,10 @@ gchar *screem_markup_next_tag( const gchar *text, gint pos,
 				*name = NULL;
 			}
 		} else {
-			if( start ) {
-				*start = g_utf8_pointer_to_offset( text, tag );
-			}
-			if( end ) {
-				*end = g_utf8_pointer_to_offset( text, tend );
-			}
+			*start = g_utf8_pointer_to_offset( text, tag );
+			*end = g_utf8_pointer_to_offset( text, tend );
 
-			ret = g_strndup( tag, 
-					g_utf8_pointer_to_offset( temp, tend ) + 1 );
+			ret = g_strndup( tag, tend - tag ); 
 		}
 	}
 	 
@@ -797,7 +838,7 @@ gchar *screem_markup_change_case( const gchar *text, gboolean upper )
 			g_string_append_utf8_len( str, offset, len );
 		}
 		
-		pos = end + 1;
+		pos = end;
 		offset = g_utf8_offset_to_pointer( text, start );
 		
 		/* non markup tag, so we don't change case */
@@ -808,7 +849,7 @@ gchar *screem_markup_change_case( const gchar *text, gboolean upper )
 				g_string_append_utf8_len( str, 
 						offset, len );
 			}
-			c = '>';
+			c = '\0';
 		} else { 		
 			tag = offset;
 			c = g_utf8_get_char( tag );
@@ -829,7 +870,9 @@ gchar *screem_markup_change_case( const gchar *text, gboolean upper )
 			g_string_append_unichar( str, c );
 			tag = g_utf8_next_char( tag );
 		}
-		g_string_append_unichar( str, c );
+		if( c != '\0' ) {
+			g_string_append_unichar( str, c );
+		}
 		g_free( name );
 		g_free( next );
 	}
@@ -868,7 +911,8 @@ gboolean screem_markup_next_tag_close( const gchar *text,
 GSList *screem_markup_build_attributes_list( const gchar *tag, GSList *list )
 {
 	const gchar *tmp;
-
+	gunichar c;
+	
 	tmp = tag;
 	do {
 		const gchar *equals;
@@ -881,8 +925,8 @@ GSList *screem_markup_build_attributes_list( const gchar *tag, GSList *list )
 		if( *tmp == '\0' || *tmp == '>' || *tmp == '/' ) {
 			break;
 		}
-		
-		equals = strchr( tmp, '=' );
+	
+		equals = g_utf8_strchr( tmp, -1, '=' );
 		if( ! equals ) {
 			/* assume single name attr,
 			   such as noshade in html on an hr */
@@ -899,20 +943,17 @@ GSList *screem_markup_build_attributes_list( const gchar *tag, GSList *list )
 			value = NULL;
 			tmp = equals;
 		} else {
-			equals ++;
-			if( *equals == '"' ) {
-				equals ++;
+			equals = g_utf8_next_char( equals );
+			equals = g_utf8_skip_space( equals );
+			c = g_utf8_get_char( equals );
+			if( c == '"' || c == '\'' ) {
+				equals = g_utf8_next_char( equals );
 				tmp = equals;
 				do {
-					tmp = strchr( tmp, '"' );
-				} while( tmp && *(tmp -1) == '\\' );
-			} else if( *equals == '\'' ) {
-				equals ++;
-				tmp = equals;
-				do {
-					tmp = strchr( tmp, '\'' );
-				} while( tmp && *(tmp -1) == '\\' );
-			} else if( *equals == '\\' ) {
+					tmp = g_utf8_strchr( tmp,
+							-1, c );
+				} while( tmp && *(tmp-1) == '\\' );
+			} else if( c == '\\' ) {
 				equals ++;
 				if( *equals == '"' ) {
 					equals ++;
@@ -983,12 +1024,14 @@ GSList *screem_markup_get_links( ScreemDTD *dtd, const gchar *text )
 				} else {
 					g_free( temp->data );
 				}
-			} else if( ! g_strcasecmp("src", 
-						  (const gchar*)tmp->data ) ||
+			} else if( ! g_strcasecmp("src",
+					(const gchar*)tmp->data ) ||
 				   ! g_strcasecmp("data",
-						  (const gchar*)tmp->data ) ||
+					(const gchar*)tmp->data ) ||
 				   ! g_strcasecmp("href",
-						  (const gchar*)tmp->data) ) {
+					(const gchar*)tmp->data) ||
+				   ! g_strcasecmp("url",
+					(const gchar*)tmp->data ) ) {
 				/* fallback mode if no dtd is passed,
 				   then we check src="" href="" and data="" */
 				ret = g_slist_prepend( ret, temp->data );
@@ -1084,7 +1127,7 @@ static void process_attrs( GString *str, const gchar *tag,
 			chars = g_utf8_pointer_to_offset( offset,
 					aoffset );
 			g_string_append_len( str, offset,
-					aoffset - offset );//chars );
+					aoffset - offset );
 			offset = aoffset;
 		}
 		offset += strlen( aname );
@@ -1224,10 +1267,10 @@ static void screem_markup_fix_html_links( ScreemSite *site,
 
 		poffset_text = offset_text;
 		offset_text = g_utf8_offset_to_pointer( offset_text,
-							end + 1 );
+							end );
 
 		start += pos;
-		end += pos + 1;
+		end += pos;
 	
 		if( ! tag ) {
 			pos = last;
@@ -1247,7 +1290,9 @@ static void screem_markup_fix_html_links( ScreemSite *site,
 						tag,
 						strlen( "<!-- #BeginEditable " ) ) ) {
 					ineditable = TRUE;
-				} else if( ! strcmp( "<!-- #EndEditable --", tag ) ) {
+				} else if( ! strncmp( "<!-- #EndEditable ", 
+						tag, 
+						strlen( "<!-- #EndEditable " ) ) ) {
 					ineditable = FALSE;
 				}
 
@@ -1270,7 +1315,6 @@ static void screem_markup_fix_html_links( ScreemSite *site,
 				/* append tag as is */
 				g_string_append( str, tag );
 			}
-			g_string_append_c( str, '>' );
 			
 			g_free( tag );
 			g_free( name );
@@ -1312,8 +1356,8 @@ static void screem_markup_fix_css_links( ScreemSite *site,
 
 	gunichar c;
 
-	pageroot = g_dirname( origpagepath );
-	destroot = g_dirname( pagepath );
+	pageroot = g_path_get_dirname( origpagepath );
+	destroot = g_path_get_dirname( pagepath );
 
 	isdir = screem_uri_is_dir( source );
 
@@ -1528,7 +1572,7 @@ void screem_markup_update_from_template( ScreemSite *site,
 	const gchar *template_path;
 	const gchar *page_path;
 	
-	if( ! screem_page_load( page ) ) {
+	if( ! screem_page_load( page, NULL ) ) {
 		return;
 	}
 		
@@ -1655,6 +1699,10 @@ gchar *screem_markup_basic_html( ScreemDTDDB *db, const gchar *doctype,
 	gchar *systemid;
 
 	gchar *text;
+	gchar *tmp;
+	gchar *spaces;
+	gfloat tabwidth;
+	GConfClient *client;
 	
 	const gchar *ntext = "\
 <html>\n\
@@ -1690,6 +1738,26 @@ gchar *screem_markup_basic_html( ScreemDTDDB *db, const gchar *doctype,
 	} else {
 		ntext = frame;
 	}
+
+	client = gconf_client_get_default();
+	tabwidth = gconf_client_get_float( client,
+			"/apps/screem/editor/tabwidth",
+			NULL );
+	if( tabwidth == 0.0 ) {
+		tabwidth = 8.0;
+	}
+	if( gconf_client_get_bool( client,
+				"/apps/screem/editor/spaces_instead_of_tabs",
+				NULL ) ) {
+		spaces = g_new0( gchar, (gint)(tabwidth + 1) );
+		memset( spaces, ' ', tabwidth );
+		tmp = find_text( ntext, "\t", spaces, NULL );
+ 		g_free( spaces );
+	} else {
+		tmp = g_strdup( ntext );
+	}
+	g_object_unref( client );
+	
 	dtd = screem_dtd_db_get_dtd( db, doctype, NULL ); 
 	root = screem_dtd_db_get_root( db, doctype, NULL );
 	if( ! root ) {
@@ -1700,10 +1768,170 @@ gchar *screem_markup_basic_html( ScreemDTDDB *db, const gchar *doctype,
 	text = g_strconcat( "<!DOCTYPE ", root,
 				   " PUBLIC \"", doctype, 
 				   "\" \"", systemid,
-				   "\">", "\n", ntext, NULL );
+				   "\">", "\n", tmp, NULL );
+	g_free( tmp );
 	g_free( systemid );
 	g_free( root );
 
 	return text;
 }
 
+gchar *screem_markup_get_charset( const gchar *text, 
+		gint *retpos, gint *retlen )
+{
+	guint len;
+	guint pos;
+
+	guint start;
+	guint end;
+	gchar *tag;
+	gchar *name;	
+	
+	gchar *ret;
+	
+	len = strlen( text );
+	pos = 0;
+
+	ret = NULL;
+
+	/* first we look for <?xml encoding="" ?>
+	   then we check <meta http-equiv="content-type" />
+	   then we check <meta name="charset" /> 
+	   if we have an <?xml ?> without an encoding="" we
+	   enforce UTF-8 no matter what <meta /> exist */
+	
+	while( ( ! ret ) && ( pos < len ) &&
+		( tag = screem_markup_next_tag( text, pos,
+					       &start, &end, 
+					       &name ) ) ) {
+		g_assert( name );
+		pos = end;
+		if( ! strcmp( "?xml", name ) ) {
+			/* xml PI element */
+			GSList *list;
+			GSList *attr;
+
+			list = screem_markup_build_attributes_list( tag,
+					NULL );
+			for( attr = list; attr; attr = attr->next ) {
+				if(! strcmp( "encoding", 
+					     attr->next->data ) &&
+				   attr->data ) {
+					ret = g_strdup( attr->data );
+					break;
+				} else {
+					attr = NULL;
+					break;
+				}
+				attr = attr->next;
+			}
+				
+			if( ! attr ) {
+				ret = g_strdup( "UTF-8" );
+			}
+
+			g_slist_foreach( list, (GFunc)g_free, NULL );
+			g_slist_free( list );
+		} else if( ! g_strcasecmp( "meta", name ) ) {
+			/* meta element */
+			GSList *list;
+			GSList *attr;
+			gboolean content;
+			
+			list = screem_markup_build_attributes_list( tag,
+					NULL );
+			content = FALSE;
+
+			for( attr = list; attr; attr = attr->next ) {
+				if( ! strcmp( "http-equiv", 
+					      attr->next->data ) && 
+				    attr->data &&
+				    ! strcmp( "content-type",
+					      attr->data ) ) {
+					content = TRUE;
+					break;
+				} else if(! strcmp( "name", 
+						    attr->next->data) &&
+					  attr->data && 
+					  ! strcmp( "charset",
+						    attr->data)){
+					break;
+				}
+				attr = attr->next;
+			}
+			if( attr ) {
+				for( attr = list; attr; 
+				     attr = attr->next ) {
+					if( ! strcmp( "content",
+						      attr->next->data ) ) {
+						break;
+					}
+					attr = attr->next;
+				}
+			}
+			if( attr && attr->data && content ) {
+				gchar *val;
+
+				val = strstr( attr->data, "charset=" );
+				if( val ) {
+					GString *tmp;
+
+					val += strlen( "charset=" );
+					tmp = g_string_new( NULL );
+					while( *val != ' ' &&
+					       *val != '\0' &&
+					       *val != '"' ) {
+						g_string_append_c( tmp,
+							   *val);
+						val ++;
+					}
+					ret = g_strdup( tmp->str );
+					g_string_free( tmp, TRUE );
+				}
+			} else if( attr ) {
+				ret = g_strdup( attr->data );
+			}
+
+			g_slist_foreach( list, (GFunc)g_free, NULL );
+			g_slist_free( list );
+		} else if( g_strcasecmp( "html", name ) &&
+			   g_strcasecmp( "head", name ) &&
+			   g_strcasecmp( "title", name ) &&
+			   g_strcasecmp( "link", name ) &&
+			   g_strcasecmp( "script", name ) &&
+			   g_strcasecmp( "style", name ) &&
+			   g_strcasecmp( "!--", name ) &&
+			   g_strcasecmp( "base", name ) &&
+			   g_strcasecmp( "isindex", name ) &&
+			   g_strcasecmp( "!DOCTYPE", name ) &&
+			   *name != '/' && *name != '?' ) {
+			/* limiting case, bit of a hack for html,
+			   valid xml docs are supposed to have the
+			   <?xml ?> element and have indeterminate 
+			   element names so we can't stop if the doc is
+			   invalid */
+			pos = len;
+		}
+		g_free( tag );
+		g_free( name );
+	}
+
+	if( ret ) {
+		if( retpos ) {
+			*retpos = start;
+		}
+		if( retlen ) {
+			*retlen = end - start;
+		}
+	} else {
+		if( retpos ) {
+			*retpos = 0;
+		}
+		if( retlen ) {
+			*retlen = len;
+		}
+	}
+
+	return ret;
+}
+
