@@ -1,12 +1,35 @@
+/*  Screem:  screem-mdi.c
+ *
+ *  Copyright (C) 2001 David A Knight
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ *  For contact information with the author of this source code please see
+ *  the AUTHORS file.  If there is no AUTHORS file present then check the
+ *  about box under the help menu for a contact address
+ */
 #include "config.h"
 
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnome/gnome-macros.h>
 
 #include <libgnomeui/gnome-popup-menu.h>
 
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
+#include <libgnomevfs/gnome-vfs-mime-handlers.h>
 
 #include <gconf/gconf-client.h>
 
@@ -30,10 +53,10 @@
 
 #include "screemmarshal.h"
 
-#include "libegg/menu/egg-menu-merge.h"
-
 #include "ephy-event-box.h"
 
+#include "gdl-stock.h"
+
 static void screem_mdi_class_init( ScreemMDIClass *klass );
 static void screem_mdi_instance_init( ScreemMDI *mdi );
 static void screem_mdi_finalize( GObject *object );
@@ -44,7 +67,7 @@ static void screem_mdi_get_prop( GObject *object, guint prop_id,
 
 
 static void screem_mdi_window_set( ScreemMDI *mdi );
-static void screem_mdi_switch_doc( EggAction *action, gpointer data );
+static void screem_mdi_switch_doc( GtkAction *action, gpointer data );
 static GtkWidget *screem_mdi_create_tab( ScreemMDI *mdi, 
 					 ScreemPage *page );
 static gboolean screem_mdi_notebook_add( ScreemMDI *mdi, 
@@ -65,7 +88,7 @@ static void screem_mdi_tab_popup_switch( GtkWidget *widget,
 static void screem_mdi_real_remove_document( ScreemMDI *mdi,
 					     ScreemPage *page,
 					     gint index);
-static void screem_mdi_document_dialog( EggAction *action,
+static void screem_mdi_document_dialog( GtkAction *action,
 					gpointer data );
 static void screem_mdi_documents_change( GtkTreeSelection *sel,
 					 gpointer data );
@@ -109,8 +132,8 @@ struct ScreemMDIPrivate {
 	GtkTooltips *tip;
 
 	ScreemWindow *window;
-	EggMenuMerge *merge;
-	EggActionGroup *group;
+	GtkUIManager *merge;
+	GtkActionGroup *group;
 	
 	ScreemPage *current;
 
@@ -149,8 +172,9 @@ static void screem_mdi_class_init( ScreemMDIClass *klass )
 			      G_STRUCT_OFFSET( ScreemMDIClass, 
 					       changed),
 			      NULL, NULL,
-			      screem_marshal_VOID__OBJECT,
-			      G_TYPE_NONE, 1,
+			      screem_marshal_VOID__OBJECT_OBJECT,
+			      G_TYPE_NONE, 2,
+			      G_TYPE_OBJECT,
 			      G_TYPE_OBJECT );
 }
 
@@ -169,14 +193,15 @@ static void screem_mdi_instance_init( ScreemMDI *mdi )
 	gtk_notebook_set_tab_pos( priv->notebook, position );
 	
 	gtk_notebook_set_show_border( priv->notebook, FALSE );
-	gtk_notebook_set_tab_border( priv->notebook, 0 );
 	g_signal_connect( G_OBJECT( priv->notebook ), "switch_page",
 			  G_CALLBACK( screem_mdi_notebook_change ),
 			  mdi );
 	g_object_set_data( G_OBJECT( priv->notebook ), "mdi", mdi );
 	gtk_widget_show( GTK_WIDGET( priv->notebook ) );
 
-	priv->group = egg_action_group_new( "ScreemMDI" );
+	priv->group = gtk_action_group_new( "ScreemMDI" );
+	gtk_action_group_set_translation_domain( priv->group, 
+			GETTEXT_PACKAGE );	
 
 	priv->store = gtk_list_store_new( MDI_MAX_COLS,
 					  G_TYPE_STRING,
@@ -204,9 +229,11 @@ static void screem_mdi_finalize( GObject *object )
 /*	we should only be called from the ScreemWindow finalize
  	signal so don't bother doing these
   	
-  	egg_menu_merge_remove_action_group( priv->merge, priv->group );
-	egg_menu_merge_remove_ui( priv->merge, priv->uiid );*/
+  	gtk_ui_manager_remove_action_group( priv->merge, priv->group );
+	gtk_ui_manager_remove_ui( priv->merge, priv->uiid );*/
 
+	g_object_unref( priv->group );
+	
 	g_list_free( priv->documents );
 
 	if( priv->dialog ) {
@@ -262,18 +289,18 @@ static void screem_mdi_window_set( ScreemMDI *mdi )
 	ScreemMDIPrivate *priv;
 	GConfClient *client;
 	GString *str;
-	const gchar *start = "<Root><menu><submenu name=\"Documents\"><placeholder name=\"DocumentsTop\">";
-	const gchar *pattern = "<menuitem name=\"%s\" verb=\"%s\" />";
-	const gchar *end = "</placeholder></submenu></menu></Root>";
+	const gchar *start = "<ui><menubar><menu action=\"Documents\"><placeholder name=\"DocumentNav\">";
+	const gchar *pattern = "<menuitem action=\"%s\" />";
+	const gchar *end = "</placeholder></menu></menubar></ui>";
 	gint docs;
 	gint i;
-	EggActionGroupEntry *entries;
-	EggAction *action;	
+	GtkActionEntry *entries;
+	GtkAction *action;	
 	gchar *name;
 	
 	priv = mdi->priv;
 
-	priv->merge = EGG_MENU_MERGE( priv->window->merge );
+	priv->merge = GTK_UI_MANAGER( priv->window->merge );
 	
 	client = gconf_client_get_default();
 	
@@ -284,41 +311,40 @@ static void screem_mdi_window_set( ScreemMDI *mdi )
 	if( docs < 1 ) {
 		priv->docs = docs = 12;
 	}
+	str = g_string_new( start );
 
+	g_string_append( str, "<menuitem action=\"SwitchDocManage\"/>" );
+	g_string_append( str, "</placeholder><placeholder name=\"DocumentsBottom\">" );
+	
 	/* + 1 so we can have a more... entry to show
 	   a dialog with a list of all documents */
-	entries = g_new0( EggActionGroupEntry, docs + 1 );
+	entries = g_new0( GtkActionEntry, docs + 1 );
 
 	entries[ docs ].name = "SwitchDocManage";
 	entries[ docs ].label = _( "_Manage Documents" );
 	entries[ docs ].tooltip = _( "Show a dialog listing all documents open in this window," );
 	entries[ docs ].stock_id = NULL;
 	entries[ docs ].callback = G_CALLBACK( screem_mdi_document_dialog );
-	entries[ docs ].user_data = mdi;
-
-	str = g_string_new( start );
 	for( i = 0; i < docs; ++ i ) {
 		name = g_strdup_printf( "SwitchDoc%i", i );
-		g_string_append_printf( str, pattern, name, name );
+		g_string_append_printf( str, pattern, name );
 		entries[ i ].name = name;
 		entries[ i ].label = _( "No Document" );
 		entries[ i ].tooltip = _( "No Document" );
 		entries[ i ].stock_id = NULL;
-		if( i < 10 ) {
+		if( i < 9 ) {
 			entries[ i ].accelerator = g_strdup_printf( "<Alt>%i", i + 1 );
 		}
 		entries[ i ].callback = G_CALLBACK( screem_mdi_switch_doc );
-		entries[ i ].user_data = mdi;
 	}
-	g_string_append( str, "<separator name=\"documenttopsep\"/><menuitem name=\"SwitchDocManage\" verb=\"SwitchDocManage\"/>" );
 	
 	g_string_append( str, end );
 
-	egg_action_group_add_actions( priv->group, entries, 
-					(guint)docs + 1 );
+	gtk_action_group_add_actions( priv->group, entries, 
+					(guint)docs + 1, mdi );
 	for( i = 0; i < docs; ++ i ) {
 		name = g_strdup_printf( "SwitchDoc%i", i );
-		action = egg_action_group_get_action( priv->group,
+		action = gtk_action_group_get_action( priv->group,
 						name );
 		g_object_set( G_OBJECT( action ),
 				"visible", FALSE,
@@ -326,22 +352,22 @@ static void screem_mdi_window_set( ScreemMDI *mdi )
 		g_free( name );
 
 		/* sneak this in here to avoid an extra for loop */
-		g_free( entries[ i ].name );
-		g_free( entries[ i ].accelerator );
+		g_free( (gchar*)entries[ i ].name );
+		g_free( (gchar*)entries[ i ].accelerator );
 	}
 	g_free( entries );
 	
-	egg_menu_merge_insert_action_group( priv->merge, priv->group,
+	gtk_ui_manager_insert_action_group( priv->merge, priv->group,
 					    1 );
-	priv->uiid = egg_menu_merge_add_ui_from_string( priv->merge,
+	priv->uiid = gtk_ui_manager_add_ui_from_string( priv->merge,
 							str->str,
 							str->len,
 							NULL );
 
-	action = egg_action_group_get_action( EGG_ACTION_GROUP( priv->window->action_group ), "PreviousDocument" );
+	action = gtk_action_group_get_action( GTK_ACTION_GROUP( priv->window->action_group ), "PreviousDocument" );
 	g_object_set( G_OBJECT( action ),
 			"sensitive", FALSE, NULL );
-	action = egg_action_group_get_action( EGG_ACTION_GROUP( priv->window->action_group ), "NextDocument" );
+	action = gtk_action_group_get_action( GTK_ACTION_GROUP( priv->window->action_group ), "NextDocument" );
 	g_object_set( G_OBJECT( action ),
 			"sensitive", FALSE, NULL );
 	
@@ -350,16 +376,20 @@ static void screem_mdi_window_set( ScreemMDI *mdi )
 	g_object_unref( client );
 }
 
-static void screem_mdi_switch_doc( EggAction *action, gpointer data )
+static void screem_mdi_switch_doc( GtkAction *action, gpointer data )
 {
 	ScreemMDI *mdi;
 	GList *tmp;
 	gint i;
+	gchar *name;
 	
 	mdi = SCREEM_MDI( data );
 
-	sscanf( action->name, "SwitchDoc%i", &i );
-
+	g_object_get( G_OBJECT( action ), "name", &name, NULL );
+	
+	sscanf( name, "SwitchDoc%i", &i );
+	g_free( name );
+	
 	tmp = g_list_nth( mdi->priv->documents, i );
 	if( tmp ) {
 		screem_mdi_set_current( mdi, SCREEM_PAGE( tmp->data ) );
@@ -404,6 +434,9 @@ static GtkWidget *screem_mdi_create_tab( ScreemMDI *mdi,
 	g_signal_connect( G_OBJECT( page ), "notify::changed",
 			  G_CALLBACK( screem_mdi_page_change ), 
 			  clabel );
+	g_signal_connect( G_OBJECT( page ), "notify::mime-type",
+			  G_CALLBACK( screem_mdi_page_change ), 
+			  clabel );
 	
 	g_signal_connect( G_OBJECT( page ), "can_undo",
 			  G_CALLBACK( screem_mdi_can_undo ),
@@ -411,10 +444,7 @@ static GtkWidget *screem_mdi_create_tab( ScreemMDI *mdi,
 	g_signal_connect( G_OBJECT( page ), "can_redo",
 			  G_CALLBACK( screem_mdi_can_redo ),
 			  mdi );
-	
-	screem_mdi_page_path_change( page, NULL, label );
-	screem_mdi_page_change( page, NULL, clabel );
-					
+						
 	gtk_widget_show( label );
 	gtk_widget_show( clabel );
 	event = ephy_event_box_new();
@@ -424,7 +454,7 @@ static GtkWidget *screem_mdi_create_tab( ScreemMDI *mdi,
 	gtk_box_pack_start( GTK_BOX( tbox ), event, TRUE, TRUE, 0 );
 	
 	close = gtk_button_new();
-	img = gtk_image_new_from_stock( GTK_STOCK_CLOSE,
+	img = gtk_image_new_from_stock( GDL_STOCK_CLOSE,
 					GTK_ICON_SIZE_MENU );
 	gtk_icon_size_lookup_for_settings( gtk_widget_get_settings( img ),
 			GTK_ICON_SIZE_MENU, &w, &h );
@@ -439,6 +469,10 @@ static GtkWidget *screem_mdi_create_tab( ScreemMDI *mdi,
 			mdi );
 	
 	box = gtk_hbox_new( 0, FALSE );
+	img = gtk_image_new_from_stock( GTK_STOCK_NEW,
+			GTK_ICON_SIZE_MENU );
+	g_object_set_data( G_OBJECT( clabel ), "img", img );
+	gtk_box_pack_start( GTK_BOX( box ), img, FALSE, FALSE, 0 );
 	gtk_box_pack_start( GTK_BOX( box ), label, TRUE, TRUE, 0 );
 	gtk_box_pack_start( GTK_BOX( box ), clabel, FALSE, FALSE, 0 );
 	gtk_widget_show_all( box );
@@ -462,22 +496,36 @@ static GtkWidget *screem_mdi_create_tab( ScreemMDI *mdi,
 
 	gtk_widget_show_all( tbox );
 	
+	screem_mdi_page_path_change( page, NULL, label );
+	screem_mdi_page_change( page, NULL, clabel );
+	
 	return tbox;
 }
 
 static gboolean screem_mdi_notebook_add( ScreemMDI *mdi, 
 					 ScreemPage *page )
 {
+	ScreemApplication *app;
 	ScreemMDIPrivate *priv;
 	GtkWidget *tab;
 	GtkWidget *hack;
 	const gchar *pathname;
 	gboolean loaded;
+	gchar *msg;
+	GError *error;
 
 	priv = mdi->priv;
 	
 	pathname = screem_page_get_pathname( page );
-
+	
+	if( priv->window && pathname ) {
+		msg = g_strdup_printf( _( "Loading page: %s" ), pathname );
+		screem_window_show_message( priv->window, msg, FALSE );
+		g_free( msg );
+		app = priv->window->application;
+		screem_application_set_cursor( app, GDK_WATCH );
+	}
+	
 	g_signal_handlers_block_matched( G_OBJECT( priv->notebook ),
 					G_SIGNAL_MATCH_FUNC|
 					G_SIGNAL_MATCH_DATA,
@@ -485,7 +533,7 @@ static gboolean screem_mdi_notebook_add( ScreemMDI *mdi,
 					screem_mdi_notebook_change,
 					mdi );
 
-	if( ! pathname || screem_page_load( page ) ) {
+	if( ! pathname || screem_page_load( page, &error ) ) {
 		tab = screem_mdi_create_tab( mdi, page );
 		hack = gtk_hbox_new( 0, FALSE );
 		gtk_widget_show( hack );
@@ -496,24 +544,49 @@ static gboolean screem_mdi_notebook_add( ScreemMDI *mdi,
 		screem_page_set_open( page, TRUE );
 		loaded = TRUE;
 
+		if( priv->window && pathname ) {
+			msg = g_strdup_printf( _( "Loaded page: %s" ), pathname );
+			screem_window_show_message( priv->window, msg,
+					FALSE );
+			g_free( msg );
+			app = priv->window->application;
+			screem_application_set_cursor( app, 
+					GDK_LAST_CURSOR );
+		}
 	} else {
 		/* failed to open */
-		GtkWidget *alert;
 		gchar *temp;
+		gchar *primary;
+		const gchar *secondary;
 			
-		pathname = screem_page_get_pathname( page );
-		temp = gnome_vfs_unescape_string_for_display( pathname );
-			
-		alert = gtk_message_dialog_new( GTK_WINDOW( priv->window ),
-						GTK_DIALOG_MODAL |
-						GTK_DIALOG_DESTROY_WITH_PARENT,
-						GTK_MESSAGE_ERROR,
-						GTK_BUTTONS_OK,
-						_("Failed to load %s"),
-						temp );
-		g_free( temp );
-		gtk_dialog_run( GTK_DIALOG( alert ) );
-		gtk_widget_destroy( alert );
+		if( priv->window && pathname ) {
+			msg = g_strdup_printf( _( "Failed to load page: %s" ), pathname );
+			screem_window_show_message( priv->window, msg,
+					FALSE );
+			g_free( msg );
+			app = priv->window->application;
+			screem_application_set_cursor( app, 
+					GDK_LAST_CURSOR );
+		}
+		
+		if( pathname ) {
+			temp = gnome_vfs_unescape_string_for_display( pathname );
+			secondary = error->message;
+		} else {
+			temp = g_strdup( "" );
+			secondary = "";
+		}
+		primary = g_strdup_printf( _( "Failed to load %s" ),
+				temp );
+		screem_hig_alert( GTK_STOCK_DIALOG_ERROR,
+				primary, secondary, 
+				GTK_WIDGET( priv->window ) );
+		g_free( primary );
+	
+		if( error ) {
+			g_error_free( error );
+		}
+		
 		loaded = FALSE;
 	}
 
@@ -578,7 +651,7 @@ static void screem_mdi_page_path_change( ScreemPage *page,
 	gchar *temp;
 	gint i;
 	gchar *name;
-	EggAction *action;
+	GtkAction *action;
 	GtkTreeIter it;
 	gboolean modified;
 	ScreemPage *cpage;
@@ -610,7 +683,7 @@ static void screem_mdi_page_path_change( ScreemPage *page,
 	}
 	if( i < priv->docs ) {
 		name = g_strdup_printf( "SwitchDoc%i", i );
-		action = egg_action_group_get_action( priv->group,
+		action = gtk_action_group_get_action( priv->group,
 							name );
 		g_free( name );
 		name = screem_escape_underlines( filename ); 
@@ -681,6 +754,8 @@ static void screem_mdi_page_change( ScreemPage *page,
 	ScreemMDI *mdi;
 	ScreemMDIPrivate *priv;
 	GtkWidget *label;
+	GtkWidget *img;
+	GdkPixbuf *pixbuf;
 	ScreemWindow *window;
 	const gchar *pathname;
 	gchar *temp;
@@ -689,25 +764,43 @@ static void screem_mdi_page_change( ScreemPage *page,
 	GtkTreeIter it;
 	ScreemPage *cpage;
 
+	const gchar *type;
+	const gchar *showtype;
+	
 	label = GTK_WIDGET( data );
+	img = g_object_get_data( G_OBJECT( label ), "img" );
 	mdi = SCREEM_MDI( g_object_get_data( G_OBJECT( label ), "mdi" ) );
 	priv = mdi->priv;
 	window = priv->window;
+	
+	type = screem_page_get_mime_type( page );
+	showtype = gnome_vfs_mime_get_description( type );
+	if( ! showtype ) {
+		showtype = type;
+	}
+
+	pixbuf = screem_file_browser_get_icon( browser,
+			NULL, 16, 16, FALSE, (gchar**)&type );
+	if( pixbuf ) {
+		gtk_image_set_from_pixbuf( GTK_IMAGE( img ), pixbuf );
+		g_object_unref( pixbuf );
+	}
 
 	pathname = screem_page_get_pathname( page );
 	if( ! pathname ) {
 		pathname = _( "Untitled" );
+		temp = g_strconcat( pathname, " ", showtype, NULL );
+	} else {
+		temp = gnome_vfs_unescape_string_for_display( pathname );
 	}
-
-	temp = gnome_vfs_unescape_string_for_display( pathname );
 	
 	modified = screem_page_get_changed( page );
 		
 	if( modified ) {
-		gtk_label_set( GTK_LABEL( label ), "*" );
-		title = g_strdup_printf( "%s * - Screem", temp );
+		gtk_label_set_text( GTK_LABEL( label ), "*" );
+		title = g_strdup_printf( "* %s - Screem", temp );
 	} else {
-		gtk_label_set( GTK_LABEL( label ), "" );
+		gtk_label_set_text( GTK_LABEL( label ), "" );
 		title = g_strdup_printf( "%s - Screem", temp );
 	}
 
@@ -784,8 +877,8 @@ static void screem_mdi_tab_clicked( GtkWidget *label,
 		GtkWidget *menuitem;
 		GList *docs;
 		
-		menu = egg_menu_merge_get_widget( priv->merge,
-						"/popups/tabmenu" );
+		menu = gtk_ui_manager_get_widget( priv->merge,
+						"/tabmenu" );
 
 		g_object_set_data( G_OBJECT( window ), 
 				"tab_popup", page );
@@ -824,7 +917,7 @@ static void screem_mdi_tab_clicked( GtkWidget *label,
 			g_free( base );
 		}
 		
-		gnome_popup_menu_do_popup_modal( menu, 0, 0, event, 0, label );
+		screem_popup_menu_do_popup_modal( menu, 0, 0, event, 0, label );
 		g_object_set_data( G_OBJECT( window ), "tab_popup", NULL );
 		
 		/* remove menuitem from menu as we don't want the
@@ -849,7 +942,7 @@ static void screem_mdi_real_remove_document( ScreemMDI *mdi,
 	gchar *name;
 	gchar *filename;
 	gchar *pathname;
-	EggAction *action;
+	GtkAction *action;
 	GtkTreeIter it;
 	ScreemPage *cpage;
 	
@@ -910,7 +1003,7 @@ static void screem_mdi_real_remove_document( ScreemMDI *mdi,
 						&pathname );
 			
 			name = g_strdup_printf( "SwitchDoc%i", index );
-			action = egg_action_group_get_action( priv->group,
+			action = gtk_action_group_get_action( priv->group,
 								name );
 			g_free( name );
 			name = screem_escape_underlines( filename ); 
@@ -930,7 +1023,7 @@ static void screem_mdi_real_remove_document( ScreemMDI *mdi,
 		   reached */
 		while( index < priv->docs ) {
 			name = g_strdup_printf( "SwitchDoc%i", index );
-			action = egg_action_group_get_action( priv->group,
+			action = gtk_action_group_get_action( priv->group,
 								name );
 			g_object_set( G_OBJECT( action ),
 					"label", _( "No Document" ),
@@ -944,27 +1037,33 @@ static void screem_mdi_real_remove_document( ScreemMDI *mdi,
 	/* we don't get a changed signal when the last
 	   page is closed, so we need handle it here */
 	if( ! priv->documents ) {
-		action = egg_action_group_get_action( EGG_ACTION_GROUP( priv->window->action_group ), "PreviousDocument" );
+		action = gtk_action_group_get_action( GTK_ACTION_GROUP( priv->window->action_group ), "PreviousDocument" );
 		g_object_set( G_OBJECT( action ),
 				"sensitive", FALSE,
 				NULL );
-		action = egg_action_group_get_action( EGG_ACTION_GROUP( priv->window->action_group ), "NextDocument" );
+		action = gtk_action_group_get_action( GTK_ACTION_GROUP( priv->window->action_group ), "NextDocument" );
 		g_object_set( G_OBJECT( action ),
 				"sensitive", FALSE,
 				NULL );
 		g_signal_emit( G_OBJECT( mdi ), 
 				screem_mdi_signals[ CHANGED ],
-			0, NULL );
+				0, NULL, page );
 	}
 }
 
+static void screem_mdi_document_dialog_response( GtkWidget *dialog,
+		gint id, gpointer data )
+{
+	screem_session_store_dialog( SCREEM_SESSION( data ), dialog );
+}
 
-static void screem_mdi_document_dialog( EggAction *action,
+static void screem_mdi_document_dialog( GtkAction *action,
 					gpointer data )
 {
 	ScreemMDI *mdi;
 	ScreemMDIPrivate *priv;
-	GConfClient *client;
+	ScreemApplication *app;
+	ScreemSession *session;
 	gchar *glade_path;
 	GladeXML *xml;
 	GtkCellRenderer *renderer;
@@ -975,16 +1074,19 @@ static void screem_mdi_document_dialog( EggAction *action,
 	mdi = SCREEM_MDI( data );
 	priv = mdi->priv;
 	
+	app = SCREEM_APPLICATION( priv->window->application );
+	session = screem_application_get_session( app );
+	
 	if( ! priv->dialog ) {
-		client = gconf_client_get_default();
-		glade_path = gconf_client_get_string( client,
-					      "/apps/screem/general/glade_path",
-					      NULL );
+		glade_path = screem_get_glade_path();
 		xml = glade_xml_new( glade_path, "documents", NULL );
 		g_free( glade_path );
-		g_object_unref( client );
 
 		priv->dialog = glade_xml_get_widget( xml, "documents" );
+		g_signal_connect( G_OBJECT( priv->dialog ),
+				"response",
+				G_CALLBACK( screem_mdi_document_dialog_response ),
+				session );
 		g_signal_connect( G_OBJECT( priv->dialog ),
 				"delete_event",
 				G_CALLBACK( gtk_widget_destroy ),
@@ -1042,6 +1144,10 @@ static void screem_mdi_document_dialog( EggAction *action,
 		g_object_set_data( G_OBJECT( widget ), "mdi", mdi );
 		
 		glade_xml_signal_autoconnect( xml );
+		screem_session_restore_dialog( session, priv->dialog );
+	}
+	if( ! GTK_WIDGET_VISIBLE( priv->dialog ) ) {
+		screem_session_restore_dialog( session, priv->dialog );
 	}
 	gtk_widget_show( priv->dialog );
 }
@@ -1204,7 +1310,7 @@ static void screem_mdi_can_undo( GtkSourceBuffer *buffer,
 	ScreemMDI *mdi;
 	ScreemMDIPrivate *priv;
 	ScreemPage *page;
-	EggAction *action;
+	GtkAction *action;
 	
 	mdi = SCREEM_MDI( data );
 	priv = mdi->priv;
@@ -1212,7 +1318,7 @@ static void screem_mdi_can_undo( GtkSourceBuffer *buffer,
 	page = screem_mdi_get_current( mdi );
 	
 	if( page && ( GTK_SOURCE_BUFFER( page ) == buffer ) ) {
-		action = egg_action_group_get_action( EGG_ACTION_GROUP( priv->window->action_group ), "Undo" );
+		action = gtk_action_group_get_action( GTK_ACTION_GROUP( priv->window->action_group ), "Undo" );
 		g_object_set( G_OBJECT( action ), "sensitive", can_undo,
 				NULL );
 	}
@@ -1225,14 +1331,14 @@ static void screem_mdi_can_redo( GtkSourceBuffer *buffer,
 	ScreemMDI *mdi;
 	ScreemMDIPrivate *priv;
 	ScreemPage *page;
-	EggAction *action;
+	GtkAction *action;
 	
 	mdi = SCREEM_MDI( data );
 	priv = mdi->priv;
 	
 	page = screem_mdi_get_current( mdi );
 	if( page && ( GTK_SOURCE_BUFFER( page ) == buffer ) ) {
-		action = egg_action_group_get_action( EGG_ACTION_GROUP( priv->window->action_group ), "Redo" );
+		action = gtk_action_group_get_action( GTK_ACTION_GROUP( priv->window->action_group ), "Redo" );
 		g_object_set( G_OBJECT( action ), "sensitive", can_redo,
 				NULL );
 	}
@@ -1243,16 +1349,16 @@ static void screem_mdi_close_button( GtkWidget *widget,
 {
 	ScreemPage *page;
 	ScreemMDIPrivate *priv;
-	EggActionGroup *group;
-	EggAction *action;
+	GtkActionGroup *group;
+	GtkAction *action;
 	
 	priv = mdi->priv;
 	group = priv->window->action_group;
-	action = egg_action_group_get_action( group, "Close Document" );
+	action = gtk_action_group_get_action( group, "Close Document" );
 	
 	page = g_object_get_data( G_OBJECT( widget ), "page" );
 	g_object_set_data( G_OBJECT( priv->window ), "tab_popup", page );
-	egg_action_activate( action );
+	gtk_action_activate( action );
 	
 	g_object_set_data( G_OBJECT( priv->window ), "tab_popup", NULL );
 }
@@ -1348,7 +1454,7 @@ gboolean screem_mdi_set_current( ScreemMDI *mdi, ScreemPage *page )
 	gint len;
 	gint index;
 	gboolean ret;
-	EggAction *action;
+	GtkAction *action;
 	gboolean can;
 	
 	g_return_val_if_fail( SCREEM_IS_MDI( mdi ), FALSE );
@@ -1386,10 +1492,10 @@ gboolean screem_mdi_set_current( ScreemMDI *mdi, ScreemPage *page )
 	
 		len = g_list_length( priv->documents );
 
-		action = egg_action_group_get_action( EGG_ACTION_GROUP( priv->window->action_group ), "PreviousDocument" );
+		action = gtk_action_group_get_action( GTK_ACTION_GROUP( priv->window->action_group ), "PreviousDocument" );
 		g_object_set( G_OBJECT( action ),
 				"sensitive", index > 0, NULL );
-		action = egg_action_group_get_action( EGG_ACTION_GROUP( priv->window->action_group ), "NextDocument" );
+		action = gtk_action_group_get_action( GTK_ACTION_GROUP( priv->window->action_group ), "NextDocument" );
 		g_object_set( G_OBJECT( action ),
 				"sensitive", index < ( len - 1 ), 
 				NULL );
@@ -1402,14 +1508,14 @@ gboolean screem_mdi_set_current( ScreemMDI *mdi, ScreemPage *page )
 						mdi );
 		g_signal_emit( G_OBJECT( mdi ),
 			       screem_mdi_signals[ CHANGED ],
-			       0, page );
+			       0, page, cpage );
 	
 		if( page ) {
-			action = egg_action_group_get_action( EGG_ACTION_GROUP( priv->window->action_group ), "Undo" );
+			action = gtk_action_group_get_action( GTK_ACTION_GROUP( priv->window->action_group ), "Undo" );
 			can = gtk_source_buffer_can_undo( GTK_SOURCE_BUFFER( page ) );
 			g_object_set( G_OBJECT( action ), "sensitive", can,
 				NULL );
-			action = egg_action_group_get_action( EGG_ACTION_GROUP( priv->window->action_group ), "Redo" );
+			action = gtk_action_group_get_action( GTK_ACTION_GROUP( priv->window->action_group ), "Redo" );
 			can = gtk_source_buffer_can_redo( GTK_SOURCE_BUFFER( page ) );
 			g_object_set( G_OBJECT( action ), "sensitive", can,
 					NULL );
