@@ -32,20 +32,14 @@
 
 #include <glade/glade.h>
 
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnome/gnome-util.h>
-#include <libgnomeui/gnome-file-entry.h>
-#include <libgnomeui/gnome-entry.h>
-#include <libgnomeui/gnome-color-picker.h>
+#include <libgnomeui/libgnomeui.h>
 
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 
-#include <gtk/gtkcombo.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtktogglebutton.h>
-#include <gtk/gtknotebook.h>
-#include <gtk/gtkimage.h>
+#include <gtk/gtk.h>
 
 #include <string.h>
 
@@ -59,33 +53,12 @@
 
 #include "fileops.h"
 
-void screem_page_druid_finish( GtkWidget *widget );
-void screem_page_druid_cancel( GtkWidget *widget );
-
-void screem_page_druid_switch_page( GtkNotebook *book,
-				    GtkNotebookPage *page,
-				    guint page_num );
-
-gboolean screem_page_druid_init_details( GtkWidget *widget);
-gboolean screem_page_druid_init_templates( GtkWidget *widget );
-gboolean screem_page_druid_init_colours( GtkWidget *widget );
-gboolean screem_page_druid_init_preview( GtkWidget *widget );
+#include "support.h"
 
+static void charset_changed( GtkWidget *entry, GtkWidget *combo );
 static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 					   ScreemWindow *window );
 
-
-/* MUST have an entry in the array for each page in the druid,
-   use NULL if there is to be no init function */
-typedef gboolean (*InitPage)( GtkWidget *widget );
-static const InitPage init_funcs[] = {
-	NULL,
-	screem_page_druid_init_details,
-	screem_page_druid_init_templates,
-	screem_page_druid_init_colours,
-	screem_page_druid_init_preview
-};
-
 void screem_page_druid_begin( ScreemApplication *application,
 			      ScreemWindow *window )
 {
@@ -93,18 +66,18 @@ void screem_page_druid_begin( ScreemApplication *application,
 	ScreemPage *page;
 	GladeXML *xml;
 	GtkWidget *druid;
-	GtkWidget *book;
-	GtkWidget *image;
 	gchar *gladepath;
+	ScreemSession *session;
+	
+	session = screem_application_get_session( application );
 
 	site = screem_window_get_current( window );
 
-	gladepath = gconf_client_get_string( application->client,
-					     "/apps/screem/general/glade_path",
-					     NULL );
-	xml = glade_xml_new( gladepath, "pagedruid", NULL );
+	gladepath = screem_get_glade_path();
+	xml = glade_xml_new( gladepath, "new_page_druid", NULL );
+	g_free( gladepath );
 
-	druid = glade_xml_get_widget( xml, "pagedruid" );
+	druid = glade_xml_get_widget( xml, "new_page_druid" );
 	page = screem_page_new( G_OBJECT( application ) );
 
 	g_object_set_data( G_OBJECT( druid ), "page", page );
@@ -112,41 +85,44 @@ void screem_page_druid_begin( ScreemApplication *application,
 	g_object_set_data( G_OBJECT( druid ), "window", window );
 	g_object_set_data( G_OBJECT( druid ), "site", site );
 
-	book = glade_xml_get_widget( xml, "druid-notebook" );
-	gtk_notebook_set_show_tabs( GTK_NOTEBOOK( book ), FALSE );
-
-	image = glade_xml_get_widget( xml, "druid-logo" );
-	gtk_image_set_from_file( GTK_IMAGE( image ),
-				 DATADIR"/pixmaps/screem.png" );
-
 	glade_xml_signal_autoconnect( xml );
 
 	gtk_window_set_transient_for( GTK_WINDOW( druid ),
 				      GTK_WINDOW( window ) );
 
-	screem_page_druid_switch_page( GTK_NOTEBOOK( book ),
-				       NULL, 0 );
+	screem_session_restore_dialog( session, druid );
+
+	gtk_widget_show_all( druid );
 }
 
-
-void screem_page_druid_finish( GtkWidget *widget )
+void screem_page_druid_finish( GnomeDruidPage *druid_page,
+		GtkWidget *widget )
 {
 	ScreemSite *site;
 	ScreemPage *page;
+	const gchar *pathname;
 	ScreemWindow *window;
 	GladeXML *xml;
-	GtkWidget *book;
-
-	xml = glade_get_widget_tree( GTK_WIDGET( widget ) );
-	widget = glade_xml_get_widget( xml, "pagedruid" );
+	ScreemApplication *application;
+	ScreemSession *session;
+	GError *error;
+	
+	xml = glade_get_widget_tree( GTK_WIDGET( druid_page ) );
+	widget = glade_xml_get_widget( xml, "new_page_druid" );
 
 	site = SCREEM_SITE( g_object_get_data( G_OBJECT( widget ), "site" ) );
 	page = SCREEM_PAGE( g_object_get_data( G_OBJECT( widget ), "page" ) );
 	window = SCREEM_WINDOW( g_object_get_data( G_OBJECT( widget ), 
 						   "window" ) );
+	application = SCREEM_APPLICATION( g_object_get_data( G_OBJECT( widget ),
+							     "application" ) );
 
+	pathname = screem_page_get_pathname( page );
 	screem_page_set_changed( page, TRUE );
-	if( screem_page_save( page ) ) {
+	error = NULL;
+	if( ( ! pathname ) ||
+		   ( overwrite( pathname ) && 
+		     screem_page_save( page, &error ) ) ) {
 		if( ! screem_site_add_page( site, page ) ) {
 			/* already got a page in the site with the
 			   same pathname, FIXME: what now? */
@@ -157,24 +133,26 @@ void screem_page_druid_finish( GtkWidget *widget )
 			
 		}
 	} else {
-		GtkWidget *msg;
+		/* failed to save the page */
+		gchar *primary;
+		const gchar *secondary;
 
-		msg = gtk_message_dialog_new( GTK_WINDOW( window ),
-					GTK_DIALOG_MODAL |
-					GTK_DIALOG_DESTROY_WITH_PARENT,
-					GTK_MESSAGE_ERROR,
-					GTK_BUTTONS_OK,
-					_( "Failed to create %s" ),
-					screem_page_get_pathname( page ) );
-		gtk_dialog_run( GTK_DIALOG( msg ) );
-		gtk_widget_destroy( msg );
-		g_object_unref( page );
+		primary = g_strdup_printf( _( "Failed to create: %s" ),
+				pathname );
+		secondary = "";
+		if( error ) {
+			secondary = error->message;
+		}
+		screem_hig_alert( GTK_STOCK_DIALOG_ERROR,
+				primary, secondary,
+				GTK_WIDGET( window ) );
+		if( error ) {
+			g_error_free( error );
+		}
 	}
-
-	book = glade_xml_get_widget( xml, "druid-notebook" );
-	g_signal_handlers_disconnect_by_func( G_OBJECT( book ),
-					      screem_page_druid_switch_page,
-					      NULL );
+	g_object_unref( page );
+	session = screem_application_get_session( application );
+	screem_session_store_dialog( session, widget );
 
 	gtk_widget_destroy( widget );
 }
@@ -184,144 +162,152 @@ void screem_page_druid_cancel( GtkWidget *widget )
 {
 	ScreemPage *page;
 	GladeXML *xml;
-	GtkWidget *book;
+	ScreemApplication *app;
+	ScreemSession *session;
 
 	xml = glade_get_widget_tree( widget );
-	widget = glade_xml_get_widget( xml, "pagedruid" );
+	widget = glade_xml_get_widget( xml, "new_page_druid" );
 
 	page = SCREEM_PAGE( g_object_get_data( G_OBJECT( widget ), "page" ) );
 	g_object_unref( page );
 
-	book = glade_xml_get_widget( xml, "druid-notebook" );
-	g_signal_handlers_disconnect_by_func( G_OBJECT( book ),
-					      screem_page_druid_switch_page,
-					      NULL );
+	app = SCREEM_APPLICATION( g_object_get_data( G_OBJECT( widget ),
+				"application" ) );
+	session = screem_application_get_session( app );
+	screem_session_store_dialog( session, widget );
 
 	gtk_widget_destroy( widget );
 }
 
-void screem_page_druid_switch_page( GtkNotebook *book,
-				    GtkNotebookPage *page,
-				    guint page_num )
+void screem_page_druid_setup_prepare( GnomeDruidPage *druid_page,
+		GtkWidget *druid )
 {
 	GladeXML *xml;
-
-	GtkWidget *back;
-	GtkWidget *forward;
-	GtkWidget *apply;
-	GtkWidget *current;
-
-	xml = glade_get_widget_tree( GTK_WIDGET( book ) );
-
-	back = glade_xml_get_widget( xml, "back" );
-	forward = glade_xml_get_widget( xml, "forward" );
-	apply = glade_xml_get_widget( xml, "apply" );
-
-	if( page_num == 0 ) {
-		gtk_widget_set_sensitive( apply, FALSE );
-		gtk_widget_set_sensitive( back, FALSE );
-		gtk_widget_set_sensitive( forward, TRUE );
-	} else if( page_num == g_list_length( book->children ) - 1 ) {
-		gtk_widget_set_sensitive( apply, TRUE );
-		gtk_widget_set_sensitive( back, TRUE );
-		gtk_widget_set_sensitive( forward, FALSE );
-	} else {
-		gtk_widget_set_sensitive( apply, FALSE );
-		gtk_widget_set_sensitive( back, TRUE );
-		gtk_widget_set_sensitive( forward, TRUE );
-	}
-
-	/* call init function for page if needed */
-	current = gtk_notebook_get_nth_page( book, page_num );
-	if( ! g_object_get_data( G_OBJECT( current ), "inited" ) ) {
-		if( init_funcs[ page_num ] && 
-		    init_funcs[ page_num ]( current ) ) {
-			g_object_set_data( G_OBJECT( current ), "inited",
-					   GINT_TO_POINTER( 1 ) );
-		}
-	}
-}
-
-gboolean screem_page_druid_init_details( GtkWidget *widget )
-{
-	/* add all loaded doctypes to "doctype" combo */
+	GtkWidget *widget;
+	ScreemSite *site;
+	const gchar *sitepath;
 	ScreemApplication *application;
 	ScreemDTDDB *db;
-	GladeXML *xml;
-	GtkWidget *druid;
+	GtkListStore *list;
 	gchar *default_dtd;
+	GtkWidget *entry;
 	gchar *default_charset;
 
-	xml = glade_get_widget_tree( GTK_WIDGET( widget ) );
+	xml = glade_get_widget_tree( GTK_WIDGET( druid_page ) );
 
-	if( xml ) {
-		ScreemSite *site;
-		const gchar *sitepath;
-		GnomeVFSURI *uri;
-		gchar *spath;
-		
-		druid = glade_xml_get_widget( xml, "pagedruid" );
+	druid = glade_xml_get_widget( xml, "new_page_druid" );
 
-		site = SCREEM_SITE( g_object_get_data( G_OBJECT( druid ),
-						       "site" ) );
-		sitepath = screem_site_get_pathname( site );
+	if( g_object_get_data( G_OBJECT( druid ), "setup_prepared" ) ) {
+		return;
+	}
+	g_object_set_data( G_OBJECT( druid ), "setup_prepared",
+			GINT_TO_POINTER( 1 ) );
 
-		uri = NULL;
-		if( sitepath ) {
-			uri = gnome_vfs_uri_new( sitepath );
-		}
-		if( ! uri ) {
-			spath = NULL;
-		} else if( gnome_vfs_uri_is_local( uri ) ) {
-			/* strip method */
-			spath = gnome_vfs_get_local_path_from_uri( sitepath );
-			gnome_vfs_uri_unref( uri );
-		} else {
-			spath = gnome_vfs_uri_to_string( uri, 0 );
-			gnome_vfs_uri_unref( uri );
-		}
+	site = SCREEM_SITE( g_object_get_data( G_OBJECT( druid ),
+				"site" ) );
+/*	sitepath = screem_site_get_pathname( site );*/
+	sitepath = screem_get_local_site_path( site );
+	if( sitepath ) {
 		widget = glade_xml_get_widget( xml, "page_path" );
 		gnome_file_entry_set_default_path( GNOME_FILE_ENTRY( widget ),
-						   spath );
-		widget = glade_xml_get_widget( xml, "page_stylesheet" );
-		gnome_file_entry_set_default_path( GNOME_FILE_ENTRY( widget ),
-						   spath );
-		g_free( spath );
-					   
-		widget = glade_xml_get_widget( xml, "doctype" );
-		
-		application = g_object_get_data( G_OBJECT( druid ), 
-						 "application" );
-		db = screem_application_get_dtd_db( application );
-		screem_dtd_db_fill_combo( db, widget );
-
-		default_dtd = gconf_client_get_string( application->client,
-						       "/apps/screem/editor/default_dtd",
-						       NULL );
-		widget = GTK_COMBO( widget )->entry;
-		gtk_entry_set_text( GTK_ENTRY( widget ), default_dtd );
-		
-		g_free( default_dtd );
-		
-		widget = glade_xml_get_widget( xml, "page_author" );
-		widget = gnome_entry_gtk_entry( GNOME_ENTRY(widget) );
-		gtk_entry_set_text( GTK_ENTRY( widget ), 
-				    g_get_real_name() );
-				
-		widget = glade_xml_get_widget( xml, "page_charset" );
-		widget = GTK_COMBO( widget )->entry;
-		default_charset = gconf_client_get_string( application->client,
-							   "/apps/screem/editor/default_charset",
-							   NULL );
-		gtk_entry_set_text( GTK_ENTRY( widget ), default_charset );
-		g_free( default_charset );
+						   sitepath );
 	}
+					   
+	widget = glade_xml_get_widget( xml, "doctype" );
+		
+	application = g_object_get_data( G_OBJECT( druid ), 
+			"application" );
+	db = screem_application_get_dtd_db( application );
 
-	return TRUE;
+	list = screem_dtd_db_get_store( db );
+	gtk_combo_box_set_model( GTK_COMBO_BOX( widget ), 
+			GTK_TREE_MODEL( list ) );
+	gtk_combo_box_entry_set_text_column( GTK_COMBO_BOX_ENTRY( widget ),
+				0 );
+
+	default_dtd = gconf_client_get_string( application->client,
+			"/apps/screem/editor/default_dtd", NULL );
+	if( ! default_dtd ) {
+		default_dtd = g_strdup( "-//W3C//DTD HTML 4.01 Transitional//EN" );
+	}
+	widget = GTK_BIN( widget )->child;
+	gtk_entry_set_text( GTK_ENTRY( widget ), default_dtd );
+		
+	g_free( default_dtd );
+		
+	widget = glade_xml_get_widget( xml, "page_charset" );
+	entry = GTK_BIN( widget )->child;
+	list = GTK_LIST_STORE( screem_application_get_encodings( application ) );
+	gtk_combo_box_set_model( GTK_COMBO_BOX( widget ), 
+			GTK_TREE_MODEL( list ) ); 
+	gtk_combo_box_entry_set_text_column( GTK_COMBO_BOX_ENTRY( widget ),
+				0 );
+
+	default_charset = gconf_client_get_string( application->client,
+			"/apps/screem/editor/default_charset", NULL );
+	if( ! default_charset ) {
+		g_get_charset( (const gchar**)&default_charset );
+		default_charset = g_strdup( default_charset );
+	}
+	g_signal_connect( G_OBJECT( entry ), "changed",
+			G_CALLBACK( charset_changed ),
+			widget );
+	gtk_entry_set_text( GTK_ENTRY( entry ), default_charset );
+	g_free( default_charset );
 }
 
+static void charset_changed( GtkWidget *entry, GtkWidget *combo )
+{
+	GtkTreeModel *model;
+	GtkTreeIter *it;
+	const gchar *value;
+	gchar *colval;
+	gchar *disval;
+	
+	value = gtk_entry_get_text( GTK_ENTRY( entry ) );
+	model = gtk_combo_box_get_model( GTK_COMBO_BOX( combo ) );
+	it = screem_support_find_in_list( GTK_LIST_STORE( model ), 1, 
+			value );
+	if( ! it ) {
+		it = screem_support_find_in_list( GTK_LIST_STORE( model ), 0, value );
+	}
+	if( it ) {
+		gtk_tree_model_get( model, it,	
+				0, &disval,
+				1, &colval, -1 );
+		if( ! strcmp( value, colval ) || 
+		    ! strcmp( value, disval ) ) {
+			gtk_combo_box_set_active_iter( GTK_COMBO_BOX( combo ), it );
+		}
+		gtk_tree_iter_free( it );
+		g_free( disval );
+		g_free( colval );
+	}
 
-gboolean screem_page_druid_init_templates( GtkWidget *widget )
+}
+
+void screem_page_druid_details_prepare( GnomeDruidPage *druid_page,
+		GtkWidget *druid )
+{
+	GladeXML *xml;
+	GtkWidget *widget;
+
+	xml = glade_get_widget_tree( GTK_WIDGET( druid_page ) );
+	druid = glade_xml_get_widget( xml, "new_page_druid" );
+	
+	if( g_object_get_data( G_OBJECT( druid ), "details_prepared" ) ) {
+		return;
+	}
+	g_object_set_data( G_OBJECT( druid ), "details_prepared",
+			GINT_TO_POINTER( 1 ) );
+
+	widget = glade_xml_get_widget( xml, "page_author" );
+	widget = gnome_entry_gtk_entry( GNOME_ENTRY(widget) );
+	gtk_entry_set_text( GTK_ENTRY( widget ), g_get_real_name() );
+}
+
+void screem_page_druid_template_prepare( GnomeDruidPage *druid_page,
+		GtkWidget *druid )
 {
 	GtkListStore *store;
 	GtkTreeView *view;
@@ -330,137 +316,147 @@ gboolean screem_page_druid_init_templates( GtkWidget *widget )
 	GladeXML *xml;
 	ScreemSite *site;
 	const gchar *template;
-	gchar *spath;
-	
-	xml = glade_get_widget_tree( GTK_WIDGET( widget ) );
-	if( xml ) {
-		widget = glade_xml_get_widget( xml, "pagedruid" );
+	const gchar *sitepath;
+	GtkWidget *widget;
 
-		site = SCREEM_SITE( g_object_get_data( G_OBJECT( widget ),
-						       "site" ) );
+	xml = glade_get_widget_tree( GTK_WIDGET( druid_page ) );
+	widget = glade_xml_get_widget( xml, "new_page_druid" );
+
+	if( g_object_get_data( G_OBJECT( widget ),
+				"template_prepared" ) ) {
+		return;
+	}
+	g_object_set_data( G_OBJECT( widget ), "template_prepared",
+			GINT_TO_POINTER( 1 ) );
+
+	site = SCREEM_SITE( g_object_get_data( G_OBJECT( widget ),
+				"site" ) );
 		
-		view = GTK_TREE_VIEW( glade_xml_get_widget( xml,
-							    "templatelist" ) );
+	view = GTK_TREE_VIEW( glade_xml_get_widget( xml, 
+				"templatelist" ) );
 
-		store = gtk_list_store_new( 2, G_TYPE_STRING, G_TYPE_STRING,
-					    NULL );
-
-		gtk_tree_view_set_model( view, GTK_TREE_MODEL( store ) );
+	store = gtk_list_store_new( 2, G_TYPE_STRING, G_TYPE_STRING,
+			NULL );
+	gtk_tree_view_set_model( view, GTK_TREE_MODEL( store ) );
+	g_object_unref( store );
 		
-		renderer = gtk_cell_renderer_text_new();
-		col = gtk_tree_view_column_new();
-		gtk_tree_view_column_set_title( col, "Name" );
-		gtk_tree_view_column_pack_start( col, renderer, TRUE );
-		gtk_tree_view_append_column( view, col );
-		gtk_tree_view_column_set_attributes( col, renderer, 
-						     "text", 0, NULL );
+	renderer = gtk_cell_renderer_text_new();
+	col = gtk_tree_view_column_new();
+	gtk_tree_view_column_set_title( col, "Name" );
+	gtk_tree_view_column_pack_start( col, renderer, TRUE );
+	gtk_tree_view_append_column( view, col );
+	gtk_tree_view_column_set_attributes( col, renderer, 
+			"text", 0, NULL );
 		
-		renderer = gtk_cell_renderer_text_new();
-		col = gtk_tree_view_column_new();
-		gtk_tree_view_column_set_title( col, "Path" );
-		gtk_tree_view_column_pack_start( col, renderer, TRUE );
-		gtk_tree_view_append_column( view, col );
-		gtk_tree_view_column_set_attributes( col, renderer, 
-						     "text", 1, NULL );
+	renderer = gtk_cell_renderer_text_new();
+	col = gtk_tree_view_column_new();
+	gtk_tree_view_column_set_title( col, "Path" );
+	gtk_tree_view_column_pack_start( col, renderer, TRUE );
+	gtk_tree_view_append_column( view, col );
+	gtk_tree_view_column_set_attributes( col, renderer, 
+			"text", 1, NULL );
 
-		widget = glade_xml_get_widget( xml, 
-					"custom_template" );
-		spath = screem_get_local_site_path( site );
-		if( spath ) {
-			gnome_file_entry_set_default_path( GNOME_FILE_ENTRY( widget ), spath );
-			g_free( spath );
-		}
+	widget = glade_xml_get_widget( xml, "custom_template" );
+	/*sitepath = screem_site_get_pathname( site );*/
+	sitepath = screem_get_local_site_path( site );
+	if( sitepath ) {
+		gnome_file_entry_set_default_path( GNOME_FILE_ENTRY( widget ), sitepath );
+	}
 		
-		template = screem_site_get_template_path( site );
-		if( template ) {
-			widget = glade_xml_get_widget( xml, 
-					"custom_template" );
+	template = screem_site_get_template_path( site );
+	if( template ) {
+		widget = glade_xml_get_widget( xml, "custom_template" );
 
-			gnome_file_entry_set_filename( GNOME_FILE_ENTRY( widget ), template );
-			widget = glade_xml_get_widget( xml,
-					"use_custom" );
-			gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( widget ), TRUE );
-		}
+		gnome_file_entry_set_filename( GNOME_FILE_ENTRY( widget ), template );
+		widget = glade_xml_get_widget( xml, "use_custom" );
+		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( widget ), TRUE );
 	}
 
-	return TRUE;
 }
 
-
-gboolean screem_page_druid_init_colours( GtkWidget *widget )
+void screem_page_druid_appearance_prepare( GnomeDruidPage *druid_page,
+		GtkWidget *druid )
 {
 	GladeXML *xml;
+	GtkWidget *widget;
 	gboolean template;
+	ScreemSite *site;
+	const gchar *sitepath;
 
-	xml = glade_get_widget_tree( GTK_WIDGET( widget ) );
+	xml = glade_get_widget_tree( GTK_WIDGET( druid_page ) );
+	druid = glade_xml_get_widget( xml, "new_page_druid" );
 
-	if( xml ) {
-		widget = glade_xml_get_widget( xml, "notemplate" );
-		template = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) );
+	if( g_object_get_data( G_OBJECT( druid ), 
+				"appearance_prepared" ) ) {
+		return;
+	}
+	g_object_set_data( G_OBJECT( druid ), "appearance_prepared",
+			GINT_TO_POINTER( 1 ) );
 
-		widget = glade_xml_get_widget( xml, "override" );
+	widget = glade_xml_get_widget( xml, "notemplate" );
+	template = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) );
+	widget = glade_xml_get_widget( xml, "override" );
 
-		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( widget ),
+	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( widget ),
 					      template );
-		gtk_widget_set_sensitive( widget, ! template );
+	gtk_widget_set_sensitive( widget, ! template );
+
+	site = SCREEM_SITE( g_object_get_data( G_OBJECT( druid ),
+				"site" ) );
+	/*sitepath = screem_site_get_pathname( site );*/
+	sitepath = screem_get_local_site_path( site );
+	if( sitepath ) {
+		widget = glade_xml_get_widget( xml, "page_stylesheet" );
+		gnome_file_entry_set_default_path( GNOME_FILE_ENTRY( widget ),
+						   sitepath );
 	}
 
-	return FALSE;
 }
 
-
-gboolean screem_page_druid_init_preview( GtkWidget *widget )
+void screem_page_druid_preview_prepare( GnomeDruidPage *druid_page,
+		GtkWidget *druid )
 {
 	GladeXML *xml;
-	GtkWidget *druid;
 	ScreemPreview *preview;
 	ScreemPage *page;
+	GList *children;
+	ScreemWindow *window;
+	GtkWidget *widget;
 
-	xml = glade_get_widget_tree( GTK_WIDGET( widget ) );
+	xml = glade_get_widget_tree( GTK_WIDGET( druid_page ) );
 
-	if( xml ) {
-		GList *children;
-		ScreemWindow *window;
-		druid = glade_xml_get_widget( xml, "pagedruid" );
+	druid = glade_xml_get_widget( xml, "new_page_druid" );
 
-		window = SCREEM_WINDOW( g_object_get_data( G_OBJECT( druid ),
-							   "window" ) );
+	window = SCREEM_WINDOW( g_object_get_data( G_OBJECT( druid ),
+				"window" ) );
 
-		widget = glade_xml_get_widget( xml, "page_preview" );
-		children = gtk_container_get_children( GTK_CONTAINER( widget ) );
-		children = children->next;
+	widget = glade_xml_get_widget( xml, "page_preview" );
+	children = gtk_container_get_children( GTK_CONTAINER( widget ) );
+	children = children->next;
 
-		if( ! children ) {
-			preview = screem_preview_new();
-			gtk_box_pack_start( GTK_BOX( widget ),
-					    GTK_WIDGET( preview ),
-					    TRUE, TRUE, 0 );
-		} else {
-			preview = SCREEM_PREVIEW( children->data );
-		}
-		gtk_widget_show_all( GTK_WIDGET( preview ) );
-
-		
-		/* now take the page template, colours, stylesheet etc
-		   and construct a page */
-		widget = glade_xml_get_widget( xml, "pagedruid" );
-		page = SCREEM_PAGE( g_object_get_data( G_OBJECT( widget ),
-						       "page" ) );
-
-		g_return_val_if_fail( page != NULL, FALSE );
-		screem_page_druid_create_page( page, xml,
-					       window );
-
-		screem_page_set_mime_type( page, "text/html" );
-
-		g_object_set( G_OBJECT( preview ), 
-			      "page", page,			      
-			      "online", FALSE,
-			      NULL );
-		screem_view_display( SCREEM_VIEW( preview ) );
+	if( ! children ) {
+		preview = screem_preview_new();
+		gtk_box_pack_start( GTK_BOX( widget ),
+				    GTK_WIDGET( preview ),
+				    TRUE, TRUE, 0 );
+	} else {
+		preview = SCREEM_PREVIEW( children->data );
 	}
+	gtk_widget_show_all( GTK_WIDGET( preview ) );
 
-	return FALSE;
+	/* now take the page template, colours, stylesheet etc
+	   and construct a page */
+	widget = glade_xml_get_widget( xml, "new_page_druid" );
+	page = SCREEM_PAGE( g_object_get_data( G_OBJECT( widget ),
+				"page" ) );
+
+	g_return_if_fail( page != NULL );
+	screem_page_druid_create_page( page, xml, window );
+	screem_page_set_mime_type( page, "text/html" );
+
+	g_object_set( G_OBJECT( preview ), "page", page, 
+			"online", FALSE, NULL );
+	screem_view_display( SCREEM_VIEW( preview ) );
 }
 
 static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
@@ -476,8 +472,9 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 	
 	const gchar *title;
 	const gchar *author;
-	const gchar *charset;
+	gchar *charset;
 	const gchar *keywords;
+	const gchar *description;
 
 	gchar *template;
 
@@ -497,6 +494,7 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 	gboolean setauthor;
 	gboolean setgenerator;
 	gboolean setkeywords;
+	gboolean setdescription;
 	gboolean setcontent;
 	gboolean setscript;
 	gboolean setstyle;
@@ -506,6 +504,11 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 	ScreemSite *site;
 	const gchar *sitepath;
 	gchar *spath;
+
+	GConfClient *client;
+	gfloat tabwidth;
+	gchar *spaces;
+	gchar *tmp;
 	
 	app = window->application;
 	db = screem_application_get_dtd_db( app );
@@ -518,21 +521,34 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 
 	widget = glade_xml_get_widget( xml, "page_stylesheet" );
 	stylesheet = gnome_file_entry_get_full_path( GNOME_FILE_ENTRY( widget ), FALSE );
-	if( stylesheet && ! screem_site_get_fake_flag( site ) ) {
-		spath = relative_path( stylesheet,
-				screem_site_get_pathname( site ) );
-		g_free( stylesheet );
-		stylesheet = spath;
-		spath = NULL;
+	if( stylesheet ) {
+		if( ! screem_site_get_fake_flag( site ) ) {
+			spath = relative_path( stylesheet,
+					screem_site_get_pathname( site ) );
+			g_free( stylesheet );
+			stylesheet = spath;
+			spath = NULL;
+		} else if( pathname ) {
+			tmp = g_path_get_dirname( pathname );
+			if( tmp &&
+			! strncmp( tmp, stylesheet, strlen( tmp ) ) ) {
+				spath = relative_path( stylesheet,
+						tmp );
+				g_free( stylesheet );
+				stylesheet = spath;
+				spath = NULL;
+			}
+			g_free( tmp );
+		}
 	}
 	
 	widget = glade_xml_get_widget( xml, "doctype" );
-	widget = GTK_COMBO( widget )->entry;
+	widget = GTK_BIN( widget )->child;
 	doctype = gtk_entry_get_text( GTK_ENTRY( widget ) );
 
 	/* bit of a hack */
 	xhtml = ( strstr( doctype, " XHTML " ) != NULL );
-	if( ! xhtml ) {
+	if( pathname && ! xhtml ) {
 		xhtml = ! strcmp( ".xml", 
 				g_extension_pointer( pathname ) );
 		xhtml |= ! strcmp( ".xhtml",
@@ -548,8 +564,24 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 	author = gtk_entry_get_text( GTK_ENTRY( widget ) );
 
 	widget = glade_xml_get_widget( xml, "page_charset" );
-	widget = GTK_COMBO( widget )->entry;
-	charset = gtk_entry_get_text( GTK_ENTRY( widget ) );
+	widget = GTK_BIN( widget )->child;
+	charset = (gchar*)gtk_entry_get_text( GTK_ENTRY( widget ) );
+	if( ! ( name = g_utf8_strchr( charset, -1, '(' ) ) ) {
+		charset = g_strdup( charset );	
+	} else {
+		charset = g_utf8_next_char( name );
+		name = g_utf8_strchr( charset, -1, ')' );
+		if( ! name ) {
+			charset = g_strdup( charset );
+		} else {
+			charset = g_strndup( charset,
+					name - charset );
+		}
+	}
+
+	widget = glade_xml_get_widget( xml, "page_description" );
+	widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
+	description = gtk_entry_get_text( GTK_ENTRY( widget ) );
 
 	widget = glade_xml_get_widget( xml, "page_keywords" );
 	widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
@@ -577,12 +609,27 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 
 	widget = glade_xml_get_widget( xml, "frameset" );
 	frameset = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) );
+	
+	screem_application_set_cursor( app, GDK_WATCH );
 
 	/* if we are using a template load it */
 	if( template ) {
 		screem_page_set_pathname( page, template );
-		screem_page_load( page );
-		text = screem_page_get_data( page );
+		if( screem_page_load( page, NULL ) ) {
+			text = screem_page_get_data( page );
+		} else {
+			/* FIXME: report error */
+	
+
+			/* fallback to no template */
+			text = screem_markup_basic_html( db, doctype,
+				frameset, xhtml );
+			screem_page_set_data( page, text );
+
+			g_free( template );
+			template = NULL;
+		}
+		screem_page_set_pathname( page, NULL );
 	} else {
 		/* create a basic page */
 
@@ -594,8 +641,8 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 	newtext = g_string_new( NULL );
 	pos = start = end = 0;
 
-	setauthor = setgenerator = setkeywords = setcontent = 
-		setscript = setstyle = FALSE;
+	setauthor = setgenerator = setdescription = 
+		setkeywords = setcontent = setscript = setstyle = FALSE;
 
 	while( ( tag = screem_markup_next_tag( text, pos, &start, &end, &name ) ) ) {
 		GSList *attrs;
@@ -640,6 +687,18 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 				}
 				setgenerator = TRUE;
 			}
+			if( ! setdescription ) {
+				g_string_append( newtext,
+						 "\t<meta name=\"description\" content=\"" );
+				g_string_append( newtext, description );
+				if( ! xhtml ) {
+					g_string_append( newtext, "\" >\n\t" );
+				} else {
+					g_string_append( newtext, "\" />\n\t" );
+				}
+				setdescription = TRUE;
+
+			}
 			if( ! setkeywords ) {
 				g_string_append( newtext,
 						 "\t<meta name=\"keywords\" content=\"" );
@@ -694,7 +753,8 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 			}
 		} else if( ! g_strcasecmp( "title", name ) && *title != '\0' ) {
 			processed = TRUE;
-			g_string_append_len( newtext, text + start, end - start + 1 );
+			g_string_append_len( newtext,
+					text + start, end - start );
 			g_string_append( newtext, title );
 			/* shift to start of closing tag for <title> */
 			while( text[ pos ] != '<' && text[ pos ] != '\0' ) {
@@ -705,6 +765,7 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 			   name="author"
 			   name="generator"
 			   name="keywords"
+			   name="description"
 
 			   and http-equiv="content-type" partly
 			*/
@@ -719,6 +780,7 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 				if( ! g_strcasecmp( "name", attr->data ) &&
 				    ( ! g_strcasecmp( "author", list->data ) ||
 				      ! g_strcasecmp( "generator", list->data ) ||
+				      ! g_strcasecmp( "description", list->data ) ||
 				      ! g_strcasecmp( "keywords", list->data ) ) ) {
 					nme = list;
 				} else if( ! g_strcasecmp( "http-equiv", attr->data ) &&
@@ -749,6 +811,9 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 				} else if( ! g_strcasecmp( "generator", 
 							   type ) ) {
 					value = g_strdup( PACKAGE_STRING );
+				} else if( ! g_strcasecmp( "description",
+							type ) ) {
+					value = g_strdup( description );
 				} else if( ! g_strcasecmp( "keywords",
 							   type ) ) {
 					value = g_strdup( keywords );
@@ -774,6 +839,9 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 					} else if( ! g_strcasecmp( "generator",
 								   list->data ) ) {
 						setgenerator = TRUE;
+					} else if( ! g_strcasecmp( "desription",
+								list->data ) ) {
+						setdescription = TRUE;
 					} else if( ! g_strcasecmp( "keywords", 
 								   list->data ) ) {
 						setkeywords = TRUE;
@@ -810,7 +878,7 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 			GSList *vlink;
 			GSList *alink;
 			GSList *background;
-			guint8 r, g, b, a;
+			GdkColor color;
 			gchar *value;
 
 			processed = TRUE;
@@ -840,9 +908,8 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 			widget = glade_xml_get_widget( xml, "bgcolor" );
 			if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) ) ) {
 				widget = glade_xml_get_widget( xml, "bgcolorpicker" );
-				gnome_color_picker_get_i8( GNOME_COLOR_PICKER( widget ),
-							   &r, &g, &b, &a );
-				value = g_strdup_printf( "#%.2x%.2x%.2x", r, g, b );
+				gtk_color_button_get_color( GTK_COLOR_BUTTON( widget ), &color );
+				value = screem_gdk_color_to_string( &color );
 				if( bgcolor ) {
 					g_free( bgcolor->data );
 					bgcolor->data = value;
@@ -855,9 +922,8 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 			widget = glade_xml_get_widget( xml, "text" );
 			if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) ) ) {
 				widget = glade_xml_get_widget( xml, "textpicker" );
-				gnome_color_picker_get_i8( GNOME_COLOR_PICKER( widget ),
-							   &r, &g, &b, &a );
-				value = g_strdup_printf( "#%.2x%.2x%.2x", r, g, b );
+				gtk_color_button_get_color( GTK_COLOR_BUTTON( widget ), &color );
+				value = screem_gdk_color_to_string( &color );
 				if( text ) {
 					g_free( text->data );
 					text->data = value;
@@ -870,9 +936,8 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 			widget = glade_xml_get_widget( xml, "link" );
 			if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) ) ) {
 				widget = glade_xml_get_widget( xml, "linkpicker" );
-				gnome_color_picker_get_i8( GNOME_COLOR_PICKER( widget ),
-							   &r, &g, &b, &a );
-				value = g_strdup_printf( "#%.2x%.2x%.2x", r, g, b );
+				gtk_color_button_get_color( GTK_COLOR_BUTTON( widget ), &color );
+				value = screem_gdk_color_to_string( &color );
 				if( link ) {
 					g_free( link->data );
 					link->data = value;
@@ -885,9 +950,8 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 			widget = glade_xml_get_widget( xml, "vlink" );
 			if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) ) ) {
 				widget = glade_xml_get_widget( xml, "vlinkpicker" );
-				gnome_color_picker_get_i8( GNOME_COLOR_PICKER( widget ),
-							   &r, &g, &b, &a );
-				value = g_strdup_printf( "#%.2x%.2x%.2x", r, g, b );
+				gtk_color_button_get_color( GTK_COLOR_BUTTON( widget ), &color );
+				value = screem_gdk_color_to_string( &color );
 				if( vlink ) {
 					g_free( vlink->data );
 					vlink->data = value;
@@ -900,9 +964,8 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 			widget = glade_xml_get_widget( xml, "alink" );
 			if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) ) ) {
 				widget = glade_xml_get_widget( xml, "alinkpicker" );
-				gnome_color_picker_get_i8( GNOME_COLOR_PICKER( widget ),
-							   &r, &g, &b, &a );
-				value = g_strdup_printf( "#%.2x%.2x%.2x", r, g, b );
+				gtk_color_button_get_color( GTK_COLOR_BUTTON( widget ), &color );
+				value = screem_gdk_color_to_string( &color );
 				if( alink ) {
 					g_free( alink->data );
 					alink->data = value;
@@ -925,6 +988,8 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 				}
 				g_string_append_c( newtext, '"' );
 			}
+			g_string_append_c( newtext, '>' );
+
 		} 
 
 		if( ! processed ) {
@@ -936,33 +1001,59 @@ static void screem_page_druid_create_page( ScreemPage *page, GladeXML *xml,
 		g_free( tag );
 	}
 	g_string_append( newtext, text + end );
+	g_free( text );
 
-	screem_page_set_data( page, newtext->str );
+	client = gconf_client_get_default();
+	tabwidth = gconf_client_get_float( client,
+			"/apps/screem/editor/tabwidth",
+			NULL );
+	if( tabwidth == 0.0 ) {
+		tabwidth = 8.0;
+	}
+	if( gconf_client_get_bool( client,
+				"/apps/screem/editor/spaces_instead_of_tabs",
+				NULL ) ) {
+		spaces = g_new0( gchar, (gint)(tabwidth + 1) );
+		memset( spaces, ' ', tabwidth );
+		text = find_text( newtext->str, "\t", spaces, NULL );
+ 		g_free( spaces );
+	} else {
+		text = g_strdup( newtext->str );
+	}
+	g_object_unref( client );
 	
 	g_string_free( newtext, TRUE );
-	g_free( text );
+	screem_page_set_data( page, text );
 	
-	if( screem_site_get_fake_flag( site ) ) {
-		if( g_path_is_absolute( pathname ) ) {
-			spath = NULL;
-		} else {
+	g_free( text );
+
+	spath = NULL;
+	uri = NULL;
+	if( pathname ) {
+		if( screem_site_get_fake_flag( site ) &&
+			! g_path_is_absolute( pathname ) ) {
 			spath = g_get_current_dir();
+		} else {
+			sitepath = screem_site_get_pathname( site );
+			spath = g_strdup( sitepath );
 		}
-	} else {
-		sitepath = screem_site_get_pathname( site );
-		spath = g_strdup( sitepath );
 	}
 	if( spath ) {	
 		uri = relative_to_full( pathname, spath );	
 		g_free( spath );
-	} else {
+	} else if( pathname ) {
 		uri = g_strdup( pathname );
 	}
 
-	screem_page_set_pathname( page, uri );
-
-	g_free( uri );
+	if( uri ) {
+		screem_page_set_pathname( page, uri );
+		g_free( uri );
+	}
 	g_free( pathname );
 	g_free( stylesheet );
 	g_free( template );
+	g_free( charset );
+		
+	screem_application_set_cursor( app, GDK_LAST_CURSOR );
 }
+
