@@ -34,6 +34,8 @@
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 
+#include <libgnome/libgnome.h>
+
 #include <gconf/gconf-client.h>
 
 #include <glib/gconvert.h>
@@ -63,6 +65,7 @@
 #include "screemmarshal.h"
 
 enum {
+	SAVING,
 	SAVED,
 	/* these 2 are fired from the ScreemPageModel */
 	BUILDING,
@@ -123,6 +126,7 @@ struct ScreemPagePrivate {
 	GnomeVFSMonitorHandle *monitor;
 	gboolean modified;
 
+	gboolean compress;
 };
 
 void screem_page_undo( ScreemPage *page )
@@ -143,42 +147,21 @@ void screem_page_redo( ScreemPage *page )
 	}
 }
 
-GSList *screem_page_get_mime_types( ScreemPage *page )
-{
-	GSList *ret;
-	const GSList *langs;
-	GtkSourceLanguagesManager *lm;
-
-	
-	g_return_val_if_fail( SCREEM_IS_PAGE( page ), NULL );
-
-	ret = NULL;
-
-	lm = screem_application_load_syntax_tables();
-	langs = gtk_source_languages_manager_get_available_languages( lm );
-	while( langs ) {
-		GtkSourceLanguage *lang;
-		GSList *types;
-
-		lang = GTK_SOURCE_LANGUAGE( langs->data );
-		types = gtk_source_language_get_mime_types( lang );
-		ret = g_slist_concat( ret, types );
-
-		langs = langs->next;
-	}
-
-	return ret;
-}
-
 ScreemPage *screem_page_new( GObject *application )
 {
 	ScreemPage *page;
 	GType type;
 
 	type = screem_page_get_type();
-
+	
 	page = SCREEM_PAGE( g_object_new( type, "tag-table", NULL, "app", application, NULL ) );
 
+	/* need to do this here so the initial mime type is
+	 * correctly set otherwise we don't get highlighting.
+	 * can't do it in screem_page_init() as the
+	 * GtkSourceBuffer construction hasn't finished */
+	screem_page_set_mime_type( page, page->private->mime_type );
+	
 	return page;
 }
 
@@ -218,14 +201,7 @@ void screem_page_set_pathname( ScreemPage *page, const gchar *path )
 }
 
 static void screem_page_set_pathname_real( ScreemPage *page, const gchar *path )
-{
-	GtkSourceLanguage *lang;
-	GtkSourceLanguagesManager *lm;
-	GtkTextTagTable *table;
-	GtkTextTag *tag;
-
-	lm = screem_application_load_syntax_tables();
-	
+{	
 	if( page->private->pathname && path != page->private->pathname ) {
 		g_free( page->private->pathname );
 	}
@@ -237,10 +213,13 @@ static void screem_page_set_pathname_real( ScreemPage *page, const gchar *path )
 	
 	if( path ) {
 		page->private->mime_type = 
-			screem_get_mime_type( path );
+			screem_get_mime_type( path, FALSE );
 	}
 	if( ! page->private->mime_type ) {
-		page->private->mime_type = g_strdup( "text/plain" );
+		page->private->mime_type = gconf_client_get_string( page->private->client, "/apps/screem/default_mimetype", NULL );
+		if( ! page->private->mime_type ) {
+			page->private->mime_type = g_strdup( "text/html" );
+		}
 	}
 
 	if( path != page->private->pathname ) {
@@ -250,18 +229,7 @@ static void screem_page_set_pathname_real( ScreemPage *page, const gchar *path )
 			page->private->pathname = NULL;
 		}
 	}
-
-	lang = gtk_source_languages_manager_get_language_from_mime_type( lm, page->private->mime_type );
-	if( lang ) {
-		gtk_source_buffer_set_language( GTK_SOURCE_BUFFER( page ), lang );
-		table = gtk_text_buffer_get_tag_table( GTK_TEXT_BUFFER( page ) );
-		tag = gtk_text_tag_new( SCREEM_INVALID_MARKUP_TAG );
-		gtk_text_tag_table_add( table, tag );
-		g_object_set( G_OBJECT( tag ),
-			"background", "#ffffffff0000",
-			"foreground", "#000000000000",
-			NULL );
-	} 
+	
 	if( page->private->monitor ) {
 		gnome_vfs_monitor_cancel( page->private->monitor );
 		page->private->monitor = NULL;
@@ -290,35 +258,27 @@ void screem_page_set_mime_type( ScreemPage *page, const gchar *mime_type )
 
 static void screem_page_set_mime_type_real( ScreemPage *page, const gchar *mime_type )
 {
-	GtkSourceLanguage *lang;
-	GtkSourceLanguagesManager *lm;
-	GtkTextTagTable *table;
-	GtkTextTag *tag;
-
+	gchar *type;
+	
 	g_return_if_fail( SCREEM_IS_PAGE( page ) );
 	
-	lm = screem_application_load_syntax_tables();
-	
+	type = NULL;
 	if( ! mime_type ) {
-		mime_type = "text/plain";
+		type = gconf_client_get_string( page->private->client, "/apps/screem/default_mimetype", NULL );
+		if( ! type ) {
+			type = g_strdup( "text/html" );
+		}
+		mime_type = type;
 	}
-
+	
 	if( page->private->mime_type ) {
 		g_free( page->private->mime_type );
 	}
 	page->private->mime_type = g_strdup( mime_type );
 
-	lang = gtk_source_languages_manager_get_language_from_mime_type( lm, page->private->mime_type );
-	gtk_source_buffer_set_language( GTK_SOURCE_BUFFER( page ), lang );
-	table = gtk_text_buffer_get_tag_table( GTK_TEXT_BUFFER( page ) );
-	tag = gtk_text_tag_new( SCREEM_INVALID_MARKUP_TAG );
-	gtk_text_tag_table_add( table, tag );
-	g_object_set( G_OBJECT( tag ),
-		"background", "#ffffffff0000",
-		"foreground", "#000000000000",
-		NULL );
-
 	screem_page_model_force_check( page->private->model );
+
+	g_free( type );
 }
 
 const gchar *screem_page_get_mime_type( ScreemPage *page )
@@ -344,29 +304,20 @@ gchar *screem_page_get_charset( ScreemPage *page )
 
 void screem_page_set_data( ScreemPage *page, const gchar *data )
 {
-	gchar *cdata;
-	
 	g_return_if_fail( SCREEM_IS_PAGE( page ) );
 
-
 	gtk_source_buffer_begin_not_undoable_action( GTK_SOURCE_BUFFER( page ) );
-	
-	/* ensure the text is UTF8, otherwise GtkTextBuffer can't
-	   hold it */
 
-	cdata = screem_support_charset_convert( data );
-	
-	if( cdata ) {
-		gtk_text_buffer_set_text( GTK_TEXT_BUFFER( page ),
-					  cdata,
-					  strlen( cdata ) );
-		g_free( cdata );
+	if( ! data ) {
+		data = "";
 	}
+	gtk_text_buffer_set_text( GTK_TEXT_BUFFER( page ), data, 
+				strlen( data ) );
+	
 	screem_page_model_force_check( page->private->model );
 		
 	page->private->loaded = TRUE;
 	gtk_source_buffer_end_not_undoable_action( GTK_SOURCE_BUFFER( page ) );
-
 }
 
 gchar *screem_page_get_data( ScreemPage *page )
@@ -406,23 +357,32 @@ gboolean screem_page_get_changed( ScreemPage *page )
 	return page->private->changed;
 }
 
-gboolean screem_page_load( ScreemPage *page )
+gboolean screem_page_load( ScreemPage *page, GError **error )
 {
+	ScreemPagePrivate *priv;
 	const gchar *path;
+	gboolean compress;
+	gchar *charset;
 
 	GString *data;
 
-        g_return_val_if_fail( SCREEM_IS_PAGE( page ), FALSE );
+    g_return_val_if_fail( SCREEM_IS_PAGE( page ), FALSE );
 
+	if( error ) {
+		*error = NULL;
+	}
+	
 	if( screem_page_is_loaded( page ) ) {
 		return TRUE;
 	}
 
+	priv = page->private;
 	path = screem_page_get_pathname( page );
 
 	g_return_val_if_fail( path != NULL, FALSE );
 	
-	data = load_file( path );
+	priv->compress = FALSE;
+	data = load_file( path, &compress, &charset, error );
 
 	if( data ) {
 		screem_page_set_data( page, data->str );
@@ -430,7 +390,14 @@ gboolean screem_page_load( ScreemPage *page )
 
 		screem_page_set_changed( page, FALSE );
 
-		page->private->loaded = TRUE;
+		priv->loaded = TRUE;
+		priv->compress = compress;
+	
+		if( screem_page_model_is_default_charset( priv->model ) ) {
+			/* set to charset we detected at load time */
+			g_object_set( G_OBJECT( priv->model ), "charset", charset, NULL );
+		}	
+		g_free( charset );
 	}
 
 	page->private->modified = FALSE;
@@ -438,17 +405,18 @@ gboolean screem_page_load( ScreemPage *page )
 	return ( data != NULL );
 }
 
-gboolean screem_page_revert( ScreemPage *page )
+gboolean screem_page_revert( ScreemPage *page, GError **error )
 {
 	g_return_val_if_fail( SCREEM_IS_PAGE( page ), FALSE );
 
 	page->private->loaded = FALSE;
 
-	return screem_page_load( page );
+	return screem_page_load( page, error );
 }
 
-gboolean screem_page_save( ScreemPage *page )
+gboolean screem_page_save( ScreemPage *page, GError **error )
 {
+	ScreemPagePrivate *priv;
 	gchar *data;
 	gboolean retval;
 
@@ -457,11 +425,20 @@ gboolean screem_page_save( ScreemPage *page )
 	
 	g_return_val_if_fail( SCREEM_IS_PAGE( page ), FALSE );
 
+	if( error ) {
+		*error = NULL;
+	}
+	
 	if( ( ! screem_page_get_changed( page ) ) ||
 	    ( ! screem_page_save_check( page ) ) ) {
 		return TRUE;
 	}
 
+	priv = page->private;
+	
+	g_signal_emit( G_OBJECT( page ),
+		       screem_page_signals[ SAVING ], 0 );
+	
 	data = screem_page_get_data( page );
 
 	charset = screem_page_get_charset( page );
@@ -474,10 +451,15 @@ gboolean screem_page_save( ScreemPage *page )
 		charset = gconf_client_get_string( client,
 						   "/apps/screem/editor/default_charset",
 						   NULL );
+		if( ! charset ) {
+			g_get_charset( (const gchar**)&charset );
+			charset = g_strdup( charset );
+		}
 		g_object_unref( G_OBJECT( client ) );
 	}
 
-	temp = screem_support_charset_convert_to( data, charset );
+	temp = screem_support_charset_convert_to( data, charset,
+			"UTF-8" );
 	if( temp ) {
 		g_free( data );
 		data = temp;
@@ -485,10 +467,10 @@ gboolean screem_page_save( ScreemPage *page )
 	g_free( charset );
 
 	/* stop monitoring while we write to the file ourselves */
-	if( page->private->monitor ) {
-		gnome_vfs_monitor_cancel( page->private->monitor );
+	if( priv->monitor ) {
+		gnome_vfs_monitor_cancel( priv->monitor );
 	}
-	page->private->monitor = NULL;
+	priv->monitor = NULL;
 
 	retval = save_file( screem_page_get_pathname( page ),
 			    data, 
@@ -496,24 +478,25 @@ gboolean screem_page_save( ScreemPage *page )
 			    GNOME_VFS_PERM_USER_WRITE |
 			    GNOME_VFS_PERM_GROUP_READ |
 			    GNOME_VFS_PERM_GROUP_WRITE |
-			    GNOME_VFS_PERM_OTHER_READ );
+			    GNOME_VFS_PERM_OTHER_READ,
+			    priv->compress, error );
 
 	g_free( data );
 
 	if( retval ) {
 		g_signal_emit( G_OBJECT( page ),
-				       screem_page_signals[ SAVED ], 0 );
+			       screem_page_signals[ SAVED ], 0 );
+		screem_page_set_changed( page, FALSE );
 	}
 	
-	screem_page_set_changed( page, FALSE );
-	page->private->modified = FALSE;
+	priv->modified = FALSE;
 
       	if( ! screem_page_get_mime_type( page ) ) {
 		screem_page_set_pathname( page, 
 					  screem_page_get_pathname( page ) );
 	} else {
-		gnome_vfs_monitor_add( &page->private->monitor,
-				       page->private->pathname,
+		gnome_vfs_monitor_add( &priv->monitor,
+				       priv->pathname,
 				       GNOME_VFS_MONITOR_FILE,
 				       screem_page_monitor_cb, page );
 	}
@@ -527,8 +510,7 @@ gboolean screem_page_is_file_page( const gchar *path )
 	
 	g_return_val_if_fail( path != NULL, FALSE );
 	
-
-	mime_type = screem_get_mime_type( path );
+	mime_type = screem_get_mime_type( path, FALSE );
 
 	ret =  screem_page_is_mime_type_page( mime_type );
 	g_free( mime_type );
@@ -539,14 +521,24 @@ gboolean screem_page_is_file_page( const gchar *path )
 gboolean screem_page_is_mime_type_page( const gchar *mime_type )
 {
 	gboolean is_page;
-
+	const gchar *end;
+	
 	is_page = FALSE;
 	
-	if(  mime_type ) {
+	if( mime_type ) {
 		is_page = ! strncmp( mime_type, "text/", strlen( "text/" ) );
 		is_page |= ! strcmp( mime_type, "application/x-php" );
 		is_page |= ! strcmp( mime_type, "application/x-asp" );
 		is_page |= ! strcmp( mime_type, "application/x-cgi" );
+		is_page |= ! strcmp( mime_type, "application/x-screem-tag-tree" );
+		if( ! is_page && ! strncmp( "application/", mime_type,
+					strlen( "application/" ) ) ) {
+			
+			end = g_utf8_strrchr( mime_type, -1, '+'  );
+			if( end ) {
+				is_page |= ! strcmp( end, "+xml" );
+			}
+		}
 	}
 	
 	return is_page;
@@ -588,8 +580,7 @@ void screem_page_add_bookmark( ScreemPage *page, gint line, const gchar *name )
 	if( bookmark ) {
 		/* already got a bookmark with this name, just update
 		   the line */
-		g_free( bookmark->line_number );
-		bookmark->line_number = g_strdup_printf( "%d", line );
+		bookmark->line_number = line;
 
 		gtk_tree_model_foreach( GTK_TREE_MODEL(private->bookmarkstore),
 					bookmark_store_find, bookmark );	
@@ -600,7 +591,7 @@ void screem_page_add_bookmark( ScreemPage *page, gint line, const gchar *name )
 	} else {
 
 		bookmark = g_new0( ScreemPageBookMark, 1 );
-		bookmark->line_number = g_strdup_printf( "%d", line );
+		bookmark->line_number = line;
 		bookmark->name = g_string_new( name );
 
 		gtk_list_store_append( private->bookmarkstore, &it );
@@ -668,7 +659,7 @@ gint screem_page_get_bookmark( ScreemPage *page, const gchar *name )
 
 	bookmark = g_hash_table_lookup( table, name );
 	if( bookmark ) {
-		return atoi( bookmark->line_number );
+		return bookmark->line_number;
 	} else {
 		return -1;
 	}
@@ -790,7 +781,41 @@ gboolean screem_page_is_dynamic( ScreemPage *page )
 
 gboolean screem_page_is_markup( ScreemPage *page )
 {
-	return screem_page_is_feature( page, "markup" );
+	gboolean ret;
+
+	ret = ( screem_page_is_feature( page, "markup" ) ||
+		screem_page_is_xml( page ) );
+	
+	return ret;
+}
+
+gchar *screem_page_template_path( ScreemPage *page )
+{
+	gchar *text;
+	gint len;
+	gchar *name;
+	gchar *ret;
+	gchar *end;
+
+	g_return_val_if_fail( SCREEM_IS_PAGE( page ), FALSE );
+
+	if( ! screem_page_is_markup( page ) ) {
+		return NULL;
+	}
+	
+	text = screem_page_get_data( page );
+	name = find_text( text, 
+			"<!-- #BeginTemplate[ \t]+\"[^\"]*\"[ \t]*-->",
+			NULL, &len );
+	ret = NULL;
+	if( name ) {
+		name += strlen( "<!-- #BeginTemplate \"" );
+		end = strchr( name, '"' );
+		ret = g_strndup( name, end - name );
+	}
+	g_free( text );
+
+	return ret;
 }
 
 gboolean screem_page_is_template( ScreemPage *page, gchar **tag,
@@ -823,7 +848,8 @@ gboolean screem_page_is_template( ScreemPage *page, gchar **tag,
 				
 		pathname = screem_page_get_pathname( page );
 		if( ! gnome_vfs_uris_match( pathname, *path ) ) {
-			/* page is not the master template, it just uses it */
+			/* page is not the master template, it just 
+			 * uses it */
 			g_free( *path );
 			g_free( *tag );
 			ret = FALSE;
@@ -900,13 +926,33 @@ gboolean screem_page_select_context( ScreemPage *page,
 			pos, start, end, select_text );
 }
 
+gboolean screem_page_select_parent_context( ScreemPage *page,
+				  guint pos,
+				  guint *start, guint *end,
+				  gboolean select_text )
+{
+	return screem_page_model_select_parent_context( page->private->model,
+			pos, start, end, select_text );
+}
+
+gboolean screem_page_select_content( ScreemPage *page,
+				  guint pos,
+				  guint *start, guint *end,
+				  gboolean select_text )
+{
+	return screem_page_model_select_content( page->private->model,
+			pos, start, end, select_text );
+}
+
 gchar *screem_page_query_context( ScreemPage *page,
 		guint pos, gboolean query_text,
 		gboolean build, 
-		guint *depth, guint *start, guint *end )
+		guint *depth, guint *start, guint *end,
+		xmlNodePtr *node )
 {
 	return screem_page_model_query_context( page->private->model,
-			pos, query_text, build, depth, start, end );
+			pos, query_text, build, depth, start, end,
+			node );
 }
 
 ScreemDTD *screem_page_get_dtd( ScreemPage *page )
@@ -934,6 +980,99 @@ void screem_page_model_emit_built( ScreemPage *page )
 		       screem_page_signals[ BUILT ], 0 );
 }
 
+gboolean screem_page_is_xml( ScreemPage *page )
+{
+	ScreemPagePrivate *priv;
+	gboolean ret;
+	ScreemDTD *dtd;
+	const gchar *mime;
+	const gchar *pathname;
+	gchar *id;
+	gchar *data;
+	const gchar *tmp;
+	
+	g_return_val_if_fail( SCREEM_IS_PAGE( page ), FALSE );
+	
+	priv = page->private;
+	
+	id = NULL;
+	ret = TRUE;
+	dtd = screem_page_get_dtd( page );
+	mime = screem_page_get_mime_type( page );
+	pathname = screem_page_get_pathname( page );
+	
+	ret = ! strcmp( "text/xml", mime );
+	if( ! ret ) {
+		ret = ( gnome_vfs_mime_type_get_equivalence( priv->mime_type, "text/xml" ) == GNOME_VFS_MIME_PARENT );
+	}
+	if( ! ret ) {
+		tmp = strrchr( priv->mime_type, '+' );
+		ret = ( tmp && ! strcmp( tmp, "+xml" ) );
+	}
+	if( ! ret ) {
+		if( dtd ) {
+			g_object_get( G_OBJECT( dtd ), 
+					"public", &id, NULL );
+		}
+		if( id ) {
+			ret = ( strstr( id, " XHTML " ) != NULL );
+			ret |= ( strstr( id, " xhtml " ) != NULL );
+			g_free( id );
+		}
+		if( pathname && ! ret ) {
+			ret = ! strcmp( ".xml", 
+					g_extension_pointer( pathname ) );
+			ret |= ! strcmp( ".xhtml",
+					g_extension_pointer( pathname ) );
+		}
+		if( ! ret ) {
+			data = screem_page_get_data( page );
+			ret = ! strncmp( "<?xml", data, 
+					strlen( "<?xml" ) );
+			g_free( data );
+		}
+	}
+	
+	return ret;
+}
+
+void screem_page_set_lang( ScreemPage *page )
+{
+	ScreemPagePrivate *priv;
+	GtkSourceLanguage *lang;
+	GtkSourceLanguagesManager *lm;
+	GtkTextTagTable *table;
+	GtkTextTag *tag;
+
+	g_return_if_fail( SCREEM_IS_PAGE( page ) );
+	
+	priv = page->private;
+	
+	lm = screem_application_load_syntax_tables();
+
+	lang = gtk_source_languages_manager_get_language_from_mime_type( lm, priv->mime_type );
+
+	if( ! lang && screem_page_is_xml( page ) ) {
+		lang = gtk_source_languages_manager_get_language_from_mime_type( lm, "text/xml" );
+	}
+	
+	if( lang ) {
+		gtk_source_buffer_set_language( GTK_SOURCE_BUFFER( page ), lang );
+		table = gtk_text_buffer_get_tag_table( GTK_TEXT_BUFFER( page ) );
+		if( ! gtk_text_tag_table_lookup( table,
+					SCREEM_INVALID_MARKUP_TAG ) ) {
+ 
+			tag = gtk_text_tag_new( SCREEM_INVALID_MARKUP_TAG );
+			gtk_text_tag_table_add( table, tag );
+			g_object_set( G_OBJECT( tag ),
+				"background", "#ffffffff0000",
+				"foreground", "#000000000000",
+				NULL );
+			g_object_unref( tag );
+		} 
+	}
+}
+
 /* G Object stuff */
 #define PARENT_TYPE GTK_TYPE_SOURCE_BUFFER
 
@@ -1008,6 +1147,15 @@ static void screem_page_class_init( ScreemPageClass *klass )
 							    G_PARAM_READWRITE)
 					);
 	
+	screem_page_signals[ SAVING ] =
+		g_signal_new( "saving",
+			      G_OBJECT_CLASS_TYPE( object_class ),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET( ScreemPageClass, 
+					       saving),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0 );
 	screem_page_signals[ SAVED ] =
 		g_signal_new( "saved",
 			      G_OBJECT_CLASS_TYPE( object_class ),
@@ -1054,14 +1202,19 @@ static void screem_page_init( ScreemPage *page )
 
 	/* NAME, LINE, BOOKMARK STRUCT */
 	page->private->bookmarkstore = gtk_list_store_new( 3, G_TYPE_STRING,
-							   G_TYPE_STRING,
+							   G_TYPE_INT,
 							   G_TYPE_POINTER,
 							   NULL );	
-	page->private->mime_type = g_strdup( "text/plain" );
+	page->private->mime_type = gconf_client_get_string( page->private->client, "/apps/screem/default_mimetype", NULL );
+	if( ! page->private->mime_type ) {
+		page->private->mime_type = g_strdup( "text/html" );
+	}
 	
 	g_signal_connect( G_OBJECT( page ),
 			  "changed", G_CALLBACK( changed ), page );
 
+	gtk_text_buffer_add_selection_clipboard( GTK_TEXT_BUFFER( page ),
+			gtk_clipboard_get( GDK_SELECTION_PRIMARY ) );
 }
 
 static void screem_page_set_prop( GObject *object, guint prop_id,
