@@ -26,28 +26,13 @@
 
 #include <gconf/gconf-client.h>
 
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 
-#include <libgnomeui/gnome-font-picker.h>
 #include <libgnomeui/gnome-entry.h>
 #include <libgnomeui/gnome-file-entry.h>
 #include <libgnomeui/gnome-icon-entry.h>
-#include <libgnomeui/gnome-color-picker.h>
 
-#include <gtk/gtkentry.h>
-#include <gtk/gtktogglebutton.h>
-#include <gtk/gtkspinbutton.h>
-#include <gtk/gtkdialog.h>
-#include <gtk/gtkcombo.h>
-#include <gtk/gtkliststore.h>
-#include <gtk/gtktreeview.h>
-#include <gtk/gtktreeselection.h>
-#include <gtk/gtkcellrenderer.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkframe.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkpaned.h>
-#include <gtk/gtknotebook.h>
+#include <gtk/gtk.h>
 
 #include <gdk/gdkcolor.h>
 #include <gdk/gdkkeysyms.h>
@@ -80,6 +65,7 @@ static ScreemPrefStruct misc_prefs[] = {
 	{ "default_filename_entry", "/apps/screem/default_filename",       SCREEM_PREF_STRING,	0 },
 	{ "default_charset_combo",  "/apps/screem/editor/default_charset", SCREEM_PREF_STRING,	0 },
 	{ "default_dtd_combo",      "/apps/screem/editor/default_dtd",     SCREEM_PREF_STRING,	0 },
+	{ "mimetype",      	    "/apps/screem/default_mimetype",     	   SCREEM_PREF_STRING,	0 },
 	{ NULL, NULL, 0, 0 }
 };
 
@@ -99,7 +85,12 @@ static ScreemPrefStruct editor_features[] = {
 	{ "indent", 		  "/apps/screem/editor/auto_indent", 	 SCREEM_PREF_TOGGLE,	0 },
 	{ "tagclose", 		  "/apps/screem/editor/intelliclose",    SCREEM_PREF_TOGGLE,	0 },
 	{ "wordwrap",		  "/apps/screem/editor/wordwrap",	 SCREEM_PREF_TOGGLE,	0 },
+	{ "margin",		  "/apps/screem/editor/right_margin",	 SCREEM_PREF_TOGGLE,	0 },
+	{ "margin_size",  "/apps/screem/editor/margin_column",	 SCREEM_PREF_SPIN,	0 },
+
 	{ "autocomplete",	  "/apps/screem/editor/autocomplete",    SCREEM_PREF_TOGGLE,    0 },
+	{ "tooltips",	  "/apps/screem/editor/tooltips",    SCREEM_PREF_TOGGLE,    0 },
+	{ "spaces", 		  "/apps/screem/editor/spaces_instead_of_tabs", 	 SCREEM_PREF_TOGGLE,	0 },
 	{ "tabwidth", 	   	  "/apps/screem/editor/tabwidth",	 SCREEM_PREF_SPIN,	0 },
 	{ "autoenton",		  "/apps/screem/editor/entityinsertion", SCREEM_PREF_RADIO,	0 },
 	{ "autoentoff",		  "/apps/screem/editor/entityinsertion", SCREEM_PREF_RADIO,	1 },
@@ -120,16 +111,11 @@ static ScreemPrefStruct browser_prefs[] = {
 	{ "browser1_icon",    "/apps/screem/Browser1_icon",    SCREEM_PREF_STRING,	0 },
 	{ "browser2_icon",    "/apps/screem/Browser2_icon",    SCREEM_PREF_STRING,	0 },
 	{ "browser3_icon",    "/apps/screem/Browser3_icon",    SCREEM_PREF_STRING,	0 },
-	{ NULL, NULL, 0, 0 }
-};
+	{ "browser1_name",    "/apps/screem/Browser1_name",    SCREEM_PREF_STRING,		0 },
+	{ "browser2_name",    "/apps/screem/Browser2_name",    SCREEM_PREF_STRING,		0 },
+	{ "browser3_name",    "/apps/screem/Browser3_name",    SCREEM_PREF_STRING,		0 },
 
-static const gchar *charsets[] = {
-	"ISO-8859-1",
-	"ISO-8859-2",
-	"ISO-8859-5",
-	"ISO-8859-7",
-	"UTF-8",
-	NULL
+	{ NULL, NULL, 0, 0 }
 };
 
 static void screem_preferences_helper_select( GtkTreeSelection *sel );
@@ -192,48 +178,50 @@ static void screem_preferences_add_page( GtkWidget *dialog,
 
 void screem_preferences_edit( ScreemApplication *application )
 {
+	ScreemSession *session;
 	ScreemDTDDB *db;
-	GConfClient *client;
 	gchar *gladepath;
 	GladeXML *xml;
 	GtkWidget *widget;
-	gint i;
 	static GtkWidget *dialog = NULL;
-	GList *list;
-
+	GtkTreeModel *list;
+	
 	if( dialog ) {
 		gtk_widget_show( dialog );
 		gdk_window_raise( GDK_WINDOW( dialog->window ) );
 		return;
 	}
 
+	session = screem_application_get_session( application );
 	db = screem_application_get_dtd_db( application );
 
-	client = gconf_client_get_default();
-	
 	/* create dialog */
-	gladepath = gconf_client_get_string( client, 
-					     "/apps/screem/general/glade_path",
-					     NULL );
-	if( ! gladepath ) {
-		gladepath = g_strconcat( GLADE_PATH, "/screem.glade",
-					 NULL );
-	}
+	gladepath = screem_get_glade_path();
 
 	xml = glade_xml_new( gladepath, "preferences", NULL );
 	dialog = glade_xml_get_widget( xml, "preferences" );
+	g_object_set_data( G_OBJECT( dialog ), "session", session );
 	glade_xml_signal_autoconnect( xml );
 		
 	widget = glade_xml_get_widget( xml, "default_dtd_combo" );
-	screem_dtd_db_fill_combo( db, widget );
-	
+	list = GTK_TREE_MODEL( screem_dtd_db_get_store( db ) );
+	gtk_combo_box_set_model( GTK_COMBO_BOX( widget ), list ); 
+	gtk_combo_box_entry_set_text_column( GTK_COMBO_BOX_ENTRY( widget ),
+			0 );
+
 	widget = glade_xml_get_widget( xml, "default_charset_combo" );
-	for( list = NULL, i = 0; charsets[ i ]; ++ i ) {
-		list = g_list_prepend( list, (gpointer)charsets[ i ] );
-	}
-	list = g_list_reverse( list );
-	gtk_combo_set_popdown_strings( GTK_COMBO( widget ), list );
-	g_list_free( list );
+	list = screem_application_get_encodings( application );
+	gtk_combo_box_set_model( GTK_COMBO_BOX( widget ), list ); 
+	gtk_combo_box_entry_set_text_column( GTK_COMBO_BOX_ENTRY( widget ),
+			0 );
+
+	widget = glade_xml_get_widget( xml, "mimetype" );
+
+	list = GTK_TREE_MODEL( screem_support_get_mime_types_store() );
+	gtk_combo_box_set_model( GTK_COMBO_BOX( widget ), list ); 
+	g_object_unref( list );
+	gtk_combo_box_entry_set_text_column( GTK_COMBO_BOX_ENTRY( widget ),
+			0 );
 	
 	screem_preferences_edit_dtds( application, xml );
 
@@ -253,9 +241,9 @@ void screem_preferences_edit( ScreemApplication *application )
 	screem_preferences_add_page( dialog, editor_features );
 	screem_preferences_add_page( dialog, browser_prefs );
 	
+	screem_session_restore_dialog( session, dialog );
+	
 	gtk_widget_show( dialog );
-
-	g_object_unref( client );
 }
 
 static void screem_preferences_toggle_notify( GConfClient *client,
@@ -344,19 +332,16 @@ static void screem_preferences_colour_notify( GConfClient *client,
 		GdkColor colour;
 		GtkWidget *widget;
 		GdkColor pcolour;
-		gushort junk;
 	
 		value = gconf_value_get_string( entry->value );
 		widget = GTK_WIDGET( data );
-		gnome_color_picker_get_i16( GNOME_COLOR_PICKER( widget ),
-				    &pcolour.red, &pcolour.green, 
-				    &pcolour.blue, &junk );
+		gtk_color_button_get_color( GTK_COLOR_BUTTON( widget ),
+				&pcolour );
 
 		if( gdk_color_parse( value, &colour ) && 
 		    ! gdk_color_equal( &colour, &pcolour ) ) {
-			gnome_color_picker_set_i16( GNOME_COLOR_PICKER(widget),
-						    colour.red, colour.green,
-						    colour.blue, 0 );
+			gtk_color_button_set_color( GTK_COLOR_BUTTON(widget),
+					&pcolour );
 		}
 	}
 }
@@ -398,45 +383,74 @@ static void screem_preferences_string_notify( GConfClient *client,
 					      GConfEntry *entry,
 					      gpointer data )
 {
-	if( entry->value && entry->value->type == GCONF_VALUE_STRING ) {
-		const gchar *value;
-		GtkWidget *widget;
-		GtkWidget *gtkentry;
+	const gchar *value;
+	GtkWidget *widget;
+	GtkWidget *gtkentry;
+	const gchar *cvalue;
 
-		value = gconf_value_get_string( entry->value );
-		widget = GTK_WIDGET( data );
-		gtkentry = NULL;
-		
-		if( GNOME_IS_ENTRY( widget ) ) {
-			gtkentry = gnome_entry_gtk_entry( GNOME_ENTRY(widget));
-		} else if( GTK_IS_ENTRY( widget ) ) {
-			gtkentry = widget;
-		} else if( GTK_IS_COMBO( widget ) ) {
-			gtkentry = GTK_COMBO( widget )->entry;
-		} else if( GNOME_IS_FILE_ENTRY( widget ) ) {
-			gtkentry = gnome_file_entry_gtk_entry( GNOME_FILE_ENTRY( widget ) );
-		} else if( GNOME_IS_ICON_ENTRY( widget ) ) {
-			const gchar *cvalue;
-			cvalue = gnome_icon_entry_get_filename( GNOME_ICON_ENTRY( widget ) );
-			if( ! cvalue || strcmp( cvalue, value ) )
-				gnome_icon_entry_set_filename( GNOME_ICON_ENTRY( widget ),
-							      value);
-		} else if( GNOME_IS_FONT_PICKER( widget ) ) {
-			const gchar *cvalue;
-			cvalue = gnome_font_picker_get_font_name( GNOME_FONT_PICKER( widget ) );
-			if( ! cvalue || strcmp( cvalue, value ) )
-				gnome_font_picker_set_font_name( GNOME_FONT_PICKER( widget ), value );
-			
-		}
-		if( gtkentry ) {
-			const gchar *cvalue;
-			cvalue = gtk_entry_get_text( GTK_ENTRY( gtkentry ) );
-			if( value && strcmp( value, cvalue ) ) {
-				gtk_entry_set_text( GTK_ENTRY( gtkentry ),
-						    value );
+	GtkTreeModel *model;
+	GtkTreeIter *it;
+	gint cols;
+	gchar *colval;
+	
+	if( entry->value && entry->value->type != GCONF_VALUE_STRING ) {
+		return;
+	}
+
+	value = gconf_value_get_string( entry->value );
+	widget = GTK_WIDGET( data );
+	gtkentry = NULL;
+	colval = NULL;	
+	
+	if( GNOME_IS_ENTRY( widget ) ) {
+		gtkentry = gnome_entry_gtk_entry( GNOME_ENTRY(widget));
+	} else if( GTK_IS_ENTRY( widget ) ) {
+		gtkentry = widget;
+	} else if( GTK_IS_COMBO_BOX_ENTRY( widget ) ) {
+		gtkentry = GTK_BIN( widget )->child;
+
+		/* combo boxes are special, they have display text
+		 * and possibly value text as well, for our
+		 * purposes they are up to 2 column GtkListStores,
+		 * 1 column display == value, 2 column col 2 == value */
+		model = gtk_combo_box_get_model( GTK_COMBO_BOX( widget ) );
+		cols = gtk_tree_model_get_n_columns( model );
+
+		if( cols == 2 ) {
+			it = screem_support_find_in_list( GTK_LIST_STORE( model ), 1, value );
+			if( ! it ) {
+				it = screem_support_find_in_list( GTK_LIST_STORE( model ), 0, value );
+			}
+			if( it ) {
+				gtk_tree_model_get( model, it,	
+					0, &colval, -1 );
+				value = colval;
+				gtk_tree_iter_free( it );
 			}
 		}
+	} else if( GNOME_IS_FILE_ENTRY( widget ) ) {
+		gtkentry = gnome_file_entry_gtk_entry( GNOME_FILE_ENTRY( widget ) );
+	} else if( GNOME_IS_ICON_ENTRY( widget ) ) {
+		const gchar *cvalue;
+		cvalue = gnome_icon_entry_get_filename( GNOME_ICON_ENTRY( widget ) );
+		if( ! cvalue || strcmp( cvalue, value ) )
+			gnome_icon_entry_set_filename( GNOME_ICON_ENTRY( widget ),
+						      value);
+	} else if( GTK_IS_FONT_BUTTON( widget ) ) {
+		const gchar *cvalue;
+		cvalue = gtk_font_button_get_font_name( GTK_FONT_BUTTON( widget ) );
+		if( ! cvalue || strcmp( cvalue, value ) )
+			gtk_font_button_set_font_name( GTK_FONT_BUTTON( widget ), value );
+		
 	}
+	if( gtkentry ) {
+		cvalue = gtk_entry_get_text( GTK_ENTRY( gtkentry ) );
+		if( value && strcmp( value, cvalue ) ) {
+			gtk_entry_set_text( GTK_ENTRY( gtkentry ),
+					    value );
+		}
+	}
+	g_free( colval );
 }
 
 static gboolean screem_preferences_icon_changed( GtkWidget *widget )
@@ -461,65 +475,106 @@ static gboolean screem_preferences_icon_changed( GtkWidget *widget )
 	return TRUE;
 }
 
-static gboolean screem_preferences_string_set( GtkWidget *widget )
+static gboolean screem_preferences_string_set( GtkWidget *widget,
+		GtkWidget *parent )
 {
 	GtkWidget *entry = NULL;
 	const gchar *cvalue = NULL;
 	const gchar *key;
 
+	GtkTreeModel *model;
+	GtkTreeIter *it;
+	gint cols;
+	gchar *colval;
+	gchar *disval;
+
 	GConfClient *client;
 
 	client = gconf_client_get_default();
 	
 	key = g_object_get_data( G_OBJECT( widget ), "key" );
-
+	colval = NULL;
+	
+	if( parent ) {
+		widget = parent;
+	}
+	
 	if( GNOME_IS_ENTRY( widget ) ) {
 		entry = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
 		cvalue = gtk_entry_get_text( GTK_ENTRY( entry ) );
-	} else if( GTK_IS_COMBO( widget ) ) {
-		entry = GTK_COMBO( widget )->entry;
+	} else if( GTK_IS_COMBO_BOX_ENTRY( widget ) ) {
+		entry = GTK_BIN( widget )->child;
 		cvalue = gtk_entry_get_text( GTK_ENTRY( entry ) );
+
+		/* combo boxes are special, they have display text
+		 * and possibly value text as well, for our
+		 * purposes they are up to 2 column GtkListStores,
+		 * 1 column display == value, 2 column col 2 == value */
+		model = gtk_combo_box_get_model( GTK_COMBO_BOX( widget ) );
+		cols = gtk_tree_model_get_n_columns( model );
+		if( cols == 2 ) {
+			it = screem_support_find_in_list( GTK_LIST_STORE( model ), 1, cvalue );
+			if( ! it ) {
+				it = screem_support_find_in_list( GTK_LIST_STORE( model ), 0, cvalue );
+			}
+			if( it ) {
+				gtk_tree_model_get( model, it,	
+					0, &disval,
+					1, &colval, -1 );
+				cvalue = colval;
+				gtk_tree_iter_free( it );
+
+				if( strcmp( cvalue, disval ) ) {
+					gtk_entry_set_text( GTK_ENTRY( entry ), disval );
+				}
+				g_free( disval );
+			}
+		}
 	} else if( GNOME_IS_FILE_ENTRY( widget ) ) {
 		entry = gnome_file_entry_gtk_entry( GNOME_FILE_ENTRY( widget ) );
 		cvalue = gtk_entry_get_text( GTK_ENTRY( entry ) );	
 	} else if( GTK_IS_ENTRY( widget ) ) {
 		cvalue = gtk_entry_get_text( GTK_ENTRY( widget ) );
 	}
-	    
+	
 	if( cvalue ) {
 		gconf_client_set_string( client, key, 
 					 cvalue, NULL );
 	}
 
+	g_free( colval );
+	
 	g_object_unref( client );
 	
 	return TRUE;
 }
 
 
-static void screem_preferences_font_set( GtkWidget *fp,
-					  const gchar *font )
+static void screem_preferences_font_set( GtkWidget *fp )
 {
 	GConfClient *client;
 	const gchar *key;
+	const gchar *font;
 
 	client = gconf_client_get_default();
 	key = g_object_get_data( G_OBJECT( fp ), "key" );
+	font = gtk_font_button_get_font_name( GTK_FONT_BUTTON( fp ) );
 	gconf_client_set_string( client, key, font, NULL );
 	g_object_unref( client );
 }
 
-static void screem_preferences_colour_set( GnomeColorPicker *cp,
-					   guint r, guint g, guint b,
-					   guint a )
+static void screem_preferences_colour_set( GtkColorButton *cb )
 {
 	GConfClient *client;
 	const gchar *key;
 	gchar *value;
+	GdkColor color;
 
 	client = gconf_client_get_default();
-	key = g_object_get_data( G_OBJECT( cp ), "key" );
-	value = g_strdup_printf( "#%.4x%.4x%.4x", r, g, b );
+	key = g_object_get_data( G_OBJECT( cb ), "key" );
+	gtk_color_button_get_color( cb, &color );
+	value = g_strdup_printf( "#%.4x%.4x%.4x", 
+			color.red, color.green, color.blue );
 	gconf_client_set_string( client, key, value, NULL );
 	g_free( value );
 	g_object_unref( client );
@@ -552,6 +607,7 @@ static gboolean screem_preferences_radio_changed( GtkWidget *toggle )
 
 void screem_preferences_edit_macros( ScreemApplication *application )
 {
+	ScreemSession *session;
 	GtkListStore *store;
 	GList *macros;
 
@@ -562,25 +618,16 @@ void screem_preferences_edit_macros( ScreemApplication *application )
 	gint i;
 	GtkTreeSelection *selection;
 
-	GConfClient *client;
 	gchar *gladepath;
 	GladeXML *xml;
 	GtkWidget *widget;
 
-	client = gconf_client_get_default();
+	session = screem_application_get_session( application );
 	
-	gladepath = gconf_client_get_string( client, 
-					     "/apps/screem/general/glade_path",
-					     NULL );
-	if( ! gladepath ) {
-		gladepath = g_strconcat( GLADE_PATH, "/screem.glade",
-					 NULL );
-	}
-
+	gladepath = screem_get_glade_path();
 	xml = glade_xml_new( gladepath, "macros", NULL );
 	glade_xml_signal_autoconnect( xml );	
 
-	
 	store = gtk_list_store_new( 3, G_TYPE_STRING, G_TYPE_STRING,
 				    G_TYPE_POINTER, NULL );
 	/* build model */
@@ -621,10 +668,12 @@ void screem_preferences_edit_macros( ScreemApplication *application )
 			  NULL );
 
 	widget = glade_xml_get_widget( xml, "macros" );
+	g_object_set_data( G_OBJECT( widget ), "session", session );
 
+	screem_session_restore_dialog( session, widget );
+	
 	gtk_widget_show_all( widget );
 	g_free( gladepath );
-	g_object_unref( G_OBJECT( client ) );
 }
 
 /* also used for the update button as we perform a check to see
@@ -653,7 +702,7 @@ void screem_preferences_add_macro( GtkWidget *widget )
 	combo = gtk_entry_get_text( GTK_ENTRY( widget ) );
 
 	widget = glade_xml_get_widget( xml, "macroaction" );
-	widget = GTK_COMBO( widget )->entry;
+	widget = GTK_BIN( widget )->child;
 	action = gtk_entry_get_text( GTK_ENTRY( widget ) );
 
 	macro = g_new0( ScreemEditorMacro, 1 );
@@ -728,7 +777,7 @@ static void screem_preferences_macro_select( GtkTreeSelection *sel )
 				    g_value_get_string( &nvalue ) );
 
 		widget = glade_xml_get_widget( xml, "macroaction" );
-		widget = GTK_COMBO( widget )->entry;
+		widget = GTK_BIN( widget )->child;
 		gtk_entry_set_text( GTK_ENTRY( widget ),
 				    g_value_get_string( &pvalue ) );
 
@@ -743,7 +792,7 @@ static void screem_preferences_macro_select( GtkTreeSelection *sel )
 		widget = glade_xml_get_widget( xml, "macrocombo" );
 		gtk_entry_set_text( GTK_ENTRY( widget ), "" );
 		widget = glade_xml_get_widget( xml, "macroaction" );
-		widget = GTK_COMBO( widget )->entry;
+		widget = GTK_BIN( widget )->child;
 		gtk_entry_set_text( GTK_ENTRY( widget ), "" );
 
 		widget = glade_xml_get_widget( xml, "remove" );
@@ -781,6 +830,7 @@ gboolean screem_preferences_grab_key( GtkWidget *widget, GdkEventKey *event )
 
 void screem_preferences_edit_helpers( ScreemApplication *application  )
 {
+	ScreemSession *session;
 	GtkListStore *store;
 	GList *helpers;
 	GtkWidget *view;
@@ -790,21 +840,14 @@ void screem_preferences_edit_helpers( ScreemApplication *application  )
 	const gchar *titles[] = { "Name", "Path", "Icon", NULL };
 	gint i;
 
-	GConfClient *client;
 	gchar *gladepath;
 	GladeXML *xml;
 	GtkWidget *widget;
+	GtkTreeIter it;
 
-	client = gconf_client_get_default();
+	session = screem_application_get_session( application );
 	
-	gladepath = gconf_client_get_string( client, 
-					     "/apps/screem/general/glade_path",
-					     NULL );
-	if( ! gladepath ) {
-		gladepath = g_strconcat( GLADE_PATH, "/screem.glade",
-					 NULL );
-	}
-
+	gladepath = screem_get_glade_path();
 	xml = glade_xml_new( gladepath, "helpers", NULL );
 	glade_xml_signal_autoconnect( xml );	
 
@@ -816,7 +859,6 @@ void screem_preferences_edit_helpers( ScreemApplication *application  )
 				    NULL );
 	/* build model */
 	for(helpers = application->helpers; helpers; helpers = helpers->next) {
-		GtkTreeIter it;
 		ScreemHelper *helper;
 		const gchar *name;
 		const gchar *pathname;
@@ -856,16 +898,32 @@ void screem_preferences_edit_helpers( ScreemApplication *application  )
 				 GTK_TREE_MODEL( store ) );
 	g_object_unref( store );
 
+	widget = glade_xml_get_widget( xml, "helpertype" );
+	store = screem_support_get_mime_types_store();
+	gtk_list_store_prepend( store, &it );
+	gtk_list_store_set( store, &it,
+			0, _( "All Types" ),
+			1, "*",
+			-1 );
+	gtk_combo_box_set_model( GTK_COMBO_BOX( widget ), 
+			GTK_TREE_MODEL( store ) ); 
+	gtk_combo_box_entry_set_text_column( GTK_COMBO_BOX_ENTRY( widget ),
+			0 );
+	gtk_combo_box_set_active( GTK_COMBO_BOX( widget ), 0 );
+	g_object_unref( store );
+	
 	selection = gtk_tree_view_get_selection( GTK_TREE_VIEW( view ) );
 	g_signal_connect( G_OBJECT( selection ), "changed",
 			  G_CALLBACK( screem_preferences_helper_select ),
 			  NULL );
 
 	widget = glade_xml_get_widget( xml, "helpers" );
+	g_object_set_data( G_OBJECT( widget ), "session", session );
+
+	screem_session_restore_dialog( session, widget );
 
 	gtk_widget_show_all( widget );
 	g_free( gladepath );
-	g_object_unref( G_OBJECT( client ) );
 }
 
 /* also used for the update button as we perform a check to see
@@ -885,6 +943,9 @@ void screem_preferences_add_helper( GtkWidget *widget )
 
 	HelperInputMode imode;
 	HelperOutputMode omode;
+
+	gboolean exec;
+	gchar *exectype;
 	
 	gboolean found;
 	ScreemHelper *old;
@@ -893,7 +954,6 @@ void screem_preferences_add_helper( GtkWidget *widget )
 	xml = glade_get_widget_tree( widget );
 
 	view = glade_xml_get_widget( xml, "helperlist" );
-	model = gtk_tree_view_get_model( GTK_TREE_VIEW( view ) );
 
 	application = SCREEM_APPLICATION( g_object_get_data( G_OBJECT( view ),
 							     "application" ) );
@@ -929,14 +989,37 @@ void screem_preferences_add_helper( GtkWidget *widget )
 		omode = SCREEM_HELPER_STDOUT_INSERT;
 	}
 	
+	widget = glade_xml_get_widget( xml, "helperexec" );
+	exec = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( widget ) );
+	widget = glade_xml_get_widget( xml, "helpertype" );
+	model = gtk_combo_box_get_model( GTK_COMBO_BOX( widget ) );
+	if( gtk_combo_box_get_active_iter( GTK_COMBO_BOX( widget ), &it ) ) {
+		gtk_tree_model_get( model, &it,
+				1, &exectype,
+				-1 );
+	} else {
+		widget = GTK_BIN( widget )->child;
+		exectype = (gchar*)gtk_entry_get_text( GTK_ENTRY( widget ) );
+		exectype = g_strdup( exectype );
+	}
+	
 	helper = screem_helper_new( name, pathname, iconpath,
 				    imode, omode );
 
+	g_object_set( G_OBJECT( helper ), 
+			"exec_on_save", exec,
+			"exec_type", exectype,
+			NULL );
+	g_free( exectype );
+	
 	screem_application_remove_helper( application, helper );
 	screem_application_add_helper( application, helper );
 
+	screem_helper_save( helper );
+	
 	found = FALSE;
 	old = NULL;
+	model = gtk_tree_view_get_model( GTK_TREE_VIEW( view ) );
 	if( gtk_tree_model_get_iter_first( GTK_TREE_MODEL( model ),
 					&it ) ) {
 		do {
@@ -1022,42 +1105,42 @@ static void screem_preferences_helper_select( GtkTreeSelection *sel )
 	xml = glade_get_widget_tree( GTK_WIDGET( view ) );
 
 	if( gtk_tree_selection_get_selected( sel, &model, &it ) ) {
-		GValue nvalue = { 0 };
-		GValue pvalue = { 0 };
-		GValue ivalue = { 0 };
-		GValue hvalue = { 0 };
-		GladeXML *xml;
-		GtkWidget *widget;
 		ScreemHelper *helper;
+		gchar *name;
+		gchar *path;
+		gchar *icon;
 		gint imode;
 		gint omode;
-		
-		gtk_tree_model_get_value( model, &it, 0, &nvalue );
-		gtk_tree_model_get_value( model, &it, 1, &pvalue );
-		gtk_tree_model_get_value( model, &it, 2, &ivalue );
-		gtk_tree_model_get_value( model, &it, 3, &hvalue );
-		
-		xml = glade_get_widget_tree( GTK_WIDGET( view ) );
-		
+		gboolean exec;
+		gchar *exectype;
+		GtkTreeIter *tit;
+	
+		gtk_tree_model_get( model, &it, 3, &helper, -1 );
+		g_object_get( G_OBJECT( helper ), 
+				"name", &name,
+				"pathname", &path,
+				"iconpath", &icon,
+				"input", &imode, 
+				"output", &omode, 
+				"exec_on_save", &exec,
+				"exec_type", &exectype,
+				NULL );
+			
 		widget = glade_xml_get_widget( xml, "helpername" );
 		widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
-		gtk_entry_set_text( GTK_ENTRY( widget ),
-				    g_value_get_string( &nvalue ) );
+		gtk_entry_set_text( GTK_ENTRY( widget ), name );
+		g_free( name );
 
 		widget = glade_xml_get_widget( xml, "helperpath" );
 		widget = gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(widget));
-		gtk_entry_set_text( GTK_ENTRY( widget ),
-				    g_value_get_string( &pvalue ) );
+		gtk_entry_set_text( GTK_ENTRY( widget ), path );
+		g_free( path );
 
 		widget = glade_xml_get_widget( xml, "helpericon" );
 		gnome_icon_entry_set_filename( GNOME_ICON_ENTRY( widget ),
-					       g_value_get_string( &ivalue ) );
+				icon );
+		g_free( icon );
 
-		helper = g_value_get_pointer( &hvalue );
-		g_assert( helper );
-		g_object_get( G_OBJECT( helper ), 
-				"input", &imode, 
-				"output", &omode, NULL );
 		widget = glade_xml_get_widget( xml, iwidgets[ imode ] );
 		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( widget ),
 						TRUE );
@@ -1065,15 +1148,28 @@ static void screem_preferences_helper_select( GtkTreeSelection *sel )
 		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( widget ),
 						TRUE );
 
+		widget = glade_xml_get_widget( xml, "helperexec" );
+		gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( widget ), exec );
+		widget = glade_xml_get_widget( xml, "helpertype" );
+		model = gtk_combo_box_get_model( GTK_COMBO_BOX( widget ) );
+		tit = screem_support_find_in_list( GTK_LIST_STORE( model ), 1, exectype );
+		if( ! tit ) {
+			tit = screem_support_find_in_list( GTK_LIST_STORE( model ), 0, exectype );
+		}
+		if( tit ) {
+			gtk_combo_box_set_active_iter( GTK_COMBO_BOX( widget ), tit );
+			gtk_tree_iter_free( tit );
+		} else {
+			widget = GTK_BIN( widget )->child;
+			gtk_entry_set_text( GTK_ENTRY( widget ), 
+					exectype );
+			g_free( exectype );
+		}
+		
 		widget = glade_xml_get_widget( xml, "helperremove" );
 		gtk_widget_set_sensitive( widget, TRUE );
 		widget = glade_xml_get_widget( xml, "helperupdate" );
 		gtk_widget_set_sensitive( widget, TRUE );
-
-		g_value_unset( &nvalue );
-		g_value_unset( &pvalue );
-		g_value_unset( &ivalue );
-		g_value_unset( &hvalue );
 	} else {
 		widget = glade_xml_get_widget( xml, "helpername" );
 		widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
@@ -1416,9 +1512,11 @@ screem_preferences_tag_tree_toggle( GtkCellRendererToggle *toggle,
 
 		gtk_tree_model_get( model, &it,
 				TAG_FILE_OBJ_COL, &tfile, -1 );
-	
+
+		gdk_threads_leave();
 		g_object_set( G_OBJECT( tfile ), 
 				"active", active, NULL );
+		gdk_threads_enter();
 	}
 }
 
@@ -1572,7 +1670,6 @@ void screem_preferences_add_tag_file( GtkWidget *widget )
 {
 	GladeXML *xml;
 	GladeXML *dxml;
-	GConfClient *client;
 	gchar *gladepath;
 	const gchar *prompt;
 	gchar *txt;
@@ -1585,17 +1682,9 @@ void screem_preferences_add_tag_file( GtkWidget *widget )
 	widget = glade_xml_get_widget( xml, "tagtreeview" );
 	model = gtk_tree_view_get_model( GTK_TREE_VIEW( widget ) );
 	
-	client = gconf_client_get_default();
 
-	gladepath = gconf_client_get_string( client, 
-					     "/apps/screem/general/glade_path",
-					     NULL );
-	if( ! gladepath ) {
-		gladepath = g_strconcat( GLADE_PATH, "/screem.glade",
-					 NULL );
-	}
-	
-	dxml = glade_xml_new( gladepath, "open_location", NULL );
+	gladepath = screem_get_glade_path();
+	dxml = glade_xml_new( gladepath, "open_file_location", NULL );
 
 	widget = glade_xml_get_widget( dxml, "prompt" );
 	prompt = _( "Enter the URL of the tag tree to add" );
@@ -1603,13 +1692,15 @@ void screem_preferences_add_tag_file( GtkWidget *widget )
 	gtk_label_set_markup( GTK_LABEL( widget ), txt );
 	g_free( txt );
 	
-	widget = glade_xml_get_widget( dxml, "open_location" );
+	widget = glade_xml_get_widget( dxml, "open_file_location" );
 	title = g_strdup_printf( "%s - Screem",
 			_( "Add Tag Tree from Location" ) );
 	gtk_window_set_title( GTK_WINDOW( widget ),
 				title );
 	g_free( title );
 
+	glade_xml_signal_autoconnect( dxml );
+	
 	if( gtk_dialog_run( GTK_DIALOG( widget ) ) == GTK_RESPONSE_OK ) {
 		widget = glade_xml_get_widget( dxml, "url" );
 		url = gtk_entry_get_text( GTK_ENTRY( widget ) );
@@ -1623,7 +1714,7 @@ void screem_preferences_add_tag_file( GtkWidget *widget )
 			gdk_threads_leave();
 			screem_tag_tree_add_uri( url, 
 					SCREEM_TAG_TREE_FORMAT );	
-gdk_threads_enter();
+			gdk_threads_enter();
 		} else {
 			gtk_tree_iter_free( it );
 			widget = gtk_message_dialog_new( NULL,
@@ -1637,10 +1728,9 @@ gdk_threads_enter();
 		}
 	}
 
-	widget = glade_xml_get_widget( dxml, "open_location" );
+	widget = glade_xml_get_widget( dxml, "open_file_location" );
 	gtk_widget_destroy( widget );
 	
-	g_object_unref( client );
 	g_object_unref( dxml );
 }
 
@@ -1666,19 +1756,12 @@ void screem_preferences_page_setup( void )
 	guint handle;
 	gdouble sval;
 	
-	client = gconf_client_get_default();
-	
-	gladepath = gconf_client_get_string( client, 
-					     "/apps/screem/general/glade_path",
-					     NULL );
-	if( ! gladepath ) {
-		gladepath = g_strconcat( GLADE_PATH, "/screem.glade",
-					 NULL );
-	}
-
+	gladepath = screem_get_glade_path();
 	xml = glade_xml_new( gladepath, "page_setup", NULL );
 	glade_xml_signal_autoconnect( xml );	
 
+	client = gconf_client_get_default();
+	
 	for( handles = NULL, i = 0; widgets[ i ]; ++ i ) {
 		
 		widget = glade_xml_get_widget( xml, widgets[ i ] );
@@ -1717,15 +1800,19 @@ void screem_preferences_page_setup( void )
 	gtk_widget_show( widget );
 
 	g_free( gladepath );
-	g_object_unref( client );
+	g_object_unref( G_OBJECT( client ) );
 }
 
 void screem_preferences_dialog_destroy( GtkWidget *widget )
 {
+	ScreemSession *session;
 	GConfClient *client;
 	GSList *handles;
 	GSList *tmp;
 	GladeXML *xml;
+
+	session = g_object_get_data( G_OBJECT( widget ), "session" );
+	screem_session_store_dialog( session, widget );
 	
 	handles = g_object_get_data( G_OBJECT( widget ), "handles" );
 	if( handles ) {
@@ -1739,7 +1826,9 @@ void screem_preferences_dialog_destroy( GtkWidget *widget )
 	}
 
 	xml = glade_get_widget_tree( widget );
-	g_object_unref( G_OBJECT( xml ) );
+	if( xml ) {
+		g_object_unref( G_OBJECT( xml ) );
+	}
 	
 	gtk_widget_destroy( widget );
 }
@@ -1817,9 +1906,8 @@ static void screem_preferences_add_pref( GtkWidget *dialog,
 					  G_CALLBACK( screem_preferences_colour_set ),
 					  NULL );
 			if( gdk_color_parse( sval, &colour ) ) {
-				gnome_color_picker_set_i16( GNOME_COLOR_PICKER(widget),
-							    colour.red, colour.green,
-							    colour.blue, 0 );
+				gtk_color_button_set_color( GTK_COLOR_BUTTON( widget ),
+						&colour );
 			}
 			break;
 		case SCREEM_PREF_STRING:
@@ -1836,12 +1924,12 @@ static void screem_preferences_add_pref( GtkWidget *dialog,
 						  G_CALLBACK( screem_preferences_string_set ),
 						  NULL );
 				gtkentry = widget;
-			} else if( GTK_IS_COMBO( widget ) ) {
-				gtkentry = GTK_COMBO( widget )->entry;
+			} else if( GTK_IS_COMBO_BOX_ENTRY( widget ) ) {
+				gtkentry = GTK_BIN( widget )->child;
 				g_object_set_data( G_OBJECT( gtkentry ), "key", (gchar*)key );
 				g_signal_connect( G_OBJECT( gtkentry ), "changed",
 						  G_CALLBACK( screem_preferences_string_set ),
-						  NULL );
+						  widget );
 			} else if( GNOME_IS_FILE_ENTRY( widget ) ) {
 				g_signal_connect( G_OBJECT( widget ), "changed",
 						  G_CALLBACK( screem_preferences_string_set ),
@@ -1853,11 +1941,11 @@ static void screem_preferences_add_pref( GtkWidget *dialog,
 						  NULL );
 				gnome_icon_entry_set_filename( GNOME_ICON_ENTRY( widget ),
 							       sval);
-			} else if( GNOME_IS_FONT_PICKER( widget ) ) {
+			} else if( GTK_IS_FONT_BUTTON( widget ) ) {
 				g_signal_connect( G_OBJECT( widget ), "font_set",
 						  G_CALLBACK( screem_preferences_font_set ),
 						  NULL );
-				gnome_font_picker_set_font_name( GNOME_FONT_PICKER( widget ), sval );
+				gtk_font_button_set_font_name( GTK_FONT_BUTTON( widget ), sval );
 			}
 			if( gtkentry && sval ) {
 				gtk_entry_set_text( GTK_ENTRY( gtkentry ), sval );
@@ -1878,3 +1966,4 @@ static void screem_preferences_add_pref( GtkWidget *dialog,
 	g_object_set_data( G_OBJECT( dialog ), "handles", handles );
 	g_object_unref( client );
 }
+
