@@ -31,10 +31,14 @@
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-async-ops.h>
 
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 
 #include <libgtkhtml/gtkhtml.h>
 
+#ifdef SCREEM_MOZ
+#include <gtkmozembed.h>
+#endif
+
 #include "screem-view.h"
 
 #include "screem-preview.h"
@@ -159,9 +163,11 @@ static void screem_preview_display_view( ScreemView *view )
 		screem_site_get_preview_static( site ) ) ||
 	      ( dynamic && screem_site_get_preview_dynamic( site ) ) ) ) {
 		gchar *uri;
+		gchar *tmp;
 		
-		pagepath = g_basename( pagepath );
-		uri = g_strconcat( "http://", baseuri, "/", pagepath, NULL );
+		tmp = g_path_get_basename( pagepath );
+		uri = g_strconcat( "http://", baseuri, "/", tmp, NULL );
+		g_free( tmp );
 		
 		screem_preview_uri( SCREEM_PREVIEW( view ),
 				    uri, online );
@@ -189,7 +195,6 @@ static void screem_preview_page( ScreemPreview *preview, ScreemPage *page,
 		const gchar *pathname;
 		const gchar *mime_type;
 		gchar *data;
-		gchar *dirname;
 		gchar *base_uri;
 		GnomeVFSURI *uri;
 		gchar *charset;
@@ -208,7 +213,8 @@ static void screem_preview_page( ScreemPreview *preview, ScreemPage *page,
 		}
 
 		charset = screem_page_get_charset( page );
-		tmp = screem_support_charset_convert_to( data, charset );
+		tmp = screem_support_charset_convert_to( data, 
+				charset, "UTF-8" );
 		if( tmp ) {
 			g_free( data );
 			data = tmp;
@@ -218,28 +224,25 @@ static void screem_preview_page( ScreemPreview *preview, ScreemPage *page,
 		if( pathname ) {
 			uri = gnome_vfs_uri_new( pathname );
 			if( uri ) {
-				dirname = gnome_vfs_uri_extract_dirname( uri );
-				base_uri = 
-					g_strconcat( gnome_vfs_uri_get_scheme( uri ),
-						     "://",
-						     dirname, "/", NULL );
+				if( ! uri->parent ) {
+					tmp = gnome_vfs_uri_to_string( uri, 0 );
+					base_uri = g_path_get_dirname( tmp );
+					g_free( tmp );
+				} else {
+					base_uri = gnome_vfs_uri_to_string( uri->parent, 0 );
+				}
 			} else {
-				dirname = NULL;
 				base_uri = g_strdup( "file:///" );
 			}
 		} else {
 			base_uri = NULL;
 			uri = NULL;
-			dirname = NULL;
 		}
 		screem_preview_display( preview, data, base_uri, online );
 		
 		if( uri ) {
 			gnome_vfs_uri_unref( uri );
 		}
-		if( dirname ) {
-			g_free( dirname );
-		}
 		if( base_uri ) {
 			g_free( base_uri );
 		}
@@ -308,6 +311,7 @@ void screem_preview_display( ScreemPreview *preview, const gchar *text,
 void screem_preview_uri( ScreemPreview *preview, const gchar *uri, 
 			 gboolean online )
 {
+#ifndef SCREEM_MOZ
 	HtmlDocument *document;
 	HtmlDocument *old;
 
@@ -321,11 +325,12 @@ void screem_preview_uri( ScreemPreview *preview, const gchar *uri,
 	if( preview->private->base_uri ) {
 		g_free( preview->private->base_uri );
 	}
-	preview->private->base_uri = g_dirname( uri );
+	preview->private->base_uri = g_path_get_dirname( uri );
 
 	html_document_open_stream( document, "text/html" );
 
 	url_requested( document, uri, document->current_stream, preview );
+#endif
 }
 
 void screem_preview_print( ScreemView *view, gboolean prview )
@@ -336,12 +341,15 @@ void screem_preview_print( ScreemView *view, gboolean prview )
 static void url_requested( HtmlDocument *document, const gchar *url,
 			   HtmlStream *stream, ScreemPreview *preview )
 {
+	ScreemSite *site;
+	ScreemPage *page;
 	GnomeVFSURI *uri;
-	ScreemWindow *window;
 	GnomeVFSAsyncHandle *handle;
 	gboolean online;
+	gboolean dynamic;
 	gchar *full;
 	gchar *msg;
+	const gchar *sitebase;
 	
 	if( document != HTML_VIEW( preview->private->html )->document ) {
 		html_stream_close( stream );
@@ -350,17 +358,53 @@ static void url_requested( HtmlDocument *document, const gchar *url,
 	}
 	
 	g_object_get( G_OBJECT( preview ), 
-			"window", &window,
-			"online", &online, NULL );
-
-	full = relative_to_full( url, preview->private->base_uri );
+			"online", &online, 
+			"site", &site,
+			"page", &page,
+			NULL );
+	
+	dynamic = FALSE;
+	if( page ) {	
+		dynamic = screem_page_is_dynamic( page );
+	}
+	full = NULL;
+	if( g_path_is_absolute( url ) ) {
+		sitebase = NULL;
+		if( site && ! screem_site_get_fake_flag( site ) ) {
+			if( ( dynamic && screem_site_get_preview_dynamic( site ) ) ||
+			    ( ! dynamic && screem_site_get_preview_static( site ) ) ) {
+				sitebase = screem_site_get_http_url( site );
+			} else {
+				sitebase = screem_site_get_pathname( site );
+			}
+		} else if( ! uri_exists( url, NULL ) ) {
+			sitebase = preview->private->base_uri;
+		}
+		if( sitebase ) {
+			full = g_strconcat( sitebase, url, NULL );
+		}
+	}
+	if( ! full ) {
+		full = relative_to_full( url, 
+				preview->private->base_uri );
+	}
+	
 	if( full ) {
 		uri = gnome_vfs_uri_new( full );
 	} else {
 		uri = NULL;
 	}
 
+	if( page ) {
+		g_object_unref( page );
+	}
+	if( site ) {
+		g_object_unref( site );
+	}
+	
 	if( uri ) {
+		g_free( full );
+		full = gnome_vfs_uri_to_string( uri, 0 );
 		if( gnome_vfs_uri_is_local( uri ) || online ) {
 			g_mutex_lock( preview->private->streams_lock );
 			
@@ -384,7 +428,8 @@ static void url_requested( HtmlDocument *document, const gchar *url,
 		} else {
 			msg = g_strconcat( "Not fetching remote uri: ",
 						full, NULL );
-			screem_window_show_message( window, msg );
+			screem_view_show_message( SCREEM_VIEW( preview ),
+					msg );
 			g_free( msg );
 			html_stream_close( stream );
 		}
@@ -392,7 +437,8 @@ static void url_requested( HtmlDocument *document, const gchar *url,
 		
 		msg = g_strconcat( _( "Invalid URI in page: " ),
 				   full, NULL );
-		screem_window_show_error( window, msg );
+		screem_view_show_error( SCREEM_VIEW( preview ),
+					msg );
 		g_free( msg );
 		html_stream_close( stream );
 	}
@@ -575,7 +621,8 @@ static void screem_preview_init( ScreemPreview *preview )
 {
 	GtkWidget *html;
 	GtkWidget *sw;
-
+	gchar *profile;
+	
 	ScreemPreviewPrivate *private;
 
 	SCREEM_VIEW( preview )->window_set = screem_preview_window_set;
@@ -601,12 +648,13 @@ static void screem_preview_init( ScreemPreview *preview )
 
 	gtk_container_add( GTK_CONTAINER( preview ), sw );
 #else
+	gtk_moz_embed_set_profile_path( "/home/david/.gnome2/epiphany/mozilla", "epiphany" );
 	gtk_moz_embed_set_comp_path( g_getenv( "MOZILLA_FIVE_HOME" ) );
 	html = private->html = gtk_moz_embed_new();
 
 	gtk_moz_embed_set_chrome_mask( html, GTK_MOZ_EMBED_FLAG_ALLCHROME );
 	
-	gtk_container_add( GTK_CONTAINER( sw ), html );
+	gtk_container_add( GTK_CONTAINER( preview ), html );
 #endif
 	preview->private->streams = g_hash_table_new( NULL, NULL );
 	preview->private->streams_lock = g_mutex_new();
