@@ -20,7 +20,7 @@
 #include "config.h"
 
 #include <libgnome/gnome-macros.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 
 #include <libgnomevfs/gnome-vfs-utils.h>
 
@@ -40,6 +40,8 @@
 #include "screem-search.h"
 #include "screem-window.h"
 
+#include "support.h"
+
 static void screem_search_class_init( ScreemSearchClass *klass );
 static void screem_search_instance_init( ScreemSearch *search );
 static void screem_search_finalize( GObject *object );
@@ -49,8 +51,11 @@ static void screem_search_get_prop( GObject *object, guint prop_id,
 				GValue *value, GParamSpec *spec );
 
 struct ScreemSearchPrivate {
-	gint found_at;
-	guint length;
+	gint found_at;		/* this is a byte offset */
+	gint found_c;		/* this is a utf8 char offset */
+	
+	guint length;		/* this is the byte length */
+	guint length_c;		/* this is the utf8 char length */
 	
 	gboolean matches;
 	
@@ -371,6 +376,9 @@ static void screem_search_has_match( gpointer data,
 	ScreemPage *page;
 	const gchar *path;
 	gchar *text;
+	GError *error;
+	gchar *temp;
+	gchar *primary;
 	
 	g_return_if_fail( SCREEM_IS_SEARCH( search ) );
 	g_return_if_fail( SCREEM_IS_PAGE( data ) );
@@ -386,15 +394,24 @@ static void screem_search_has_match( gpointer data,
 	path = screem_page_get_pathname( page );
 
 	/* is path excluded? */
+	error = NULL;
 	if( ( ! screem_site_is_excluded( site, path ) ) &&
-		screem_page_load( page ) ) {
-		
+		screem_page_load( page, &error ) ) {
+
 		text = screem_page_get_data( page );
-		if( find_text( text, priv->find_pattern, 
-					NULL, NULL ) ) {
+		if( screem_search_static( text,	priv->find_pattern, 
+				priv->regexp, 0, &priv->length ) != -1 ) {
 			screem_search_add_match( search, page );
 		}
 		g_free( text );
+	} else if( error ) {
+		temp = gnome_vfs_unescape_string_for_display( path );
+		primary = g_strdup_printf( _( "Failed to load: %s" ), temp );
+		screem_hig_alert( GTK_STOCK_DIALOG_ERROR,
+				primary, error->message, GTK_WIDGET( window ) );
+		g_free( primary );
+		g_free( temp );
+		g_error_free( error );
 	}
 }
 
@@ -729,29 +746,26 @@ gchar *screem_search_get_replace_pattern( const ScreemSearch *search )
 void screem_search_show_dialog( ScreemSearch *search, 
 				gboolean doreplace )
 {
-	GConfClient *client;
 	gchar *glade_path;
 	GladeXML *xml;
 
 	GtkWidget *widget;
 	gchar *pattern;
 	
+	guint start;
+	guint end;
+	GtkTextIter it;
+	GtkTextIter eit;
+	ScreemPage *page;
+	
 	g_return_if_fail( SCREEM_IS_SEARCH( search ) );
 	g_return_if_fail( SCREEM_IS_WINDOW( search->priv->window ) );
 	
 	if( ! search->priv->dialog ) {
-		client = gconf_client_get_default();
-
-		glade_path = gconf_client_get_string( client, 
-				"/apps/screem/general/glade_path",
-				NULL );
-	
+		glade_path = screem_get_glade_path();
 		xml = glade_xml_new( glade_path, "find_dialog", NULL );
-	
 		g_free( glade_path );
 	
-		g_object_unref( client );
-
 		search->priv->dialog = glade_xml_get_widget( xml,
 				"find_dialog" );
 	
@@ -833,7 +847,8 @@ void screem_search_show_dialog( ScreemSearch *search,
 				  G_CALLBACK( screem_search_all_toggle ),
 				  search );
 	
-		
+		screem_search_set_search_type( search,
+				SCREEM_SEARCH_SINGLE );
 		screem_search_setup_match_view( search );
 
 		glade_xml_signal_autoconnect( xml );
@@ -849,7 +864,15 @@ void screem_search_show_dialog( ScreemSearch *search,
 	/* find pattern */
 	widget = glade_xml_get_widget( xml, "find_entry" );
 	widget = gnome_entry_gtk_entry( GNOME_ENTRY( widget ) );
-	pattern = screem_search_get_find_pattern( search );
+	if( screem_editor_has_selection( search->priv->editor,
+				&start, &end ) ) {
+		page = screem_window_get_document( search->priv->window );
+		gtk_text_buffer_get_iter_at_offset( GTK_TEXT_BUFFER( page ), &it, start );
+		gtk_text_buffer_get_iter_at_offset( GTK_TEXT_BUFFER( page ), &eit, end );
+		pattern = gtk_text_buffer_get_text( GTK_TEXT_BUFFER( page ), &it, &eit, FALSE );
+	} else {
+		pattern = screem_search_get_find_pattern( search );
+	} 
 	if( pattern ) {
 		gtk_entry_set_text( GTK_ENTRY( widget ), pattern );
 		g_free( pattern );
@@ -857,9 +880,11 @@ void screem_search_show_dialog( ScreemSearch *search,
 		gtk_entry_set_text( GTK_ENTRY( widget), "" );
 	}
 
+	/* pattern overide if we have selected text */
+	
 	/* replace pattern */
 	if( doreplace ) {
-		pattern = screem_search_get_find_pattern( search );
+		pattern = screem_search_get_replace_pattern( search );
 	} else {
 		pattern = NULL;
 	}
@@ -921,6 +946,7 @@ void screem_search_show_dialog( ScreemSearch *search,
 	}
 
 	gtk_list_store_clear( GTK_LIST_STORE( search->priv->match_model ) );
+	search->priv->matches = FALSE;
 
 	widget = glade_xml_get_widget( xml, "replace" );
 	gtk_widget_set_sensitive( widget, FALSE );
@@ -936,6 +962,7 @@ void screem_search_do_find( ScreemSearch *search )
 {
 	ScreemSearchPrivate *priv;
 	ScreemWindow *window;
+	ScreemApplication *app;
 	ScreemEditor *editor;
 	ScreemSite *site;
 	const GList *pages;
@@ -952,6 +979,7 @@ void screem_search_do_find( ScreemSearch *search )
 	priv = search->priv;
 
 	window = priv->window;
+	app = window->application;
 	editor = priv->editor;
 	site = screem_window_get_current( window );
 	
@@ -975,11 +1003,13 @@ void screem_search_do_find( ScreemSearch *search )
 
 		/* if we don't have a list of matches,
 		 * then we need to search the list of files */
+		screem_application_set_cursor( app, GDK_WATCH );
 		g_list_foreach( (GList *)pages, 
 				(GFunc)screem_search_has_match,
 				search );
-
+		screem_application_set_cursor( app, GDK_LAST_CURSOR );
 	}
+	/* FIXME: we leak pages if SCREEM_SEARCH_SITE */
 
 	switch( screem_search_get_from( search ) ) {
 		case SCREEM_SEARCH_START:
@@ -1001,10 +1031,22 @@ void screem_search_do_find( ScreemSearch *search )
 	page = screem_window_get_document( window );
 	text = screem_page_get_data( page );
 
+	if( ! priv->find_pattern ) {
+		screem_search_set_find_pattern( search, "" );
+	}
+
+	screem_application_set_cursor( app, GDK_WATCH );
 	priv->found_at = screem_search_static( text, 
 			priv->find_pattern, 
 			priv->regexp, 
 			pos, &priv->length );
+	priv->found_c = g_utf8_pointer_to_offset( text, 
+			text + priv->found_at );
+	priv->length_c = g_utf8_pointer_to_offset( text,
+			text + priv->found_at + priv->length ) - 
+		priv->found_c;
+	
+	screem_application_set_cursor( app, GDK_LAST_CURSOR );
 
 	widget = NULL;
 	if( priv->dialog ) {
@@ -1015,8 +1057,8 @@ void screem_search_do_find( ScreemSearch *search )
 	if( priv->found_at != -1 ) {
 		g_signal_emit( G_OBJECT( search ),
 			       screem_search_signals[ FOUND ],
-			       0, page, priv->found_at,
-			       priv->length );
+			       0, page, priv->found_c,
+			       priv->length_c );
 		if( widget ) {
 			gtk_widget_set_sensitive( widget, TRUE );
 		}
@@ -1048,18 +1090,19 @@ void screem_search_do_replace( ScreemSearch *search )
 	if( ! priv->replace_pattern ) {
 		screem_search_set_replace_pattern( search, "" );
 	}
-	
-	screem_editor_delete_forward( editor, priv->found_at,
-			priv->length );
-	screem_editor_insert( editor, priv->found_at,
+
+	screem_editor_delete_forward( editor, priv->found_c,
+			priv->length_c );
+	screem_editor_insert( editor, priv->found_c,
 			priv->replace_pattern );
 	priv->found_at += strlen( priv->replace_pattern );
-
+	
+	priv->found_c += g_utf8_strlen( priv->replace_pattern, -1 );
 	while( gtk_events_pending() ) {
 		gtk_main_iteration();
 	}
 	
-	screem_editor_set_pos( editor, priv->found_at );
+	screem_editor_set_pos( editor, priv->found_c );
 	screem_search_do_find( search );
 }
 
