@@ -66,6 +66,9 @@ struct ScreemSessionPrivate {
 	GnomeClient *client;
 
 	GString *output;
+
+	/* hash table of classes and pos / dimensions */
+	GHashTable *dialogs;
 };
 
 GNOME_CLASS_BOILERPLATE( ScreemSession, screem_session,
@@ -75,24 +78,21 @@ GNOME_CLASS_BOILERPLATE( ScreemSession, screem_session,
 static void screem_session_class_init( ScreemSessionClass *klass )
 {
 	GObjectClass *obj_class;
+	GParamSpec *spec;
 
 	obj_class = G_OBJECT_CLASS( klass );
 	obj_class->finalize = screem_session_finalize;
 	obj_class->get_property = screem_session_get_prop;
 	obj_class->set_property = screem_session_set_prop;
 
-
-	g_object_class_install_property(obj_class,
-					PROP_APP,
-					g_param_spec_object("app",
-							    "Application",
-							    "The Application",
-							    G_TYPE_OBJECT,
-							    G_PARAM_READWRITE |
-							    G_PARAM_CONSTRUCT)
-					);
+	spec = g_param_spec_object( "app", "Application",
+				    "The Application",
+				    G_TYPE_OBJECT,
+				    G_PARAM_READWRITE |
+				    G_PARAM_CONSTRUCT );
+	g_object_class_install_property( obj_class, PROP_APP, spec );
 }
-
+	
 static void screem_session_instance_init( ScreemSession *session )
 {
 	ScreemSessionPrivate *priv;
@@ -106,6 +106,9 @@ static void screem_session_instance_init( ScreemSession *session )
         g_signal_connect( G_OBJECT( client ), "die",
 			  G_CALLBACK( gtk_main_quit ), NULL );
 
+	priv->dialogs = g_hash_table_new_full( g_str_hash, g_str_equal,
+			(GDestroyNotify)g_free,
+			(GDestroyNotify)g_free );
 }
 
 static void screem_session_finalize( GObject *object )
@@ -120,6 +123,8 @@ static void screem_session_finalize( GObject *object )
 
 	priv = session->priv;
 	
+	g_hash_table_destroy( priv->dialogs );
+	
 	g_free( priv );
 	
 	GNOME_CALL_PARENT( G_OBJECT_CLASS, finalize, (object) );
@@ -197,7 +202,7 @@ save_yourself( GnomeClient *client, gint phase, GnomeSaveStyle save_style,
                gint is_fast, ScreemSession *session )
 {
 	gchar *argv[] = { "screem", "--load-session", NULL };
-        gchar *discard_argv[] = { "rm", "-r", NULL, NULL };
+        gchar *discard_argv[] = { "rm", "-f", NULL, NULL };
 
 	argv[ 2 ] = get_session_filename( SESSION_GNOME );
 	gnome_client_set_restart_command( client, 3, argv );
@@ -307,10 +312,12 @@ static void screem_session_load_site( ScreemSession *session,
 							if( *url == '\0' && ! page ) {
 								page = screem_page_new( G_OBJECT( app ) );
 								screem_site_add_page( site, page );
+								g_object_unref( page );
 							} else if( fake && ! page ) {
 								page = screem_page_new( G_OBJECT( app ) );
 								screem_page_set_pathname( page, url );
 								screem_site_add_page( site, page );
+								g_object_unref( page );
 							}
 							xmlFree( url );
 						} else {
@@ -322,6 +329,7 @@ static void screem_session_load_site( ScreemSession *session,
 					}
 				}
 				docs = g_list_reverse( docs );
+				/* FIXME: name gets leaked */
 				screem_site_set_documents( site, name, docs );
 			} else if( name ) {
 				xmlFree( name );
@@ -341,7 +349,8 @@ static void screem_session_save_windows( ScreemWindow *window,
 	gint y;
 	gint w;
 	gint h;
-	
+	gboolean max;
+
 	ScreemApplication *app;
 	ScreemSite *site;
 	const gchar *sname;
@@ -359,10 +368,17 @@ static void screem_session_save_windows( ScreemWindow *window,
 	
 	gtk_window_get_position( GTK_WINDOW( window ), &x, &y );
 	gtk_window_get_size( GTK_WINDOW( window ), &w, &h );
-	
+	max = window->max;
+	if( max ) {
+		x = window->x;
+		y = window->y;
+		w = window->w;
+		h = window->h;
+	} 
 	g_string_append_printf( output, 
-				"<window name=\"%s\" x=\"%i\" y=\"%i\" w=\"%i\" h=\"%i\">",
-				name, x, y, w, h );
+				"<window name=\"%s\" x=\"%i\" y=\"%i\" w=\"%i\" h=\"%i\" max=\"%i\">",
+				name, x, y, w, h, max );
+	
 	
 	site = screem_window_get_current( window );
 	sname = screem_site_get_name( site );
@@ -443,15 +459,14 @@ static void screem_session_save_hint( ScreemSession *session )
 	
 	gtk_window_get_position( GTK_WINDOW( hint ), &x, &y );
 
-	if( GTK_WIDGET_VISIBLE( hint ) ) {
-		g_string_append_printf( output, "<hints x=\"%i\" y=\"%i\"/>",
+	g_string_append_printf( output, "<hints x=\"%i\" y=\"%i\"/>",
 				x, y );
-	}
-
+	
 	g_object_unref( app );
 }
 
-static void screem_session_load_hint( ScreemSession *session, xmlNodePtr node )
+static void screem_session_load_hint( ScreemSession *session, 
+		xmlNodePtr node )
 {
 	ScreemApplication *app;
 	GtkWidget *hint;
@@ -476,9 +491,73 @@ static void screem_session_load_hint( ScreemSession *session, xmlNodePtr node )
 	g_object_unref( app );
 }
 
+static gboolean save_dialog( gpointer key, gpointer value,
+		gpointer data )
+{
+	GString *output;
+	const gchar *name;
+	GdkRectangle *rect;
+
+	name = (const gchar *)key;
+	rect = (GdkRectangle*)value;
+	output = (GString*)data;
+
+	g_string_append_printf( output, "<dialog name=\"%s\" x=\"%i\" y=\"%i\" w=\"%i\" h=\"%i\" />", name, rect->x, rect->y, rect->width, rect->height );
+
+	return FALSE;
+}
+
+static void screem_session_save_dialogs( ScreemSession *session )
+{
+	ScreemSessionPrivate *priv;
+	
+	g_return_if_fail( SCREEM_IS_SESSION( session ) );
+	g_return_if_fail( session->priv->output != NULL );
+	
+	priv = session->priv;
+	g_hash_table_foreach( priv->dialogs, (GHFunc)save_dialog,
+			priv->output );
+}
+
+static void screem_session_load_dialog( ScreemSession *session, 
+		xmlNodePtr node )
+{
+	ScreemSessionPrivate *priv;
+	xmlChar *tmp;
+	gchar *name;
+	GdkRectangle *rect;
+	gint x;
+	gint y;
+	gint w;
+	gint h;
+	
+	priv = session->priv;
+	
+	name = xmlGetProp( node, "name" );
+	tmp = xmlGetProp( node, "x" );
+	x = atoi( tmp );
+	xmlFree( tmp );
+	tmp = xmlGetProp( node, "y" );
+	y = atoi( tmp );
+	xmlFree( tmp );
+	tmp = xmlGetProp( node, "w" );
+	w = atoi( tmp );
+	xmlFree( tmp );
+	tmp = xmlGetProp( node, "h" );
+	h = atoi( tmp );
+	xmlFree( tmp );
+
+	rect = g_new0( GdkRectangle, 1 );
+	rect->x = x;
+	rect->y = y;
+	rect->width = w;
+	rect->height = h;
+	g_hash_table_insert( priv->dialogs, name, rect );
+}
+
 /* public stuff */
 
-ScreemSession *screem_session_new( ScreemApplication *app )
+ScreemSession *screem_session_new( GObject *app )
 {
 	ScreemSession *session;
 
@@ -542,6 +621,7 @@ void screem_session_load( ScreemSession *session,
 				gint y;
 				gint w;
 				gint h;
+				gint max;
 				
 				tmp = xmlGetProp( node, "x" );
 				x = atoi( tmp );
@@ -555,8 +635,19 @@ void screem_session_load( ScreemSession *session,
 				tmp = xmlGetProp( node, "h" );
 				h = atoi( tmp );
 				xmlFree( tmp );
+				tmp = xmlGetProp( node, "max" );
+				max = 0;
+				if( tmp ) {
+					max = atoi( tmp );
+				}
+				xmlFree( tmp );
 
 				win = screem_application_create_window( app );
+				win->x = x;
+				win->y = y;
+				win->w = w;
+				win->h = h;
+				win->max = max;
 
 				screem_session_load_window( session, win, 
 							node->children );
@@ -565,8 +656,16 @@ void screem_session_load( ScreemSession *session,
 								w, h );
 				gtk_widget_show( GTK_WIDGET( win ) );
 				gtk_window_move( GTK_WINDOW( win ), x, y );
+				gtk_window_resize( GTK_WINDOW( win ),
+						w, h );
+				if( max ) {
+					gtk_window_maximize( GTK_WINDOW( win ) );
+				}
 			} else if( ! strcmp( "hints", node->name ) ) {
 				screem_session_load_hint( session, node );
+			} else if( ! strcmp( "dialog", node->name ) ) {
+				screem_session_load_dialog( session,
+						node );
 			}
 
 			node = node->next;
@@ -631,13 +730,16 @@ gboolean screem_session_save( ScreemSession *session, const gchar *filename )
 	}
 
 	screem_session_save_hint( session );
+	screem_session_save_dialogs( session );
 	
 	g_string_append( priv->output, "</screemsession>" );
 	
 	file = get_session_filename( filename );
 	ret = save_file( file, priv->output->str, 
 			 GNOME_VFS_PERM_USER_READ |
-			 GNOME_VFS_PERM_USER_WRITE );
+			 GNOME_VFS_PERM_USER_WRITE,
+			 FALSE,
+			 NULL );
 	g_free( file );
 	
 	g_string_free( priv->output, TRUE );
@@ -646,4 +748,98 @@ gboolean screem_session_save( ScreemSession *session, const gchar *filename )
 	return ret;
 }
 
+void screem_session_set_dialog( ScreemSession *session, 
+		const gchar *name, gint x, gint y, gint w, gint h )
+{
+	ScreemSessionPrivate *priv;
+	GdkRectangle *rect;
+
+	priv = session->priv;
+
+	rect = g_hash_table_lookup( priv->dialogs, name );
+	if( ! rect ) {
+		rect = g_new0( GdkRectangle, 1 );
+		g_hash_table_insert( priv->dialogs, g_strdup( name ), 
+				rect );
+	}
+	rect->x = x;
+	rect->y = y;
+	if( w != -1 ) {
+		rect->width = w;
+	}
+	if( h != -1 ) {
+		rect->height = h;
+	}
+}
+
+gboolean screem_session_get_dialog( ScreemSession *session, 
+		const gchar *name, gint *x, gint *y, gint *w, gint *h )
+{
+	ScreemSessionPrivate *priv;
+	GdkRectangle *rect;
+
+	priv = session->priv;
+	
+	rect = g_hash_table_lookup( priv->dialogs, name );
+	if( rect ) {
+		if( x ) {
+			*x = rect->x;
+		}
+		if( y ) {
+			*y = rect->y;
+		}
+		if( w ) {
+			*w = rect->width;
+		}
+		if( h ) { 
+			*h = rect->height;
+		}
+	}
+
+	return ( rect != NULL );
+}
+
+void screem_session_restore_dialog( ScreemSession *session,
+		GtkWidget *widget )
+{
+	const gchar *role;
+	gint x;
+	gint y;
+	gint w;
+	gint h;
+	
+	g_return_if_fail( GTK_IS_WINDOW( widget ) );
+	
+	role = gtk_window_get_role( GTK_WINDOW( widget ) );
+
+	g_return_if_fail( role != NULL );
+	
+	if( session && screem_session_get_dialog( session, role,
+				&x, &y, &w, &h ) ) {
+		gtk_window_move( GTK_WINDOW( widget ), x, y );
+		gtk_window_resize( GTK_WINDOW( widget ), w, h );
+	}
+}
+
+void screem_session_store_dialog( ScreemSession *session,
+		GtkWidget *widget )
+{
+	const gchar *role;
+	gint x;
+	gint y;
+	gint w;
+	gint h;
+	
+	g_return_if_fail( GTK_IS_WINDOW( widget ) );
+	
+	role = gtk_window_get_role( GTK_WINDOW( widget ) );
+
+	g_return_if_fail( role != NULL );
+	
+	if( session ) {
+		gtk_window_get_size( GTK_WINDOW( widget ), &w, &h );
+		gtk_window_get_position( GTK_WINDOW( widget ), &x, &y );
+		screem_session_set_dialog( session, role, x, y, w, h );
+	}
+}
 
