@@ -33,15 +33,13 @@
 #include <libgnomeui/gnome-dialog.h>
 #include <libgnomeui/gnome-entry.h>
 #include <libgnomeui/gnome-file-entry.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-file-info.h>
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
 
-#include <bonobo/bonobo-file-selector-util.h>
-
 #include <string.h>
 
 #include <sys/stat.h>
@@ -70,8 +68,6 @@
 
 #include "screem-file-browser.h"
 
-#include "libegg/menu/egg-menu.h"
-
 void screem_site_settings_dialog_cb( GtkWidget *widget, gint button );
 void screem_site_settings_change_url( GtkWidget *widget, gpointer data );
 
@@ -95,6 +91,9 @@ static void screem_site_create_template_removed( ScreemFileBrowser *browser,
 						 gpointer data );
 static xmlNodePtr screem_site_template_locate_uri( xmlNodePtr node, const gchar *uri );
 
+static void screem_site_template_file_op( GnomeVFSMonitorEventType type,
+		const gchar *uri, gpointer data );
+
 void screem_site_close_site( ScreemSite *site )
 {
        	g_return_if_fail( site != NULL );
@@ -113,9 +112,12 @@ gboolean screem_site_open_with_filename( ScreemWindow *window,
 	gchar *path = NULL;
 	GnomeVFSURI *uri;
 	GnomeVFSFileInfo *info;
+	gchar *msg;
 
 	g_return_val_if_fail( filename != NULL, FALSE );
 
+	screem_application_set_cursor( application, GDK_WATCH );
+
 	uri = gnome_vfs_uri_new( filename );
 	
 	/* is the filename local? */
@@ -135,11 +137,17 @@ gboolean screem_site_open_with_filename( ScreemWindow *window,
 	/* ensure we have a dir */
 	info = gnome_vfs_file_info_new();
 
-	gnome_vfs_get_file_info( path, info, 
-				 GNOME_VFS_FILE_INFO_FOLLOW_LINKS );
+	if( gnome_vfs_get_file_info( path, info, 
+				 GNOME_VFS_FILE_INFO_FOLLOW_LINKS ) != 
+			GNOME_VFS_OK ) {
+		g_free( path );
+		screem_application_set_cursor( application, 
+				GDK_LAST_CURSOR );
+		return FALSE;
+	}
 
 	if( info->type != GNOME_VFS_FILE_TYPE_DIRECTORY )  {
-		gchar *temp = g_dirname( path );
+		gchar *temp = g_path_get_dirname( path );
 		g_free( path );
 		path = temp;
 	}
@@ -153,6 +161,8 @@ gboolean screem_site_open_with_filename( ScreemWindow *window,
 		if( window ) {
 			screem_window_set_current( window, site );
 		}
+		screem_application_set_cursor( application, 
+				GDK_LAST_CURSOR );
 		return TRUE;
 	}
 
@@ -160,23 +170,50 @@ gboolean screem_site_open_with_filename( ScreemWindow *window,
 	screem_site_set_pathname( site, path );
 	screem_site_set_name( site, _( "Imported Site" ) );
 
-	g_free( path );
+	if( window ) {
+		msg = g_strdup_printf( _( "Loading site: %s" ), path );
+		screem_window_show_message( window, msg, FALSE );
+		g_free( msg );
+		gdk_threads_leave(); 
+		while( g_main_context_iteration(NULL,FALSE) ) { }
+		gdk_threads_enter();
+	}
+	
 	if( ! screem_site_load( site ) ) {
 		/* we failed to load the site */
 		g_object_unref( G_OBJECT( site ) );
+		if( window ) {
+			msg = g_strdup_printf( _( "Failed to load site: %s" ), path );
+			screem_window_show_message( window, msg, FALSE );
+			g_free( msg );
+			g_free( path );
+		}
+		screem_application_set_cursor( application, 
+				GDK_LAST_CURSOR );
 		return FALSE;
 	}
+		
+	g_free( path );
+		
 	/* add this site to the list of loaded sites */
 	screem_application_add_site( application, site );
+	screem_application_set_cursor( application, GDK_LAST_CURSOR );
 
 	if( window ) {
+		screem_window_set_current( window, site );
+		
+		msg = g_strdup_printf( _( "Loaded site: %s" ), screem_site_get_name( site ) );
+		screem_window_show_message( window, msg, FALSE );
+		g_free( msg );
+		gdk_threads_leave(); 
+		while( g_main_context_iteration(NULL,FALSE) ) { }
+		gdk_threads_enter();
+		
 		/* if we are importing a site open the site settings dialog */
 		if( screem_site_get_is_import( site ) ) {
 			screem_site_settings_dialog( site, window );
 			screem_site_set_is_import( site, FALSE );
 		}
-
-		screem_window_set_current( window, site );
 	}
 
 	/* site loaded, check for cvs update,
@@ -191,10 +228,8 @@ gboolean screem_site_open_with_filename( ScreemWindow *window,
 		GtkWidget *toggle;
 		gboolean ask;
 		gchar *gladepath;
-
-		gladepath = gconf_client_get_string( application->client,
-						     "/apps/screem/general/glade_path",
-						     NULL );
+	
+		gladepath = screem_get_glade_path();
 		xml = glade_xml_new( gladepath,
 				     "auto_cvs_update", NULL );
 		g_free( gladepath );
@@ -219,18 +254,18 @@ gboolean screem_site_open_with_filename( ScreemWindow *window,
 		/* FIXME can't update from session load as window == NULL */
 		if( button == 0 && window ) {
 			/* do a cvs update */
-			EggAction *action;
+			GtkAction *action;
 			
-			action = egg_action_group_get_action( EGG_ACTION_GROUP( window->action_group ),
+			action = gtk_action_group_get_action( GTK_ACTION_GROUP( window->action_group ),
 								"UpdateSite" );
-			egg_action_activate( action );
+			gtk_action_activate( action );
 		} else if( ! ask ) {
 			/* don't ask was selected and the dialog was
 			   just closed */
 			screem_site_set_auto_update( site, FALSE );
 		}
 	}
-
+	
 	return TRUE;
 }
 
@@ -240,21 +275,20 @@ gboolean screem_site_open_with_filename( ScreemWindow *window,
  *
  * return values: none
  */
-void screem_site_settings_dialog( ScreemSite *site, ScreemWindow *window )
+void screem_site_settings_dialog( ScreemSite *site, 
+		ScreemWindow *window )
 {
+	ScreemApplication *app;
+	ScreemSession *session;
 	GtkWidget *dialog1;
      	GtkWidget *entry;
 	GtkWidget *toggle;
-	UploadMethods method;
 	ScreemSitePerms permissions;
 	ScreemSiteSymlinks symlinks;
 	const gchar *text;
 	GladeXML *xml;
-	GConfClient *client;
 	gchar *gladepath;
 
-	gint width;
-	gint height;
 	gint button;
 		
 	gchar *spath;
@@ -262,10 +296,10 @@ void screem_site_settings_dialog( ScreemSite *site, ScreemWindow *window )
 	g_return_if_fail( site != NULL );
 	g_return_if_fail( ! screem_site_get_fake_flag( site ) );
 
-	client = gconf_client_get_default();
-	gladepath = gconf_client_get_string( client, 
-					     "/apps/screem/general/glade_path",
-					     NULL );
+	app = SCREEM_APPLICATION( window->application );
+	session = screem_application_get_session( app );
+	
+	gladepath = screem_get_glade_path();	
 	xml = glade_xml_new( gladepath, "site_settings", NULL );
 	g_free( gladepath );
 
@@ -359,9 +393,9 @@ void screem_site_settings_dialog( ScreemSite *site, ScreemWindow *window )
 	gtk_entry_set_text( GTK_ENTRY( entry ), text );
 	
 	/* remote method */
-	method = screem_site_get_remote_method( site );
-	toggle = glade_xml_get_widget( xml, upload_strings[ method ] );
-	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( toggle ), TRUE );
+	entry = glade_xml_get_widget( xml, "remote_method" );
+	gtk_combo_box_set_active( GTK_COMBO_BOX( entry ), 
+			screem_site_get_remote_method( site ) );
 	
 	/* passive ftp */
 	if( screem_site_get_passive_ftp( site ) ) {
@@ -429,78 +463,23 @@ void screem_site_settings_dialog( ScreemSite *site, ScreemWindow *window )
 	gtk_window_set_transient_for( GTK_WINDOW( dialog1 ),
 				      GTK_WINDOW( window ) );
 
-	/* get size from gconf */
-	width = gconf_client_get_int( client, 
-				      "/apps/screem/ui/site_settings_width",
-					NULL );
-	height = gconf_client_get_int( client, 
-				      "/apps/screem/ui/site_settings_height",
-					NULL );
-	if( width > -1 && height > -1 ) {
-		gtk_window_resize( GTK_WINDOW( dialog1 ), width, height );
-	}
+	screem_session_restore_dialog( session, dialog1 );
+	
 	button = gtk_dialog_run( GTK_DIALOG( dialog1 ) );
 	
 	if( button == GTK_RESPONSE_CLOSE ) {
 		screem_site_settings_dialog_cb( dialog1, button );
 	}
-	g_object_unref( client );
+	
+	screem_session_store_dialog( session, dialog1 );
+	
 	gtk_widget_destroy( dialog1 );
 	g_object_unref( xml );
 }
 
-void screem_site_settings_size_allocate( GtkWidget *widget, GtkAllocation *req )
-{
-	GConfClient *client;
-	gint width;
-	gint height;
-
-	client = gconf_client_get_default();
-
-	width = gconf_client_get_int( client, 
-				      "/apps/screem/ui/site_settings_width",
-					NULL );
-	height = gconf_client_get_int( client, 
-				      "/apps/screem/ui/site_settings_height",
-					NULL );
-	if( req->width != width ) {
-		gconf_client_set_int( client, "/apps/screem/ui/site_settings_width",
-					req->width, NULL );
-	}
-	if( req->height != height ) {
-		gconf_client_set_int( client, "/apps/screem/ui/site_settings_height",
-					req->height, NULL );
-	}
-	
-	g_object_unref( client );
-}
-
 void screem_site_settings_change_url( GtkWidget *widget, gpointer data )
 {
-	const gchar *host;
-	GtkWidget *url;
-	GtkWidget *rhost;
-	GtkWidget *path;
-	GtkWidget *webdav;
-	GladeXML *xml;
-	const gchar *pathname;
-	gchar *baseuri;
 
-	xml = glade_get_widget_tree( widget );
-
-	webdav = glade_xml_get_widget( xml, upload_strings[ WEBDAV ] );
-	if( gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( webdav ) ) ) {
-	       	url = glade_xml_get_widget( xml, "http_url_entry" );
-		path = glade_xml_get_widget( xml, "remote_path_entry" );
-		rhost = glade_xml_get_widget( xml, "remote_url_entry" );
-		
-		host = gtk_entry_get_text( GTK_ENTRY( rhost ) );
-		pathname = gtk_entry_get_text( GTK_ENTRY( path ) );
-		
-		baseuri = g_strconcat( "http://", host, pathname, NULL );
-		gtk_entry_set_text( GTK_ENTRY( url ), baseuri );
-		g_free( baseuri );
-	}
 }
 
 void screem_site_settings_dialog_cb( GtkWidget *widget, gint button ) 
@@ -574,27 +553,9 @@ void screem_site_settings_dialog_cb( GtkWidget *widget, gint button )
 	screem_site_set_http_url( site, gtk_entry_get_text( GTK_ENTRY( entry ) ) );
 	
 	/* remote method */
-	toggle = glade_xml_get_widget( xml, "local" );
-	if(gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(toggle) )) {
-		screem_site_set_remote_method( site, LOCAL );
-	}
-	toggle = glade_xml_get_widget( xml, "ftp" );
-	if(gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(toggle) )) {
-		screem_site_set_remote_method( site, FTP );
-	}
-	toggle = glade_xml_get_widget( xml, "webdav" );
-	if(gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(toggle) )) {
-		screem_site_set_remote_method( site, WEBDAV );
-	}
-	toggle = glade_xml_get_widget( xml, "rsh" );
-	if(gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(toggle) )) {
-		screem_site_set_remote_method( site, RSH );
-	}
-	toggle = glade_xml_get_widget( xml, "ssh" );
-	if(gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(toggle) )) {
-		screem_site_set_remote_method( site, SSH );
-	}
-	
+	entry = glade_xml_get_widget( xml, "remote_method" );
+	screem_site_set_remote_method( site,
+			gtk_combo_box_get_active( GTK_COMBO_BOX( entry ) ) );
 	
 	/* passive ftp */
 	toggle = glade_xml_get_widget( xml, "ftpmode" );
@@ -969,19 +930,21 @@ static void screem_site_setup_lists( ScreemSite *site, GladeXML *xml )
 
 void screem_site_create_template( ScreemWindow *window )
 {
+	ScreemApplication *app;
 	gchar *uri;
 
+	app = window->application;
 	uri = screem_create_tmp_file( "screem-XXXXXX", NULL, TRUE );
 	
 	if( uri ) {
 		if( ! mkdir_recursive( uri,
-				       GNOME_VFS_PERM_USER_ALL ) ) {
+				       GNOME_VFS_PERM_USER_ALL,
+				       screem_application_file_op, app ) ) {
 			g_free( uri );
 			uri = NULL;
 		}
 	}
 	if( uri ) {
-		GConfClient *client;
 		gchar *glade_path;
 		GladeXML *xml;
 		ScreemFileBrowser *browser;
@@ -999,12 +962,7 @@ void screem_site_create_template( ScreemWindow *window )
 						     "screem_template",
 						     NULL ) );
 
-		client = gconf_client_get_default();
-		
-		glade_path = gconf_client_get_string(client,
-						     "/apps/screem/general/glade_path",
-						     NULL );
-
+		glade_path = screem_get_glade_path();
 		xml = glade_xml_new( glade_path, "site_template_window",
 				     NULL );
 
@@ -1070,7 +1028,6 @@ void screem_site_create_template( ScreemWindow *window )
 		glade_xml_signal_autoconnect( xml );
 
 		g_free( glade_path );
-		g_object_unref( client );
 	} else {
 		GtkWidget *widget;
 	
@@ -1104,9 +1061,10 @@ void screem_site_create_template_response( GtkWidget *dialog,
 	switch( response ) {
 	case 0:
 		/* load */
-		filename = bonobo_file_selector_open( NULL, FALSE,
-						      _("Select Site Template"),
-						      NULL, NULL );
+		filename = screem_fileselect( _( "Select Site Template" ), 
+						GTK_WINDOW( window ),
+						GTK_FILE_CHOOSER_ACTION_OPEN,
+						NULL, NULL );
 		if( filename ) {
 			xmlDocPtr newdoc;
 			ScreemSite *site;
@@ -1115,8 +1073,10 @@ void screem_site_create_template_response( GtkWidget *dialog,
 
 			url = screem_create_tmp_file( "screem-XXXXXX", ".xml", TRUE );
 			if( url ) {
-				if(!mkdir_recursive(url,
-						    GNOME_VFS_PERM_USER_ALL)) {
+				if(!mkdir_recursive( url,
+						     GNOME_VFS_PERM_USER_ALL,
+						     screem_site_template_file_op,
+						     browser ) ) {
 					g_free( url );
 					url = NULL;
 				}
@@ -1138,7 +1098,8 @@ void screem_site_create_template_response( GtkWidget *dialog,
 							   G_DIR_SEPARATOR_S,
 							   ".project.screem",
 							   NULL );
-				delete_file( projectfile );
+				delete_file( projectfile, 
+					screem_site_template_file_op, browser );
 				g_free( projectfile );
 				
 				g_object_set_data( G_OBJECT( dialog ), "doc",
@@ -1151,15 +1112,17 @@ void screem_site_create_template_response( GtkWidget *dialog,
 				screem_file_browser_scan_directory( browser,
 								    url,
 								    FILE_BROWSE_ROOT | FILE_BROWSE_RECURSE );
-				delete_dir( uri );
+				delete_dir( uri,
+					screem_site_template_file_op, browser );
 				g_free( (gchar*)uri );
 			}
 		}
 		break;
 	case 1:
-		filename = bonobo_file_selector_save( NULL, FALSE,
-						      _("Select directory to save site as"),
-						      NULL, NULL, NULL);
+		filename = screem_fileselect( _("Select directory to save site as"), 
+						GTK_WINDOW( window ),
+						GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER, NULL, NULL );
+			
 		if( filename ) {
 			xmlSaveFile( filename, doc );
 			g_free( filename );
@@ -1186,7 +1149,7 @@ static void screem_site_create_template_finish( GtkWidget *widget,
 	if( doc ) {
 		xmlFreeDoc( doc );
 	}
-	delete_dir( uri );
+	delete_dir( uri, screem_site_template_file_op, browser );
 	g_free( uri );
 	g_object_unref( browser );
 }
@@ -1237,18 +1200,22 @@ static void screem_site_create_template_select( GtkTreeSelection *selection,
 			xmlNodePtr node;
 			gchar *name;
 			gchar *template;
-
+			
 			list = g_list_append( NULL, &sit );
 			tmp = screem_file_browser_get_pathnames( browser,
 								 list, FALSE );
 			g_list_free( list );
-			uri = url = (const gchar*)tmp->data;
-			g_list_free( tmp );
-			if( ! strncmp( "file://", url, strlen( "file://" ) ) ) {
-				url += strlen( "file://" );
+			uri = url = NULL;
+			if( tmp ) {
+				uri = url = (const gchar*)tmp->data;
+				g_list_free( tmp );
+			} 
+			if( uri ) {
+				if( ! strncmp( "file://", url, strlen( "file://" ) ) ) {
+					url += strlen( "file://" );
+				}
+				url += strlen( base );
 			}
-			url += strlen( base );
-
 			node = xmlDocGetRootElement( doc );
 			node = screem_site_template_locate_uri( node, url );
 			if( node ) {
@@ -1423,6 +1390,10 @@ static xmlNodePtr screem_site_template_locate_uri( xmlNodePtr node, const gchar
 	gint i;
 	gchar **names;
 
+	if( ! uri ) {
+		return NULL;
+	}
+	
 	names = g_strsplit( uri, G_DIR_SEPARATOR_S, -1 );
 	for( i = 0; node && names[ i ]; ++ i ) {
 		xmlNodePtr child;
@@ -1458,6 +1429,19 @@ static xmlNodePtr screem_site_template_locate_uri( xmlNodePtr node, const gchar
 	return node;
 }
 
+static void screem_site_template_file_op( GnomeVFSMonitorEventType type,
+		const gchar *uri, gpointer data )
+{
+	ScreemFileBrowser *browser;
+	gchar *dirname;
+
+	browser = SCREEM_FILE_BROWSER( data );
+	dirname = g_path_get_dirname( uri );
+	screem_file_browser_file_mod( browser, dirname, uri, type );
+	g_free( dirname );
+}
+
+
 void screem_site_create_template_add( GtkWidget *widget )
 {
 	GladeXML *xml;
@@ -1508,7 +1492,7 @@ void screem_site_create_template_add( GtkWidget *widget )
 
 		/* if url isn't a directory, we want its parent */
 		if( ! screem_uri_is_dir( url ) ) {
-			temp = g_dirname( url );
+			temp = g_path_get_dirname( url );
 		} else {
 			temp = g_strdup( url );
 		}
@@ -1520,11 +1504,14 @@ void screem_site_create_template_add( GtkWidget *widget )
 
 		if( isdir ) {
 			created = mkdir_recursive( uri,  
-						   GNOME_VFS_PERM_USER_ALL );
+						GNOME_VFS_PERM_USER_ALL,
+						screem_site_template_file_op,
+						browser );
 		} else {
 			created = save_file( uri, "", 
 					     GNOME_VFS_PERM_USER_READ |
-					     GNOME_VFS_PERM_USER_WRITE );
+					     GNOME_VFS_PERM_USER_WRITE,
+					     FALSE, NULL );
 		}
 		if( created ) {
 			xmlNodePtr node;
@@ -1578,7 +1565,7 @@ void screem_site_create_template_remove( GtkWidget *widget )
 		g_list_free( list );
 		url = (const gchar*)tmp->data;
 		g_list_free( tmp );
-		delete_dir( url );
+		delete_dir( url, screem_site_template_file_op, browser );
 	}
 }
 
@@ -1586,14 +1573,15 @@ gboolean screem_site_save_confirmation( ScreemSite *site )
 {
 	gint button;
 	GList *list;
+	GList *tmp;
 	gboolean ret;
 	
 	ret = TRUE;
 	button = GTK_RESPONSE_YES;
-	for( list = screem_site_get_pages( site ); 
+	for( tmp = list = screem_site_get_pages( site ); 
 		list && button != GTK_RESPONSE_CANCEL ;
 		list = list->next ) {
-		button = screem_page_save_confirmation( SCREEM_PAGE( list->data ) );
+		button = screem_page_save_confirmation( SCREEM_PAGE( list->data ), TRUE );
 			
 		if( button == GTK_RESPONSE_CANCEL ||
 		    button == GTK_RESPONSE_NONE ||
@@ -1601,6 +1589,7 @@ gboolean screem_site_save_confirmation( ScreemSite *site )
 			ret = FALSE;
 		}
 	}
+	g_list_free( tmp );
 
 	return ret;
 }
