@@ -38,23 +38,7 @@
 
 #include <glade/glade.h>
 
-#include <gconf/gconf-client.h>
-
-#include <gtk/gtkmain.h>
-
-#include <gtk/gtktreeview.h>
-#include <gtk/gtktreeselection.h>
-#include <gtk/gtkcellrenderer.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkcellrendererpixbuf.h>
-#include <gtk/gtkdialog.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtktogglebutton.h>
-#include <gtk/gtkoptionmenu.h>
-#include <gtk/gtkmenuitem.h>
-#include <gtk/gtktooltips.h>
+#include <gtk/gtk.h>
 
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
 #include <libgnomevfs/gnome-vfs-directory.h>
@@ -64,7 +48,7 @@
 
 #include <libgnomeui/gnome-popup-menu.h>
 #include <libgnomeui/gnome-stock-icons.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 
 
 #include <sys/types.h>
@@ -79,6 +63,8 @@
 
 #include "fileops.h"
 
+#include "support.h"
+
 typedef struct {
 	gchar *uri;
 	gchar *mime_type;
@@ -101,22 +87,32 @@ static const GtkTargetEntry drop_types[] = {
 };
 static const int num_drop_types = sizeof(drop_types) / sizeof(drop_types[ 0 ]);
 
-static GnomeUIInfo dnd_menu[] = {
-        { GNOME_APP_UI_ITEM, N_("_Move here"), N_("Move file"),
-          0, NULL, NULL,
-          GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BLANK, 0,
-          GDK_CONTROL_MASK, NULL },
-        { GNOME_APP_UI_ITEM, N_("_Copy here"), N_("Copy file"),
-          0, NULL, NULL,
-          GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BLANK, 0,
-          GDK_CONTROL_MASK, NULL },
-        GNOMEUIINFO_SEPARATOR,
-        { GNOME_APP_UI_ITEM, N_("Cancel drag"), N_("Cancel drag"),
-          0, NULL, NULL,
-          GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_BLANK, 0,
-          GDK_CONTROL_MASK, NULL },
-        GNOMEUIINFO_END
+/* DnD menu */
+static void dnd_cb( GtkAction *action, gpointer data );
+
+static const gchar *dnd_menu = "\
+<ui>\
+<popup action=\"site_view_dnd_menu\">\
+<menuitem action=\"move\"/>\
+<menuitem action=\"copy\"/>\
+<separator name=\"site_view_dnd_sep1\"/>\
+<menuitem action=\"cancel drag\"/>\
+</popup>\
+</ui>";
+static GtkActionEntry dnd_actions[] = {
+	{ "site_view_dnd_menu", "", NULL, NULL, NULL, NULL },
+
+	{ "move", NULL, N_( "_Move here" ), NULL, N_( "Move file" ),
+	  G_CALLBACK( dnd_cb ) },
+	  
+	{ "copy", NULL, N_( "_Copy here" ), NULL,  N_( "Copy file" ),
+	  G_CALLBACK( dnd_cb ) },
+
+	  { "cancel drag", NULL, N_( "Cancel drag" ), NULL,
+	  N_( "Cancel the drag and drop operation" ),
+	  G_CALLBACK( dnd_cb ) }
 };
+static guint dnd_actions_n = G_N_ELEMENTS( dnd_actions );
 
 
 static void screem_site_view_row_deleted( GtkTreeModel *model, 
@@ -164,7 +160,6 @@ static void screem_site_view_fm_activated( GtkTreeView *view,
 static GList *screem_site_view_calc_stats( GtkTreeModel *model, 
 					   GtkTreeIter *it, GList *types );
 
-
 gint screem_site_view_compare_func( GtkTreeModel *model, 
 				    GtkTreeIter *a, GtkTreeIter *b,
 				    ScreemSite *site )
@@ -248,8 +243,7 @@ void screem_site_view_set_val_at( GtkTreeStore *store, GtkTreeIter *iter,
 				  ScreemSiteViewNodeInfo *info)
 {
 	ScreemSite *site;
-	gchar flags[ 3 ] = { '.', '.', '.' };
-	gchar *flagstxt;
+	gchar flags[ 4 ] = { '.', '.', '.', '\0' };
 	
 	site = info->site;
 
@@ -263,16 +257,12 @@ void screem_site_view_set_val_at( GtkTreeStore *store, GtkTreeIter *iter,
 		flags[ 1 ] = 'A';
 	}
 
-	flagstxt = g_strdup_printf( "%c%c%c", 
-				    flags[ 0 ], flags[ 1 ], flags[ 2 ] );
-
 	/* name is what we want to display */
 	gtk_tree_store_set( store, iter, 
-			    FILE_BROWSER_USER_COL, flagstxt, 
+			    FILE_BROWSER_USER_COL, flags, 
 			    FILE_BROWSER_DATA_COL, info, 
 			    FILE_BROWSER_URI_COL, info->fullname,
 			    -1 );
-	g_free( flagstxt );
 }
 
 void screem_site_view_destroy( ScreemSite *site )
@@ -292,13 +282,15 @@ void screem_site_view_destroy( ScreemSite *site )
 	g_free( view );
 }
 
-void screem_site_init_tree_view( GtkWidget *tree )
+void screem_site_init_tree_view( GtkWidget *tree,
+		GtkUIManager *merge, GtkActionGroup *group )
 {
 	GtkCellRenderer *prenderer;
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
 	GtkTreeSelection *selection;
 	GtkTooltips *tip;
+	GError *error;
 	
 	gtk_tree_view_set_rules_hint( GTK_TREE_VIEW( tree ), TRUE );
 	gtk_tree_view_set_headers_visible( GTK_TREE_VIEW( tree ),
@@ -379,6 +371,19 @@ void screem_site_init_tree_view( GtkWidget *tree )
 	g_signal_connect( G_OBJECT( tree ), "row_expanded",
 			  G_CALLBACK( screem_site_view_fm_expanded ),
 			  NULL );
+
+	/* dnd popup */
+	gtk_action_group_add_actions( GTK_ACTION_GROUP( group ),
+			dnd_actions, dnd_actions_n, tree );
+	error = NULL;
+	g_object_set_data( G_OBJECT( tree ), "merge", merge );
+	if( ! gtk_ui_manager_add_ui_from_string( GTK_UI_MANAGER( merge ),
+				dnd_menu, strlen( dnd_menu ),
+				&error ) ) {
+		g_message( "Site View dnd menu string error = %s", 
+				error->message );
+		g_error_free( error );
+	}
 }
 
 GtkTreeModel *screem_site_get_model( ScreemSite *site )
@@ -410,8 +415,9 @@ gboolean screem_site_view_create_dir( ScreemSite *site, GtkWidget *widget,
 	node_info = g_value_get_pointer( &value );
 
 	if( ! node_info ) {
-		newdir = g_strconcat( screem_site_get_pathname( site ),
-				      "/", dir, NULL );
+		newdir = g_build_path( G_DIR_SEPARATOR_S,
+				screem_site_get_pathname( site ),
+				dir, NULL );
 	} else if( node_info->type != SCREEM_SITE_VIEW_FOLDER ) {
 		GtkTreeIter pit;
 		GValue pvalue = { 0 };
@@ -433,7 +439,9 @@ gboolean screem_site_view_create_dir( ScreemSite *site, GtkWidget *widget,
 			       GNOME_VFS_PERM_USER_ALL |
 			       GNOME_VFS_PERM_GROUP_ALL |
 			       GNOME_VFS_PERM_OTHER_READ |
-			       GNOME_VFS_PERM_OTHER_EXEC );
+			       GNOME_VFS_PERM_OTHER_EXEC,
+			       screem_site_view_file_op, 
+			       site->private->view );
 
 	g_free( newdir );
 
@@ -445,7 +453,6 @@ void screem_site_view_show_properties( ScreemSite *site,
 				       GtkWindow *window )
 {
 	GladeXML *xml;
-	GConfClient *client;
 	GtkWidget *dialog;
 	GtkWidget *widget;
 	gchar *gladepath;
@@ -468,18 +475,20 @@ void screem_site_view_show_properties( ScreemSite *site,
 	gchar *name;
 
 	ScreemFileBrowser *browser;
+
+	ScreemApplication *app;
+	ScreemSession *session;
+
+	g_object_get( G_OBJECT( site ), "app", &app, NULL );
+	session = screem_application_get_session( app );
+	g_object_unref( app );
 	
 	browser = site->private->view->browser;
 	
-	client = gconf_client_get_default();
-	gladepath = gconf_client_get_string( client,
-					     "/apps/screem/general/glade_path",
-					     NULL );
+	gladepath = screem_get_glade_path();
 	xml = glade_xml_new( gladepath, "file_properties", NULL );
 	g_free( gladepath );
 
-	g_object_unref( client );
-
 	if( screem_site_get_fake_flag( site ) ) {
 		widget = glade_xml_get_widget( xml, "exclude" );
 		gtk_widget_set_sensitive( widget, FALSE );
@@ -493,7 +502,8 @@ void screem_site_view_show_properties( ScreemSite *site,
 
 	vfsinfo = gnome_vfs_file_info_new();
 	options = GNOME_VFS_FILE_INFO_GET_MIME_TYPE |
-		GNOME_VFS_FILE_INFO_FOLLOW_LINKS;
+		GNOME_VFS_FILE_INFO_FOLLOW_LINKS |
+		GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS;
 
 	gnome_vfs_get_file_info( uri, vfsinfo, options );
 
@@ -519,7 +529,7 @@ void screem_site_view_show_properties( ScreemSite *site,
 	{
 		gchar *dirname;
 
-		dirname = g_dirname( uri );	
+		dirname = g_path_get_dirname( uri );	
 		gtk_label_set_text( GTK_LABEL( label ), dirname );
 		g_free( dirname );
 	}
@@ -540,9 +550,10 @@ void screem_site_view_show_properties( ScreemSite *site,
 		label = glade_xml_get_widget( xml, "type" );
 		gtk_label_set_text( GTK_LABEL( label ), _( "Unknown" ) );
 	}
-
-	gtk_image_set_from_pixbuf( GTK_IMAGE( image ), pixbuf );
-	g_object_unref( pixbuf );
+	if( pixbuf ) {
+		gtk_image_set_from_pixbuf( GTK_IMAGE( image ), pixbuf );
+		g_object_unref( pixbuf );
+	}
 	
 	label = glade_xml_get_widget( xml, "modified" );
 	if( vfsinfo->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_MTIME ) {
@@ -566,10 +577,15 @@ void screem_site_view_show_properties( ScreemSite *site,
 	userinfo = getpwuid( vfsinfo->uid );
 	if( userinfo ) {
 		gchar *t;
+		gchar *tmp;
 
 		label = glade_xml_get_widget( xml, "user" );
 		t = g_strconcat( userinfo->pw_name, " - ",
 				 userinfo->pw_gecos, NULL );
+		tmp = strchr( t, ',' );
+		if( tmp ) {
+			*tmp = '\0';
+		}
 		gtk_label_set_text( GTK_LABEL( label ), t );
 		g_free( t );
 	}
@@ -677,7 +693,9 @@ void screem_site_view_show_properties( ScreemSite *site,
 	glade_xml_signal_autoconnect( xml );
 
 	gtk_window_set_transient_for( GTK_WINDOW( dialog ), window );
+	screem_session_restore_dialog( session, dialog );
 	gtk_dialog_run( GTK_DIALOG( dialog ) );
+	screem_session_store_dialog( session, dialog );
 
 	name = g_object_get_data( G_OBJECT( dialog ), "fullname" );
 	if( name ) {
@@ -687,6 +705,7 @@ void screem_site_view_show_properties( ScreemSite *site,
 	g_object_set_data( G_OBJECT( dialog ), "fullname", NULL );
 			   
 	gtk_widget_destroy( dialog );
+	g_object_unref( xml );
 }
 
 void screem_site_view_properties_rename( GtkWidget *widget )
@@ -718,24 +737,28 @@ void screem_site_view_properties_rename( GtkWidget *widget )
 
 		page = screem_site_locate_page( site, fullname );
 		
-		dirname = g_dirname( fullname );
-		newpath = g_strconcat( dirname, G_DIR_SEPARATOR_S,
-				       name, NULL );
+		dirname = g_path_get_dirname( fullname );
+		newpath = g_build_filename( dirname, name, NULL );
 
 		if( page ) {
 			screem_page_set_pathname( page, newpath );
 		}
 		if( screem_uri_is_dir( fullname ) ) {
-	       		done = copy_dir( fullname, newpath, FALSE );
+	       		done = copy_dir( fullname, newpath, FALSE,
+					screem_site_view_file_op,
+					site->private->view );
 		} else {
-			done = copy_file( fullname, newpath );
+			done = copy_file( fullname, newpath, 
+					screem_site_view_file_op,
+					site->private->view );
 		}
 		if( done ) {
 			screem_site_file_change( site,
 						 fullname,
 						 newpath );
 			
-			delete_dir( fullname );
+			delete_dir( fullname, screem_site_view_file_op,
+					site->private->view );
 			
 			g_object_set_data( G_OBJECT( widget ), "fullname",
 					   newpath );
@@ -885,6 +908,7 @@ void screem_site_view_properties_ascii( GtkWidget *widget )
 						    "fullname" );
 
 	if( fullname ) {
+		g_print( "FULLNAME: %s\n", fullname );
 		if( ascii ) {
 			screem_site_add_ascii( site, fullname );
 		} else {
@@ -1042,6 +1066,23 @@ static void screem_site_view_dnd_delete( GtkWidget *widget,
 	g_signal_stop_emission_by_name( G_OBJECT( widget ), "drag_data_delete" );
 }
 
+static void dnd_cb( GtkAction *action, gpointer data )
+{
+	gchar *name;
+	gint i;
+	gint item;
+
+	g_object_get( G_OBJECT( action ), "name", &name, NULL );
+	item = -1;
+	for( i = 1; i < dnd_actions_n; ++ i ) {
+		if( ! strcmp( name, dnd_actions[ i ].name ) ) {
+			item = i - 1;
+			break;
+		}
+	}
+	g_object_set_data( G_OBJECT( data ), "dnd_popup_item",
+			GINT_TO_POINTER( item ) );
+}
 
 static void screem_site_view_drop( GtkWidget *widget, GdkDragContext *context,
 				   gint x, gint y, 
@@ -1051,7 +1092,6 @@ static void screem_site_view_drop( GtkWidget *widget, GdkDragContext *context,
 {
 	gboolean moving;
 	gchar **uris;
-	const gchar *site_path;
 	GtkTreeModel *model;
 	GtkTreePath *path;
 	GtkTreeIter it;
@@ -1059,17 +1099,25 @@ static void screem_site_view_drop( GtkWidget *widget, GdkDragContext *context,
 	GValue value = {0};
 	const gchar *target_uri;
 	ScreemSite *site;
+	GtkUIManager *merge;
 
 	site = SCREEM_SITE( g_object_get_data( G_OBJECT( widget ), "site" ) );
 
 	g_signal_stop_emission_by_name( G_OBJECT( widget ),
 					"drag_data_received" );
 	
+	merge = g_object_get_data( G_OBJECT( widget ), "merge" );
+	
 	if( context->action == GDK_ACTION_ASK ) {
 		GtkWidget *popup;
 		gint item;
-		popup = gnome_popup_menu_new( dnd_menu );
-		item = gnome_popup_menu_do_popup_modal( popup, 0, 0, 0, 0, widget );
+		popup = gtk_ui_manager_get_widget( GTK_UI_MANAGER( merge ),
+					"/site_view_dnd_menu" );
+		g_object_set_data( G_OBJECT( widget), "dnd_popup_item",
+			GINT_TO_POINTER( -1 ) );
+                screem_popup_menu_do_popup_modal( popup, 0, 0, 0, 0, 
+				widget );
+		item = GPOINTER_TO_INT( g_object_get_data( G_OBJECT( widget ), "dnd_popup_item" ) );
 		switch( item ) {
 		case 0:
 			context->action = GDK_ACTION_MOVE;
@@ -1086,8 +1134,6 @@ static void screem_site_view_drop( GtkWidget *widget, GdkDragContext *context,
 
 	moving = ( context->action == GDK_ACTION_MOVE );
 
-	site_path = screem_site_get_pathname( site );
-
 	model = gtk_tree_view_get_model( GTK_TREE_VIEW( widget ) );
 
 	gtk_tree_view_get_dest_row_at_pos( GTK_TREE_VIEW( widget ),
@@ -1157,30 +1203,37 @@ static gboolean screem_site_view_drop_uris( ScreemSite *site,
 					    gboolean moving )
 {
 	gint i;
+	GnomeVFSURI *source;
+	GnomeVFSURI *target;
+	GnomeVFSURI *realtarget;
+	gchar *basename;
+	GnomeVFSFileInfo *vfsinfo;
+	GnomeVFSFileInfoOptions options;
+	gboolean res;
+	gchar *tgt;
 
 	/* processed all uris, they have been
 	   dropped onto target_uri */
 	for( i = 0; uris[ i ] && *uris[ i ] != '\0'; ++ i ) {
-		GnomeVFSURI *source;
-		GnomeVFSURI *target;
-		GnomeVFSURI *realtarget;
-		gchar *basename;
-		GnomeVFSFileInfo *vfsinfo;
-		GnomeVFSFileInfoOptions options;
-		gboolean res;
-		gchar *tgt;
 		/* make sure we aren't dropping onto the
 		   same uri, first we need to append the
 		   basename of source onto target */
 		source = gnome_vfs_uri_new( uris[ i ] );
-		target = gnome_vfs_uri_new( target_uri );
+		
 		basename = gnome_vfs_uri_extract_short_name( source );
+		
+		target = gnome_vfs_uri_new( target_uri );
 		realtarget = gnome_vfs_uri_append_file_name(target,
 							    basename );
-		gnome_vfs_uri_unref( target );
-		target = realtarget;
-		
-		if( ! gnome_vfs_uri_equal( source, target ) ) {
+				
+		if( gnome_vfs_uri_equal( source, target ) ) {
+			/* copy to same location, don't do it. */
+			gnome_vfs_uri_unref( realtarget );
+
+		} else if( ! gnome_vfs_uri_equal( source, realtarget ) ) {
+			gnome_vfs_uri_unref( target );
+			target = realtarget;
+
 			tgt = gnome_vfs_uri_to_string(target,
 						      GNOME_VFS_URI_HIDE_NONE);
 			
@@ -1189,9 +1242,14 @@ static gboolean screem_site_view_drop_uris( ScreemSite *site,
 			gnome_vfs_get_file_info( uris[ i ], vfsinfo, 
 						 options );
 			if( vfsinfo->type == GNOME_VFS_FILE_TYPE_DIRECTORY ) {
-				res = copy_dir( uris[ i ], tgt, TRUE );
+				res = copy_dir( uris[ i ], tgt, 
+						FALSE,
+						screem_site_view_file_op,
+						site->private->view  );
 			} else {
-				res = copy_file( uris[ i ], tgt );
+				res = copy_file( uris[ i ], tgt, 
+						screem_site_view_file_op,
+						site->private->view );
 			}
 			gnome_vfs_file_info_unref( vfsinfo );
 			
@@ -1217,14 +1275,16 @@ static gboolean screem_site_view_drop_uris( ScreemSite *site,
 								 uris[i],tgt );
 					/* will call delete_file()
 					   if needed */
-					delete_dir( uris[ i ] );
+					delete_dir( uris[ i ], 
+						screem_site_view_file_op,
+						site->private->view );
 				}
 			}
 			g_free( tgt );
 			g_free( basename );
-			gnome_vfs_uri_unref( target );
-			gnome_vfs_uri_unref( source );
 		}
+		gnome_vfs_uri_unref( target );
+		gnome_vfs_uri_unref( source );
 	}
 		
 	return TRUE;
@@ -1349,8 +1409,8 @@ static GList *screem_site_view_calc_stats( GtkTreeModel *model,
 {
 	GnomeVFSFileInfo *info;
 	ScreemSiteViewNodeInfo *ninfo;
-	GValue value = { 0 };
 	GtkTreeIter iter;
+	const gchar *mime_type;
 
 	if( ! it ) {
 		/* get root */
@@ -1367,61 +1427,60 @@ static GList *screem_site_view_calc_stats( GtkTreeModel *model,
 		return types;
 	}
 
-	gtk_tree_model_get_value( model, it, FILE_BROWSER_DATA_COL, &value );
-	ninfo = g_value_get_pointer( &value );
-	g_value_unset( &value );
-	
-	info = gnome_vfs_file_info_new();
-
-	gnome_vfs_get_file_info( ninfo->fullname, 
-				 info, GNOME_VFS_FILE_INFO_GET_MIME_TYPE );
-
-	if( info->type == GNOME_VFS_FILE_TYPE_DIRECTORY &&
-	    gtk_tree_model_iter_has_child( model, it ) ) {
-		GtkTreeIter child;
-
-		if( gtk_tree_model_iter_children( model, &child, it ) ) {
-			types = screem_site_view_calc_stats( model, &child,
-							     types );
+	do {
+		gtk_tree_model_get( model, it, 
+				FILE_BROWSER_DATA_COL, &ninfo,
+				FILE_BROWSER_MIME_COL, 
+				(gchar**)&mime_type, -1 );
+		if( ! ninfo || ! ninfo->fullname ) {
+			continue;
 		}
-	} else {
-		GList *list;
-		const gchar *mime_type;
+		
+		info = gnome_vfs_file_info_new();
 
-		mime_type = gnome_vfs_file_info_get_mime_type( info );
+		gnome_vfs_get_file_info( ninfo->fullname, 
+				 info, 0 );
 
-		if( ! mime_type ) {
-			mime_type = _( "unknown" );
-		}
+		if( info->type == GNOME_VFS_FILE_TYPE_DIRECTORY &&
+		    gtk_tree_model_iter_has_child( model, it ) ) {
+			GtkTreeIter child;
 
-		for( list = types; list; list = list->next ) {
-			Stats *stats;
+			if( gtk_tree_model_iter_children( model, &child, it ) ) {
+				types = screem_site_view_calc_stats( model, &child,
+								     types );
+			}
+		} else {
+			GList *list;
 
-			stats = (Stats*)list->data;
-			if( ! strcmp( stats->mime_type, mime_type ) ) {
-				stats->number ++;
-				stats->total_size += info->size;
-				break;
+			if( ! mime_type ) {
+				mime_type = _( "unknown" );
+			}
+
+			for( list = types; list; list = list->next ) {
+				Stats *stats;
+
+				stats = (Stats*)list->data;
+				if( ! strcmp( stats->mime_type,
+							mime_type ) ) {
+					stats->number ++;
+					stats->total_size += info->size;
+					break;
+				}
+			}
+			if( ! list ) {
+				Stats *stats;
+			
+				stats = g_new0( Stats, 1 );
+				stats->uri = g_strdup( ninfo->fullname );
+				stats->number = 1;
+				stats->total_size = info->size;
+				stats->mime_type = g_strdup( mime_type );
+			
+				types = g_list_prepend( types, stats );
 			}
 		}
-		if( ! list ) {
-			Stats *stats;
-			
-			stats = g_new0( Stats, 1 );
-			stats->uri = g_strdup( ninfo->fullname );
-			stats->number = 1;
-			stats->total_size = info->size;
-			stats->mime_type = g_strdup( mime_type );
-			
-			types = g_list_prepend( types, stats );
-		}
-	}
-
-	gnome_vfs_file_info_unref( info );
-
-	if( gtk_tree_model_iter_next( model, it ) ) {
-		types = screem_site_view_calc_stats( model, it, types );
-	}
+		gnome_vfs_file_info_unref( info );
+	} while( gtk_tree_model_iter_next( model, it ) );
 
 	return types;
 }
@@ -1440,6 +1499,9 @@ GtkTreeModel *screem_site_get_statistics( ScreemSite *site )
 	store = screem_file_browser_get_model( browser );
 	store = gtk_tree_model_sort_get_model( GTK_TREE_MODEL_SORT( store ) );
 
+	/* FIXME: should do this in an idle handler, processing
+	 * 1 file / dir at a time */
+	
 	list = screem_site_view_calc_stats( store, NULL, NULL );
 
 	model = GTK_TREE_MODEL( gtk_list_store_new( 5, 
@@ -1558,6 +1620,7 @@ void screem_site_view_added( ScreemFileBrowser *browser, const gchar *uri,
 	ScreemSiteViewNodeInfo *info;
 	GtkTreeModel *model;
 	gboolean is_dir;
+	gchar *tmp;
 
 	site = SCREEM_SITE( data );
 
@@ -1572,31 +1635,44 @@ void screem_site_view_added( ScreemFileBrowser *browser, const gchar *uri,
 	model = screem_file_browser_get_model( browser );
 	model = gtk_tree_model_sort_get_model( GTK_TREE_MODEL_SORT( model ) );
 	
-	is_dir = screem_uri_is_dir( uri );
-
-	/* mime type hack */
-	if( ! mime_type ) {
-		mime_type = "text/plain";
+	is_dir = FALSE;
+	if( mime_type ) {
+		is_dir = ! strcmp( "x-directory/normal", mime_type );
+	} else {
+		/* mime type hack */
+		mime_type = "application/octet-stream";
 	}
 	
+	tmp = NULL;
 	if( ! is_dir ) {
+		/* if mime type is application/octet-stream
+		 * do a slow mime type check, we don't want
+		 * to miss any pages
+		 */
+		if( ! strcmp( "application/octet-stream", mime_type ) ){
+			tmp = screem_get_mime_type( uri, FALSE );
+			if( tmp ) {
+				mime_type = tmp;
+			}
+		}
+		
 		info = screem_site_view_node_info_new( SCREEM_SITE_VIEW_FILE,
 						       site,
-						       uri,
+						       (gpointer)uri,
 						       NULL );
 		info->view = private;
 
-		if( screem_page_is_file_page( uri ) &&
+		if( screem_page_is_mime_type_page( mime_type ) &&
 			! screem_site_locate_page( site, uri ) ) {
 			ScreemPage *page;
-			
+	
 			page = screem_page_new( G_OBJECT( app ) );
 			screem_page_set_pathname( page, uri );
 			if( ! screem_site_add_page( site, page ) ) {
-				g_object_unref( page );
-				page = screem_site_locate_page( site, uri );
-				g_assert( page );
+				/* shouldn't actually happen */
+				g_assert( FALSE );
 			}
+			g_object_unref( page );
 		}
 
 		if( ! strncmp( "image/", mime_type, strlen( "image/" ) ) ) {
@@ -1753,7 +1829,7 @@ void screem_site_view_added( ScreemFileBrowser *browser, const gchar *uri,
 	} else {
 		info = screem_site_view_node_info_new( SCREEM_SITE_VIEW_FOLDER,
 						       site,
-						       uri,
+						       (gpointer)uri,
 						       NULL );
 		info->view = private;
 	}
@@ -1762,6 +1838,7 @@ void screem_site_view_added( ScreemFileBrowser *browser, const gchar *uri,
 					     it, info );
 	}
 	g_object_unref( app );
+	g_free( tmp );
 }
 
 void screem_site_view_icon_change( ScreemFileBrowser *browser, const gchar *uri,
@@ -1802,3 +1879,18 @@ void screem_site_view_icon_change( ScreemFileBrowser *browser, const gchar *uri,
 	
 	g_free( url );
 }
+
+
+void screem_site_view_file_op( GnomeVFSMonitorEventType type,
+				const gchar *uri, gpointer data )
+{
+	ScreemSiteView *view;
+	ScreemSite *site;
+	ScreemApplication *app;
+	
+	view = (ScreemSiteView*)data;
+	site = view->site;
+	app = SCREEM_APPLICATION( site->private->application );
+	screem_application_file_op( type, uri, app );
+}
+
