@@ -33,7 +33,7 @@
 #include <dirent.h>
 
 #include <glib/gfileutils.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnome/gnome-exec.h>
 
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
@@ -48,6 +48,8 @@
 
 #include "fileops.h"
 
+#include "screem-application.h"
+
 #include "screem-site.h"
 #include "screem-site-view.h"
 
@@ -73,12 +75,23 @@ const gchar *upload_strings[] = {
 	"ftp",
 	"webdav",
 	"rsh",
-	"ssh"
+	"ssh",
+	""
+};
+const gchar *upload_schemes[] = {
+	"file://",
+	"ftp://",
+	"dav://",
+	"rsh://",
+	"sftp://",
+	""
 };
 
 enum {
 	PROP_0,
-	PROP_APP
+	PROP_APP,
+	PROP_BROWSER,
+	PROP_NAME
 };
 
 static void screem_site_class_init( ScreemSiteClass *klass );
@@ -89,8 +102,6 @@ static void screem_site_get_prop( GObject *object, guint prop_id,
 				  GValue *value, GParamSpec *spec );
 static void screem_site_finalize( GObject *object );
 
-static void screem_site_set_pages( ScreemSite *site, GList *list );
-
 static void screem_site_view_initialise( ScreemSite *site );
 
 static void screem_site_create_from_template( ScreemSite *site, 
@@ -122,30 +133,9 @@ ScreemSite *screem_site_new( GObject *application )
 	return site;
 }
 
-void screem_site_purge( ScreemSite *site )
-{
-	const GList *list;
-	ScreemPage *page;
-
-	list = screem_site_get_pages( site );
-	while( list ) {
-		page = SCREEM_PAGE( list->data );
-		screem_site_remove_page( site, page );
-		g_object_unref( G_OBJECT( page ) );
-		list = screem_site_get_pages( site );
-	}
-	screem_todo_clear( SCREEM_TODO( screem_site_get_todo( site ) ) ); 
-}
-
 void screem_site_set_name( ScreemSite *site, const gchar *name )
 {
-
-	/* does the site already have a name set? */
-	if( site->private->name ) {
-		g_free( site->private->name );
-	}
-	
-	site->private->name = g_strdup( name );
+	g_object_set( G_OBJECT( site ), "name", name, NULL );
 }
 
 const gchar* screem_site_get_name( ScreemSite *site )
@@ -158,24 +148,20 @@ const gchar* screem_site_get_name( ScreemSite *site )
 void screem_site_set_pathname( ScreemSite *site, const gchar *pathname )
 {
 	GnomeVFSURI *uri;
-	gchar *path;
+	gsize len;
 
 	/* does the site already have a pathname set? */
 	if( site->private->pathname ) {
 		g_free( site->private->pathname );
 	}
 
-	if( pathname[ strlen( pathname ) - 1 ] != G_DIR_SEPARATOR ) {
-		path = g_strconcat( pathname, G_DIR_SEPARATOR_S, NULL );
-	} else {
-		path = g_strdup( pathname );
+	uri = gnome_vfs_uri_new( pathname );
+	site->private->pathname = 
+		gnome_vfs_uri_to_string( uri, GNOME_VFS_URI_HIDE_NONE );
+	len = g_utf8_strlen( site->private->pathname, -1 );
+	if( site->private->pathname[ len - 1 ] == G_DIR_SEPARATOR ) {
+		site->private->pathname[ len - 1 ] = '\0';
 	}
-
-	uri = gnome_vfs_uri_new( path );
-	g_free( path );
-	site->private->pathname = gnome_vfs_uri_to_string( uri,
-							   GNOME_VFS_URI_HIDE_NONE );
-
 	gnome_vfs_uri_unref( uri );
 }
 
@@ -599,7 +585,9 @@ gboolean screem_site_create( ScreemSite *site )
 			       GNOME_VFS_PERM_USER_ALL |
 			       GNOME_VFS_PERM_GROUP_ALL |
 			       GNOME_VFS_PERM_OTHER_READ |
-			       GNOME_VFS_PERM_OTHER_EXEC ) ) {
+			       GNOME_VFS_PERM_OTHER_EXEC,
+			       screem_site_view_file_op,
+			       site->private->view ) ) {
 		return FALSE;
 	}
 
@@ -618,8 +606,8 @@ gboolean screem_site_create( ScreemSite *site )
 			  G_CALLBACK( screem_site_view_icon_change ),
 			  site );
 	screem_file_browser_set_sort_func( site->private->view->browser,
-					   screem_site_view_compare_func,
-					   site );
+			(GtkTreeIterCompareFunc)screem_site_view_compare_func,
+			site );
     
 	/* write the initial project file */
 	screem_site_write_project_file( site );
@@ -674,7 +662,7 @@ gboolean screem_site_load( ScreemSite *site )
 		gchar *name;
 	
 		name = g_build_filename( pathname, info->name, NULL );
-		mime_type = screem_get_mime_type( name );
+		mime_type = screem_get_mime_type( name, FALSE );
 		
 		if( mime_type &&
 		! strcmp( "application/x-screem", mime_type ) ) {
@@ -706,11 +694,12 @@ gboolean screem_site_load( ScreemSite *site )
 
 	project_file = NULL;
 	if( file ) {
-		project_file = g_strconcat( pathname, file->data, NULL );
+		project_file = g_build_filename( pathname, file->data, NULL );
 	}
 	g_list_foreach( project_files, (GFunc)g_free, NULL );
 
-	if( ! screem_site_parse_project_file( site, project_file ) ) {
+	if( project_file &&
+	   ! screem_site_parse_project_file( site, project_file ) ) {
 		screem_site_set_is_import( site, TRUE );
 	}
 	if( project_file ) {
@@ -731,8 +720,8 @@ gboolean screem_site_load( ScreemSite *site )
 			  G_CALLBACK( screem_site_view_icon_change ),
 			  site );
 	screem_file_browser_set_sort_func( site->private->view->browser,
-					   screem_site_view_compare_func,
-					   site );
+			(GtkTreeIterCompareFunc)screem_site_view_compare_func,
+			site );
 	screem_file_browser_scan_directory( site->private->view->browser,
 					    pathname, -1 );
 
@@ -911,7 +900,8 @@ gboolean screem_site_write_project_file( ScreemSite *site )
 	if( FALSE && name && strlen( name ) ) {
 		project_file = g_strconcat( pathname, name, ".screem", NULL );
 	} else {
-		project_file = g_strconcat( pathname,".project.screem", NULL );
+		project_file = g_build_filename( pathname,
+				".project.screem", NULL );
 	}
 
 
@@ -923,7 +913,9 @@ gboolean screem_site_write_project_file( ScreemSite *site )
 
 		retval = save_file( project_file, mem,
 				    GNOME_VFS_PERM_USER_READ | 
-				    GNOME_VFS_PERM_USER_WRITE );
+				    GNOME_VFS_PERM_USER_WRITE,
+				    FALSE,
+				    NULL );
 		xmlFree( mem );
 		xmlFreeDoc( doc );
 		g_free( project_file );
@@ -934,11 +926,8 @@ gboolean screem_site_write_project_file( ScreemSite *site )
 
 ScreemPage* screem_site_locate_page( ScreemSite *site, const gchar *path )
 {
-	const GList *list;
 	ScreemPage *page = NULL;
-	const gchar *pathname;
 	gchar *pname;
-	gchar *pthname;
 	GnomeVFSURI *uri;
 
 	g_return_val_if_fail( site, NULL );
@@ -954,81 +943,168 @@ ScreemPage* screem_site_locate_page( ScreemSite *site, const gchar *path )
 		pname = g_strdup( path );
 	}
 
-	for( list = screem_site_get_pages(site ); list && ( ! page );
-	     list = list->next ) {
-		page = SCREEM_PAGE( list->data );
-		pathname = screem_page_get_pathname( page );
-		if( pathname ) {
-			uri = gnome_vfs_uri_new( pathname );
-			pthname = gnome_vfs_uri_to_string( uri, 0 );
-			gnome_vfs_uri_unref( uri );
-
-			if( strcmp( pname, pthname ) ) { 
-				page = NULL;
-			}
-			g_free( pthname );
-		} else {
-			page = NULL;
-		}
-	}
-
+	page = g_hash_table_lookup( site->private->pages,
+			pname );
 	g_free( pname );
 
 	return page;
 }
 
+static void screem_site_page_saving( ScreemPage *page, ScreemSite *site )
+{
+	ScreemApplication *app;
+
+	g_object_get( G_OBJECT( site ), "app", &app, NULL );
+
+	screem_application_exec_helpers( app, page );
+	
+	g_object_unref( app );
+}
+
+static void screem_site_page_saved( ScreemPage *page, ScreemSite *site )
+{
+	ScreemSitePrivate *priv;
+	const gchar *pathname;
+	const gchar *sitepath;
+	gchar *dirname;
+	
+	priv = site->private;
+	
+	sitepath = screem_site_get_pathname( site );
+	
+	pathname = screem_page_get_pathname( page );
+	dirname = g_path_get_dirname( pathname );
+
+	/* attempt to add to the file browser model */
+	screem_file_browser_file_mod( priv->view->browser, dirname,
+				pathname, GNOME_VFS_MONITOR_EVENT_CREATED );
+
+	/* check template */
+	screem_site_check_template( page, site );
+
+	g_free( dirname );
+}
+
 gboolean screem_site_add_page( ScreemSite *site, ScreemPage *page )
 {
-	const GList *list;
-	GList *nlist;
-
+	ScreemSitePrivate *priv;
+	const gchar *pathname;
+	
 	g_return_val_if_fail( site != NULL, FALSE );
 	g_return_val_if_fail( page != NULL, FALSE );
 
-	list = screem_site_get_pages( site );
-	nlist = g_list_append( (GList*)list, page );
-		
+	priv = site->private;
+	
+	pathname = screem_page_get_pathname( page );
+	if( ! pathname && ! g_slist_find( priv->untitled, page ) ) {
+		/* untitled page, stick in untitled list */
+		priv->untitled = g_slist_prepend( priv->untitled, page );
+	} else {
+		g_assert( ! screem_site_locate_page( site, pathname ) );
+	
+		g_hash_table_insert( priv->pages, 
+				(gpointer)pathname, page );
+	}
+
+	/* hookup to the saving signal */
+	g_signal_connect( G_OBJECT( page ), "saving",
+			G_CALLBACK( screem_site_page_saving ),
+			site );
+	
 	/* hook up to the save signal */
 	g_signal_connect( G_OBJECT( page ), "saved",
-			G_CALLBACK( screem_site_check_template ),
+			G_CALLBACK( screem_site_page_saved ),
 			site );
-		
-	screem_site_set_pages( site, nlist );
 
+	/* keep a ref on page, pathname in priv->pages will always
+	 * be valid in this case */
+	g_object_ref( page );
+	
 	return TRUE;
 }
-ScreemPage* screem_site_remove_page_with_path( ScreemSite *site, const gchar *path )
+
+void screem_site_remove_page_with_path( ScreemSite *site, const gchar *path )
 {
 	ScreemPage *page;
 	
-	g_return_val_if_fail( site != NULL, NULL );
-	g_return_val_if_fail( path != NULL, NULL );
+	g_return_if_fail( site != NULL );
+	g_return_if_fail( path != NULL );
 
 	page = screem_site_locate_page( site, path );
-
-	return screem_site_remove_page( site, page );
-}
-
-ScreemPage* screem_site_remove_page( ScreemSite *site, ScreemPage *page )
-{
-	g_return_val_if_fail( SCREEM_IS_SITE( site ), NULL );
-	g_return_val_if_fail( SCREEM_IS_PAGE( page ), NULL );
-	
 	if( page ) {
-		GList *list;
-
-		list = (GList*)screem_site_get_pages( site );
-
-		list = g_list_remove( list, page );
-		screem_site_set_pages( site, list );
-
+		g_hash_table_remove( site->private->pages, path );
 	}
-	return page;
 }
 
-const GList* screem_site_get_pages( ScreemSite *site )
+void screem_site_remove_page( ScreemSite *site, ScreemPage *page )
 {
-	return site->private->pages;
+	const gchar *pathname;
+	
+	g_return_if_fail( SCREEM_IS_SITE( site ) );
+	g_return_if_fail( SCREEM_IS_PAGE( page ) );
+
+	pathname = screem_page_get_pathname( page );
+	if( ! pathname ) {
+		/* untitled page */
+		site->private->untitled = g_slist_remove( site->private->untitled, page );
+	} else {
+		screem_site_remove_page_with_path( site, pathname );
+	}
+}
+
+static void screem_site_get_pages_real( ScreemPage *page,
+		gpointer data )
+{
+	GList **ret;
+
+	ret = (GList**)data;
+	*ret = g_list_prepend( *ret, page );
+}
+
+GList *screem_site_get_pages( ScreemSite *site )
+{
+	GList *ret;
+	
+	ret = NULL;
+	screem_site_foreach_page( site,
+			(GFunc)screem_site_get_pages_real,
+			&ret );
+	return ret;
+}
+
+static void screem_site_foreach_page_real( gpointer key,
+		gpointer value,
+		gpointer data )
+{
+	ScreemSite *site;
+	ScreemPage *page;
+	GFunc func;
+	
+	site = SCREEM_SITE( data );
+	func = g_object_get_data( G_OBJECT( site ), "func" );
+	data = g_object_get_data( G_OBJECT( site ), "data" );
+	
+	page = SCREEM_PAGE( value );
+
+	func( page, data );
+}
+
+void screem_site_foreach_page( ScreemSite *site,
+		GFunc func, gpointer user_data )
+{
+	GSList *tmp;
+	
+	g_return_if_fail( SCREEM_IS_SITE( site ) );
+	g_return_if_fail( func != NULL );
+
+	g_object_set_data( G_OBJECT( site ), "func", func );
+	g_object_set_data( G_OBJECT( site ), "data", user_data );
+	
+	g_hash_table_foreach( site->private->pages,
+			screem_site_foreach_page_real, site );
+	for( tmp = site->private->untitled; tmp; tmp = tmp->next ) {
+		func( tmp->data, user_data );
+	}
 }
 
 GObject *screem_site_get_todo( ScreemSite *site )
@@ -1036,44 +1112,59 @@ GObject *screem_site_get_todo( ScreemSite *site )
 	return site->private->tasks;
 }
 
+static void screem_site_file_change_real( ScreemPage *page,
+		gpointer data )
+{
+	const gchar *pathname;
+	gchar *oldpathname;
+	gchar *temp;
+	gpointer *info;
+	ScreemSite *site;
+	const gchar *src;
+	const gchar *dest;
+	
+	pathname = screem_page_get_pathname( page );
+	info = (gpointer*)data;
+	site = info[ 0 ];
+	src = info[ 1 ];
+	dest = info[ 2 ];
+
+	/* if pathname is in src then we
+	   need to update the pathname to point
+	   to the new location */
+	oldpathname = g_strdup( pathname );
+	if( ! strncmp( pathname, src, strlen( src ) ) ) {
+		temp = g_strconcat( dest,
+				    pathname + strlen( src ),
+				    NULL );
+		screem_page_set_pathname( page, temp );
+		g_free( temp );
+	}
+	if( screem_page_load( page, NULL ) ) {
+		pathname = screem_page_get_pathname( page );
+		screem_markup_fix_links( site, page, pathname,
+					 oldpathname,
+					 src, dest );
+	} else {
+		/* FIXME: report error */
+	}
+	g_free( oldpathname );
+
+}
+
 void screem_site_file_change( ScreemSite *site, const gchar *src, 
 			      const gchar *dest )
 {
+	gpointer info[ 3 ];
+	
 	/* if we are a fake site we do not want to do this */
 	if( ! screem_site_get_fake_flag( site ) ) {
-		/* go through all pages in the site and update any links */
-		const GList *list;
-
-		for( list = screem_site_get_pages( site ); list; 
-		     list = list->next ) {
-			ScreemPage *page;
-			const gchar *pathname;
-			gchar *oldpathname;
-			gchar *temp;
-				
-			page = SCREEM_PAGE( list->data );
-			
-			pathname = screem_page_get_pathname( page );
-
-			/* if pathname is in src then we
-			   need to update the pathname to point
-			   to the new location */
-			oldpathname = g_strdup( pathname );
-			if( ! strncmp( pathname, src, strlen( src ) ) ) {
-				temp = g_strconcat( dest,
-						    pathname + strlen( src ),
-						    NULL );
-				screem_page_set_pathname( page, temp );
-				g_free( temp );
-			}
-			screem_page_load( page );
-			
-			pathname = screem_page_get_pathname( page );
-			screem_markup_fix_links( site, page, pathname,
-						 oldpathname,
-						 src, dest );
-			g_free( oldpathname );
-		}
+		info[ 0 ] = site;
+		info[ 1 ] = (gpointer)src;
+		info[ 2 ] = (gpointer)dest;
+		screem_site_foreach_page( site,
+				(GFunc)screem_site_file_change_real,
+				info );
 	}
 }
 
@@ -1137,7 +1228,21 @@ gchar *screem_get_local_site_path( ScreemSite *site )
 			/* strip method */
 			spath = gnome_vfs_get_local_path_from_uri( pathname );
 			gnome_vfs_uri_unref( uri );
-		} else {
+		} else if( ! strncmp( "file:", pathname,
+					strlen( "file:" ) ) ) {
+			/* not local, still a file uri so we
+			 * will hack a little as hostnames never
+			 * seem to be used on file: by anything.
+			 * this will occur if a path is nfs
+			 * mounted etc. */
+			spath = gnome_vfs_uri_to_string( uri, 
+					GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD |
+					GNOME_VFS_URI_HIDE_USER_NAME |
+					GNOME_VFS_URI_HIDE_PASSWORD |
+					GNOME_VFS_URI_HIDE_HOST_NAME |
+					GNOME_VFS_URI_HIDE_HOST_PORT );
+			gnome_vfs_uri_unref( uri );
+		} else 	{
 			spath = gnome_vfs_uri_to_string( uri, 0 );
 			gnome_vfs_uri_unref( uri );
 		}
@@ -1147,16 +1252,12 @@ gchar *screem_get_local_site_path( ScreemSite *site )
 }
 
 /* static stuff */
-static void screem_site_set_pages( ScreemSite *site, GList *list )
-{
-	site->private->pages = list;
-}
-
 static void screem_site_view_initialise( ScreemSite *site )
 {
 	ScreemSiteView *view;
 
 	view = site->private->view = g_new0( ScreemSiteView, 1 );
+	view->site = site;
 
 	view->browser = screem_file_browser_new();
 	screem_file_browser_set_mode( site->private->view->browser,
@@ -1193,7 +1294,9 @@ static void screem_site_create_from_template( ScreemSite *site,
 						 GNOME_VFS_PERM_USER_ALL |
 						 GNOME_VFS_PERM_GROUP_ALL |
 						 GNOME_VFS_PERM_OTHER_READ |
-						 GNOME_VFS_PERM_OTHER_EXEC );
+						 GNOME_VFS_PERM_OTHER_EXEC,
+						 screem_site_view_file_op,
+						 site->private->view );
 				g_free( name );
 			}
 			name = xmlGetProp( node, "template" );
@@ -1231,7 +1334,9 @@ static void screem_site_create_from_template( ScreemSite *site,
 				}
 				if( template ) {
 					/* copy template to dir */
-					copy_file( template, dir );
+					copy_file( template, dir,
+						screem_site_view_file_op,
+						site->private->view );
 					g_free( template );
 				} else {
 					/* create empty file */
@@ -1240,7 +1345,8 @@ static void screem_site_create_from_template( ScreemSite *site,
 						   GNOME_VFS_PERM_USER_WRITE |
 						   GNOME_VFS_PERM_GROUP_READ |
 						   GNOME_VFS_PERM_GROUP_WRITE |
-						   GNOME_VFS_PERM_OTHER_READ );
+						   GNOME_VFS_PERM_OTHER_READ,
+						   FALSE, NULL );
 				}
 				g_free( dir );
 				g_free( name );
@@ -1250,30 +1356,52 @@ static void screem_site_create_from_template( ScreemSite *site,
 	}
 }
 
+static void screem_site_check_template_real( ScreemPage *page,
+		gpointer data )
+{
+	ScreemSite *site;
+	ScreemPage *p;
+	gpointer *info;
+	const gchar *tag;
+	const gchar *path;
+	GSList *blocks;
+	
+	info = (gpointer*)data;
+	site = SCREEM_SITE( info[ 0 ] );
+	p = SCREEM_PAGE( info[ 1 ] );
+	tag = (const gchar*)info[ 2 ];
+	path = (const gchar*)info[ 3 ];
+	blocks = (GSList*)info[ 4 ];
+	
+	if( p != page ) {
+		screem_markup_update_from_template( site, p, page, tag, blocks );
+
+	}
+}
+
 static void screem_site_check_template( ScreemPage *page, ScreemSite *site )
 {
 	gchar *tag;
 	gchar *path;
 	GSList *blocks;
-
+	gpointer data[ 5 ];
+	
 	tag = NULL;
 
 	blocks = NULL;
 	if( ( ! screem_site_get_fake_flag( site ) ) &&
 	    screem_page_is_template( page, &tag, &path, &blocks ) ) {
-		const GList *pages;
-				
-		for( pages = screem_site_get_pages( site );
-			pages; pages = pages->next ) {
-			ScreemPage *p;
-			
-			p = SCREEM_PAGE( pages->data );
-			if( p != page ) {
-				screem_markup_update_from_template( site, page, p, tag, blocks );
-			}
-		}
+		data[ 0 ] = site;
+		data[ 1 ] = page;
+		data[ 2 ] = tag;
+		data[ 3 ] = path;
+		data[ 4 ] = blocks;
+		screem_site_foreach_page( site,
+				(GFunc)screem_site_check_template_real,
+				data );
 		g_slist_foreach( blocks, (GFunc)g_free, NULL );
 		g_slist_free( blocks );
+		g_free( tag );
 	}		
 }
 
@@ -1366,6 +1494,15 @@ static GList *screem_site_manage_get( ScreemSite *site,
 	return ret;
 }
 
+static void screem_site_set_name_real( ScreemSite *site,
+		const gchar *name )
+{
+	if( site->private->name != name ) {
+		g_free( site->private->name );
+		site->private->name = g_strdup( name );
+	}
+}
+
 /* G Object stuff */
 #define PARENT_TYPE G_TYPE_OBJECT
 
@@ -1374,7 +1511,8 @@ static gpointer parent_class;
 static void screem_site_class_init( ScreemSiteClass *klass )
 {
 	GObjectClass *object_class;
-
+	GParamSpec *spec;
+	
 	object_class = G_OBJECT_CLASS( klass );
 	parent_class = g_type_class_peek_parent( klass );
 
@@ -1382,15 +1520,19 @@ static void screem_site_class_init( ScreemSiteClass *klass )
 	object_class->set_property = screem_site_set_prop;
 	object_class->finalize = screem_site_finalize;
 
-	g_object_class_install_property(object_class,
-					PROP_APP,
-					g_param_spec_object("app",
-							    "Application",
-							    "The Application",
-							    G_TYPE_OBJECT,
-							    G_PARAM_READWRITE |
-							    G_PARAM_CONSTRUCT)
-					);
+	spec = g_param_spec_object( "app", "Application", "The Application",
+			G_TYPE_OBJECT, G_PARAM_READWRITE | G_PARAM_CONSTRUCT );
+	g_object_class_install_property( object_class, PROP_APP, spec );
+							
+	spec = g_param_spec_object( "browser", "Browser", 
+			"The site file browser",
+			G_TYPE_OBJECT, G_PARAM_READABLE );
+	g_object_class_install_property( object_class, PROP_BROWSER, spec );
+
+	spec =g_param_spec_string( "name", "Site name",
+			"The name of the site", "", G_PARAM_READWRITE );
+	g_object_class_install_property( object_class, PROP_NAME, spec );
+
 }
 
 static void screem_site_init( ScreemSite *site )
@@ -1417,6 +1559,11 @@ static void screem_site_init( ScreemSite *site )
 	site->private->asciis = gtk_list_store_new( 2, 
 						    G_TYPE_STRING,
 						    G_TYPE_BOOLEAN );
+
+	site->private->pages = g_hash_table_new_full( g_str_hash,
+			g_str_equal,
+			(GDestroyNotify)NULL,
+			(GDestroyNotify)g_object_unref );
 }
 
 static void screem_site_set_prop( GObject *object, guint prop_id,
@@ -1430,6 +1577,10 @@ static void screem_site_set_prop( GObject *object, guint prop_id,
 		case PROP_APP:
 			site->private->application = g_value_get_object( value );
 			break;
+		case PROP_NAME:
+			screem_site_set_name_real( site,
+					g_value_get_string( value ) );
+			break;
 		default:
 			break;
 	}
@@ -1446,6 +1597,14 @@ static void screem_site_get_prop( GObject *object, guint prop_id,
 		case PROP_APP:
 			g_value_set_object( value, site->private->application );
 			break;
+		case PROP_BROWSER:
+			g_value_set_object( value, 
+					site->private->view->browser );
+			break;
+		case PROP_NAME:
+			g_value_set_string( value,
+					site->private->name );
+			break;
 		default:
 			break;
 	}
@@ -1455,12 +1614,18 @@ static void screem_site_finalize( GObject *object )
 {
 	ScreemSite *site;
 	ScreemSitePrivate *private;
+	GSList *tmp;
 
 	site = SCREEM_SITE( object );
 
 	private = site->private;
 
-	screem_site_purge( site );
+	for( tmp = private->untitled; tmp; tmp = tmp->next ) {
+		g_object_unref( G_OBJECT( tmp->data ) );
+	}
+	g_slist_free( private->untitled );
+	
+	screem_todo_clear( SCREEM_TODO( screem_site_get_todo( site ) ) ); 
 	g_object_unref( screem_site_get_todo( site ) );
 
 	screem_site_view_destroy( site );
@@ -1475,7 +1640,6 @@ static void screem_site_finalize( GObject *object )
 	g_free( private->cvs_root );
 	g_free( private->template_path );
 
-	/* table already cleared by screem_site_purge() */
 	g_hash_table_destroy( private->documents );
 
 	g_object_unref( private->excludes );
@@ -1483,10 +1647,13 @@ static void screem_site_finalize( GObject *object )
 	g_object_unref( private->asciis );
 
 	if( private->auto_open ) {
-		g_list_foreach( private->auto_open, (GFunc)g_free, NULL );
+		g_list_foreach( private->auto_open, 
+				(GFunc)g_free, NULL );
 		g_list_free( private->auto_open );
 	}
 
+	g_hash_table_destroy( private->pages );
+	
 	g_free( private );
 
 	G_OBJECT_CLASS( parent_class )->finalize( object );
