@@ -30,13 +30,17 @@
 #include <libgnome/gnome-i18n.h>
 #include <gtk/gtktreestore.h>
 #include <gtk/gtktreemodelsort.h>
-#include <glib/gmarkup.h>
-#include <glib/gqueue.h>
 #include <gdk/gdk.h>
 
+#include <libxml/tree.h>
+#include <libxml/parser.h>
+#include <libxml/xpath.h>
+
 #include "fileops.h"
 #include "screem-file-browser.h"
 
+#include "support.h"
+
 static void screem_tag_file_class_init( ScreemTagFileClass *klass );
 static void screem_tag_file_instance_init( ScreemTagFile *tag_file );
 static void screem_tag_file_finalize( GObject *object );
@@ -61,35 +65,6 @@ static gboolean screem_tag_file_clear_foreach( GtkTreeModel *model,
 						GtkTreeIter *iter,
 						gpointer data );
 
-static void screem_tag_file_start_element( GMarkupParseContext *context,
-					const gchar *element,
-					const gchar **attributes,
-					const gchar **values,
-					gpointer data,
-					GError **error );
-static void screem_tag_file_end_element( GMarkupParseContext *context,
-					const gchar *element,
-					gpointer data,
-					GError **error );
-static void screem_tag_file_text( GMarkupParseContext *context,
-				const gchar *text,
-				gsize len,
-				gpointer data,
-				GError **error );
-static void screem_tag_file_error( GMarkupParseContext *context,
-				GError *error,
-				gpointer data );
-static void screem_tag_file_parse_node( ScreemTagFilePrivate *priv,
-					const gchar *element,
-					const gchar **attributes,
-					const gchar **values );
-static void screem_tag_file_parse_insert( ScreemTagFilePrivate *priv,
-					  const gchar **attributes,
-					  const gchar **values );
-
-static void screem_tag_file_parse_param( ScreemTagFilePrivate *priv,
-					 const gchar **attributes,
-					 const gchar **values );
 
 enum {
 	PROP_0,
@@ -119,43 +94,9 @@ struct ScreemTagFilePrivate {
 
 	ScreemTagFileFormat format;
 
-	GQueue *stack;
 	/* current parent path */
 	GtkTreePath *path;
 };
-
-static GMarkupParser screem_parser_cbs = {
-	screem_tag_file_start_element,
-	screem_tag_file_end_element,
-	screem_tag_file_text,
-	NULL,
-	screem_tag_file_error
-};
-
-typedef enum {
-	PARSER_STATE_NONE = 0,
-	PARSER_STATE_REF,
-	PARSER_STATE_GROUP,
-	PARSER_STATE_ENTRY,
-	PARSER_STATE_FUNCTION,
-	PARSER_STATE_INSERT,
-	PARSER_STATE_DESCRIPTION,
-	PARSER_STATE_PARAM,
-	PARSER_STATE_PROPERTY
-} ScreemTagTreeParserState;
-
-static const gchar *tag_file_tags[] = {
-	"",
-	"ref",
-	"group",
-	"entry",
-	"function",
-	"insert",
-	"description",
-	"param",
-	"property",
-};
-
 GNOME_CLASS_BOILERPLATE( ScreemTagFile, screem_tag_file,
 			  GObject,
 			  G_TYPE_OBJECT )
@@ -219,6 +160,9 @@ static void screem_tag_file_finalize( GObject *object )
 	screem_tag_file_clear( tag_file );
 
 	g_free( priv->filename );
+	g_free( priv->name );
+	g_free( priv->description );
+	g_free( priv->type );
 	
 	g_object_unref( priv->browser );
 	
@@ -241,7 +185,7 @@ static void screem_tag_file_set_prop( GObject *object, guint prop_id,
 			priv->active = g_value_get_boolean( value );
 			if( priv->active && ! priv->loaded ) {
 				screem_tag_file_load( tag_file,
-						SCREEM_TAG_TREE_FORMAT);
+						priv->format );
 			}
 			break;
 		default:
@@ -391,571 +335,6 @@ static gboolean screem_tag_file_clear_foreach( GtkTreeModel *model,
 	return FALSE;
 }
 
-static void screem_tag_file_start_element( GMarkupParseContext *context,
-					const gchar *element,
-					const gchar **attributes,
-					const gchar **values,
-					gpointer data,
-					GError **error )
-{
-	ScreemTagFile *tag_file;
-	ScreemTagFilePrivate *priv;
-	ScreemTagTreeParserState state;
-	GQueue *stack;
-	
-	tag_file = SCREEM_TAGFILE( data );
-	priv = tag_file->priv;
-
-	stack = priv->stack;
-	
-	state = GPOINTER_TO_INT( g_queue_peek_head( stack ) );
-
-	switch( state ) {
-
-		/* no parser state, valid elements == <ref> */
-		case PARSER_STATE_NONE:
-			if( ! strcmp( tag_file_tags[ PARSER_STATE_REF ],
-					element ) ) {
-				/* got ref state */
-
-
-				state = PARSER_STATE_REF;
-				g_queue_push_head( stack,
-						GINT_TO_POINTER( state ) );
-				screem_tag_file_parse_node( priv,
-						element,
-						attributes,
-						values );
-			} else {
-				/* error */
-				g_set_error( error, 
-					G_MARKUP_ERROR, 
-					G_MARKUP_ERROR_UNKNOWN_ELEMENT,
-					element );
-			}
-			
-			break;
-
-		/* <ref> allows <group> */
-		case PARSER_STATE_REF:
-			if( ! strcmp( tag_file_tags[ PARSER_STATE_GROUP ],
-					element ) ) {
-				/* got group state */
-
-				state = PARSER_STATE_GROUP;
-				g_queue_push_head( stack,
-						GINT_TO_POINTER( state ) );
-				screem_tag_file_parse_node( priv,
-						element,
-						attributes,
-						values );
-			} 
-			/* not in our DTD, but simplifies the
-			 * current doctype node */
-			else if( ! strcmp( tag_file_tags[ PARSER_STATE_ENTRY ],
-						element ) ) {
-				state = PARSER_STATE_ENTRY;
-				g_queue_push_head( stack,
-						GINT_TO_POINTER( state ) );
-				screem_tag_file_parse_node( priv, 
-						element,
-						attributes,
-						values );
-			} else {
-				/* error */
-				g_set_error( error, 
-					G_MARKUP_ERROR, 
-					G_MARKUP_ERROR_PARSE,
-					element );
-			}
-
-			break;
-	
-		/* <group> allows <group> <entry> or <function> */
-		case PARSER_STATE_GROUP:
-			if( ! strcmp( tag_file_tags[ PARSER_STATE_GROUP ],
-					element ) ) {
-				/* got group state */
-
-				state = PARSER_STATE_GROUP;
-				g_queue_push_head( stack,
-						GINT_TO_POINTER( state ) );
-			} else 	if( ! strcmp( tag_file_tags[ PARSER_STATE_ENTRY ],
-						element ) ) {
-				/* got entry state */
-
-				state = PARSER_STATE_ENTRY;
-				g_queue_push_head( stack,
-						GINT_TO_POINTER( state ) );
-			} else if( ! strcmp( tag_file_tags[ PARSER_STATE_FUNCTION ],
-						element ) ) {
-				/* got function state */
-
-				state = PARSER_STATE_FUNCTION;
-				g_queue_push_head( stack,
-						GINT_TO_POINTER( state ) );
-			} else {
-				/* error */
-				g_set_error( error, 
-					G_MARKUP_ERROR, 
-					G_MARKUP_ERROR_PARSE,
-					element );
-				break;
-			}
-			screem_tag_file_parse_node( priv, element,
-						attributes,
-						values );
-			break;
-	
-		/* <entry> allows <insert> <description> <property> */
-		case PARSER_STATE_ENTRY:
-			if( ! strcmp( tag_file_tags[ PARSER_STATE_INSERT ],
-					element ) ) {
-				/* got insert state */
-				screem_tag_file_parse_insert( priv,
-						attributes,
-						values );
-
-				state = PARSER_STATE_INSERT;
-				g_queue_push_head( stack,
-						GINT_TO_POINTER( state ) );
-			} else 	if( ! strcmp( tag_file_tags[ PARSER_STATE_DESCRIPTION ],
-						element ) ) {
-				/* got description state */
-
-				state = PARSER_STATE_DESCRIPTION;
-				g_queue_push_head( stack,
-						GINT_TO_POINTER( state ) );
-			} else if( ! strcmp( tag_file_tags[ PARSER_STATE_PROPERTY ],
-						element ) ) {
-				/* got property state */
-
-				state = PARSER_STATE_PROPERTY;
-				g_queue_push_head( stack,
-						GINT_TO_POINTER( state ) );
-				screem_tag_file_parse_param( priv,
-						attributes,
-						values );
-			} else {
-				/* error */
-				g_set_error( error, 
-					G_MARKUP_ERROR, 
-					G_MARKUP_ERROR_PARSE,
-					element );
-			}
-			break;
-
-		/* <function> allows <insert> <description> <param> */
-		case PARSER_STATE_FUNCTION:
-			if( ! strcmp( tag_file_tags[ PARSER_STATE_INSERT ],
-					element ) ) {
-				/* got insert state */
-				screem_tag_file_parse_insert( priv,
-						attributes,
-						values );
-
-				state = PARSER_STATE_INSERT;
-				g_queue_push_head( stack,
-						GINT_TO_POINTER( state ) );
-			} else 	if( ! strcmp( tag_file_tags[ PARSER_STATE_DESCRIPTION ],
-						element ) ) {
-				/* got description state */
-
-				state = PARSER_STATE_DESCRIPTION;
-				g_queue_push_head( stack,
-						GINT_TO_POINTER( state ) );
-			} else if( ! strcmp( tag_file_tags[ PARSER_STATE_PARAM ],
-						element ) ) {
-				/* got param state */
-
-				state = PARSER_STATE_PARAM;
-				g_queue_push_head( stack,
-						GINT_TO_POINTER( state ) );
-				screem_tag_file_parse_param( priv,
-						attributes,
-						values );
-			} else {
-				/* error, does comply with out
-				 * DTD, however that is because
-				 * we don't support <return> <dialog>
-				 * <info> in the Bluefish reference
-				 * files, ignore the error
-				 * 
-				g_set_error( error, 
-					G_MARKUP_ERROR, 
-					G_MARKUP_ERROR_PARSE,
-					element );
-				*/
-
-			}
-			break;
-
-		/* <insert> allows only #PCDATA 
-		 * <description> allows only #PCDATA
-		 * <param> allows only #PCDATA
-		 * <property> allows only #PCDATA */
-		case PARSER_STATE_INSERT:
-		case PARSER_STATE_DESCRIPTION:
-		case PARSER_STATE_PARAM:
-		case PARSER_STATE_PROPERTY:
-			/* error */
-			g_set_error( error, 
-				G_MARKUP_ERROR, 
-				G_MARKUP_ERROR_PARSE,
-				element );
-			break;
-	}
-}
-
-static void screem_tag_file_end_element( GMarkupParseContext *context,
-					const gchar *element,
-					gpointer data,
-					GError **error )
-{
-	ScreemTagFile *tag_file;
-	ScreemTagFilePrivate *priv;
-	ScreemTagTreeParserState state;
-	GQueue *stack;
-	
-	tag_file = SCREEM_TAGFILE( data );
-	priv = tag_file->priv;
-
-	stack = priv->stack;
-	
-	state = GPOINTER_TO_INT( g_queue_peek_head( stack ) );
-
-	if( ! strcmp( tag_file_tags[ state ], element ) ) {
-
-		switch( state ) {
-			case PARSER_STATE_REF:
-			case PARSER_STATE_GROUP:
-			case PARSER_STATE_ENTRY:
-			case PARSER_STATE_FUNCTION:
-				gtk_tree_path_up( priv->path );
-				break;
-		}
-		g_queue_pop_head( stack );
-	} else {
-		/* error, but as we accept Bluefish function
-		 * files it is possible to receive an end
-		 * tag for one of the tags we don't support,
-		 * ignore the error
-		g_set_error( error, 
-			G_MARKUP_ERROR, 
-			G_MARKUP_ERROR_PARSE,
-			element );
-			*/
-	}
-}
-
-static void screem_tag_file_text( GMarkupParseContext *context,
-				const gchar *text,
-				gsize len,
-				gpointer data,
-				GError **error )
-{
-	ScreemTagFile *tag_file;
-	ScreemTagFilePrivate *priv;
-	ScreemTagTreeParserState state;
-	GQueue *stack;
-
-	GtkTreeIter it;
-
-	NodeInfo *info;
-	NodeAttribute *attr;
-
-	tag_file = SCREEM_TAGFILE( data );
-	priv = tag_file->priv;
-
-	stack = priv->stack;
-	
-	state = GPOINTER_TO_INT( g_queue_peek_head( stack ) );
-
-	info = NULL;
-	if( priv->path ) {
-		/* get parent iter */
-		if( ! gtk_tree_model_get_iter( priv->model, 
-					&it, priv->path ) ) {
-			/* error */
-			return;
-		}
-		gtk_tree_model_get( priv->model, &it,
-				FILE_BROWSER_DATA_COL, &info, -1 );
-	} else {
-		/* error */
-		return;
-	}
-
-	switch( state ) {
-		case PARSER_STATE_INSERT:
-			info->text = g_strdup( text );
-			info->text = g_strstrip( info->text );
-			break;
-		case PARSER_STATE_DESCRIPTION:
-			info->description = g_strdup( text );
-			info->description = g_strstrip( info->description );
-			break;
-		case PARSER_STATE_PARAM:
-		case PARSER_STATE_PROPERTY:
-			if( info->attributes ) {
-				attr = (NodeAttribute*)info->attributes->data;
-				if( attr ) {
-					attr->description = g_strdup( text );
-					attr->description = g_strstrip( attr->description );
-
-				}
-			}
-			break;
-		default:
-			/* ignore */
-			break;
-	}
-}
-
-static void screem_tag_file_error( GMarkupParseContext *context,
-				GError *error,
-				gpointer data )
-{
-	ScreemTagFile *tag_file;
-	ScreemTagFilePrivate *priv;
-	ScreemTagTreeParserState state;
-	GQueue *stack;
-	
-	tag_file = SCREEM_TAGFILE( data );
-	priv = tag_file->priv;
-
-	stack = priv->stack;
-	
-	state = GPOINTER_TO_INT( g_queue_peek_head( stack ) );
-	
-}
-
-static void screem_tag_file_parse_node( ScreemTagFilePrivate *priv,
-					const gchar *element,
-					const gchar **attributes,
-					const gchar **values )
-{
-	ScreemTagTreeParserState state;
-	GQueue *stack;
-
-	GtkTreeIter it;
-	GtkTreeIter *parent;
-
-	GdkPixbuf *pixbuf;
-	const gchar *name;
-	const gchar *description;
-	const gchar *type;
-	gint i;
-	NodeInfo *info;
-	
-	stack = priv->stack;
-	
-	state = GPOINTER_TO_INT( g_queue_peek_head( stack ) );
-
-	parent = NULL;
-	if( priv->path ) {
-		if( ! gtk_tree_model_get_iter( priv->model, 
-					&it, priv->path ) ) {
-			/* error */
-			return;
-		}
-		parent = &it;
-	} else if( state != PARSER_STATE_REF ) {
-		/* error, can only not have a path for <ref> */
-		return;
-	}
-
-	gtk_tree_store_append( GTK_TREE_STORE( priv->model ),
-					&it, parent );
-	pixbuf = NULL;
-	name = description = type = NULL;
-	for( i = 0; attributes[ i ]; ++ i ) {
-		if( ! strcmp( "name", attributes[ i ] ) ) {
-			name = values[ i ];
-		} else if( ! strcmp( "description", attributes[ i ] ) ) {
-			description = values[ i ];
-		} else if( ! strcmp( "type", attributes[ i ] ) ) {
-			type = values[ i ];
-		}
-	}
-	if( ! name ) {
-		name = _( "Unknown Resource" );
-	}
-	
-	switch( state ) {
-		case PARSER_STATE_REF:
-			pixbuf = screem_file_browser_get_icon( priv->browser,
-					"/", 16, 16, FALSE, NULL );
-			g_free( priv->name );
-			g_free( priv->description );
-			g_free( priv->type );
-			priv->name = g_strdup( name );
-			priv->description = NULL;
-			if( description ) {
-				priv->description = g_strdup( description );
-			}
-			if( type ) {
-				priv->type = g_strdup( type );
-			}
-			break;
-		case PARSER_STATE_GROUP:
-			pixbuf = screem_file_browser_get_icon( priv->browser,
-					"/", 16, 16, FALSE, NULL );
-			break;
-		case PARSER_STATE_ENTRY:
-		case PARSER_STATE_FUNCTION:
-			description = NULL;
-			break;
-		default:
-			/* error */
-			return;
-			break;
-	}
-	priv->path = gtk_tree_model_get_path( priv->model, &it );
-
-	info = g_new0( NodeInfo, 1 );
-	info->tfile = priv->tfile;
-	info->name = g_strdup( name );
-	if( description ) {
-		info->description = g_strdup( description );
-	}
-	
-	gtk_tree_store_set( GTK_TREE_STORE( priv->model ),
-			&it,
-			FILE_BROWSER_NAME_COL, name,
-			FILE_BROWSER_ICON_COL, pixbuf,
-			FILE_BROWSER_DATA_COL, info,
-			-1 );
-	if( pixbuf ) {
-		g_object_unref( pixbuf );
-	}
-}
-
-static void screem_tag_file_parse_insert( ScreemTagFilePrivate *priv,
-					const gchar **attributes,
-					const gchar **values )
-{
-	ScreemTagTreeParserState state;
-	GQueue *stack;
-
-	GtkTreeIter it;
-
-	gint i;
-	NodeInfo *info;
-
-	const gchar *attr;
-	const gchar *val;
-	GdkPixbuf *pixbuf;
-	gchar *pathname;
-	
-	stack = priv->stack;
-	
-	state = GPOINTER_TO_INT( g_queue_peek_head( stack ) );
-
-	info = NULL;
-	if( priv->path ) {
-		/* get parent iter */
-		if( ! gtk_tree_model_get_iter( priv->model, 
-					&it, priv->path ) ) {
-			/* error */
-			return;
-		}
-		gtk_tree_model_get( priv->model, &it,
-				FILE_BROWSER_DATA_COL, &info, -1 );
-	} else {
-		/* error */
-		return;
-	}
-
-	info->isfunc = ( state == PARSER_STATE_FUNCTION );
-	
-	for( i = 0; attributes[ i ]; ++ i ) {
-		attr = attributes[ i ];
-		val = values[ i ];
-		
-		if( ! strcmp( "href", attr ) ) {
-			/* external reference */
-			pathname = relative_to_full( val,
-						RESOURCE_PATH );
-			info->file = pathname;
-			pixbuf = screem_file_browser_get_icon( priv->browser,
-					pathname, 16, 16, FALSE, NULL );
-			gtk_tree_store_set( GTK_TREE_STORE( priv->model ),
-					&it,
-					FILE_BROWSER_ICON_COL,
-					pixbuf, -1 );
-		} else if( ! strcmp( "open", attr ) ) {
-			/* open text */
-			info->open = g_strdup( val );	
-		} else if( ! strcmp( "close", attr ) ) {
-			/* close text */
-			info->close = g_strdup( val );
-		}
-	}
-
-}
-
-
-static void screem_tag_file_parse_param( ScreemTagFilePrivate *priv,
-					 const gchar **attributes,
-					 const gchar **values )
-{
-	ScreemTagTreeParserState state;
-	GQueue *stack;
-
-	GtkTreeIter it;
-
-	gint i;
-	NodeInfo *info;
-
-	NodeAttribute *attribute;
-	const gchar *attr;
-	const gchar *val;
-	
-	stack = priv->stack;
-	
-	state = GPOINTER_TO_INT( g_queue_peek_head( stack ) );
-
-	info = NULL;
-	if( priv->path ) {
-		/* get parent iter */
-		if( ! gtk_tree_model_get_iter( priv->model, 
-					&it, priv->path ) ) {
-			/* error */
-			return;
-		}
-		gtk_tree_model_get( priv->model, &it,
-				FILE_BROWSER_DATA_COL, &info, -1 );
-	} else {
-		/* error */
-		return;
-	}
-
-	attribute = g_new0( NodeAttribute, 1 );
-	
-	for( i = 0; attributes[ i ]; ++ i ) {
-		attr = attributes[ i ];
-		val = values[ i ];
-
-		if( ! strcmp( "name", attr ) ) {
-			attribute->name = g_strdup( val );
-			attribute->name = g_strstrip( attribute->name );
-		} else if( ! strcmp( "required", attr ) ) {
-			attribute->required = ( *val == '1' );
-		} else if( ! strcmp( "vallist", attr ) ) {
-			attribute->vallist = ( *val == '1' );
-		} else if( ! strcmp( "type", attr ) ) {
-			attribute->type = g_strdup( val );
-		} else if( ! strcmp( "default", attr ) ) {
-			attribute->defvalue = g_strdup( val );
-			attribute->defvalue = g_strstrip( attribute->defvalue );
-		}
-	}
-	info->attributes = g_slist_prepend( info->attributes, 
-					attribute );
-
-}
-
 static gboolean screem_tag_file_autocomplete_fill( GtkTreeModel *model,
 						GtkTreePath *path,
 						GtkTreeIter *iter,
@@ -991,6 +370,581 @@ static gboolean screem_tag_file_autocomplete_fill( GtkTreeModel *model,
 	return FALSE;
 }
 
+static gboolean screem_tag_file_autocomplete_tip_fill( GtkTreeModel *model,
+						GtkTreePath *path,
+						GtkTreeIter *iter,
+						gpointer data )
+{
+	GSList **ret;
+	ScreemTagFile *tfile;
+	NodeInfo *info;
+	const gchar *str;
+	const gchar *prefix;
+
+	tfile = SCREEM_TAGFILE( g_object_get_data( G_OBJECT( model ),
+						"tagfile" ) );
+	prefix = tfile->priv->prefix;
+	ret = (GSList**)data;
+
+	gtk_tree_model_get( model, iter,
+			FILE_BROWSER_DATA_COL, &info,
+			-1 );
+
+	str = info->tip;
+	if( ! str ) {
+		str = info->open;
+	}
+	if( ! str ) {
+		str = info->close;
+	}
+	if( ! str ) {
+		str = info->text;
+	}
+	if( str && 
+	   ! strncmp( str, prefix, strlen( prefix ) ) ) {
+		*ret = g_slist_prepend( *ret, (gchar*)str );
+	}
+
+	return FALSE;
+}
+
+static gint lookup_lang( gconstpointer a, gconstpointer b )
+{
+	gint ret;
+	
+	if( a && b ) {
+		ret = strcmp( a, b );
+	} else if( b ) {
+		ret = -1;
+	} else {
+		ret = 1;
+	}
+
+	return ret;
+}
+
+static gboolean find_node( const gchar *expr, gboolean lang,
+		xmlXPathContextPtr ctx, 
+		xmlNodePtr *node, xmlXPathObjectPtr *o )
+{
+	xmlXPathObjectPtr obj;
+	xmlNodeSetPtr nodes;
+	xmlNodePtr n;
+	const GList *langs;
+	const GList *tmp;
+	gint i;
+	gchar *xmllang;
+	gchar *lexpr;
+	
+	langs = gnome_i18n_get_language_list( "LC_COLLATE" );
+
+	if( ! node ) {
+		node = &n;
+	}
+	if( o ) {
+		*o = NULL;
+	}
+	
+	/* first we want a <ref> matching the locale */
+	*node = NULL;
+	obj = NULL;
+	tmp = NULL;
+	if( ! lang ) {
+		lexpr = g_strconcat( expr, "[@xml:lang]", NULL );
+		obj = xmlXPathEvalExpression( lexpr, ctx );
+		g_free( lexpr );
+	}
+	if( obj ) {
+		/* got some results */
+		nodes = obj->nodesetval;
+		for( i = 0; ( ( *node = xmlXPathNodeSetItem( nodes, i ) ) ); ++ i ) {
+			xmllang = xmlGetNsProp( *node, "lang",
+					XML_XML_NAMESPACE );
+			tmp = g_list_find_custom( (GList*)langs, 
+					(const gchar *)xmllang,
+					(GCompareFunc)lookup_lang );
+			g_free( xmllang );
+			if( tmp ) {
+				if( o ) {
+					*o = obj;
+				}
+				break;
+			}
+		}
+		if( ( ! *node ) || ! o ) {
+			xmlXPathFreeObject( obj );
+		}
+	}
+	if( ! *node ) {
+		/* failed to find one, fall back */
+		lexpr = g_strconcat( expr, "[not(@xml:lang)]", NULL );
+		obj = xmlXPathEvalExpression( lexpr, ctx );
+		g_free( lexpr );
+		if( obj ) {
+			nodes = obj->nodesetval;
+			*node = xmlXPathNodeSetItem( nodes, 0 );
+			if( o ) {
+				*o = obj;
+			} else {
+				xmlXPathFreeObject( obj );
+			}
+		}
+	}
+
+	return ( tmp != NULL );
+}
+
+static void process_param( xmlXPathContextPtr ctx, xmlNodePtr node, 
+		gboolean lang, gpointer userdata )
+{
+	ScreemTagFile *tfile;
+	ScreemTagFilePrivate *priv;
+	xmlNodePtr snode;
+	gboolean slang;
+
+	gchar *name;
+	gchar *desc;
+	gchar *required;
+	gchar *vallist;
+	gchar *defval;
+	gchar *type;
+
+	GtkTreeIter it;
+	NodeInfo *info;
+	NodeAttribute *attr;
+
+	tfile = SCREEM_TAGFILE( userdata );
+	priv = tfile->priv;
+	
+	/* process attrs */
+	name = xmlGetProp( node, "name" );
+	required = xmlGetProp( node, "required" );
+	vallist = xmlGetProp( node, "vallist" );
+	defval = xmlGetProp( node, "default" );
+	type = xmlGetProp( node, "type" );
+	desc = NULL;
+
+	/* process description */
+	ctx->node = node;
+	slang = find_node( "text()", lang, ctx, &snode, NULL );
+	if( snode ) {
+		desc = g_strdup( snode->content );
+	}
+
+	gtk_tree_model_get_iter( priv->model, &it, priv->path );
+	gtk_tree_model_get( priv->model, &it,
+			FILE_BROWSER_DATA_COL, &info, -1 );
+	
+	attr = g_new0( NodeAttribute, 1 );
+	if( name ) {
+		attr->name = g_strstrip( name );
+	}
+	if( required ) {
+		attr->required = ( *required == '1' );
+		g_free( required );
+	}
+	if( vallist ) {
+		attr->vallist = ( *vallist == '1' );
+		g_free( vallist );
+	}
+	if( defval ) {
+		attr->defvalue = g_strstrip( defval );
+	}
+	if( type ) {
+		attr->type = type;
+	}
+	if( desc ) {
+		attr->description = g_strstrip( desc );
+	}
+
+	info->attributes = g_slist_prepend( info->attributes, attr );
+}
+
+static void process_entry( xmlXPathContextPtr ctx, xmlNodePtr node, 
+		gboolean lang, gpointer userdata )
+{
+	ScreemTagFile *tfile;
+	ScreemTagFilePrivate *priv;
+	xmlXPathObjectPtr obj;
+	xmlNodeSetPtr set;
+	xmlNodePtr snode;
+	gboolean slang;
+	gint i;
+
+	gchar *name;
+	gchar *desc;
+
+	gchar *open;
+	gchar *close;
+	gchar *href;
+	gchar *data;
+	gchar *pathname;
+	
+	GtkTreeIter it;
+	GtkTreeIter pit;
+	GdkPixbuf *pixbuf;
+	NodeInfo *info;
+	GtkTreePath *path;
+
+	gboolean isfunc;
+	
+	tfile = SCREEM_TAGFILE( userdata );
+	priv = tfile->priv;
+	
+	isfunc = ( ! strcmp( "function", node->name ) );
+	
+	/* process attrs */
+	name = xmlGetProp( node, "name" );
+	desc = NULL;
+
+	/* process description */
+	ctx->node = node;
+	slang = find_node( "description", lang, ctx, &snode, NULL );
+	if( snode ) {
+		/* get text */
+		ctx->node = snode;
+		slang = find_node( "text()", slang, ctx,
+				&snode, NULL );
+		if( snode ) {
+			desc = g_strdup( snode->content );
+		}
+	}
+
+	pixbuf = NULL;
+	
+	info = g_new0( NodeInfo, 1 );
+	info->name = name;
+	info->isfunc = isfunc;
+	if( desc ) {
+		info->description = desc;
+	}
+	
+	gtk_tree_model_get_iter( priv->model, &pit, priv->path );
+	gtk_tree_store_append( GTK_TREE_STORE( priv->model ), 
+			&it, &pit );
+	gtk_tree_store_set( GTK_TREE_STORE( priv->model ), &it,
+			FILE_BROWSER_NAME_COL, name,
+			FILE_BROWSER_ICON_COL, pixbuf,
+			FILE_BROWSER_DATA_COL, info,
+			-1 );
+	path = gtk_tree_model_get_path( priv->model, &it );
+
+
+	/* process insert */
+	ctx->node = node;
+	slang = find_node( "insert", lang, ctx, &snode, NULL );
+	if( snode ) {
+		/* process attrs */
+		open = xmlGetProp( snode, "open" );
+		close = xmlGetProp( snode, "close" );
+		href = xmlGetProp( snode, "href" );
+
+		/* get text */
+		ctx->node = snode;
+		slang = find_node( "text()", slang, ctx,
+				&snode, NULL );
+		data = NULL;
+		if( snode ) {
+			data = g_strdup( snode->content );
+		}
+
+		if( href ) {
+			pathname = relative_to_full( href,
+					RESOURCE_PATH );
+			info->file = pathname;
+			pixbuf = screem_file_browser_get_icon( priv->browser,
+					pathname, 16, 16, FALSE, NULL );
+		} else {
+			if( open ) {
+				/* open text */
+				info->open = g_strdup( open );
+			} 
+			if( close ) {
+				/* close text */
+				info->close = g_strdup( close );
+			}
+		}
+		if( isfunc ) {
+			info->text = g_strconcat( info->name, "()",
+					NULL );
+			if( data ) {
+				info->tip = g_strstrip( data );
+			}
+		} else if( data ) {
+			info->text = g_strstrip( data );
+		}
+		g_free( open );
+		g_free( close );
+		g_free( href );
+	}
+	
+	/* process property / param */
+	ctx->node = node;
+	slang = find_node( "(property|param)", lang, ctx, NULL, &obj );
+	if( obj ) {
+		set = obj->nodesetval;
+		for( i = 0; i < set->nodeNr; ++ i ) {
+			snode = set->nodeTab[ i ];
+
+			/* process <property> / <param> */
+			priv->path = path;
+			process_param( ctx, snode, lang, userdata );
+		}
+		xmlXPathFreeObject( obj );
+	}
+
+	gtk_tree_store_set( GTK_TREE_STORE( priv->model ), &it,
+			FILE_BROWSER_ICON_COL, pixbuf,
+			-1 );
+
+	if( pixbuf ) {
+		g_object_unref( pixbuf );
+	}
+	gtk_tree_path_free( path );
+}
+
+static void process_group( xmlXPathContextPtr ctx, xmlNodePtr node, 
+	gboolean lang, gpointer userdata )
+{
+	ScreemTagFile *tfile;
+	ScreemTagFilePrivate *priv;
+	xmlXPathObjectPtr obj;
+	xmlNodeSetPtr set;
+	xmlNodePtr snode;
+	gboolean slang;
+	gint i;
+
+	gchar *name;
+	gchar *desc;
+
+	GtkTreeIter it;
+	GtkTreeIter pit;
+	GdkPixbuf *pixbuf;
+	NodeInfo *info;
+	GtkTreePath *path;
+	
+	tfile = SCREEM_TAGFILE( userdata );
+	priv = tfile->priv;
+	
+	/* process attrs */
+	name = xmlGetProp( node, "name" );
+	desc = xmlGetProp( node, "description" );
+	
+	/* process description */
+	if( ! desc ) {
+		ctx->node = node;
+		slang = find_node( "description", lang, ctx, 
+				&snode, NULL );
+		if( snode ) {
+			/* get text */
+			ctx->node = snode;
+			slang = find_node( "text()", slang, ctx,
+					&snode, NULL );
+			if( snode ) {
+				desc = g_strdup( snode->content );
+			}
+		}
+	}
+
+	pixbuf = screem_file_browser_get_icon( priv->browser,
+			"/", 16, 16, FALSE, NULL );
+	
+	info = g_new0( NodeInfo, 1 );
+	info->name = name;
+	if( desc ) {
+		info->description = desc;
+	}
+	
+	gtk_tree_model_get_iter( priv->model, &pit, priv->path );
+	gtk_tree_store_append( GTK_TREE_STORE( priv->model ), 
+			&it, &pit );
+	gtk_tree_store_set( GTK_TREE_STORE( priv->model ), &it,
+			FILE_BROWSER_NAME_COL, name,
+			FILE_BROWSER_ICON_COL, pixbuf,
+			FILE_BROWSER_DATA_COL, info,
+			-1 );
+	if( pixbuf ) {
+		g_object_unref( G_OBJECT( pixbuf ) );
+	}
+	path = gtk_tree_model_get_path( priv->model, &it );
+
+	/* get subgroups */
+	ctx->node = node;
+	slang = find_node( "group", lang, ctx, NULL, &obj );
+	if( obj && ( obj->nodesetval->nodeNr != 0 ) ) {
+		set = obj->nodesetval;
+		for( i = 0; i < set->nodeNr; ++ i ) {
+			snode = set->nodeTab[ i ];
+
+			/* process <group> */
+			priv->path = path;
+			process_group( ctx, snode, slang, userdata );
+		}
+		xmlXPathFreeObject( obj );
+	} else {
+		if( obj ) {
+			xmlXPathFreeObject( obj );
+		}
+		/* get entries */
+		ctx->node = node;
+		slang = find_node( "(entry|function)", lang, 
+				ctx, NULL, &obj );
+		if( obj ) {
+			set = obj->nodesetval;
+			for( i = 0; i < set->nodeNr; ++ i ) {
+				snode = set->nodeTab[ i ];
+
+				/* process <entry> */
+				priv->path = path;
+				process_entry( ctx, snode, slang,
+						userdata );
+			}
+			xmlXPathFreeObject( obj );
+		}
+	}
+
+	gtk_tree_path_free( path );
+}
+
+static void process_ref( xmlXPathContextPtr ctx, xmlNodePtr node,
+		gboolean lang, gpointer userdata )
+{
+	ScreemTagFile *tfile;
+	ScreemTagFilePrivate *priv;
+	xmlXPathObjectPtr obj;
+	xmlNodeSetPtr set;
+	xmlNodePtr gnode;
+	gint i;
+
+	gchar *name;
+	gchar *desc;
+	gchar *type;
+	
+	GtkTreeIter it;
+	GdkPixbuf *pixbuf;
+	NodeInfo *info;
+	GtkTreePath *path;
+	gboolean slang;
+	
+	tfile = SCREEM_TAGFILE( userdata );
+	priv = tfile->priv;
+	
+	/* process attrs */
+	name = xmlGetProp( node, "name" );
+	desc = xmlGetProp( node, "description" );
+	type = xmlGetProp( node, "type" );
+
+	g_free( priv->name );
+	g_free( priv->description );
+	g_free( priv->type );
+	priv->name = name;
+	if( desc ) {
+		priv->description = desc;
+	}
+	if( type ) {
+		priv->type = type;
+	}
+	
+	pixbuf = screem_file_browser_get_icon( priv->browser,
+			"/", 16, 16, FALSE, NULL );
+	
+	info = g_new0( NodeInfo, 1 );
+	info->name = g_strdup( name );
+	info->tfile = tfile;
+	if( desc ) {
+		info->description = g_strdup( desc );
+	}
+
+	gtk_tree_store_append( GTK_TREE_STORE( priv->model ), &it,
+			NULL );
+	gtk_tree_store_set( GTK_TREE_STORE( priv->model ), &it,
+			FILE_BROWSER_NAME_COL, name,
+			FILE_BROWSER_ICON_COL, pixbuf,
+			FILE_BROWSER_DATA_COL, info,
+			-1 );
+	
+	if( pixbuf ) {
+		g_object_unref( G_OBJECT( pixbuf ) );
+	}
+	path = gtk_tree_model_get_path( priv->model, &it );
+	
+	ctx->node = node;
+	
+	/* get groups */
+	lang = find_node( "group", lang, ctx, NULL, &obj );
+	if( obj  && ( obj->nodesetval->nodeNr != 0 ) ) {
+		set = obj->nodesetval;
+		for( i = 0; i < set->nodeNr; ++ i ) {
+			gnode = set->nodeTab[ i ];
+			
+			/* process <group> */
+			priv->path = path;
+			process_group( ctx, gnode, lang, userdata );
+		}
+		xmlXPathFreeObject( obj );
+	} else {
+		/* hmm no groups, cheat an treat <ref> as a group */
+		if( obj ) {
+			xmlXPathFreeObject( obj );
+		}
+		
+		/* get entries */
+		ctx->node = node;
+		slang = find_node( "(entry|function)", lang, 
+				ctx, NULL, &obj );
+		if( obj ) {
+			set = obj->nodesetval;
+			for( i = 0; i < set->nodeNr; ++ i ) {
+				gnode = set->nodeTab[ i ];
+
+				/* process <entry> */
+				priv->path = path;
+				process_entry( ctx, gnode, slang,
+						userdata );
+			}
+			xmlXPathFreeObject( obj );
+		}
+	}
+
+	gtk_tree_path_free( path );
+	priv->path = NULL;
+}
+
+static gboolean parse_from_string( const gchar *buffer, 
+		gpointer userdata )
+{
+	xmlDocPtr doc;
+	xmlXPathContextPtr ctx;
+	xmlNodePtr node;
+	gboolean lang;
+
+	doc = xmlParseMemory( buffer, strlen( buffer ) );
+	
+	if( doc ) {
+		ctx = xmlXPathNewContext( doc );
+		
+		lang = find_node( "/refs/ref", FALSE, ctx, &node, NULL );
+		if( ! node ) {
+			/* bluefish doesn't have the <refs> as
+			 * it doesn't translate <ref> */
+			lang = find_node( "/ref", FALSE, ctx, 
+					&node, NULL );
+		}
+		if( node ) {
+			/* process <ref> */
+			process_ref( ctx, node, lang, userdata );
+		} 
+	
+		xmlXPathFreeContext( ctx );
+		xmlFreeDoc( doc );
+		if( ! node ) {
+			doc = NULL;
+		}
+	}
+
+	return ( doc != NULL );
+}
+
 /* public stuff */
 
 ScreemTagFile *screem_tag_file_new( void )
@@ -1007,11 +961,33 @@ gboolean screem_tag_file_load( ScreemTagFile *tagfile,
 {
 	GString *file;
 	gboolean ret;
+	gchar *type;
 	
 	g_return_val_if_fail( SCREEM_IS_TAGFILE( tagfile ), FALSE );
 
+	if( format == SCREEM_TAG_UNKNOWN_FORMAT ) {
+		type = screem_get_mime_type( tagfile->priv->filename,
+				FALSE ); 
+		if( ! strcmp( "application/x-screem-tag-tree",
+					type ) ) {
+			format = SCREEM_TAG_TREE_FORMAT;
+		} else if( ! strcmp( "application/x-weml+xml",
+					type ) ) {
+			format = SCREEM_TAG_WEML_FORMAT;
+		} else if( ! strcmp( "text/xml", type ) ) {
+			/* treat text/xml as screem tag trees,
+			 * this is as bluefish uses .xml and
+			 * we support those files */
+			format = SCREEM_TAG_TREE_FORMAT;
+		} else {
+			g_free( type );
+			return FALSE;
+		}
+		g_free( type );
+	}
+	
 	ret = FALSE;
-	file = load_file( tagfile->priv->filename );
+	file = load_file( tagfile->priv->filename, NULL, NULL, NULL );
 	if( file ) {
 		tagfile->priv->format = format;
 		ret = screem_tag_file_set_from_string( tagfile, 
@@ -1037,9 +1013,7 @@ gboolean screem_tag_file_set_from_string( ScreemTagFile *tagfile,
 					guint length )
 {
 	ScreemTagFilePrivate *priv;
-	GMarkupParseContext *ctx;
 	gboolean ret;
-	GMarkupParser *parser_cbs;
 
 	g_return_val_if_fail( SCREEM_IS_TAGFILE( tagfile ), FALSE );
 	g_return_val_if_fail( str != NULL, FALSE );
@@ -1055,38 +1029,15 @@ gboolean screem_tag_file_set_from_string( ScreemTagFile *tagfile,
 	screem_tag_file_clear( tagfile );
 	
 	switch( priv->format ) {
+		case SCREEM_TAG_WEML_FORMAT:
+
+			break;
 		default:
-			parser_cbs = &screem_parser_cbs;
-			priv->stack = g_queue_new();
-			priv->path = NULL;
-			g_queue_push_head( priv->stack,
-					GINT_TO_POINTER( PARSER_STATE_NONE ) );
+			ret = parse_from_string( str, tagfile );
 			break;
 	}
 	
-	ctx = NULL;
-	if( parser_cbs ) {
-		ctx = g_markup_parse_context_new( parser_cbs, 0, 
-						  tagfile, NULL );
-	}
-	if( ctx ) {
-		ret = g_markup_parse_context_parse( ctx, str, 
-						length, NULL );
-		if( ret ) {
-			ret = g_markup_parse_context_end_parse( ctx,
-					NULL );
-		}
-		g_markup_parse_context_free( ctx );
-
-		priv->loaded = ret;
-	}
-	switch( priv->format ) {
-		default:
-			g_queue_free( priv->stack );
-			gtk_tree_path_free( priv->path );
-			break;
-	}
-
+	priv->loaded = ret;
 	
 	return ret;	
 }
@@ -1126,6 +1077,13 @@ const gchar *screem_tag_file_for_type( const ScreemTagFile *tagfile )
 	return tagfile->priv->type;
 }
 
+ScreemTagFileFormat screem_tag_file_get_format( const ScreemTagFile *tagfile )
+{
+	g_return_val_if_fail( SCREEM_IS_TAGFILE( tagfile ), 0 );
+
+	return tagfile->priv->format;
+}
+
 void screem_tag_file_set_name( ScreemTagFile *tagfile, 
 				const gchar *name )
 {
@@ -1156,11 +1114,20 @@ void screem_tag_file_set_uri( ScreemTagFile *tagfile,
 	tagfile->priv->filename = g_strdup( uri );
 }
 
+void screem_tag_file_set_format( ScreemTagFile *tagfile,
+		ScreemTagFileFormat format )
+{
+	g_return_if_fail( SCREEM_IS_TAGFILE( tagfile ) );
+
+	tagfile->priv->format = format;
+}
+
 GSList *screem_tag_file_autocomplete( ScreemTagFile *tagfile,
-					const gchar *prefix )
+		const gchar *prefix, gboolean fortip )
 {
 	ScreemTagFilePrivate *priv;
 	GSList *ret;
+	GtkTreeModelForeachFunc func;
 	
 	g_return_val_if_fail( SCREEM_IS_TAGFILE( tagfile ), NULL );
 	g_return_val_if_fail( prefix != NULL, NULL );
@@ -1169,9 +1136,12 @@ GSList *screem_tag_file_autocomplete( ScreemTagFile *tagfile,
 	
 	ret = NULL;
 	priv->prefix = prefix;
-	gtk_tree_model_foreach( priv->model,
-			(GtkTreeModelForeachFunc)screem_tag_file_autocomplete_fill,
-			&ret );
+	if( ! fortip ) {
+		func = screem_tag_file_autocomplete_fill;
+	} else {
+		func = screem_tag_file_autocomplete_tip_fill;
+	}
+	gtk_tree_model_foreach( priv->model, func, &ret );
 
 	return ret;			
 }
