@@ -30,17 +30,11 @@
 
 #include <glade/glade.h>
 
-#include <gtk/gtktreeview.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtksizegroup.h>
-#include <gtk/gtkdialog.h>
+#include <gtk/gtk.h>
 
 #include <gtksourceview/gtksourceview.h>
 
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
@@ -62,12 +56,6 @@
 #include "fileops.h"
 #include "support.h"
 
-#include "libegg/menu/egg-action-group.h"
-#include "libegg/menu/egg-toggle-action.h"
-#include "libegg/menu/egg-menu-merge.h"
-
-#include "libegg/toolbar/eggtoolbar.h"
-
 #include "eggtreemodelunion.h"
 
 #include "screem-tagfile.h"
@@ -88,6 +76,7 @@ struct ScreemTagTreePrivate {
 	GtkWidget *view;
 
 	GtkTreeModel *umodel;
+	GtkTreeModel *sorted;
 
 	GHashTable *paths;
 
@@ -137,12 +126,16 @@ static xmlNodePtr screem_tag_tree_get_file_node( const gchar *url );
 static gboolean screem_tag_tree_is_active( const gchar *url,
 					gchar **name,
 					gchar **desc );
-static void screem_tag_tree_search( EggAction *action,
+static void screem_tag_tree_search( GtkAction *action,
 				gpointer data );
 static gboolean screem_tag_tree_search_func( GtkTreeModel *model,
 					GtkTreePath *path,
 					GtkTreeIter *iter,
 					gpointer data );
+static gint screem_tag_tree_sort( GtkTreeModel *model,
+		GtkTreeIter *a, GtkTreeIter *b,
+		gpointer data );
+static gboolean screem_tag_tree_add_current( ScreemTagTree *tree );
 
 static void screem_tag_tree_class_init( ScreemTagTreeClass *klass );
 static void screem_tag_tree_init( ScreemTagTree *tag_tree );
@@ -173,7 +166,7 @@ gboolean screem_tag_tree_load_trees( void )
 	gchar *dot;
 
 	if( ! mutex ) {
-gdk_threads_enter();
+		gdk_threads_enter();
 		mutex = g_mutex_new();
 		filesstore = gtk_list_store_new( MAX_TAG_FILE_COLS,
 						G_TYPE_STRING,
@@ -186,7 +179,10 @@ gdk_threads_enter();
 				dot, "tagtreeconfig.xml",
 				NULL );
 		g_free( dot );
-		config = xmlParseFile( configpath );
+		config = NULL;
+		if( uri_exists( configpath, NULL ) ) {
+			config = xmlParseFile( configpath );
+		}
 		if( ! config ) {
 			config = xmlNewDoc( XML_DEFAULT_VERSION );
 			xmlDocSetRootElement( config,
@@ -209,6 +205,19 @@ GtkListStore *screem_tag_tree_get_file_model( void )
 	return filesstore;
 }
 
+static gint screem_tag_tree_find_tag_file( gconstpointer a,
+					gconstpointer b )
+{
+	const gchar *uri;
+	gint match;
+	
+	uri = screem_tag_file_get_uri( SCREEM_TAGFILE( a ) );
+	
+	match = ! gnome_vfs_uris_match( uri, (const gchar*)b );
+	
+	return match;
+}
+
 gboolean screem_tag_tree_add_uri( const gchar *uri,
 				  ScreemTagFileFormat format )
 {
@@ -218,8 +227,15 @@ gboolean screem_tag_tree_add_uri( const gchar *uri,
 	gchar *desc;
 	GtkTreeIter it;
 	xmlNodePtr node;
-	
+
+	/* don't add already existing uris */
+	if( g_slist_find_custom( treefiles, uri,
+				(GCompareFunc)screem_tag_tree_find_tag_file ) ) {
+		return TRUE;
+	}
+
 	tfile = screem_tag_file_new();
+	screem_tag_file_set_format( tfile, format );
 
 	screem_tag_file_set_uri( tfile, uri );
 	g_mutex_lock( mutex );
@@ -236,7 +252,7 @@ gboolean screem_tag_tree_add_uri( const gchar *uri,
 			"notify::active",
 			G_CALLBACK( screem_tag_file_active_set ),
 			NULL );
-gdk_threads_enter();
+	gdk_threads_enter();
 	gtk_list_store_append( filesstore, &it );
 	gtk_list_store_set( filesstore, &it,
 			TAG_FILE_NAME_COL, name,
@@ -244,22 +260,17 @@ gdk_threads_enter();
 			TAG_FILE_DESC_COL, desc,
 			TAG_FILE_OBJ_COL, tfile,
 			-1 );	
+	gdk_threads_leave();
 	g_mutex_unlock( mutex );
 	g_free( name );
 	g_free( desc );
+	
+	g_object_set( G_OBJECT( tfile ), "active", active, NULL );
+	
 	if( active ) {
-		screem_tag_file_load( tfile, format );
-		name = (gchar*)screem_tag_file_get_name( tfile );
-		desc = (gchar*)screem_tag_file_get_desc( tfile );
-		gtk_list_store_set( filesstore, &it,
-			TAG_FILE_NAME_COL, name,
-			TAG_FILE_DESC_COL, desc,
-			-1 );
-		g_object_set( G_OBJECT( tfile ),
-				"active", TRUE, NULL );
-		
 		g_mutex_lock( mutex );	
-		if( ! screem_tag_tree_get_file_node( uri ) ) {
+		node = screem_tag_tree_get_file_node( uri );
+		if( ! node ) {
 			/* new node being added, add to config */
 			node = xmlDocGetRootElement( config );
 			node = xmlNewChild( node, NULL, "tagfile", NULL );
@@ -275,8 +286,6 @@ gdk_threads_enter();
 		}
 		g_mutex_unlock( mutex );
 	}
-
-	gdk_threads_leave();
 	
 	return TRUE;
 }
@@ -284,24 +293,20 @@ gdk_threads_enter();
 void screem_tag_tree_remove_uri( const gchar *uri )
 {
 	GSList *tmp;
-	ScreemTagFile *tfile;
 	const gchar *furi;
+	ScreemTagFile *tfile;
 	GtkTreeIter *it;
 	gboolean active;
 	xmlNodePtr node;
 	
 	g_mutex_lock( mutex );
 
-	for( tmp = treefiles; tmp; tmp = tmp->next ) {
+	tmp = g_slist_find_custom( treefiles, uri,
+			(GCompareFunc)screem_tag_tree_find_tag_file );
+
+	if( tmp ) {
 		tfile = SCREEM_TAGFILE( tmp->data );
 		furi = screem_tag_file_get_uri( tfile );
-
-		if( gnome_vfs_uris_match( uri, furi ) ) {
-			break;
-		}
-	}
-	
-	if( tmp ) {
 		it = screem_support_find_in_list( filesstore,
 				TAG_FILE_URI_COL, furi );
 		g_assert( it );
@@ -319,18 +324,20 @@ void screem_tag_tree_remove_uri( const gchar *uri )
 							tfile );
 			}
 		}
+	
+		if( ( node = screem_tag_tree_get_file_node( uri ) ) ) {
+			xmlUnlinkNode( node );
+			xmlFreeNode( node );
+			screem_tag_tree_save_config();
+		}
 		g_object_unref( tfile );
 	}
-	if( ( node = screem_tag_tree_get_file_node( uri ) ) ) {
-		xmlUnlinkNode( node );
-		xmlFreeNode( node );
-		screem_tag_tree_save_config();
-	}
 	g_mutex_unlock( mutex );
 }
 
 GSList *screem_tag_tree_autocomplete( const gchar *mime_type,
-				const gchar *prefix )
+				const gchar *prefix,
+				gboolean fortip )
 {
 	GSList *ret;
 	GSList *tmp;
@@ -352,7 +359,7 @@ GSList *screem_tag_tree_autocomplete( const gchar *mime_type,
 		if( type && ! strcmp( mime_type, type ) ) {
 
 			tret = screem_tag_file_autocomplete( tfile,
-					prefix );
+					prefix, fortip );
 			ret = g_slist_concat( ret, tret );
 		}
 	}
@@ -372,35 +379,68 @@ static const gchar *screem_tag_tree_dtd_cache( ScreemTagTree *tree,
 	const GSList *elements;
 	GString *str;
 	gchar *data;
+	gchar *escaped;
 
+	g_return_val_if_fail( SCREEM_IS_DTD( dtd ), NULL );
+	
 	ret = g_object_get_data( G_OBJECT( dtd ),
 				"tagtree_string" );
 	if( ! ret ) {
+		const gchar *pub;
+		gboolean html;
+		const gchar *closing;
+		
 		elements = screem_dtd_get_elements( dtd );
 	
+		html = FALSE;
+		pub = screem_dtd_get_public_id( dtd );
+		if( pub ) {
+			html = ( strstr( pub, " HTML " ) != NULL );
+			html |= ( strstr( pub, " html " ) != NULL );
+			escaped = g_markup_escape_text( pub, strlen( pub ) );
+		} else {
+			pub = screem_dtd_get_system_id( dtd );
+			if( pub ) {
+				escaped = g_markup_escape_text( pub, strlen( pub ) );
+			} else {
+				escaped = g_strdup( "" );
+			}
+		}
+		
+		if( html ) {
+			closing = "&gt;";
+		} else {
+			closing = " /&gt;";
+		}
+		
 		str = g_string_new( "<?xml version=\"1.0\"?>" );
-		g_string_append_printf( str, "<ref name=\"%s\" description=\"%s\">",
+		g_string_append_printf( str, "<refs><ref name=\"%s\" description=\"%s:\n%s\">",
 					_( "Current Doctype" ),
-				_( "Elements for the current doctype" ) );
+				_( "Elements for the current doctype" ),
+				escaped );
+		g_free( escaped );
+
 		while( elements ) {
 			const gchar *name;
 			const gchar *desc;
 			ScreemDTDTagExistance close;
 			const GSList *attrs;
 			const gchar *adesc;
-			gchar *escaped;
 
 			name = screem_dtd_element_get_name( (ScreemDTDElement*)elements->data );
 			desc = screem_dtd_element_get_description( (ScreemDTDElement*)elements->data );		
 			close = screem_dtd_element_close_state( dtd, name );
-			attrs = screem_dtd_element_get_attrs( (ScreemDTDElement*)elements->data );
-		
-			g_string_append_printf(str, 
-					"<entry name=\"%s\"><insert open=\"&lt;%s&gt;\"",
-					name, name );
 			if( close != SCREEM_DTD_MUST_NOT ) {
+				g_string_append_printf(str, 
+						"<entry name=\"%s\"><insert open=\"&lt;%s&gt;\"",
+						name, name );
 				g_string_append_printf( str, " close=\"&lt;/%s&gt;\"",
 							name );
+			} else {
+				g_string_append_printf(str, 
+						"<entry name=\"%s\"><insert open=\"&lt;%s%s\"",
+						name, name, closing );
+
 			}
 			g_string_append( str, "/>");
 			if( desc ) {
@@ -410,7 +450,8 @@ static const gchar *screem_tag_tree_dtd_cache( ScreemTagTree *tree,
 						escaped );
 				g_free( escaped );
 			}
-
+			
+			attrs = screem_dtd_element_get_attrs( (ScreemDTDElement*)elements->data );
 			while( attrs ) {
 				const ScreemDTDAttribute *attr;
 
@@ -418,7 +459,7 @@ static const gchar *screem_tag_tree_dtd_cache( ScreemTagTree *tree,
 				adesc = screem_dtd_attribute_get_description( attr );
 				if( ! adesc ) {
 					adesc = "";
-				}
+				} 
 				escaped = g_markup_escape_text( adesc, strlen( adesc ) );
 				g_string_append_printf( str,
 						"<property name=\"%s\" required=\"%i\" vallist=\"0\" default=\"%s\">%s</property>",
@@ -436,12 +477,13 @@ static const gchar *screem_tag_tree_dtd_cache( ScreemTagTree *tree,
 			elements = elements->next;
 		}
 
-		g_string_append( str, "</ref>" );
-
+		g_string_append( str, "</ref></refs>" );
+		
 		tmp = g_slist_append( tree->private->dtdcache,
 					dtd );
 		tree->private->dtdcache = tmp;
 		ret = str->str;
+
 		g_object_set_data( G_OBJECT( dtd ), "tagtree_string",
 				   (gpointer)ret );
 		
@@ -489,8 +531,8 @@ static gboolean screem_tag_tree_click( GtkTreeView *tree_view,
 	GtkTreeIter iter;
 	GtkTreeModel *model;
 	GtkTreeSelection *sel;
-
 	ScreemTagTree *tag_tree;
+	guint pos;
 
 	tag_tree = SCREEM_TAG_TREE( data );
 
@@ -514,18 +556,18 @@ static gboolean screem_tag_tree_click( GtkTreeView *tree_view,
 			
 			vuri = gnome_vfs_uri_new( info->file );
 			if( ! vuri ) {
-				screem_window_show_error( window, _( "Invalid URI for requested resource" ) );
+				screem_window_show_error( window, _( "Invalid URI for requested resource" ), FALSE );
 			} else if( gnome_vfs_uri_is_local( vuri ) ||
 				  ! window->details->offline ) {
 				GString *data;
 				
-				data = load_file( info->file );
+				data = load_file( info->file, NULL, NULL, NULL );
 				/* now set info->text to be the file contents,
 			   	this way we don't refetch files we already have */
 				info->text = data->str;
 				g_string_free( data, FALSE );
 			} else {
-				screem_window_show_message( window, _( "Remote resources are not available in offline mode" ) );	
+				screem_window_show_message( window, _( "Remote resources are not available in offline mode" ), FALSE );	
 			}
 
 			if( vuri ) {	
@@ -542,6 +584,10 @@ static gboolean screem_tag_tree_click( GtkTreeView *tree_view,
 		 */
 		if( info && info->text ) {
 			screem_editor_insert( editor, -1, info->text );
+			if( g_str_has_suffix( info->text, ")" ) ) {
+				pos = screem_editor_get_pos( editor );
+				screem_editor_set_pos ( editor, pos - 1 );
+			}
 		}
 	} else {
 		/* hmm, no iterator */
@@ -579,7 +625,7 @@ static void screem_tagtree_sel_change( GtkTreeSelection *selection,
 		gtk_tree_model_get( model, &iter, FILE_BROWSER_DATA_COL,
 					 &info, -1 );
 		if( info && info->description ) {
-			screem_window_show_message( window, info->description );
+			screem_window_show_message( window, info->description, FALSE );
 		}
 	}
 	gtk_widget_set_sensitive( priv->toolbar, ( rows > 0 ) );
@@ -588,6 +634,7 @@ static void screem_tagtree_sel_change( GtkTreeSelection *selection,
 static void screem_tag_tree_set_model( ScreemTagTree *tree )
 {
 	ScreemTagTreePrivate *priv;
+	GtkTreeModel *model;
 	
 	g_return_if_fail( SCREEM_IS_TAG_TREE( tree ) );
 	
@@ -599,13 +646,25 @@ static void screem_tag_tree_set_model( ScreemTagTree *tree )
 					   G_TYPE_POINTER,
 					   G_TYPE_STRING,
 					   G_TYPE_POINTER,
-					   G_TYPE_STRING );
+					   G_TYPE_STRING,
+					   G_TYPE_POINTER );
 
 	priv->dtdbranch = screem_tag_file_new();
 
+	model = gtk_tree_model_sort_new_with_model( priv->umodel );
+	gtk_tree_sortable_set_sort_func( GTK_TREE_SORTABLE( model ),
+			0, screem_tag_tree_sort, tree, NULL );
+	gtk_tree_sortable_set_sort_column_id( GTK_TREE_SORTABLE( model ),
+			FILE_BROWSER_NAME_COL, GTK_SORT_ASCENDING );
+	priv->sorted = model;
+
+	gdk_threads_leave();
 	screem_tag_tree_add_file( tree, priv->dtdbranch, TRUE );
+	gdk_threads_enter();
+
 	gtk_tree_view_set_model( GTK_TREE_VIEW( tree->private->view ), 
-				 GTK_TREE_MODEL( priv->umodel ) );
+			model );
+	g_object_unref( model );
 }
 
 static gint tagtree_filter( const GnomeVFSFileInfo *info )
@@ -627,6 +686,28 @@ static gint tagtree_filter( const GnomeVFSFileInfo *info )
 
 	return ret;
 }
+#if 0
+static gint weml_filter( const GnomeVFSFileInfo *info )
+{
+	const gchar *mime_type;
+	gchar *type;
+	gint ret;
+	
+	mime_type = info->mime_type;
+
+	type = screem_get_mime_type_override( info->name, mime_type ); 
+
+	ret = -1;
+	if( type ) {
+		ret = strcmp( "application/x-weml+xml", 
+				type );
+		g_free( type );
+	}
+
+	return ret;
+
+}
+#endif
 static gboolean screem_tag_tree_add_screem_format( gpointer data )
 {
 	screem_tag_tree_add_uri( data, SCREEM_TAG_TREE_FORMAT );
@@ -645,6 +726,13 @@ static gpointer screem_tag_tree_create( gpointer data )
 		NULL,
 		NULL
 	};
+	xmlNodePtr node;
+	xmlNodePtr dead;
+	gchar *file;
+	GnomeVFSResult res;
+	
+	/* FIXME: windows should show a busy-interactive cursor
+	   while this function is running */
 
 	/* load tag trees */
 	treepaths[ 1 ] = screem_get_dot_dir();
@@ -660,10 +748,40 @@ static gpointer screem_tag_tree_create( gpointer data )
 	}
 	g_free( (gchar*)treepaths[ 1 ] );
 
+	/* now load any additional trees that may be present in the
+	 * tagtreeconfig.xml */
+	g_mutex_lock( mutex );
+	files = NULL;
+	node = xmlDocGetRootElement( config );
+	node = node->children;
+	while( node ) {
+		file = xmlGetProp( node, "src" );
+
+		if( uri_exists( file, &res ) ) {
+			files = g_slist_prepend( files, file );
+			node = node->next;
+		} else if( res == GNOME_VFS_ERROR_NOT_FOUND ){
+			/* remove if not found, we will
+			 * allow other errors to keep the uri
+			 * in the tag config */
+			dead = node;
+			node = node->next;
+			xmlUnlinkNode( dead );
+			xmlFreeNode( dead );
+		}
+	}
+	screem_tag_tree_save_config();
+	g_mutex_unlock( mutex );
+	for( tmp = files; tmp; tmp = tmp->next ) {
+		screem_tag_tree_add_screem_format( tmp->data );
+		g_free( tmp->data );
+	}
+	g_slist_free( files );
+	
 	return NULL;
 }
 
-static void add_widget( EggMenuMerge *merge, GtkWidget *widget, ScreemTagTree *tree )
+static void add_widget( GtkUIManager *merge, GtkWidget *widget, ScreemTagTree *tree )
 {
 	ScreemTagTreePrivate *priv;
 	GtkWidget *box;
@@ -671,9 +789,9 @@ static void add_widget( EggMenuMerge *merge, GtkWidget *widget, ScreemTagTree *t
 	priv = tree->private;
 	box = priv->box;
 	
-	if( EGG_IS_TOOLBAR( widget ) ) {
-		egg_toolbar_set_show_arrow( EGG_TOOLBAR( widget ), TRUE );
-		egg_toolbar_set_tooltips( EGG_TOOLBAR( widget ), TRUE );
+	if( GTK_IS_TOOLBAR( widget ) ) {
+		gtk_toolbar_set_show_arrow( GTK_TOOLBAR( widget ), TRUE );
+		gtk_toolbar_set_tooltips( GTK_TOOLBAR( widget ), TRUE );
 		
 		gtk_box_pack_start( GTK_BOX( box ), widget,
 				    FALSE, TRUE, 0 );
@@ -682,7 +800,7 @@ static void add_widget( EggMenuMerge *merge, GtkWidget *widget, ScreemTagTree *t
 	}
 }
 
-static void screem_tag_tree_tag_help( EggAction *action, ScreemTagTree *tree )
+static void screem_tag_tree_tag_help( GtkAction *action, ScreemTagTree *tree )
 {
 	ScreemTagTreePrivate *priv;
 	GtkTreeView *view;
@@ -762,7 +880,7 @@ static void screem_tag_tree_tag_help( EggAction *action, ScreemTagTree *tree )
 					  ! window->details->offline ) {
 					GString *data;
 				
-					data = load_file( info->file );
+					data = load_file( info->file, NULL, NULL, NULL );
 					/* now set info->text to be the file contents,
 				   	this way we don't refetch files we already have */
 					info->text = data->str;
@@ -826,7 +944,8 @@ static void screem_tag_tree_tag_help( EggAction *action, ScreemTagTree *tree )
 					gtk_label_set_justify( GTK_LABEL( widget ), GTK_JUSTIFY_LEFT );
 					gtk_misc_set_alignment( GTK_MISC( widget ), 0.0, 0.0 );
 
-					gtk_box_pack_start( GTK_BOX( attrbox ), widget, FALSE, TRUE, 0 );
+					gtk_box_pack_start( GTK_BOX( attrbox ), widget, FALSE, FALSE, 0 );
+					gtk_box_reorder_child( GTK_BOX( attrbox ), widget, 0 );
 				}
 				gtk_container_set_border_width( GTK_CONTAINER( attrbox ), 6 );
 				gtk_box_set_spacing( GTK_BOX( attrbox ), 6 );
@@ -909,7 +1028,7 @@ static gboolean screem_tag_tree_page_set( gpointer data )
 	tree = SCREEM_TAG_TREE( data );
 	page = tree->private->page;
 
-gdk_threads_enter();
+	gdk_threads_enter();
 	
 	if( page ) {
 		g_signal_connect(G_OBJECT( page ), "notify::dtd",
@@ -941,6 +1060,13 @@ static void screem_tag_tree_add_file( ScreemTagTree *tree,
 	g_return_if_fail( SCREEM_IS_TAGFILE( file ) );
 
 	model = screem_tag_file_get_model( file );
+
+	if( egg_tree_model_union_contains( EGG_TREE_MODEL_UNION( tree->private->umodel ), model ) ) {
+		g_warning( "attempted to add tag file twice\n" );
+		return;
+	}
+	
+	gdk_threads_enter();
 	
 	if( prepend ) {
 		egg_tree_model_union_prepend( EGG_TREE_MODEL_UNION( tree->private->umodel ), model );
@@ -948,6 +1074,8 @@ static void screem_tag_tree_add_file( ScreemTagTree *tree,
 		egg_tree_model_union_append( EGG_TREE_MODEL_UNION( tree->private->umodel ), model );
 
 	}
+
+	gdk_threads_leave();
 }
 
 static void screem_tag_tree_remove_file( ScreemTagTree *tree,
@@ -959,9 +1087,10 @@ static void screem_tag_tree_remove_file( ScreemTagTree *tree,
 	g_return_if_fail( SCREEM_IS_TAGFILE( file ) );
 
 	model = screem_tag_file_get_model( file );
-	
-	egg_tree_model_union_remove( EGG_TREE_MODEL_UNION( tree->private->umodel ), model );
 
+	if( egg_tree_model_union_contains( EGG_TREE_MODEL_UNION( tree->private->umodel ), model ) ) {
+		egg_tree_model_union_remove( EGG_TREE_MODEL_UNION( tree->private->umodel ), model );
+	}
 }
 
 static void screem_tag_file_active_set( ScreemTagFile *file,
@@ -974,6 +1103,9 @@ static void screem_tag_file_active_set( ScreemTagFile *file,
 	GSList *tmp;
 	xmlNodePtr node;
 	
+	const gchar *name;
+	const gchar *desc;
+	
 	g_mutex_lock( mutex );
 
 	uri = screem_tag_file_get_uri( file );
@@ -985,7 +1117,12 @@ static void screem_tag_file_active_set( ScreemTagFile *file,
 			TAG_FILE_URI_COL, uri );
 	g_assert( it );
 
+	name = screem_tag_file_get_name( file );
+	desc = screem_tag_file_get_desc( file );
+
 	gtk_list_store_set( filesstore, it,
+			TAG_FILE_NAME_COL, name,
+			TAG_FILE_DESC_COL, desc,
 			TAG_FILE_ACTIVE_COL, active, -1 );
 
 	gtk_tree_iter_free( it );
@@ -1096,12 +1233,11 @@ static gboolean screem_tag_tree_is_active( const gchar *url,
 	return ret;
 }
 
-static void screem_tag_tree_search( EggAction *action,
+static void screem_tag_tree_search( GtkAction *action,
 				gpointer data )
 {
 	ScreemTagTree *tree;
 	ScreemTagTreePrivate *priv;
-	GConfClient *client;
 	gchar *glade_path;
 	GtkWidget *widget;
 	GtkTreeModel *model;
@@ -1128,17 +1264,13 @@ static void screem_tag_tree_search( EggAction *action,
 		gtk_tree_path_free( path );
 	}
 	
-	client = gconf_client_get_default();
-	glade_path = gconf_client_get_string( client,
-					      "/apps/screem/general/glade_path",
-					      NULL );
+	glade_path = screem_get_glade_path();	
 	xml = glade_xml_new( glade_path, "find_tagtree", NULL );
 	g_free( glade_path );
-	g_object_unref( client );
 	
 	widget = glade_xml_get_widget( xml, "tagtrees" );
 	model = screem_top_level_model_new();
-	screem_top_level_model_set_model( SCREEM_TOP_LEVEL_MODEL( model ), priv->umodel );
+	screem_top_level_model_set_model( SCREEM_TOP_LEVEL_MODEL( model ), priv->sorted );
 	
 	selection = gtk_tree_view_get_selection( GTK_TREE_VIEW( widget ) );
 
@@ -1168,15 +1300,18 @@ static void screem_tag_tree_search( EggAction *action,
 	
 	gtk_tree_view_set_model( GTK_TREE_VIEW( widget ), model );
 	g_object_unref( model );
-
-	widget = glade_xml_get_widget( xml, "find_tagtree" );
-	gtk_widget_show_all( widget );
 	
 	if( i != -1 ) {
 		path = gtk_tree_path_new_from_indices( i, -1 );
 		gtk_tree_selection_select_path( selection, path );
+	
+		gtk_tree_view_scroll_to_cell( GTK_TREE_VIEW( widget ), 
+				path, col, TRUE, 0.0, 0.5 );
 		gtk_tree_path_free( path );
 	}
+
+	widget = glade_xml_get_widget( xml, "find_tagtree" );
+	gtk_widget_show_all( widget );
 	
 	if( gtk_dialog_run( GTK_DIALOG( widget ) ) == GTK_RESPONSE_OK ) {
 		widget = glade_xml_get_widget( xml, "pattern" );
@@ -1248,6 +1383,54 @@ static gboolean screem_tag_tree_search_func( GtkTreeModel *model,
 	return ( priv->search_result != NULL );
 }
 
+static gint screem_tag_tree_sort( GtkTreeModel *model,
+		GtkTreeIter *a, GtkTreeIter *b,
+		gpointer data )
+{
+	ScreemTagTree *tree;
+	gchar *aname;
+	gchar *bname;
+	gint ret;
+	
+	tree = SCREEM_TAG_TREE( data );
+	
+	gtk_tree_model_get( model, a,
+			FILE_BROWSER_NAME_COL, &aname,
+			-1 );
+	gtk_tree_model_get( model, b,
+			FILE_BROWSER_NAME_COL, &bname,
+			-1 );
+
+	ret = 0;
+	if( aname && ! bname ) {
+		ret = 1;
+	} else if( bname && ! aname ) {
+		ret = -1;
+	} else {
+		ret = strcmp( aname, bname );
+	}
+
+	return ret;
+}
+
+static gboolean screem_tag_tree_add_current( ScreemTagTree *tree )
+{
+	GSList *tmp;
+	
+	g_mutex_lock( mutex );
+	trees = g_slist_prepend( trees, tree );
+
+	for( tmp = treefiles; tmp; tmp = tmp->next ) {
+		screem_tag_tree_add_file( tree, 
+				SCREEM_TAGFILE( tmp->data ), FALSE );
+	}
+	g_mutex_unlock( mutex );
+
+	return FALSE;
+}
+
+
+
 /* G Object stuff */
 #define PARENT_TYPE GTK_TYPE_BIN
 
@@ -1296,25 +1479,25 @@ static void screem_tag_tree_init( ScreemTagTree *tag_tree )
 	GtkCellRenderer *prend;
 	GtkTreeViewColumn *col;
 	GtkTreeSelection *selection;
-	GSList *tmp;
 	gchar *dot;
 
-	static EggActionGroupEntry action_entries[] = {
+	static GtkActionEntry action_entries[] = {
 		{
-		   "SearchTagTree", N_( "Find in Tag Tree..." ),
-		   GTK_STOCK_FIND, NULL, N_( "Search the tag tree" ),
-		   G_CALLBACK( screem_tag_tree_search ), NULL
+		   "SearchTagTree", GTK_STOCK_FIND,
+		   N_( "Find in Tag Tree..." ),
+		   "", N_( "Search the tag tree" ),
+		   G_CALLBACK( screem_tag_tree_search )
 		},
 		{ 
-		   "Tag Help", N_( "Tag Help" ),
-		   GTK_STOCK_DIALOG_INFO, NULL, N_( "Help" ),
-	  	   G_CALLBACK( screem_tag_tree_tag_help ), NULL 
-		},
+		   "Tag Help", GTK_STOCK_DIALOG_INFO, N_( "Tag Help" ),
+		   "", N_( "Help" ),
+	  	   G_CALLBACK( screem_tag_tree_tag_help ) 
+		}
 	};
 	static guint action_n_entries=G_N_ELEMENTS( action_entries );
-	EggActionGroup *group;
-	EggMenuMerge *merge;
-	gint i;
+	GtkActionGroup *group;
+	GtkUIManager *merge;
+	GError *error;
 	
 	ScreemTagTreePrivate *priv;
 
@@ -1331,7 +1514,10 @@ static void screem_tag_tree_init( ScreemTagTree *tag_tree )
 				dot, "tagtreeconfig.xml",
 				NULL );
 		g_free( dot );
-		config = xmlParseFile( configpath );
+		config = NULL;
+		if( uri_exists( configpath, NULL ) ) {
+			config = xmlParseFile( configpath );
+		}
 		if( ! config ) {
 			config = xmlNewDoc( XML_DEFAULT_VERSION );
 			xmlDocSetRootElement( config,
@@ -1366,6 +1552,7 @@ static void screem_tag_tree_init( ScreemTagTree *tag_tree )
 	gtk_tree_view_column_pack_start( col, prend, FALSE );
 	gtk_tree_view_column_pack_start( col, rend, TRUE );
 	gtk_tree_view_column_set_resizable( col, TRUE );
+/*	gtk_tree_view_column_set_sizing( col, GTK_TREE_VIEW_COLUMN_FIXED );*/
 	gtk_tree_view_append_column( GTK_TREE_VIEW( priv->view ), col );
 	
 	gtk_tree_view_column_set_title( col, _( "Resources" ) );
@@ -1381,6 +1568,10 @@ static void screem_tag_tree_init( ScreemTagTree *tag_tree )
 	/* HIG says 1 column lists should have no header */
 	gtk_tree_view_set_headers_visible( GTK_TREE_VIEW( priv->view ), FALSE ); 
 
+/*	horrible behaviour do not enable
+ *	g_object_set( G_OBJECT( priv->view ), "fixed-height-mode", TRUE,
+			NULL );*/
+	
 	gtk_widget_show( priv->view );
 
 	g_signal_connect( G_OBJECT( priv->view ), "row_activated",
@@ -1396,35 +1587,33 @@ static void screem_tag_tree_init( ScreemTagTree *tag_tree )
 	gtk_box_pack_start( GTK_BOX( priv->box ), sw, TRUE, TRUE, 0 );
 
 
-	group = egg_action_group_new( "ScreemTagTreeActions" );
-	/* set user data field in action table */
-	for( i = 0; i < action_n_entries; ++ i ) {
-		action_entries[ i ].user_data = tag_tree;
-	}
-	egg_action_group_add_actions( EGG_ACTION_GROUP( group ),
+	group = gtk_action_group_new( "ScreemTagTreeActions" );
+	gtk_action_group_set_translation_domain( group, 
+			GETTEXT_PACKAGE );	
+
+	gtk_action_group_add_actions( GTK_ACTION_GROUP( group ),
 					action_entries,
-					action_n_entries );
-	merge = egg_menu_merge_new ();
-	egg_menu_merge_insert_action_group( merge, 
-					    EGG_ACTION_GROUP( group ), 
+					action_n_entries,
+					tag_tree );
+	merge = gtk_ui_manager_new ();
+	gtk_ui_manager_insert_action_group( merge, 
+					    GTK_ACTION_GROUP( group ), 
 					    0 );
 	g_signal_connect( merge, "add_widget", G_CALLBACK( add_widget ), tag_tree );
-  	egg_menu_merge_add_ui_from_file( merge, 
+  	if( ! gtk_ui_manager_add_ui_from_file( merge, 
 					 UIDATADIR"/screem-tag-tree-bar.xml", 
-					 NULL );
+					 &error ) ) {
+		g_message( "Tag Tree ui file error = %s", 
+				error->message );
+		g_error_free( error );
+	}
 	
 	gtk_widget_show_all( GTK_WIDGET( tag_tree ) );
 
 	screem_tag_tree_set_model( tag_tree );
 
-	g_mutex_lock( mutex );
-	trees = g_slist_prepend( trees, tag_tree );
-
-	for( tmp = treefiles; tmp; tmp = tmp->next ) {
-		screem_tag_tree_add_file( tag_tree, 
-				SCREEM_TAGFILE( tmp->data ), FALSE );
-	}
-	g_mutex_unlock( mutex );
+	g_idle_add( (GSourceFunc)screem_tag_tree_add_current,
+			tag_tree );
 }
 
 static void screem_tag_tree_finalize( GObject *object )
