@@ -35,8 +35,8 @@
 #define g_marshal_value_peek_ulong(v)    (v)->data[0].v_ulong
 #define g_marshal_value_peek_int64(v)    (v)->data[0].v_int64
 #define g_marshal_value_peek_uint64(v)   (v)->data[0].v_uint64
-#define g_marshal_value_peek_enum(v)     (v)->data[0].v_int
-#define g_marshal_value_peek_flags(v)    (v)->data[0].v_uint
+#define g_marshal_value_peek_enum(v)     (v)->data[0].v_long
+#define g_marshal_value_peek_flags(v)    (v)->data[0].v_ulong
 #define g_marshal_value_peek_float(v)    (v)->data[0].v_float
 #define g_marshal_value_peek_double(v)   (v)->data[0].v_double
 #define g_marshal_value_peek_string(v)   (v)->data[0].v_pointer
@@ -248,7 +248,44 @@ screem_marshal_VOID__STRING_STRING_POINTER (GClosure     *closure,
 
 /* VOID:OBJECT (./screemmarshal.list:32) */
 
-/* VOID:OBJECT,UINT,UINT (./screemmarshal.list:33) */
+/* VOID:OBJECT,OBJECT (./screemmarshal.list:33) */
+void
+screem_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
+                                    GValue       *return_value,
+                                    guint         n_param_values,
+                                    const GValue *param_values,
+                                    gpointer      invocation_hint,
+                                    gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__OBJECT_OBJECT) (gpointer     data1,
+                                                    gpointer     arg_1,
+                                                    gpointer     arg_2,
+                                                    gpointer     data2);
+  register GMarshalFunc_VOID__OBJECT_OBJECT callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 3);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__OBJECT_OBJECT) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_object (param_values + 1),
+            g_marshal_value_peek_object (param_values + 2),
+            data2);
+}
+
+/* VOID:OBJECT,UINT,UINT (./screemmarshal.list:34) */
 void
 screem_marshal_VOID__OBJECT_UINT_UINT (GClosure     *closure,
                                        GValue       *return_value,
@@ -287,7 +324,7 @@ screem_marshal_VOID__OBJECT_UINT_UINT (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,STRING,UINT (./screemmarshal.list:34) */
+/* VOID:STRING,STRING,UINT (./screemmarshal.list:35) */
 void
 screem_marshal_VOID__STRING_STRING_UINT (GClosure     *closure,
                                          GValue       *return_value,
@@ -326,7 +363,7 @@ screem_marshal_VOID__STRING_STRING_UINT (GClosure     *closure,
             data2);
 }
 
-/* VOID:UINT,UINT,STRING,STRING,STRING,STRING (./screemmarshal.list:35) */
+/* VOID:UINT,UINT,STRING,STRING,STRING,STRING (./screemmarshal.list:36) */
 void
 screem_marshal_VOID__UINT_UINT_STRING_STRING_STRING_STRING (GClosure     *closure,
                                                             GValue       *return_value,
@@ -371,3 +408,9 @@ screem_marshal_VOID__UINT_UINT_STRING_STRING_STRING_STRING (GClosure     *closur
             data2);
 }
 
+/* VOID:STRING (./screemmarshal.list:37) */
+
+/* VOID:BOOLEAN (./screemmarshal.list:38) */
+
+/* VOID:UINT (./screemmarshal.list:39) */
+
